/*!
 * Tine 2.0 - Expressomail 
 * Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * http://www.gnu.org/licenses/agpl.html AGPL Version 3
 */
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @copyright   Copyright (c) 2007-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 * TODO         think about adding a generic expressomail backend with the exception handler
 */
Ext.ns('Tine.Expressomail.Model');

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.MessageAttachments
 * @extends Tine.Tinebase.data.Record
 *
 * MessageAttachments Record Definition
 */
Tine.Expressomail.Model.MessageAttachments = Tine.Tinebase.data.Record.create([
      { name: 'id' },
      { name: 'attachments' }
    ], {
    appName: 'Expressomail',
    modelName: 'MessageContent',
    idProperty: 'id'
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.MessageContent
 * @extends Tine.Tinebase.data.Record
 *
 * MessageContent Record Definition
 */
Tine.Expressomail.Model.MessageContent = Tine.Tinebase.data.Record.create([
      { name: 'id' },
      { name: 'body' },
      { name: 'attachments' },
      { name: 'embedded_images' }
    ], {
    appName: 'Expressomail',
    modelName: 'MessageContent',
    idProperty: 'id'
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Message
 * @extends Tine.Tinebase.data.Record
 *
 * Message Record Definition
 */
Tine.Expressomail.Model.Message = Tine.Tinebase.data.Record.create([
      { name: 'id' },
      { name: 'account_id' },
      { name: 'subject' },
      { name: 'from_email' },
      { name: 'from_name' },
      { name: 'sender' },
      { name: 'sender_account' },
      { name: 'to' },
      { name: 'cc' },
      { name: 'bcc' },
      { name: 'sent',     type: 'date', dateFormat: Date.patterns.ISO8601Long },
      { name: 'received', type: 'date', dateFormat: Date.patterns.ISO8601Long },
      { name: 'flags' },
      { name: 'size' },
      { name: 'body',     defaultValue: undefined },
      { name: 'headers' },
      { name: 'content_type' },
      { name: 'body_content_type' },
      { name: 'structure' },
      { name: 'attachments' },
      { name: 'has_attachment', type: 'bool' },
      { name: 'original_id' },
      { name: 'draft_id' },
      { name: 'folder_id' },
      { name: 'note' },
      { name: 'add_contacts' }, // contains true if to add unknown contacts
      { name: 'added_contacts' }, // contains how many contacts where added (or -1 if error)
      { name: 'preparedParts' }, // contains invitation event record
      { name: 'reading_conf' },
      { name: 'smime' },
      { name: 'importance' },
      { name: 'smimeEml' },
      { name: 'signature_info' },      
      { name: 'embedded_images' },
      { name: 'error_message' }
    ], {
    appName: 'Expressomail',
    modelName: 'Message',
    idProperty: 'id',
    titleProperty: 'subject',
    // ngettext('Message', 'Messages', n);
    recordName: 'Message',
    recordsName: 'Messages',
    containerProperty: 'folder_id',
    // ngettext('Folder', 'Folders', n);
    containerName: 'Folder',
    containersName: 'Folders',

    /**
     * check if message has given flag
     *
     * @param  {String} flag
     * @return {Boolean}
     */
    hasFlag: function(flag) {
        var flags = this.get('flags') || [];
        return flags.indexOf(flag) >= 0;
    },

        /**
     * check if message has impertance flag
     *
     * @param  {String} flag
     * @return {Boolean}
     */
    isImportant: function() {
        var value = this.get('importance');
        if (value == true){
            return true;
        }

        return false;

    },

    /**
     * adds given flag to message
     *
     * @param  {String} flag
     * @return {Boolean} false if flag was already set before, else true
     */
    addFlag: function(flag) {
        if (! this.hasFlag(flag)) {
            var flags = Ext.unique(this.get('flags'));
            flags.push(flag);

            this.set('flags', flags);
            return true;
        }

        return false;
    },

    /**
     * check if body has been fetched
     *
     * @return {Boolean}
     */
    bodyIsFetched: function() {
        return this.get('body') !== undefined;
    },

    /**
     * clears given flag from message
     *
     * @param {String} flag
     * @return {Boolean} false if flag was not set before, else true
     */
    clearFlag: function(flag) {
        if (this.hasFlag(flag)) {
            var flags = Ext.unique(this.get('flags'));
            flags.remove(flag);

            this.set('flags', flags);
            return true;
        }

        return false;
    },

    /**
     * returns true if given record obsoletes this one
     *
     * NOTE: this does only work for Tine.widgets.grid.GridPanel::onStoreBeforeLoadRecords record comparison
     *
     * @param {Tine.Tinebase.data.Record} record
     * @return {Boolean}
     */
    isObsoletedBy: function(record) {
        return record.mtime || record.ctime > this.ctime;
    }
});

/**
 * get default message data
 *
 * @return {Object}
 */
Tine.Expressomail.Model.Message.getDefaultData = function() {
    var autoAttachNote = Tine.Expressomail.registry.get('preferences').get('autoAttachNote');

    return {
        note: autoAttachNote,
        content_type: 'text/html'
    };
};

/**
 * get filtermodel for messages
 *
 * @namespace Tine.Expressomail.Model
 * @static
 * @return {Object} filterModel definition
 */
Tine.Expressomail.Model.Message.getFilterModel = function() {
    var app = Tine.Tinebase.appMgr.get('Expressomail');

    return [
        {filtertype: 'tine.expressomail.folder.filtermodel', app: app},
        {label: app.i18n._('Subject/From/CC/Body'),field: 'query',         operators: ['contains']},
        {label: app.i18n._('Subject'),     field: 'subject',       operators: ['contains']},
        {label: app.i18n._('Body'),        field: 'body',          operators: ['contains']},
        {label: app.i18n._('From (Email)'),field: 'from_email',    operators: ['contains']},
        {label: app.i18n._('From (Name)'), field: 'from_name',     operators: ['contains']},
        {label: app.i18n._('To'),          field: 'to',            operators: ['contains']},
        {label: app.i18n._('Cc'),          field: 'cc',            operators: ['contains']},
        {label: app.i18n._('Bcc'),         field: 'bcc',           operators: ['contains']},
        {label: app.i18n._('Flags'),       field: 'flags',         filtertype: 'tinebase.multiselect', app: app, multiselectFieldConfig: {
            valueStore: Tine.Expressomail.loadFlagsStore()
        }},
        {label: app.i18n._('Received'),    field: 'received',      valueType: 'date', pastOnly: true}
    ];
};

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.messageBackend
 * @extends Tine.Tinebase.data.RecordProxy
 *
 * Message Backend
 *
 * TODO make clear/addFlags send filter as param instead of array of ids
 */
Tine.Expressomail.messageBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'Message',
    recordClass: Tine.Expressomail.Model.Message,

    /**
     * move messsages to folder
     *
     * @param  array $filterData filter data
     * @param  string $targetFolderId
     * @return  {Number} Ext.Ajax transaction id
     */
    moveMessages: function(filter, targetFolderId, options) {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.moveMessages';
        p.filterData = filter;
        p.targetFolderId = targetFolderId;

        options.beforeSuccess = function(response) {
            return [Tine.Expressomail.folderBackend.recordReader(response)];
        };

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },
    /**
     * searches all (lightweight) records matching filter
     * 
     * @param   {Object} filter
     * @param   {Object} paging
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Object} root:[records], totalcount: number
     */
    searchRecords: function(filter, paging, options) {
        options = options || {};
        options.params = options.params || {};
        
        var p = options.params;
        
        this.normalizeQuickFilterSearch(filter);

        p.method = this.appName + '.search' + this.modelName + 's';
        p.filter = (filter) ? filter : [];
        p.paging = paging;
        
        options.beforeSuccess = function(response) {
            return [this.jsonReader.read(response)];
        };
        
        // increase timeout as this can take a longer (1 minute)
        options.timeout = 300000;
        
        return this.doXHTTPRequest(options);
    },        

    /**
     * normalizes a quickFilter search: pushes the current path if it is
     * not in the filter criteria.
     * 
     * @param   {Object} filter
     */
    normalizeQuickFilterSearch: function(filter) {
        var isQuickFilter = false;

        Ext.each(filter, function(filterData) {
            isQuickFilter |= (filterData.id == "quickFilter");
        }, isQuickFilter);

        if(isQuickFilter) {
            var hasFolderData = false;
            Ext.each(filter, function(filterData) {
                hasFolderData |= (filterData.field == "path");
            }, hasFolderData);
            if(!hasFolderData) {
                folderFilterData = {
                    field: "path",
                    operator: "in",
                    value: [Tine.Tinebase.appMgr.get('Expressomail').mainScreen.GridPanel.getCurrentFolderFromTree().get("path")],
                    id: ""
                }
                filter.push(folderFilterData);
            }
        }
    },

    /**
     * fetches body and additional headers (which are needed for the preview panel) into given message
     *
     * @param {Message} message
     * @param {Function|Object} callback (NOTE: this has NOTHING to do with standard Ext request callback fn)
     */
    fetchBody: function(message, callback) {
        return this.loadRecord(message, {
            timeout: 120000, // 2 minutes
            nonblocking: true,
            scope: this,
            success: function(response, options) {
                var msg = this.recordReader({responseText: Ext.util.JSON.encode(response.data)});
                // NOTE: Flags from the server might be outdated, so we skip them
                Ext.copyTo(message.data, msg.data, Tine.Expressomail.Model.Message.getFieldNames().remove('flags'));
                if (Ext.isFunction(callback)) {
                    callback(message);
                } else if (callback.success) {
                    Ext.callback(callback.success, callback.scope, [message]);
                }
            },
            failure: function(exception) {
                if (callback.failure) {
                    Ext.callback(callback.failure, callback.scope, [exception]);
                } else {
                    this.handleRequestException(exception);
                }
            }
        });
    },

    /**
     * saves a message into a folder
     *
     * @param   {Ext.data.Record} record
     * @param   {String} folderName
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Ext.data.Record}
     */
    saveInFolder: function(record, folderName, options) {
        options = options || {};
        options.params = options.params || {};
        options.beforeSuccess = function(response) {
            return [this.recordReader(response)];
        };

        var p = options.params;
        p.method = this.appName + '.saveMessageInFolder';
        p.recordData = record.data;
        p.folderName = folderName;

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },

    /**
     * saves a draft message
     *
     * @param   {Ext.data.Record} record
     * @param   {String} folderName
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Ext.data.Record}
     */
    saveDraft: function(record, folderName, options) {
        options = options || {};
        options.params = options.params || {};
        options.beforeSuccess = function(response) {
            return [this.recordReader(response)];
        };

        var p = options.params;
        p.method = this.appName + '.saveDraftInFolder';
        p.recordData = record.data;
        p.folderName = folderName;

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },

    /**
     * report message(s) as phishing
     *
     * @param   {Ext.data.Record} record
     * @param  {Array} msgIds
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Ext.data.Record}
     */
    reportPhishing: function(record, msgIds, options)
    {
        var options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.reportPhishing';
        p.recordData = record.data;
        p.msgIds = msgIds;

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },

    /**
     * add given flags to given messages
     *
     * @param  {String/Array} ids
     * @param  {String/Array} flags
     */
    addFlags: function(ids, flags, options)
    {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.addFlags';
        p.filterData = ids;
        p.flags = flags;

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },

    /**
     * clear given flags from given messages
     *
     * @param  {String/Array} ids
     * @param  {String/Array} flags
     */
    clearFlags: function(ids, flags, options)
    {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.clearFlags';
        p.filterData = ids;
        p.flags = flags;

        // increase timeout as this can take a longer (5 minutes)
        options.timeout = 300000;

        return this.doXHTTPRequest(options);
    },

    /**
     * exception handler for this proxy
     *
     * @param {Tine.Exception} exception
     */
    handleRequestException: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
    }
});


/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Account
 * @extends Tine.Tinebase.data.Record
 *
 * Account Record Definition
 */
Tine.Expressomail.Model.Account = Tine.Tinebase.data.Record.create(Tine.Tinebase.Model.modlogFields.concat([
    { name: 'id' },
    { name: 'original_id' }, // client only, used in message compose dialog for accounts combo
    { name: 'user_id' },
    { name: 'name' },
    { name: 'type' },
    { name: 'user' },
    { name: 'host' },
    { name: 'email' },
    { name: 'password' },
    { name: 'from' },
    { name: 'organization' },
    { name: 'port' },
    { name: 'ssl' },
    { name: 'imap_status', defaultValue: 'success'}, // client only {success|failure}
    { name: 'sent_folder' },
    { name: 'trash_folder' },
    { name: 'drafts_folder' },
    { name: 'templates_folder' },
    { name: 'has_children_support', type: 'bool' },
    { name: 'delimiter' },
    { name: 'display_format' },
    { name: 'ns_personal' },
    { name: 'ns_other' },
    { name: 'ns_shared' },
    { name: 'signature' },
    { name: 'signature_position' },
    { name: 'smtp_port' },
    { name: 'smtp_hostname' },
    { name: 'smtp_auth' },
    { name: 'smtp_ssl' },
    { name: 'smtp_user' },
    { name: 'smtp_password' },
    { name: 'sieve_hostname' },
    { name: 'sieve_port' },
    { name: 'sieve_ssl' },
    { name: 'sieve_vacation_active', type: 'bool' },
    { name: 'all_folders_fetched', type: 'bool', defaultValue: false } // client only
]), {
    appName: 'Expressomail',
    modelName: 'Account',
    idProperty: 'id',
    titleProperty: 'name',
    // ngettext('Account', 'Accounts', n);
    recordName: 'Account',
    recordsName: 'Accounts',
    // ngettext('Email Accounts', 'Email Accounts', n);
    containerName: 'Email Accounts',
    containersName: 'Contas de Email',  //'Email Accounts' 

    /**
     * @type Object
     */
    lastIMAPException: null,

    /**
     * get the last IMAP exception
     *
     * @return {Object}
     */
    getLastIMAPException: function() {
        return this.lastIMAPException;
    },

    /**
     * returns sendfolder id
     * -> needed as trash is saved as globname :(
     */
    getSendFolderId: function() {
        var app = Ext.ux.PopupWindowMgr.getMainWindow().Tine.Tinebase.appMgr.get('Expressomail'),
            sendName = this.get('sent_folder'),
            accountId = this.id,
            send = sendName ? app.getFolderStore().queryBy(function(record) {
                return record.get('account_id') === accountId && record.get('globalname') === sendName;
            }, this).first() : null;

        return send ? send.id : null;
    },

    /**
     * returns trashfolder id
     * -> needed as trash is saved as globname :(
     */
    getTrashFolderId: function() {
        var app = Ext.ux.PopupWindowMgr.getMainWindow().Tine.Tinebase.appMgr.get('Expressomail'),
            trashName = this.get('trash_folder'),
            accountId = this.id,
            trash = trashName ? app.getFolderStore().queryBy(function(record) {
                return record.get('account_id') === accountId && record.get('globalname') === trashName;
            }, this).first() : null;

        return trash ? trash.id : null;
    },

    /**
     * set or clear IMAP exception and update imap_state
     *
     * @param {Object} exception
     */
    setLastIMAPException: function(exception) {
        this.lastIMAPException = exception;
        this.set('imap_status', exception ? 'failure' : 'success');
        this.commit();
    }
});

/**
 * get default data for account
 *
 * @return {Object}
 */
Tine.Expressomail.Model.Account.getDefaultData = function() {
    var defaults = (Tine.Expressomail.registry.get('defaults'))
        ? Tine.Expressomail.registry.get('defaults')
        : {};

    var currentUserDisplayName = Tine.Tinebase.registry.get('currentAccount').accountDisplayName;

    return {
        from: currentUserDisplayName,
        host: (defaults.host) ? defaults.host : '',
        port: (defaults.port) ? defaults.port : 143,
        smtp_hostname: (defaults.smtp && defaults.smtp.hostname) ? defaults.smtp.hostname : '',
        smtp_port: (defaults.smtp && defaults.smtp.port) ? defaults.smtp.port : 25,
        smtp_ssl: (defaults.smtp && defaults.smtp.ssl) ? defaults.smtp.ssl : 'none',
        sieve_port: 2000,
        sieve_ssl: 'none',
        signature: 'Sent with love from the new tine 2.0 email client ...<br/>'
            + 'Please visit <a href="http://www.tine20.com">http://www.tine20.com</a>',
        sent_folder: (defaults.sent_folder) ? defaults.sent_folder : 'Sent',
        trash_folder: (defaults.trash_folder) ? defaults.trash_folder : 'Trash'
    };
};

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.accountBackend
 * @extends Tine.Tinebase.data.RecordProxy
 *
 * Account Backend
 */
Tine.Expressomail.accountBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'Account',
    recordClass: Tine.Expressomail.Model.Account
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Record
 * @extends Ext.data.Record
 *
 * Folder Record Definition
 */
Tine.Expressomail.Model.Folder = Tine.Tinebase.data.Record.create([
      { name: 'id' },
      { name: 'localname' },
      { name: 'globalname' },
      { name: 'path' }, // /accountid/folderid/...
      { name: 'parent' },
      { name: 'parent_path' }, // /accountid/folderid/...
      { name: 'account_id' },
      { name: 'has_children',       type: 'bool' },
      { name: 'is_selectable',      type: 'bool' },
      { name: 'system_folder',      type: 'bool' },
      { name: 'imap_status' },
      { name: 'imap_timestamp',     type: 'date', dateFormat: Date.patterns.ISO8601Long },
      { name: 'imap_uidvalidity',   type: 'int' },
      { name: 'imap_totalcount',    type: 'int' },
      { name: 'cache_status' },
      { name: 'cache_recentcount',  type: 'int' },
      { name: 'cache_totalcount',   type: 'int' },
      { name: 'cache_unreadcount',  type: 'int' },
      { name: 'cache_timestamp',    type: 'date', dateFormat: Date.patterns.ISO8601Long  },
      { name: 'cache_job_actions_est',     type: 'int' },
      { name: 'cache_job_actions_done',         type: 'int' },
      { name: 'quota_usage',         type: 'int' },
      { name: 'quota_limit',         type: 'int' },
      { name: 'client_access_time', type: 'date', dateFormat: Date.patterns.ISO8601Long  }, // client only {@see Tine.Expressomail.folderBackend#updateMessageCache}
      { name: 'unread_children', type: 'Array', defaultValue: [] } // client only / array of unread child ids
], {
    // translations for system folders:
    // _('INBOX') _('Drafts') _('Sent') _('Templates') _('Junk') _('Trash')

    appName: 'Expressomail',
    modelName: 'Folder',
    idProperty: 'id',
    titleProperty: 'localname',
    // ngettext('Folder', 'Folders', n);
    recordName: 'Folder',
    recordsName: 'Folders',
    // ngettext('record list', 'record lists', n);
    containerName: 'Folder list',
    containersName: 'Folder lists',

    /**
     * is this folder the currently selected folder
     *
     * @return {Boolean}
     */
    isCurrentSelection: function() {
        if (Tine.Tinebase.appMgr.get(this.appName).getMainScreen().getTreePanel()) {
            // get active node
            var node = Tine.Tinebase.appMgr.get(this.appName).getMainScreen().getTreePanel().getSelectionModel().getSelectedNode();
            if (node && node.attributes.folder_id) {
                return node.id == this.id;
            }
        }

        return false;
    },

    /**
     * is this folder an inbox?
     *
     * @return {Boolean}
     */
    isInbox: function() {
        return Ext.util.Format.lowercase(this.get('localname')) === 'inbox';
    },

    /**
     * returns true if current folder needs an update
     */
    needsUpdate: function(updateInterval) {
        if (! this.get('is_selectable')) {
            return false;
        }

        var timestamp = this.get('client_access_time');
        return this.get('cache_status') !== 'complete' || ! Ext.isDate(timestamp) || timestamp.getElapsed() > updateInterval;
    }
});

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.folderBackend
 * @extends Tine.Tinebase.data.RecordProxy
 *
 * Folder Backend
 */
Tine.Expressomail.folderBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'Folder',
    recordClass: Tine.Expressomail.Model.Folder,

    /**
     * update message cache of given folder for given execution time and sets the client_access_time
     *
     * @param   {String} folderId
     * @param   {Number} executionTime (seconds)
     * @return  {Number} Ext.Ajax transaction id
     */
    updateMessageCache: function(folderId, executionTime, options) {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.updateMessageCache';
        p.folderId = folderId;
        p.time = executionTime;

        options.beforeSuccess = function(response) {
            var folder = this.recordReader(response);
            folder.set('client_access_time', new Date());
            return [folder];
        };

        // give 5 times more before timeout
        options.timeout = executionTime * 5000;

        return this.doXHTTPRequest(options);
    },

    /**
     * exception handler for this proxy
     *
     * @param {Tine.Exception} exception
     */
    handleRequestException: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
    }
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Vacation
 * @extends Tine.Tinebase.data.Record
 *
 * Vacation Record Definition
 */
Tine.Expressomail.Model.Vacation = Tine.Tinebase.data.Record.create(Tine.Tinebase.Model.modlogFields.concat([
    { name: 'id' },
    { name: 'reason' },
    { name: 'enabled', type: 'boolean'},
    { name: 'days' },
    { name: 'start_date', type: 'date' },
    { name: 'end_date', type: 'date' },
    { name: 'contact_ids' },
    { name: 'template_id' },
    { name: 'signature' },
    { name: 'mime' }
]), {
    appName: 'Expressomail',
    modelName: 'Vacation',
    idProperty: 'id',
    titleProperty: 'id',
    // ngettext('Vacation', 'Vacations', n);
    recordName: 'Vacation',
    recordsName: 'Vacations',
    // ngettext('record list', 'record lists', n);
    containerName: 'Vacation list',
    containersName: 'Vacation lists'
});

/**
 * get default data for vacation
 *
 * @return {Object}
 */
Tine.Expressomail.Model.Vacation.getDefaultData = function() {
    return {
        days: 7,
        mime: 'multipart/alternative'
    };
};

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.vacationBackend
 * @extends Tine.Tinebase.data.RecordProxy
 *
 * Vacation Backend
 */
Tine.Expressomail.vacationBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'Vacation',
    recordClass: Tine.Expressomail.Model.Vacation,

    /**
     * exception handler for this proxy
     *
     * @param {Tine.Exception} exception
     */
    handleRequestException: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
    }
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Rule
 * @extends Tine.Tinebase.data.Record
 *
 * Rule Record Definition
 */
Tine.Expressomail.Model.Rule = Tine.Tinebase.data.Record.create(Tine.Tinebase.Model.modlogFields.concat([
    { name: 'id', sortType: function(value) {
        // should be sorted as int
        return parseInt(value, 10);
    }
    },
    { name: 'action_type' },
    { name: 'action_argument' },
    { name: 'enabled', type: 'boolean'},
    { name: 'conditions' },
    { name: 'account_id' }
]), {
    appName: 'Expressomail',
    modelName: 'Rule',
    idProperty: 'id',
    titleProperty: 'id',
    // ngettext('Rule', 'Rules', n);
    recordName: 'Regra',
    recordsName: 'Regras',
    // ngettext('record list', 'record lists', n);
    containerName: 'Rule list',
    containersName: 'Rule lists'
});

/**
 * get default data for rules
 *
 * @return {Object}
 */
Tine.Expressomail.Model.Rule.getDefaultData = function() {
    return {
        enabled: true,
        conditions: [{
            test: 'address',
            header: 'from',
            comperator: 'contains',
            key: ''
        }],
        action_type: 'fileinto',
        action_argument: ''
    };
};

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.rulesBackend
 * @extends Tine.Tinebase.data.RecordProxy
 *
 * Rule Backend
 */
Tine.Expressomail.rulesBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'Rule',
    recordClass: Tine.Expressomail.Model.Rule,

    /**
     * searches all (lightweight) records matching filter
     *
     * @param   {Object} filter accountId
     * @param   {Object} paging
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Object} root:[records], totalcount: number
     */
    searchRecords: function(filter, paging, options) {
        options = options || {};
        options.params = options.params || {};
        var p = options.params;

        p.method = this.appName + '.get' + this.modelName + 's';
        p.accountId = filter;

        options.beforeSuccess = function(response) {
            return [this.jsonReader.read(response)];
        };

        // increase timeout as this can take a longer (1 minute)
        options.timeout = 60000;

        return this.doXHTTPRequest(options);
    },

    /**
     * save sieve rules
     *
     * @param  {String}     accountId
     * @param  {Array}      rules
     * @param  {Object}     options
     */
    saveRules: function(accountId, rules, options)
    {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.saveRules';
        p.accountId = accountId;
        p.rulesData = rules;

        return this.doXHTTPRequest(options);
    },

     /**
     * retrieve sieve rules
     *
     * @param   {String}     accountId
     * @param   {Object}     options
     * @return  {Object}     Ext.Ajax transaction results
     */
    getRules: function(accountId, options)
    {
        options = options || {};
        options.params = options.params || {};

        var p = options.params;

        p.method = this.appName + '.getRules';
        p.accountId = accountId;
        p.part = options.parts;

        options.timeout = 3000;

        return this.doXHTTPRequest(options);
    },


    /**
     * saves a single record
     *
     * NOTE: Single rule records can't be saved
     *
     * @param   {Ext.data.Record} record
     * @param   {Object} options
     * @return  {Number} Ext.Ajax transaction id
     * @success {Ext.data.Record}
     */
    saveRecord: function(record, options, additionalArguments) {
        // does nothing
    },

    /**
     * exception handler for this proxy
     *
     * @param {Tine.Exception} exception
     */
    handleRequestException: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
    }
});

/**
 * @namespace Tine.Expressomail.Model
 * @class Tine.Expressomail.Model.Flag
 * @extends Tine.Tinebase.data.Record
 *
 * Flag Record Definition
 */
Tine.Expressomail.Model.Flag = Tine.Tinebase.data.Record.create(Tine.Tinebase.Model.modlogFields.concat([
    { name: 'id' },
    { name: 'name' }
]), {
    appName: 'Expressomail',
    modelName: 'Flag',
    idProperty: 'id',
    titleProperty: 'id',
    // ngettext('Flag', 'Flags', n);
    recordName: 'Flag',
    recordsName: 'Flags',
    // ngettext('Flag list', 'Flag lists', n);
    containerName: 'Flag list',
    containersName: 'Flag lists'
});

Tine.Expressomail.Model.Acl = Ext.data.Record.create([
    {name: 'account_id'},
    {name: 'account_name', sortType: Tine.Tinebase.common.accountSortType},
    {name: 'readacl',    type: 'boolean'},
    {name: 'writeacl',     type: 'boolean'},
    {name: 'sendacl',     type: 'boolean'}
]);
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @copyright   Copyright (c) 2011-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.ns('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.GridPanelHook
 * 
 * <p>Expressomail Gridpanel Hook</p>
 * <p>
 * </p>
 * 
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @constructor
 */
Tine.Expressomail.GridPanelHook = function(config) {
    Ext.apply(this, config);

    Tine.log.info('Tine.Expressomail.GridPanelHook::Initialising Expressomail ' + this.foreignAppName + ' hooks.');
    
    // NOTE: due to the action updater this action is bound the the adb grid only!
    this.composeMailAction = new Ext.Action({
        actionType: 'add',
        text: this.app.i18n._('Compose email'),
        iconCls: this.app.getIconCls(),
        disabled: true,
        scope: this,
        actionUpdater: this.updateAction,
        handler: this.onComposeEmail,
        listeners: {
            scope: this,
            render: this.onRender
        }
    });
    
    this.composeMailBtn = Ext.apply(new Ext.Button(this.composeMailAction), {
        scale: 'medium',
        rowspan: 2,
        iconAlign: 'top'
    });


    // register in toolbar + contextmenu
    Ext.ux.ItemRegistry.registerItem(this.foreignAppName + '-' + this.recordTypeName + '-GridPanel-ActionToolbar-leftbtngrp', this.composeMailBtn, 30);
    Ext.ux.ItemRegistry.registerItem(this.foreignAppName + '-' + this.recordTypeName + '-GridPanel-ContextMenu', this.composeMailAction, 80);
    
    if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1' && Tine.Tinebase.registry.get('preferences').get('windowtype')=='Ext') {
        this.composeMailAction_encrypted = new Ext.Action({
            actionType: 'add',
            text: this.app.i18n._('Compose email (encrypted)'),
            iconCls: this.app.getIconCls(),
            disabled: true,
            scope: this,
            encrypted: true,
            actionUpdater: this.updateAction,
            handler: this.onComposeEmail,
            listeners: {
                scope: this,
                render: this.onRender
            }
        });

        this.composeMailBtn_encrypted = Ext.apply(new Ext.Button(this.composeMailAction_encrypted), {
            scale: 'medium',
            rowspan: 2,
            iconAlign: 'top'
        });
        
        Ext.ux.ItemRegistry.registerItem(this.foreignAppName + '-' + this.recordTypeName + '-GridPanel-ActionToolbar-leftbtngrp', this.composeMailBtn_encrypted, 30);
        Ext.ux.ItemRegistry.registerItem(this.foreignAppName + '-' + this.recordTypeName + '-GridPanel-ContextMenu', this.composeMailAction_encrypted, 80);
    }
};

Ext.apply(Tine.Expressomail.GridPanelHook.prototype, {
    
    /**
     * @property app
     * @type Tine.Expressomail.Application
     * @private
     */
    app: null,
    
    /**
     * foreign application name
     * @type String
     */
    foreignAppName: null,
    
    /**
     * @property composeMailAction
     * @type Tine.widgets.ActionUpdater
     * @private
     */
    composeMailAction: null,
    
    /**
     * @property composeMailBtn
     * @type Ext.Button
     * @private
     */
    composeMailBtn: null,
    
    /**
     * @property gridPanel
     * @type Tine.Addressbook.gridPanel
     * @private
     */
    gridPanel: null,
    
    contactInRelation: false,
    relationType: null,

    /**
     * set to true if a email (or multiple emails) happen to be list
     * @property listEmails
     * @private
     */
    listEmails: false,

    /**
     * get addressbook contact grid panel
     */
    getGridPanel: function() {
        if (! this.gridPanel) {
            this.gridPanel = Tine.Tinebase.appMgr.get(this.foreignAppName).getMainScreen().getCenterPanel();
        }
        return this.gridPanel;
    },
    
    /**
     * return mail addresses of given contacts 
     * 
     * @param {Array} contacts
     * @param {String} prefered
     * @return {Array}
     */
    getMailAddresses: function(records) {
        var mailAddresses = [];
        
        Ext.each(records, function(record) {
            if ((record.data.type != 'list') && (record.data.type != 'group')){
                if (this.contactInRelation && record.get('relations')) {
                    Ext.each(record.get('relations'), function(relation) {
                        if (relation.type === this.relationType) {
                        this.addMailFromAddressbook(mailAddresses, relation.related_record);
                        }
                    }, this);
                } else {
                    this.addMailFromAddressbook(mailAddresses, record);
                }
            }else {
                this.listEmails = true;
                mailAddresses.push(record.data.id);
            }
        }, this);

        Tine.log.debug('Tine.Expressomail.GridPanelHook::getMailAddresses - Got ' + mailAddresses.length + ' email addresses.');
        if (mailAddresses.length > 0) {
            Tine.log.debug(mailAddresses);
        }

        return mailAddresses;
    },
    
    /**
     * add mail address from contact (if available) and add it to mailAddresses array
     * 
     * @param {Array} mailAddresses
     * @param {Tine.Addressbook.Model.Contact|Object} contact
     */
    addMailFromAddressbook: function(mailAddresses, contact) {
        if (! contact) {
            return;
        }
        if (! Ext.isFunction(contact.beginEdit)) {
            contact = new Tine.Addressbook.Model.Contact(contact);
        }
        
        if (!contact.get("members")) {
            mailAddresses.push(Tine.Expressomail.getEmailStringFromContact(contact));
        }
    },

    /**
     * compose an email to selected contacts
     * 
     * @param {Button} btn 
     */
    onComposeEmail: function(btn) {
        var sm = this.getGridPanel().grid.getSelectionModel(),
            mailAddresses = sm.isFilterSelect ? null : this.getMailAddresses(this.getGridPanel().grid.getSelectionModel().getSelections());
        if (this.listEmails){
            var strText = 'Fetching emails from the list';
            if (mailAddresses.length > 1){
                strText = 'Fetching emails from the lists';
            }
            Ext.Msg.wait(this.app.i18n.gettext(strText), this.app.i18n.gettext('Please wait'));
            Ext.Ajax.request({
                scope: this,
                params: {
                    method: 'Addressbook.searchListContactsFormated',
                    listId: mailAddresses
                },
                success: function (response, request) {
                    var recordData = Ext.util.JSON.decode(response.responseText);
                    Ext.Msg.hide();
                    this.openEmailWindow(btn, sm, recordData);
                },
                failure: function (response, request) {
                    Ext.MessageBox.alert(this.app.i18n.gettext('Error'),
                    this.app.i18n.gettext('A error happened while fetching the List emails'));
                }
            });
        }else {
            this.openEmailWindow(btn, sm, mailAddresses);
        }
    },

    /**
     * Open the edit email window
     */
    openEmailWindow: function(btn, sm, mailAddresses){
        var popupWindow = Tine.Expressomail.MessageEditDialog.openWindow({
            encrypted: btn.encrypted,
            mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
            selectionFilter: sm.isFilterSelect ? Ext.encode(sm.getSelectionFilter()) : null,
            mailAddresses: mailAddresses ? Ext.encode(mailAddresses) : null
        });
    },

    /**
     * add to action updater the first time we render
     */
    onRender: function() {
        var actionUpdater = this.getGridPanel().actionUpdater,
            registeredActions = actionUpdater.actions;
            
        if (registeredActions.indexOf(this.composeMailAction) < 0) {
            actionUpdater.addActions([this.composeMailAction]);
        }
            
        if (registeredActions.indexOf(this.composeMailAction_encrypted) < 0) {
            actionUpdater.addActions([this.composeMailAction_encrypted]);
        }
    },
    
    /**
     * updates compose button
     * 
     * @param {Ext.Action} action
     * @param {Object} grants grants sum of grants
     * @param {Object} records
     */
    updateAction: function(action, grants, records) {
        var sm = this.getGridPanel().grid.getSelectionModel();
        action.setDisabled(this.getMailAddresses(sm.getSelections()).length == 0);
    }
});
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.FolderStore
 * @extends     Ext.data.Store
 * 
 * <p>Expressomail folder store</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * 
 * @constructor
 * Create a new  Tine.Expressomail.FolderStore
 */
Tine.Expressomail.FolderStore = function(config) {
    config = config || {};
    Ext.apply(this, config);
    
    this.reader = Tine.Expressomail.folderBackend.getReader();
    this.queriesPending = [];
    this.queriesDone = [];

    Tine.Expressomail.FolderStore.superclass.constructor.call(this);
    
    this.on('load', this.onStoreLoad, this);
    this.on('add', this.onStoreAdd, this);
    this.on('loadexception', this.onStoreLoadException, this);
};

Ext.extend(Tine.Expressomail.FolderStore, Ext.data.Store, {
    
    fields: Tine.Expressomail.Model.Folder,
    proxy: Tine.Expressomail.folderBackend,
    
    /**
     * @property queriesDone
     * @type Array
     */
    queriesDone: null,
    
    /**
     * @property queriesPending
     * @type Array
     */
    queriesPending: null,
    
    /**
     * async query
     */
    asyncQuery: function(field, value, callback, args, scope, store) {
        var result = null,
            key = store.getKey(field, value);
        
        Tine.log.info('Tine.Expressomail.FolderStore.asyncQuery: ' + key);
        
        if (store.queriesDone.indexOf(key) >= 0) {
            Tine.log.debug('result already loaded -> directly query store');
            // we need regexp here because query returns all records with path that begins with the value string otherwise
            var valueReg = new RegExp(value + '$');
            result = store.query(field, valueReg);
            args.push(result);
            callback.apply(scope, args);
        } else if (store.queriesPending.indexOf(key) >= 0) {
            Tine.log.debug('result not in store yet, but async query already running -> wait a bit');
            this.asyncQuery.defer(2500, this, [field, value, callback, args, scope, store]);
        } else {
            Tine.log.debug('result is requested the first time -> fetch from server');
            var accountId = value.match(/^\/([a-z0-9]*)/i)[1],
                folderIdMatch = value.match(/[a-z0-9]+\/([a-z0-9]*)$/i),
                folderId = (folderIdMatch) ? folderIdMatch[1] : null,
                folder = folderId ? store.getById(folderId) : null;
            
            if (folderId && ! folder) {
                Tine.log.warn('folder ' + folderId + ' not found -> performing no query at all');
                callback.apply(scope, args);
                return;
            }
            
            store.queriesPending.push(key);
            store.load({
                path: value,
                params: {filter: [
                    {field: 'account_id', operator: 'equals', value: accountId},
                    {field: 'globalname', operator: 'equals', value: (folder !== null) ? folder.get('globalname') : ''}
                ]},
                callback: function () {
                    store.queriesDone.push(key);
                    store.queriesPending.remove(key);
                    
                    // query store again (it should have the new folders now) and call callback function to add nodes
                    result = store.query(field, value);
                    args.push(result);
                    callback.apply(scope, args);
                },
                add: true
            });
        }
    },
    
    /**
     * on store load exception
     * 
     * @param {Tine.Tinebase.data.RecordProxy} proxy
     * @param {String} type
     * @param {Object} error
     * @param {Object} options
     * 
     * TODO remove loading class / remove from queriesDone?
     */
    onStoreLoadException: function(proxy, type, error, options) {
        //var node = options.params.path
        //node.getUI().removeClass("x-tree-node-loading");

        // Remove failed queries from queriesPending
        var key = this.getKey('parent_path', options.path);
        this.queriesPending.remove(key);
        Tine.Expressomail.handleRequestException(error);
    },
    
    
    /**
     * check if query has already loaded or is loading
     * 
     * @param {String} field
     * @param {String} value
     * @return {boolean}
     */
    isLoadedOrLoading: function(field, value) {
        var key = this.getKey(field, value),
            result = false;
        
        result = (this.queriesDone.indexOf(key) >= 0 || this.queriesPending.indexOf(key) >= 0);
        
        return result;
    },
    
    /**
     * get key to store query 
     * 
     * @param  {string} field
     * @param  {mixed} value
     * @return {string}
     */
    getKey: function(field, value) {
        return field + ' -> ' + value;
    },
    
    /**
     * load event handler
     * 
     * @param {Tine.Expressomail.FolderStore} store
     * @param {Tine.Expressomail.Model.Folder} records
     * @param {Object} options
     */
    onStoreLoad: function(store, records, options) {
        this.computePaths(records, options.path);
    },
    
    /**
     * add event handler
     * 
     * @param {Tine.Expressomail.FolderStore} store
     * @param {Tine.Expressomail.Model.Folder} records
     * @param {Integer} index
     */
    onStoreAdd: function(store, records, index) {
        this.computePaths(records, null);
    },

    /**
     * compute paths for folder records
     * 
     * @param {Tine.Expressomail.Model.Folder} records
     * @param {String|null} parentPath
     */
    computePaths: function(records, givenParentPath) {
        var parentPath, path;
        Ext.each(records, function(record) {
            if (givenParentPath === null) {
                var parent = this.getParent(record);
                parentPath = (parent) ? parent.get('path') : '/' + record.get('account_id');
            } else {
                parentPath = givenParentPath;
            }
            path = parentPath + '/' + record.id;
            
            if (record.get('parent_path') != parentPath || record.get('path') != path) {
                record.beginEdit();
                record.set('parent_path', parentPath);
                record.set('path', path);
                record.endEdit();
            }
        }, this);
    },
    
    /**
     * resets the query and removes all records that match it
     * 
     * @param {String} field
     * @param {String} value
     */
    resetQueryAndRemoveRecords: function(field, value) {
        this.queriesPending.remove(this.getKey(field, value));
        var toRemove = this.query(field, value);
        toRemove.each(function(record) {
            this.remove(record);
            this.queriesDone.remove(this.getKey(field, record.get(field)));
        }, this);
    },
    
    /**
     * update folder in this store
     * 
     * NOTE: parent_path and path are computed onLoad and must be preserved
     * 
     * @param {Array/Tine.Expressomail.Model.Folder} update
     * @return {Tine.Expressomail.Model.Folder}
     */
    updateFolder: function(update) {
        if (Ext.isArray(update)) {
            Ext.each(update, function(u) {this.updateFolder.call(this, u)}, this);
            return;
        }
        
        var folder = this.getById(update.id);
        
        if (folder) {
            folder.beginEdit();
            Ext.each(Tine.Expressomail.Model.Folder.getFieldNames(), function(f) {
                if (! f.match('path')) {
                    folder.set(f, update.get(f));
                }
            }, this);
            folder.endEdit();
            folder.commit();
            return folder;
        }
    },
    
    /**
     * get parent folder
     * 
     * @param {Tine.Expressomail.Model.Folder} folder
     * @return {Tine.Expressomail.Model.Folder|null}
     */
    getParent: function(folder) {
        var result = this.queryBy(function(record, id) {
            if (record.get('account_id') == folder.get('account_id') && record.get('globalname') == folder.get('parent')) {
                return true;
            }
        });
        
        return result.first() || null;
    }    
});

/*
 * Tine 2.0
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 */
Ext.ns('Tine.Expressomail');

/**
 * folder select trigger field
 * 
 * @namespace   Tine.widgets.container
 * @class       Tine.Expressomail.FolderSelectTriggerField
 * @extends     Ext.form.ComboBox
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * 
 */
Tine.Expressomail.FolderSelectTriggerField = Ext.extend(Ext.form.TriggerField, {
    
    triggerClass: 'x-form-search-trigger',
    account: null,
    allAccounts: false,
    
    /**
     * onTriggerClick
     * open ext window with (folder-)select panel that fires event on select
     * 
     * @param e
     */
    onTriggerClick: function(e) {
        if (! this.disabled && (this.account && this.account.id !== 0) || this.allAccounts) {
            this.selectPanel = Tine.Expressomail.FolderSelectPanel.openWindow({
                account: this.account,
                allAccounts: this.allAccounts,
                listeners: {
                    // NOTE: scope has to be first item in listeners! @see Ext.ux.WindowFactory
                    scope: this,
                    'folderselect': this.onSelectFolder
                }
            });
        }
    },
    
    /**
     * select folder event listener
     * 
     * @param {Ext.tree.AsyncTreeNode} node
     */
    onSelectFolder: function(node) {
        this.selectPanel.close();
        this.setValue(node.attributes.globalname);
        this.el.focus();
    }
});
Ext.reg('expressomailfolderselect', Tine.Expressomail.FolderSelectTriggerField);
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.FolderSelectPanel
 * @extends     Ext.Panel
 * 
 * <p>Account/Folder Tree Panel</p>
 * <p>Tree of Accounts with folders</p>
 * <pre>
 * TODO         show error if no account(s) available
 * TODO         make it possible to preselect folder
 * TODO         use it for folder subscriptions
 * </pre>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.FolderSelectPanel
 */
Tine.Expressomail.FolderSelectPanel = Ext.extend(Ext.Panel, {
    
    /**
     * Panel config
     * @private
     */
    frame: true,
    border: true,
    autoScroll: true,
    bodyStyle: 'background-color:white',
    selectedNode: null,
    
    /**
     * init
     * @private
     */
    initComponent: function() {
        this.addEvents(
            /**
             * @event folderselect
             * Fired when folder is selected
             */
            'folderselect'
        );

        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        
        if (! this.allAccounts) {
            this.account = this.account || this.app.getActiveAccount();
        }
        
        this.initActions();
        this.initFolderTree();
        
        Tine.Expressomail.FolderSelectPanel.superclass.initComponent.call(this);
    },
    
    /**
     * init actions
     */
    initActions: function() {
        this.action_cancel = new Ext.Action({
            text: _('Cancel'),
            minWidth: 70,
            scope: this,
            handler: this.onCancel,
            iconCls: 'action_cancel'
        });
        
        this.action_ok = new Ext.Action({
            disabled: true,
            text: _('Ok'),
            iconCls: 'action_saveAndClose',
            minWidth: 70,
            handler: this.onOk,
            scope: this
        });
        
        this.fbar = [
            '->',
            this.action_cancel,
            this.action_ok
        ];
    },
        
    /**
     * init folder tree
     */
    initFolderTree: function() {
        
        if (this.allAccounts) {

            this.root = new Ext.tree.TreeNode({
                text: 'default',
                draggable: false,
                allowDrop: false,
                expanded: true,
                leaf: false,
                id: 'root'
            });
        
            var mainApp = Ext.ux.PopupWindowMgr.getMainWindow().Tine.Tinebase.appMgr.get('Expressomail');
            mainApp.getAccountStore().each(function(record) {
                // TODO generalize this
                var node = new Ext.tree.AsyncTreeNode({
                    id: record.data.id,
                    path: '/' + record.data.id,
                    record: record,
                    globalname: '',
                    draggable: false,
                    allowDrop: false,
                    expanded: false,
                    text: record.get('name'),
                    qtip: Tine.Tinebase.common.doubleEncode(record.get('host')),
                    leaf: false,
                    cls: 'expressomail-node-account',
                    delimiter: record.get('delimiter'),
                    ns_personal: record.get('ns_personal'),
                    account_id: record.data.id
                });
            
                this.root.appendChild(node);
            }, this);
            
        } else {
            this.root = new Ext.tree.AsyncTreeNode({
                text: this.account.get('name'),
                draggable: false,
                allowDrop: false,
                expanded: true,
                leaf: false,
                cls: 'expressomail-node-account',
                id: this.account.id,
                path: '/' + this.account.id
            });
        }
        
        
        this.folderTree = new Ext.tree.TreePanel({
            id: 'expressomail-foldertree',
            rootVisible: ! this.allAccounts,
            store: this.store || this.app.getFolderStore(),
            // TODO use another loader/store for subscriptions
            loader: this.loader || new Tine.Expressomail.TreeLoaderSieve({
                folderStore: this.store,
                app: this.app
            }),
            root: this.root
        });
        this.folderTree.on('dblclick', this.onTreeNodeDblClick, this);
        this.folderTree.on('click', this.onTreeNodeClick, this);
        
        this.items = [this.folderTree];
    },
    
    /**
     * @private
     */
    afterRender: function() {
        Tine.Expressomail.FolderSelectPanel.superclass.afterRender.call(this);
        
        var title = (! this.allAccounts) 
            ? String.format(this.app.i18n._('Folders of account {0}'), this.account.get('name'))
            : this.app.i18n._('Folders of all accounts');
            
        this.window.setTitle(title);
    },

    /**
     * on folder select handler
     * 
     * @param {Ext.tree.AsyncTreeNode} node
     * @private
     */
    onTreeNodeDblClick: function(node) {
        this.selectedNode = node;
        this.onOk();
        return false;
    },
    
    /**
     * @private
     */
    onTreeNodeClick: function(node) {
        this.selectedNode = node;
        if(node.attributes.is_selectable)
            this.action_ok.setDisabled(false);
        else 
            this.action_ok.setDisabled(true);
    },
    
    /**
     * @private
     */
    onCancel: function(){
        this.purgeListeners();
        this.window.close();
    },
    
    /**
     * @private
     */
    onOk: function() {
        if (this.selectedNode) {
            this.fireEvent('folderselect', this.selectedNode);
        }
    }
});

/**
 * Expressomail FolderSelectPanel Popup
 * 
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.FolderSelectPanel.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 200,
        height: 300,
        modal: true,
        name: Tine.Expressomail.FolderSelectPanel.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.FolderSelectPanel',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail.sieve');

/**
 * @namespace  Tine.Expressomail.sieve
 * @class       Tine.Expressomail.sieve.VacationEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Sieve Filter Dialog</p>
 * <p>This dialog is editing sieve filters (vacation and rules).</p>
 * <p>
 * </p>
 *
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new VacationEditDialog
 */
 Tine.Expressomail.sieve.VacationEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {

    /**
     * @cfg {Tine.Expressomail.Model.Account}
     */
    account: null,

    /**
     * @private
     */
    windowNamePrefix: 'VacationEditWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Vacation,
    recordProxy: Tine.Expressomail.vacationBackend,
    loadRecord: true,
    tbarItems: [],
    evalGrants: false,
    readonlyReason: false,

    /**
     * overwrite update toolbars function (we don't have record grants yet)
     *
     * @private
     */
    updateToolbars: function() {
    },

    /**
     * executed after record got updated from proxy
     *
     * @private
     */
    onRecordLoad: function() {
        // interrupt process flow till dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }

        // mime type is always multipart/alternative
        this.record.set('mime', 'multipart/alternative');
        if (this.account && this.account.get('signature')) {
            this.record.set('signature', this.account.get('signature'));
        }

        this.getForm().loadRecord(this.record);

        var title = String.format(this.app.i18n._('Vacation Message for {0}'), this.account.get('name'));
        this.window.setTitle(title);

        this.reasonEditor.setDisabled(! this.record.get('enabled'));

        Tine.log.debug('Tine.Expressomail.sieve.VacationEditDialog::onRecordLoad() -> record:');
        Tine.log.debug(this.record);

        this.loadMask.hide();
    },

    /**
     * returns dialog
     *
     * NOTE: when this method gets called, all initalisation is done.
     *
     * @return {Object}
     * @private
     *
     */
    getFormItems: function() {

        this.initReasonEditor();

        var generalItems = this.getGeneralItems();

        return {
            xtype: 'tabpanel',
            deferredRender: false,
            border: false,
            activeTab: 0,
            items: [{
                title: this.app.i18n._('General'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: {
                    anchor: '100%',
                    labelSeparator: '',
                    columnWidth: 1
                },
                items: generalItems
            }, {
                title: this.app.i18n._('Advanced'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: {
                    anchor: '100%',
                    labelSeparator: '',
                    columnWidth: 1
                },
                items: [[{
                    fieldLabel: this.app.i18n._('Only send all X days to the same sender'),
                    name: 'days',
                    value: 7,
                    xtype: 'numberfield',
                    allowNegative: false,
                    minValue: 1
                }]]
            }]
        };
    },

    /**
     * init reason editor
     */
    initReasonEditor: function() {
        var reg = this.app.getRegistry(),
            readonly = reg.get('config').vacationMessageCustomAllowed && reg.get('config').vacationMessageCustomAllowed.value === 0;

        this.reasonEditor = new Ext.form.HtmlEditor({
            fieldLabel: this.app.i18n._('Incoming mails will be answered with this text:'),
            name: 'reason',
            allowBlank: true,
            disabled: true,
            height: 220,
            readOnly: readonly,
            getDocMarkup: function() {
                var markup = '<html><body></body></html>';
                return markup;
            },
            plugins: [
                new Ext.ux.form.HtmlEditor.RemoveFormat()
            ]
        });
    },

    /**
     * get items for general tab
     *
     * @return Array
     */
    getGeneralItems: function() {
        var items = [[{
            fieldLabel: this.app.i18n._('Status'),
            name: 'enabled',
            typeAhead     : false,
            triggerAction : 'all',
            lazyRender    : true,
            editable      : false,
            mode          : 'local',
            forceSelection: true,
            value: 0,
            xtype: 'combo',
            store: [
                [0, this.app.i18n._('I am available (vacation message disabled)')],
                [1, this.app.i18n._('I am not available (vacation message enabled)')]
            ],
            listeners: {
                scope: this,
                select: function (combo, record) {
                    this.reasonEditor.setDisabled(! record.data.field1);
                }
            }
        }]];

        // add vacation template items if needed
        var templates = this.app.getRegistry().get('vacationTemplates');
        if (templates.totalcount > 0) {
            items = items.concat(this.getTemplateItems(templates));
        }

        items.push([this.reasonEditor]);

        return items;
    },

    /**
     * get items for vacation templates
     *
     * @param Object templates
     * @return Array
     *
     * TODO use grid panel for x representatives?
     */
    getTemplateItems: function(templates) {
        Tine.log.debug('Tine.Expressomail.sieve.VacationEditDialog::getTemplateItems()');
        Tine.log.debug(templates);

        var commonConfig = {
                listeners: {
                    scope: this,
                    select: this.onSelectTemplateField
                },
                columnWidth: 0.5
            },
            items = [[Ext.apply({
            fieldLabel: this.app.i18n._('Start Date'),
            emptyText: this.app.i18n._('Set vacation start date ...'),
            name: 'start_date',
            xtype: 'datefield'
        }, commonConfig), Ext.apply({
            fieldLabel: this.app.i18n._('End Date'),
            emptyText: this.app.i18n._('Set vacation end date ...'),
            name: 'end_date',
            xtype: 'datefield'
        }, commonConfig)], [
            new Tine.Addressbook.SearchCombo(Ext.apply({
                fieldLabel: this.app.i18n._('Representative #1'),
                emptyText: this.app.i18n._('Choose first Representative ...'),
                blurOnSelect: true,
                name: 'contact_id1',
                selectOnFocus: true,
                forceSelection: false,
                allowBlank: true
            }, commonConfig)),
            new Tine.Addressbook.SearchCombo(Ext.apply({
                fieldLabel: this.app.i18n._('Representative #2'),
                emptyText: this.app.i18n._('Choose second Representative ...'),
                blurOnSelect: true,
                name: 'contact_id2',
                selectOnFocus: true,
                forceSelection: false,
                allowBlank: true
            }, commonConfig))
        ], [{
            fieldLabel: this.app.i18n._('Message Template'),
            xtype: 'combo',
            mode: 'local',
            listeners: {
                scope: this,
                select: this.onTemplateComboSelect
            },
            displayField: 'name',
            name: 'template_id',
            valueField: 'id',
            triggerAction: 'all',
            emptyText: this.app.i18n._('Choose Template ...'),
            editable: false,
            store: new Ext.data.JsonStore({
                id: 'timezone',
                root: 'results',
                totalProperty: 'totalcount',
                fields: ['id', 'name', 'type'], // TODO use Tine.Filemanager.Model.Node or generic File model?
                data: templates
            })
        }]
        ];

        return items;
    },

    /**
     * template field has been selected, check if new vacation message needs to be fetched
     * - do this only if template has already been selected
     */
    onSelectTemplateField: function() {
        if (this.record.get('template_id') !== '') {
            this.getVacationMessage();
        }
    },

    /**
     * template combo select event handler
     *
     * @param {} combo
     * @param {} record
     * @param {} index
     */
    onTemplateComboSelect: function(combo, record, index) {
        Tine.log.debug('Tine.Expressomail.sieve.VacationEditDialog::onTemplateComboSelect()');
        Tine.log.debug(record);

        if (record.data && record.get('type') === 'file') {
            this.getVacationMessage();
        } else {
            // TODO do something?
        }
    },

    /**
     * get vacation with template replacements message from server
     */
    getVacationMessage: function() {
        this.loadMask.show();
        this.onRecordUpdate();
        Tine.Expressomail.getVacationMessage(this.record.data, this.onGetVacationMessage.createDelegate(this));
    },

    /**
     * onGetVacationMessage
     *
     * @param {} response
     */
    onGetVacationMessage: function(response) {
        Tine.log.debug('Tine.Expressomail.sieve.VacationEditDialog::onGetMessage()');
        Tine.log.debug(response);
        this.loadMask.hide();

        if (response.message) {
            this.reasonEditor.setValue(response.message);
        }
    },

    /**
     * generic request exception handler
     *
     * @param {Object} exception
     */
    onRequestFailed: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
        this.loadMask.hide();
    },

    /**
     * executed when record gets updated from form
     */
    onRecordUpdate: function() {
        Tine.Expressomail.sieve.VacationEditDialog.superclass.onRecordUpdate.call(this);

        var contactIds = [];
        Ext.each(['contact_id1', 'contact_id2'], function(field) {
            if (this.getForm().findField(field) && this.getForm().findField(field).getValue() !== '') {
                contactIds.push(this.getForm().findField(field).getValue());
            }
        }, this);
        this.record.set('contact_ids', contactIds);
    }
});

/**
 * Expressomail Edit Popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.sieve.VacationEditDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 640,
        height: 550,
        name: Tine.Expressomail.sieve.VacationEditDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.sieve.VacationEditDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail.sieve');

/**
 * @namespace   Tine.Expressomail.sieve
 * @class       Tine.Expressomail.sieve.RuleEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 * 
 * <p>Sieve Filter Dialog</p>
 * <p>This dialog is editing a filter rule.</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010 Metaways Infosystems GmbH (http://www.metaways.de)
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new RuleEditDialog
 */
Tine.Expressomail.sieve.RuleEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {
    /**
     * @cfg {Tine.Expressomail.Model.Account}
     */
    account: null,
    
    /**
     * @private
     */
    windowNamePrefix: 'RuleEditWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Rule,
    mode: 'local',
    loadRecord: true,
    tbarItems: [],
    evalGrants: false,
    
    conditionsIsValid : false,
    validationMessage: '',
    
    /**
     * overwrite update toolbars function (we don't have record grants yet)
     * 
     * @private
     */
    updateToolbars: function() {

    },
    
    /**
     * @private
     */
    onRender: function(ct, position) {
        Tine.Expressomail.sieve.RuleEditDialog.superclass.onRender.call(this, ct, position);
        
        this.onChangeType.defer(250, this);
    },
    
    /**
     * Change type card layout depending on selected combo box entry and set field value
     */
    onChangeType: function() {
        var type = this.actionTypeCombo.getValue();
        
        var cardLayout = Ext.getCmp(this.idPrefix + 'CardLayout').getLayout();
        if (cardLayout !== 'card') {
            cardLayout.setActiveItem(this.idPrefix + type);
            if (this.record.get('action_type') == type) {
                var field = this.getForm().findField('action_argument_' + type);
                if (field !== null) {
                    field.setValue(this.record.get('action_argument'));
                }
            }
        }
    },
    
    /**
     * Custom isValid method to check both conditions and actions
     * @return {Boolean}
     */
    isValid: function(){
        var actionType = this.actionTypeCombo.getValue();
        var actionField = this.getForm().findField('action_argument_' + actionType);
        var actionIsValid = actionField == null || actionField.isValid();
        
        var conditionsIsValid = true;
        var msg = '';
        var app = Tine.Tinebase.appMgr.get('Expressomail');
        var conditions = this.conditionsPanel.getValue();
        Ext.each(conditions, function(condition){
            if(Ext.util.Format.trim(condition.value).length == 0){
                conditionsIsValid = false;
                msg = app.i18n._('All fields must be filled');
                return;
            }
            if(['header', 'headerregex'].indexOf(condition.field) >= 0 && Ext.util.Format.trim(condition.operator).length == 0){
                conditionsIsValid = false;
                msg = app.i18n._('All fields must be filled');
                return;
            }
            if(condition.field == 'size' && /^[0-9]*$/.test(Ext.util.Format.trim(condition.value)) == false ){
                conditionsIsValid = false;
                msg = app.i18n._('Size value should be numeric');
                return;
            }
            if(condition.field == 'size' && condition.value > 50){
                conditionsIsValid = false;
                msg = app.i18n._('Size value must not be more than') + " 50 MB";
                return;
            }
        });
        this.conditionsIsValid = conditionsIsValid;
        this.validationMessage = msg;
        
        return actionIsValid && conditionsIsValid;
        
    },
    
     /**
     * get validation error message
     * 
     * @return {String}
     */
    getValidationErrorMessage: function() {
        if(!this.conditionsIsValid){
            return this.validationMessage;
        }
        else {
            return _('Please fix the errors noted.');
        }
    },
    
    /**
     * executed after record got updated from proxy
     * 
     * @private
     */
    onRecordLoad: function() {
        // interrupt process flow till dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }
        
        var title = this.app.i18n._('Edit Filter Rule');
        this.window.setTitle(title);
        
        this.getForm().loadRecord(this.record);
        
        this.loadMask.hide();
    },
    
    /**
     * @private
     */
    onRecordUpdate: function() {
        Tine.Expressomail.sieve.RuleEditDialog.superclass.onRecordUpdate.call(this);
        
        this.record.set('conditions', this.getConditions());
        
        var argumentField = this.getForm().findField('action_argument_' + this.actionTypeCombo.getValue()),
            argumentValue = (argumentField !== null) ? argumentField.getValue() : '';
        this.record.set('action_argument', argumentValue);
    },
    
    /**
     * get conditions and do the mapping
     * 
     * @return {Array}
     */
    getConditions: function() {
        var conditions = this.conditionsPanel.getAllFilterData();
        var result = [],
            i = 0, 
            condition,
            test,
            comperator,
            header;
            
        for (i = 0; i < conditions.length; i++) {
            // set defaults
            comperator = conditions[i].operator;
            header = conditions[i].field;
            test = 'header';

            switch (conditions[i].field) {
                case 'from':
                case 'to':
                    test = 'address';
                    break;
                case 'fromheader':
                    header = 'From';
                    break;
                case 'size':
                    test = 'size';
                    comperator = (conditions[i].operator == 'greater') ? 'over' : 'under';
                    break;
                case 'header':
                    header = conditions[i].operator;
                    comperator = 'contains';
                    break;
                case 'headerregex':
                    header = conditions[i].operator;
                    comperator = 'regex';
                    break;
            }
            condition = {
                test: test,
                header: header,
                comperator: comperator,
                key: conditions[i].value
            };
            result.push(condition);
        }
        return result;
    },
    
    /**
     * get conditions filter data (reverse of getConditions)
     * 
     * @return {Array}
     */
    getConditionsFilter: function() {
        var conditions = this.record.get('conditions');
        var result = [],
            i = 0, 
            filter,
            operator,
            field;
            
        for (i = 0; i < conditions.length; i++) {
            field = conditions[i].header;
            switch (field) {
                case 'size':
                    operator = (conditions[i].comperator == 'over') ? 'greater' : 'less';
                    break;
                case 'from':
                case 'to':
                case 'subject':
                    operator = conditions[i].comperator;
                    break;
                default:
                    if (field == 'From') {
                        operator = conditions[i].comperator;
                        field = 'fromheader';
                    } else {
                        operator = field;
                        field = (conditions[i].comperator == 'contains') ? 'header' : 'headerregex';
                    }
            }
            filter = {
                field: field,
                operator: operator,
                value: conditions[i].key
            };
            result.push(filter);
        }
        
        return result;
    },
    
    /**
     * returns dialog
     * 
     * NOTE: when this method gets called, all initalisation is done.
     * 
     * @return {Object}
     * @private
     */
    getFormItems: function() {
        
        this.conditionsPanel = new Tine.Expressomail.sieve.RuleConditionsPanel({
            filters: this.getConditionsFilter()
        });
        
        this.actionTypeCombo = new Ext.form.ComboBox({
            hideLabel       : true,
            name            : 'action_type',
            typeAhead       : false,
            triggerAction   : 'all',
            lazyRender      : true,
            editable        : false,
            mode            : 'local',
            forceSelection  : true,
            value           : 'discard',
            columnWidth     : 0.4,
            store: Tine.Expressomail.sieve.RuleEditDialog.getActionTypes(this.app),
            listeners: {
                scope: this,
                change: this.onChangeType,
                select: this.onChangeType
            }
        });
        
        this.idPrefix = Ext.id();
        
        return [{
            xtype: 'panel',
            layout: 'border',
            autoScroll: true,
            items: [
            {
                title: this.app.i18n._('If all of the following conditions are met:'),
                region: 'north',
                border: false,
                autoScroll: true,
                items: [
                    this.conditionsPanel
                ],
                xtype: 'panel',
                listeners: {
                    scope: this,
                    afterlayout: function(ct, layout) {
                        ct.suspendEvents();
                        if (this.conditionsPanel.getHeight() < 170) {
                            ct.setHeight(this.conditionsPanel.getHeight() + 30);
                        }
                        ct.ownerCt.layout.layout();
                        ct.resumeEvents();
                    }
                }
            }, {
                title: this.app.i18n._('Do this action:'),
                region: 'center',
                border: false,
                frame: true,
                layout: 'column',
                items: [
                    this.actionTypeCombo,
                    // TODO try to add a spacer/margin between the two input fields
                /*{
                    // spacer
                    columnWidth: 0.1,
                    layout: 'fit',
                    title: '',
                    items: []
                }, */{
                    id: this.idPrefix + 'CardLayout',
                    layout: 'card',
                    activeItem: this.idPrefix + 'fileinto',
                    border: false,
                    columnWidth: 0.5,
                    defaults: {
                        border: false
                    },
                    items: [{
                        id: this.idPrefix + 'fileinto',
                        layout: 'form',
                        items: [{
                            name: 'action_argument_fileinto',
                            xtype: 'expressomailfolderselect',
                            width: 200,
                            hideLabel: true,
                            allowBlank: false,
                            account: this.account
                        }]
                    }, {
                        // TODO add email validator?
                        id: this.idPrefix + 'redirect',
                        layout: 'form',
                        items: [{
                            name: 'action_argument_redirect',
                            xtype: 'textfield',
                            vtype: 'email',
                            allowBlank: false,
                            emptyText: 'test@example.org',
                            width: 200,
                            hideLabel: true
                        }]
                    }, {
                        id: this.idPrefix + 'reject',
                        layout: 'form',
                        items: [{
                            name: 'action_argument_reject',
                            xtype: 'textarea',
                            width: 300,
                            height: 60,
                            allowBlank: false,
                            hideLabel: true
                        }]
                    }, {
                        id: this.idPrefix + 'discard',
                        layout: 'fit',
                        items: []
                    }]
                }]
            }]
        }];
    }
});

/**
 * Expressomail Edit Popup
 * 
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.sieve.RuleEditDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 700,
        height: 300,
        name: Tine.Expressomail.sieve.RuleEditDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.sieve.RuleEditDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};

/**
 * get action types for action combo and action type renderer
 * 
 * @param {} app
 * @return {Array}
 */
Tine.Expressomail.sieve.RuleEditDialog.getActionTypes = function(app) {
    if(app === undefined){
        app = Tine.Tinebase.appMgr.get('Expressomail');
    }
    return [
        ['fileinto',    app.i18n._('Move mail to folder')],
        ['redirect',    app.i18n._('Redirect mail to address')],
        ['reject',      app.i18n._('Reject mail with this text')],
        ['discard',     app.i18n._('Discard mail')]
        //['keep',        app.i18n._('Keep mail')],
    ];
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.ns('Tine.Expressomail.sieve');

/**
 * @namespace Tine.Expressomail
 * @class     Tine.Expressomail.sieve.RulesGridPanel
 * @extends   Tine.widgets.grid.GridPanel
 * Rules Grid Panel <br>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 */
Tine.Expressomail.sieve.RulesGridPanel = Ext.extend(Tine.widgets.grid.GridPanel, {
    /**
     * @cfg {Tine.Expressomail.Model.Account}
     */
    account: null,
    
    // model generics
    recordClass: Tine.Expressomail.Model.Rule,
    recordProxy: Tine.Expressomail.rulesBackend,
    
    // grid specific
    defaultSortInfo: {field: 'id', dir: 'ASC'},
    storeRemoteSort: false,
    
    // not yet
    evalGrants: false,
    usePagingToolbar: false,
    splitAddButton: false,
    
    // disable this check as records are only deleted locally
    disableDeleteActionCheckServiceMap: Ext.emptyFn,
    
    newRecordIcon: 'action_new_rule',
    editDialogClass: Tine.Expressomail.sieve.RuleEditDialog,
    
    initComponent: function() {
        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        this.initColumns();
        
        this.editDialogConfig = {
            account: this.account
        };
        
        this.supr().initComponent.call(this);
    },
    
    /**
     * Return CSS class to apply to rows depending on enabled status
     * 
     * @param {Tine.Expressomail.Model.Rule} record
     * @param {Integer} index
     * @return {String}
     */
    getViewRowClass: function(record, index) {
        var className = '';
        
        if (! record.get('enabled')) {
            className += ' expressomail-sieverule-disabled';
        }
        
        return className;
    },
    
    /**
     * init actions with actionToolbar, contextMenu and actionUpdater
     * 
     * @private
     */
    initActions: function() {
        this.action_moveup = new Ext.Action({
            text: this.app.i18n._('Move up'),
            handler: this.onMoveRecord.createDelegate(this, ['up']),
            scope: this,
            iconCls: 'action_move_up'
        });

        this.action_movedown = new Ext.Action({
            text: this.app.i18n._('Move down'),
            handler: this.onMoveRecord.createDelegate(this, ['down']),
            scope: this,
            iconCls: 'action_move_down'
        });

        this.action_enable = new Ext.Action({
            text: this.app.i18n._('Enable'),
            handler: this.onEnableDisable.createDelegate(this, [true]),
            scope: this,
            iconCls: 'action_enable'
        });

        this.action_disable = new Ext.Action({
            text: this.app.i18n._('Disable'),
            handler: this.onEnableDisable.createDelegate(this, [false]),
            scope: this,
            iconCls: 'action_disable'
        });
        
        this.supr().initActions.call(this);
    },
    
    /**
     * enable / disable rule
     * 
     * @param {Boolean} state
     */
    onEnableDisable: function(state) {
        var selectedRows = this.grid.getSelectionModel().getSelections();
        for (var i = 0; i < selectedRows.length; i++) {
            selectedRows[i].set('enabled', state);
        }
    },
    
    /**
     * move record up or down
     * 
     * @param {String} dir (up|down)
     */
    onMoveRecord: function(dir) {
        var sm = this.grid.getSelectionModel();
            
        if (sm.getCount() == 1) {
            var selectedRows = sm.getSelections();
            record = selectedRows[0];
            
            // get next/prev record
            var index = this.store.indexOf(record),
                switchRecordIndex = (dir == 'down') ? index + 1 : index - 1,
                switchRecord = this.store.getAt(switchRecordIndex);
            
            if (switchRecord) {
                // switch ids and resort store
                var oldId = record.id;
                    switchId = switchRecord.id;

                record.set('id', Ext.id());
                record.id = Ext.id();
                switchRecord.set('id', oldId);
                switchRecord.id = oldId;
                record.set('id', switchId);
                record.id = switchId;
                
                this.store.commitChanges();
                this.store.sort('id', 'ASC');
                sm.selectRecords([record]);
            }
        }
    },

    /**
     * add custom items to action toolbar
     * 
     * @return {Object}
     */
     
    initFilterPanel: function() {},
    
    getActionToolbarItems: function() {
        return [
            {
                xtype: 'buttongroup',
                columns: 1,
                frame: false,
                items: [
                    this.action_moveup,
                    this.action_movedown
                ]
            }
        ];
    },
    
    /**
     * add custom items to context menu
     * 
     * @return {Array}
     */
    getContextMenuItems: function() {
        var items = [
            '-',
            this.action_moveup,
            this.action_movedown,
            '-',
            this.action_enable,
            this.action_disable
        ];
        
        return items;
    },
    
    /**
     * init columns
     */
    initColumns: function() {
        this.gridConfig.columns = [
        {
            id: 'id',
            header: this.app.i18n._("ID"),
            width: 40,
            sortable: false,
            dataIndex: 'id',
            hidden: true
        }, {
            id: 'conditions',
            header: this.app.i18n._("Conditions"),
            width: 200,
            sortable: false,
            dataIndex: 'conditions',
            scope: this,
            renderer: this.conditionsRenderer
        }, {
            id: 'action',
            header: this.app.i18n._("Action"),
            width: 250,
            sortable: false,
            dataIndex: 'action_type',
            scope: this,
            renderer: this.actionRenderer
        }];
    },
    
    /**
     * init layout
     */
    initLayout: function() {
        this.supr().initLayout.call(this);
        
        this.items.push({
            region : 'north',
            height : 55,
            border : false,
            items  : this.actionToolbar
        });
    },
    
    /**
     * preform the initial load of grid data
     */
    initialLoad: function() {
        this.store.load.defer(10, this.store, [
            typeof this.autoLoad == 'object' ?
                this.autoLoad : undefined]);
    },
    
    /**
     * called before store queries for data
     */
    onStoreBeforeload: function(store, options) {
        Tine.Expressomail.sieve.RulesGridPanel.superclass.onStoreBeforeload.call(this, store, options);
        
        options.params.filter = this.account.id;
    },
    
    /**
     * action renderer
     * 
     * @param {Object} type
     * @param {Object} metadata
     * @param {Object} record
     * @return {String}
     */
    actionRenderer: function(type, metadata, record) {
        var types = Tine.Expressomail.sieve.RuleEditDialog.getActionTypes(this.app),
            result = type;
        
        for (i=0; i < types.length; i++) {
            if (types[i][0] == type) {
                result = types[i][1];
            }
        }
        
        if (record.get('action_argument') && record.get('action_argument') != '') {
            result += ' ' + record.get('action_argument');
        }
            
        return Ext.util.Format.htmlEncode(result);
    },

    /**
     * conditions renderer
     * 
     * @param {Object} value
     * @return {String}
     * 
     * TODO show more conditions?
     */
    conditionsRenderer: function(value) {
        var result = '',
            app = Tine.Tinebase.appMgr.get('Expressomail');
        
        // show only first condition
        if (value && value.length > 0) {
            var condition = value[0];
            
            // get header/comperator translation
            var filterModels = Tine.Expressomail.sieve.RuleConditionsPanel.getFilterModel(app),
                header, 
                comperator, 
                found = false;
            Ext.each(filterModels, function(filterModel) {
                if (condition.header == filterModel.field || (condition.header === 'From' && filterModel.field == 'fromheader')) {
                    header = filterModel.label;
                    if (condition.header == 'size') {
                        comperator = (condition.comperator == 'over') ? _('is greater than') : _('is less than');
                    } else {
                        comperator = _(condition.comperator);
                    }
                    found = true;
                }
            }, this);
            
            if (found === true) {
                result = header + ' ' + comperator + ' "' + condition.key + '"';
            } else {
                result = (condition.comperator == 'contains') ? app.i18n._('Header "{0}" contains "{1}"') : app.i18n._('Header "{0}" matches "{1}"');
                result = String.format(result, condition.header, condition.key);
            }
        }
        
        return Ext.util.Format.htmlEncode(result);
    },
    
    /**
     * on update after edit
     * 
     * @param {String} encodedRecordData (json encoded)
     */
    onUpdateRecord: function(encodedRecordData) {
        var newRecord = Tine.Expressomail.rulesBackend.recordReader({responseText: encodedRecordData});
        
        if (! newRecord.id) {
            var lastRecord = null,
                nextId = null;
            do {
                // get next free id
                lastRecord = this.store.getAt(this.store.getCount()-1);
                nextId = (lastRecord) ? (parseInt(lastRecord.id, 10) + 1).toString() : '1';
            } while (this.store.getById(newRecord.id));
            
            newRecord.set('id', nextId);
            newRecord.id = nextId;
        } else {
            this.store.remove(this.store.getById(newRecord.id));
        }
        
        this.store.addSorted(newRecord);
        
        // some eyecandy
        var row = this.getView().getRow(this.store.indexOf(newRecord));
        Ext.fly(row).highlight();
    },
    
    /**
     * generic delete handler
     */
    onDeleteRecords: function(btn, e) {
        var sm = this.grid.getSelectionModel();
        var records = sm.getSelections();
        Ext.each(records, function(record) {
            this.store.remove(record);
        });
    }
});
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail.sieve');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.sieve.RulesDialog
 * @extends     Tine.widgets.dialog.EditDialog
 * 
 * <p>Sieve Filter Dialog</p>
 * <p>This dialog is for editing sieve filters (rules).</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new RulesDialog
 */
Tine.Expressomail.sieve.RulesDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {

    /**
     * @cfg {Tine.Expressomail.Model.Account}
     */
    account: null,

    /**
     * @private
     */
    windowNamePrefix: 'VacationEditWindow_',
    appName: 'Expressomail',
//    loadRecord: false,
    mode: 'local',
    tbarItems: [],
    evalGrants: false,
    
    //private
    initComponent: function(){
        Tine.Expressomail.sieve.RulesDialog.superclass.initComponent.call(this);
        
        this.i18nRecordName = this.app.i18n._('Sieve Filter Rules');
    },
    
    /**
     * overwrite update toolbars function (we don't have record grants yet)
     * 
     * @private
     */
    updateToolbars: Ext.emptyFn,
    
    /**
     * init record to edit
     * -> we don't have a real record here
     */
    initRecord: function() {
        this.onRecordLoad();
    },
    
    /**
     * executed after record got updated from proxy
     * -> we don't have a real record here
     * 
     * @private
     */
    onRecordLoad: function() {
        // interrupt process flow till dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }
        
        var title = String.format(this.app.i18n._('Sieve Filter Rules for {0}'), this.account.get('name'));
        this.window.setTitle(title);
        
        this.loadMask.hide();
    },
        
    /**
     * returns dialog
     * 
     * NOTE: when this method gets called, all initalisation is done.
     * 
     * @return {Object}
     * @private
     * 
     */
    getFormItems: function() {
        this.rulesGrid = new Tine.Expressomail.sieve.RulesGridPanel({
            account: this.account
        });
        
        return [this.rulesGrid];
    },
    
    /**
     * apply changes handler (get rules and send them to saveRules)
     */
    onApplyChanges: function(closeWindow) {
        var rules = [];
        this.rulesGrid.store.each(function(record) {
            rules.push(record.data);
        });
        
        this.loadMask.show();
        Tine.Expressomail.rulesBackend.saveRules(this.account.id, rules, {
            scope: this,
            success: function(record) {
                if (closeWindow) {
                    this.purgeListeners();
                    this.window.close();
                }
            },
            failure: Tine.Expressomail.handleRequestException.createSequence(function() {
                this.loadMask.hide();
            }, this),
            timeout: 150000 // 3 minutes
        });
    }
});

/**
 * Expressomail Edit Popup
 * 
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.sieve.RulesDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 800,
        height: 400,
        name: Tine.Expressomail.sieve.RulesDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.sieve.RulesDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail.sieve');

/**
 * @namespace   Tine.Expressomail.sieve
 * @class       Tine.Expressomail.sieve.RuleConditionsPanel
 * @extends     Tine.widgets.grid.FilterToolbar
 * 
 * <p>Sieve Filter Conditions Panel</p>
 * <p>
 * mapping when getting filter values:
 *  field       -> test_header or 'size'
 *  operator    -> comperator
 *  value       -> key
 * </p>
 * <p>
 * </p>
 * 
 * @param       {Object} config
 * @constructor
 * Create a new RuleConditionsPanel
 */
Tine.Expressomail.sieve.RuleConditionsPanel = Ext.extend(Tine.widgets.grid.FilterToolbar, {
    
    defaultFilter: 'from',
    neverAllowSaving: true,
    showSearchButton: false,
    filterFieldWidth: 160,
    
    // unused fn
    onFiltertrigger: Ext.emptyFn,
    
    initComponent: function() {
        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        this.rowPrefix = '';
        
        this.filterModels = Tine.Expressomail.sieve.RuleConditionsPanel.getFilterModel(this.app);
        
        this.supr().initComponent.call(this);
    },
    
    /**
     * gets filter data (use getValue() if we don't have a store/plugins)
     * 
     * @return {Array} of filter records
     */
    getAllFilterData: function() {
        return this.getValue();
    }
});

/**
 * get rule conditions for filter model and condition renderer
 * 
 * @param {} app
 * @return {Array}
 */
Tine.Expressomail.sieve.RuleConditionsPanel.getFilterModel = function(app) {

    return [
        {label: app.i18n._('From (Email)'),     field: 'from',     operators: ['contains', 'regex'], emptyText: 'test@example.org'},
        {label: app.i18n._('From (Email and Name)'), field: 'fromheader',     operators: ['contains', 'regex'], emptyText: 'name or email'},
        {label: app.i18n._('To (Email)'),       field: 'to',       operators: ['contains', 'regex'], emptyText: 'test@example.org'},
        {label: app.i18n._('Subject'),          field: 'subject',  operators: ['contains', 'regex'], emptyText: app.i18n._('Subject')},
        {label: app.i18n._('Size') + " (MB)",   field: 'size',     operators: ['greater', 'less'], valueType: 'number', defaultOperator: 'greater'},
        {label: app.i18n._('Header contains'),  field: 'header',   operators: ['freeform'], defaultOperator: 'freeform', 
            emptyTextOperator: app.i18n._('Header name'), emptyText: app.i18n._('Header value')},
        {label: app.i18n._('Header regex'),     field: 'headerregex',   operators: ['freeform'], defaultOperator: 'freeform',
            emptyTextOperator: app.i18n._('Header name'), emptyText: app.i18n._('Header value')}
    ];
};
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * get expressomail tree panel context menus
 * this is used in Tine.Expressomail.TreePanel (with createDelegate)
 *
 * TODO use Ext.apply to get this
 */
Tine.Expressomail.setTreeContextMenus = function() {

    // define additional actions
    var emptyFolderAction = {
        text: this.app.i18n._('Empty Folder'),
        iconCls: 'action_folder_emptytrash',
        scope: this,
        handler: function() {
            this.ctxNode.getUI().addClass("x-tree-node-loading");
            var folderId = this.ctxNode.attributes.folder_id,
                selectedNode = this.getSelectionModel().getSelectedNode(),
                isSelectedNode = (selectedNode && this.ctxNode.id == selectedNode.id);
            
            if(isSelectedNode){
                this.app.getMainScreen().getCenterPanel().grid.getSelectionModel().clearSelections();
                this.app.getMainScreen().getCenterPanel().getStore().removeAll();
            }
            Ext.Ajax.request({
                params: {
                    method: 'Expressomail.emptyFolder',
                    folderId: folderId
                },
                scope: this,
                success: function(result, request){
                    var selectedNode = this.getSelectionModel().getSelectedNode(),
                        isSelectedNode = (selectedNode && this.ctxNode.id == selectedNode.id),
                        account = this.app.getActiveAccount(),
                        dRegexp = new RegExp('\\'+account.get('delimiter')+'$'),
                        inboxName = account.get('ns_personal').replace(dRegexp, '').toUpperCase(),
                        folderStore = this.app.getFolderStore(),
                        inboxRecord = folderStore.getAt(folderStore.findExact('globalname', inboxName));

                    if (isSelectedNode) {
                        var folder = Tine.Expressomail.folderBackend.recordReader(result);
                        folder.set('cache_unreadcount', 0);
                        folder.set('cache_totalcount', 0);
                        this.app.getFolderStore().updateFolder(folder);
                    } else {
                        var folder = folderStore.getById(folderId);
                        folder.set('cache_unreadcount', 0);
                        folder.set('cache_totalcount', 0);
                    }
                    this.ctxNode.getUI().removeClass("x-tree-node-loading");
                    this.ctxNode.removeAll();

                    inboxRecord.set('cache_status', 'incomplete');
                    this.app.checkMailsDelayedTask.delay(2000);
                },
                failure: function() {
                    this.ctxNode.getUI().removeClass("x-tree-node-loading");
                },
                timeout: 120000 // 2 minutes
            });
        }
    };

    // we need this for adding folders to account (root level)
    var addFolderToRootAction = {
        text: this.app.i18n._('Add Folder'),
        iconCls: 'action_add',
        scope: this,
        disabled: true,
        handler: function() {
            Ext.MessageBox.prompt(String.format(_('New {0}'), this.app.i18n._('Folder')), String.format(_('Please enter the name of the new {0}:'), this.app.i18n._('Folder')), function(_btn, _text) {
                if( this.ctxNode && _btn == 'ok') {
                    if (! _text) {
                        Ext.Msg.alert(String.format(_('No {0} added'), this.app.i18n._('Folder')), String.format(_('You have to supply a {0} name!'), this.app.i18n._('Folder')));
                        return;
                    }
                    Ext.MessageBox.wait(_('Please wait'), String.format(_('Creating {0}...' ), this.app.i18n._('Folder')));
                    var parentNode = this.ctxNode;

                    var params = {
                        method: 'Expressomail.addFolder',
                        name: _text
                    };

                    params.parent = '';
                    params.accountId = parentNode.id;

                    Ext.Ajax.request({
                        params: params,
                        scope: this,
                        timeout: 150000, // 2 minutes
                        success: function(_result, _request){
                            var nodeData = Ext.util.JSON.decode(_result.responseText);
                            var newNode = this.loader.createNode(nodeData);
                            parentNode.appendChild(newNode);
                            this.fireEvent('containeradd', nodeData);
                            Ext.MessageBox.hide();
                        }
                    });

                }
            }, this);
        }
    };

    var editAccountAction = {
        text: this.app.i18n._('Edit Account'),
        iconCls: 'ExpressomailIconCls',
        scope: this,
        disabled: ! Tine.Tinebase.common.hasRight('manage_accounts', 'Expressomail'),
        handler: function() {
            var record = this.accountStore.getById(this.ctxNode.attributes.account_id);
            var popupWindow = Tine.Expressomail.AccountEditDialog.openWindow({
                record: record,
                listeners: {
                    scope: this,
                    'update': function(record) {
                        var account = new Tine.Expressomail.Model.Account(Ext.util.JSON.decode(record));

                        // update tree node + store
                        this.ctxNode.setText(account.get('name'));
                        this.accountStore.reload();

                        // reload tree node + remove all folders of this account from store ?
                        this.folderStore.resetQueryAndRemoveRecords('parent_path', '/' + this.ctxNode.attributes.account_id);
                        this.ctxNode.reload(function(callback) {
                            this.selectInbox(account);
                        }, this);
                    }
                }
            });
        }
    };

    var editVacationAction = {
        text: this.app.i18n._('Edit Vacation Message'),
        iconCls: 'action_email_replyAll',
        scope: this,
        handler: function() {
            var accountId = this.ctxNode.attributes.account_id;
            var account = this.accountStore.getById(accountId);
            var record = new Tine.Expressomail.Model.Vacation({id: accountId}, accountId);

            var popupWindow = Tine.Expressomail.sieve.VacationEditDialog.openWindow({
                account: account,
                record: record
            });
        }
    };

    var editRulesAction = {
        text: this.app.i18n._('Edit Filter Rules'),
        iconCls: 'action_email_forward',
        scope: this,
        handler: function() {
            var accountId = this.ctxNode.attributes.account_id;
            var account = this.accountStore.getById(accountId);

            var popupWindow = Tine.Expressomail.sieve.RulesDialog.openWindow({
                account: account
            });
        }
    };

    var manageAclsAction = {
        text: this.app.i18n._('Share mailbox'),
        iconCls: 'action_managePermissions',
        scope: this,
        handler:function() {
                if (this.ctxNode) {
                    var node = this.ctxNode;
                    var folderId = this.ctxNode.attributes.globalname;
                    var account = this.ctxNode.attributes.account_id;
                    var window = Tine.Expressomail.AclsEditDialog.openWindow({
                        title: String.format(this.app.i18n._('Share mailbox')),
                        accountId: account,
                        // Using 'INBOX', can use folderId
                        globalName: 'INBOX'
                    });
                }
            }
    };
   var manageEmlImportAction = {
        text: this.app.i18n._('Import msg(eml)'),
        iconCls: 'action_import',
        scope: this,
        handler:function() {
                if (this.ctxNode) {
                    var window = Tine.Expressomail.ImportEmlDialog.openWindow({
                        account: this.ctxNode.attributes.account_id,
                        textName: this.ctxNode.text,
                        folderId: this.ctxNode.attributes.folder_id,
                        // Using 'INBOX', can use folderId
                        globalName: this.ctxNode.attributes.globalname
                    });
                }
            }
    };
    
    var deleteFolderChildAction = {
        text: this.app.i18n._('Delete Folder'),
        iconCls: 'action_delete',
        scope: this,
        handler:function() {
             if (this.ctxNode) {
                Ext.Msg.alert(this.app.i18n._('Warning'), this.app.i18n._('Delete your sub-folders first.'));
            }
        }
    };
    
    

    var markFolderSeenAction = {
        text: this.app.i18n._('Mark Folder as read'),
        iconCls: 'action_mark_read',
        scope: this,
        handler: function() {
            if (this.ctxNode) {
                var folderId = this.ctxNode.id,
                    filter = [{
                        field: 'folder_id',
                        operator: 'equals',
                        value: folderId
                    }, {
                        field: 'flags',
                        operator: 'notin',
                        value: ['\\Seen']
                    }
                ];

                var selectedNode = this.getSelectionModel().getSelectedNode(),
                    isSelectedNode = (selectedNode && this.ctxNode.id == selectedNode.id);

                Tine.Expressomail.messageBackend.addFlags(filter, '\\Seen', {
                    callback: function() {
                        this.app = Tine.Tinebase.appMgr.get('Expressomail');
                        var folder = this.app.getFolderStore().getById(folderId);
                        folder.set('cache_unreadcount', 0);
                        if (isSelectedNode) {
                            this.app.getMainScreen().getCenterPanel().loadGridData({
                                removeStrategy: 'keepBuffered'
                            });
                        }
                    }
                });
            }
        }
    };

    var updateFolderCacheAction = {
        text: this.app.i18n._('Update Folder List'),
        iconCls: 'action_update_cache',
        scope: this,
        handler: function() {
            if (this.ctxNode) {
                this.getSelectionModel().clearSelections();

                var folder = this.app.getFolderStore().getById(this.ctxNode.id),
                    account = folder ? this.app.getAccountStore().getById(folder.get('account_id')) :
                                       this.app.getAccountStore().getById(this.ctxNode.id);
                this.ctxNode.getUI().addClass("x-tree-node-loading");
                // call update folder cache
                Ext.Ajax.request({
                    params: {
                        method: 'Expressomail.updateFolderCache',
                        accountId: account.id,
                        folderName: folder ? folder.get('globalname') : ''
                    },
                    scope: this,
                    success: function(result, request){
                        this.ctxNode.getUI().removeClass("x-tree-node-loading");
                        // clear query to query server again and reload subfolders
                        this.folderStore.resetQueryAndRemoveRecords('parent_path', (folder ? folder.get('path') : '/') + account.id);
                        this.ctxNode.reload(function(callback) {
                            this.selectInbox(account);
                        }, this);
                    },
                    failure: function() {
                        this.ctxNode.getUI().removeClass("x-tree-node-loading");
                    }
                });
            }
        }
    };

    // mutual config options
    var config = {
        nodeName: _('Folder'),
        scope: this,
        backend: 'Expressomail',
        backendModel: 'Folder'
    };

    // system folder ctx menu
    config.actions = [markFolderSeenAction, 'add',manageEmlImportAction];
    this.contextMenuSystemFolder = Tine.widgets.tree.ContextMenu.getMenu(config);

    // user folder ctx menu
    config.actions = [markFolderSeenAction, 'add', 'rename', 'delete',manageEmlImportAction];
    this.contextMenuUserFolder = Tine.widgets.tree.ContextMenu.getMenu(config);

    // trash ctx menu
    config.actions = [markFolderSeenAction, 'add', emptyFolderAction];
    this.contextMenuTrash = Tine.widgets.tree.ContextMenu.getMenu(config);

    // account ctx menu
    this.contextMenuAccount = Tine.widgets.tree.ContextMenu.getMenu({
        nodeName: this.app.i18n.n_('Account', 'Accounts', 1),
        actions: [addFolderToRootAction, manageAclsAction, updateFolderCacheAction, editVacationAction, editRulesAction, editAccountAction],
        scope: this,
        backend: 'Expressomail',
        backendModel: 'Account'
    });
    
    config.actions = [markFolderSeenAction, 'add', 'rename', deleteFolderChildAction , manageEmlImportAction];
    this.contextMenuUserFolderChildren = Tine.widgets.tree.ContextMenu.getMenu(config);

    // context menu for unselectable folders (like public/shared namespace)
    config.actions = ['add'];
    this.unselectableFolder = Tine.widgets.tree.ContextMenu.getMenu(config);
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */
 
/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.TreeLoader
 * @extends     Tine.widgets.tree.Loader
 * 
 * <p>Expressomail Account/Folder Tree Loader</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.TreeLoader
 * 
 */
Tine.Expressomail.TreeLoader = Ext.extend(Tine.widgets.tree.Loader, {
    
    /**
     * request data
     * 
     * @param {Ext.tree.TreeNode} node
     * @param {Function} callback Function to call after the node has been loaded. The
     * function is passed the TreeNode which was requested to be loaded.
     * @param (Object) scope The cope (this reference) in which the callback is executed.
     * defaults to the loaded TreeNode.
     */
    requestData : function(node, callback, scope){
        
        if(this.fireEvent("beforeload", this, node, callback) !== false) {
            var fstore = Tine.Tinebase.appMgr.get('Expressomail').getFolderStore(),
                folder = fstore.getById(node.attributes.folder_id),
                path = (folder) ? folder.get('path') : node.attributes.path;
            
            // we need to call doQuery fn from store to transparently do async request
            fstore.asyncQuery('parent_path', path, function(node, callback, scope, data) {
                if (data) {
                    node.beginUpdate();
                    data.each(function(folderRecord) {
                        var n = this.createNode(folderRecord.copy().data);
                        if (n) {
                            node.appendChild(n);
                        }
                    }, this);
                    node.endUpdate();
                }
                this.runCallback(callback, scope || node, [node]);
            }, [node, callback, scope], this, fstore);
            
        } else {
            // if the load is cancelled, make sure we notify
            // the node that we are done
            this.runCallback(callback, scope || node, []);
        }
    },
    
    /**
     * inspectCreateNode
     * 
     * @private
     */
    inspectCreateNode: function(attr) {
        var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(attr.account_id);
        
        // NOTE cweiss 2010-06-15 this has to be precomputed on server side!
        attr.has_children = (account && account.get('has_children_support')) ? attr.has_children : true;
        if (attr.has_children == "0") {
            attr.has_children = false;
        }
        
        Ext.apply(attr, {
            leaf: !attr.has_children,
            expandable: attr.has_children,
            cls: 'x-tree-node-collapsed',
            folder_id: attr.id,
            folderNode: true,
            allowDrop: true,
            text: this.app.i18n._hidden(attr.localname)
        });
        
        // show standard folders icons 
        if (account) {
            if (account.get('trash_folder') === attr.globalname) {
                if (attr.cache_totalcount > 0) {
                    attr.cls = 'expressomail-node-trash-full';
                } else {
                    attr.cls = 'expressomail-node-trash';
                }
            }
            if (account.get('sent_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-sent';
            }
            if (account.get('drafts_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-drafts';
            }
            if (account.get('templates_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-templates';
            }
        }
        if (attr.globalname.match(/^inbox$/i)) {
            attr.cls = 'expressomail-node-inbox';
            attr.text = this.app.i18n._hidden('INBOX');
        }
        if (attr.globalname.match(/^junk$/i)) {
            attr.cls = 'expressomail-node-junk';
        }
        if (attr.globalname.match(/^inbox\/arquivo remoto$/i) || attr.globalname.match(/^inbox\/arquivo remoto\//i)) {
            attr.cls = 'expressomail-node-remote';
        }

        if (! attr.is_selectable) {
            attr.cls = 'expressomail-node-unselectable';
        }
    }
});

Ext.override(Ext.tree.TreeNodeUI, {
    updateExpandIcon : function() {
        if(this.rendered){
            var n = this.node,
                c1,
                c2,
                rc1,
                rc2,
                cls = n.isLast() ? "x-tree-elbow-end" : "x-tree-elbow",
                hasChild = n.hasChildNodes();
            if(hasChild || n.attributes.expandable){
                if(n.expanded){
                    cls += "-minus";
                    c1 = "x-tree-node-collapsed";
                    c2 = "x-tree-node-expanded";
                    rc1 = "expressomail-node-remote";
                    rc2 = "expressomail-node-remote-open";
                }else{
                    cls += "-plus";
                    c1 = "x-tree-node-expanded";
                    c2 = "x-tree-node-collapsed";
                    rc1 = "expressomail-node-remote-open";
                    rc2 = "expressomail-node-remote";
                }
                if(this.wasLeaf){
                    this.removeClass("x-tree-node-leaf");
                    this.wasLeaf = false;
                }
                if(this.c1 != c1 || this.c2 != c2){
                    Ext.fly(this.elNode).replaceClass(c1, c2);
                    if (Ext.fly(this.elNode).hasClass(rc1)) {
                        Ext.fly(this.elNode).replaceClass(rc1, rc2);
                    }
                    this.c1 = c1; this.c2 = c2;
                }
            }else{
                if(!this.wasLeaf){
                    Ext.fly(this.elNode).replaceClass("x-tree-node-expanded", "x-tree-node-collapsed");
                    delete this.c1;
                    delete this.c2;
                    this.wasLeaf = true;
                }
            }
            var ecc = "x-tree-ec-icon "+cls;
            if(this.ecc != ecc){
                this.ecNode.className = ecc;
                this.ecc = ecc;
            }
        }
    }
});
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */
 
/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.TreeLoaderSieve
 * @extends     Tine.widgets.tree.Loader
 * 
 * <p>Expressomail Account/Folder Tree Loader</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.TreeLoaderSieve
 * 
 */
Tine.Expressomail.TreeLoaderSieve = Ext.extend(Tine.widgets.tree.Loader, {
    
    /**
     * request data
     * 
     * @param {Ext.tree.TreeNode} node
     * @param {Function} callback Function to call after the node has been loaded. The
     * function is passed the TreeNode which was requested to be loaded.
     * @param (Object) scope The cope (this reference) in which the callback is executed.
     * defaults to the loaded TreeNode.
     */
    requestData : function(node, callback, scope){
        
        if(this.fireEvent("beforeload", this, node, callback) !== false) {
            var fstore = Tine.Tinebase.appMgr.get('Expressomail').getFolderStore(),
                folder = fstore.getById(node.attributes.folder_id),
                path = (folder) ? folder.get('path') : node.attributes.path;
            
            // we need to call doQuery fn from store to transparently do async request
            fstore.asyncQuery('parent_path', path, function(node, callback, scope, data) {
                if (data) {
                    node.beginUpdate();
                    data.each(function(folderRecord) {
                        var n = this.createNode(folderRecord.copy().data);
                        if (n) {
                            node.appendChild(n);
                        }
                    }, this);
                    node.endUpdate();
                }
                this.runCallback(callback, scope || node, [node]);
            }, [node, callback, scope], this, fstore);
            
        } else {
            // if the load is cancelled, make sure we notify
            // the node that we are done
            this.runCallback(callback, scope || node, []);
        }
    },
    
    /**
    * @private
    */
    createNode: function() {
        if(!arguments[0].globalname.match(/^user\/.+$/i) 
                && !arguments[0].globalname.match(/^user$/i) 
                && !arguments[0].globalname.match(/^inbox\/Arquivo Remoto\/.+$/i) 
                && !arguments[0].globalname.match(/^inbox\/Arquivo Remoto$/i)){
            this.inspectCreateNode.apply(this, arguments);
            return Tine.widgets.tree.Loader.superclass.createNode.apply(this, arguments);
            }
        },
    
    
    
    /**
     * inspectCreateNode
     * 
     * @private
     */
    inspectCreateNode: function(attr) {
        var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(attr.account_id);
        
        // NOTE cweiss 2010-06-15 this has to be precomputed on server side!
        attr.has_children = (account && account.get('has_children_support')) ? attr.has_children : true;
        if (attr.has_children == "0") {
            attr.has_children = false;
        }
        
        
        
        
        Ext.apply(attr, {
            leaf: !attr.has_children,
            expandable: attr.has_children,
            cls: 'x-tree-node-collapsed',
            folder_id: attr.id,
            folderNode: true,
            allowDrop: true,
            text: this.app.i18n._hidden(attr.localname)
        });
        
        // show standard folders icons 
        if (account) {
            if (account.get('trash_folder') === attr.globalname) {
                if (attr.cache_totalcount > 0) {
                    attr.cls = 'expressomail-node-trash-full';
                } else {
                    attr.cls = 'expressomail-node-trash';
                }
            }
            if (account.get('sent_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-sent';
            }
            if (account.get('drafts_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-drafts';
            }
            if (account.get('templates_folder') === attr.globalname) {
                attr.cls = 'expressomail-node-templates';
            }
        }
        if (attr.globalname.match(/^inbox$/i)) {
            attr.cls = 'expressomail-node-inbox';
            attr.text = this.app.i18n._hidden('INBOX');
        }
        
        if (attr.globalname.match(/^inbox\/Arquivo Remoto\/.+$/i) || attr.globalname.match(/^inbox\/Arquivo Remoto$/i) ) {
            attr.cls = 'expressomail-node-unselectable';
            attr.is_selectable = false;
            //attr.text = this.app.i18n._hidden('INBOX');
        }
        if (attr.globalname.match(/^junk$/i)) {
            attr.cls = 'expressomail-node-junk';
        }

        if (! attr.is_selectable) {
            attr.cls = 'expressomail-node-unselectable';
        }
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.FilterPanel
 * @extends     Tine.widgets.persistentfilter.PickerPanel
 *
 * <p>Expressomail Favorites Panel</p>
 *
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.FilterPanel
 */
Tine.Expressomail.FilterPanel = Ext.extend(Tine.widgets.persistentfilter.PickerPanel, {
    filterModel: 'Expressomail_Model_MessageFilter'
});

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.TreePanel
 * @extends     Ext.tree.TreePanel
 *
 * <p>Account/Folder Tree Panel</p>
 * <p>Tree of Accounts with folders</p>
 * <pre>
 * low priority:
 * TODO         make inbox/drafts/templates configurable in account
 * TODO         save tree state? @see http://examples.extjs.eu/?ex=treestate
 * TODO         disable delete action in account ctx menu if user has no manage_accounts right
 * </pre>
 *
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.TreePanel
 *
 */
Tine.Expressomail.TreePanel = function(config) {
    Ext.apply(this, config);

    this.addEvents(
        /**
         * @event containeradd
         * Fires when a folder was added
         * @param {folder} the new folder
         */
        'containeradd',
        /**
         * @event containerdelete
         * Fires when a folder got deleted
         * @param {folder} the deleted folder
         */
        'containerdelete',
        /**
         * @event containerrename
         * Fires when a folder got renamed
         * @param {folder} the renamed folder
         */
        'containerrename',
        /**
         * @event beforecontainerrename
         * Fires before any folder is renamed
         * @param {node} the renamed folder node
         */
        'beforecontainerrename'
    );

    Tine.Expressomail.TreePanel.superclass.constructor.call(this);
};

Ext.extend(Tine.Expressomail.TreePanel, Ext.tree.TreePanel, {

    /**
     * @property app
     * @type Tine.Expressomail.Application
     */
    app: null,

    /**
     * @property accountStore
     * @type Ext.data.JsonStore
     */
    accountStore: null,

    /**
     * @type Ext.data.JsonStore
     */
    folderStore: null,

    /**
     * @cfg {String} containerName
     */
    containerName: 'Folder',

    /**
     * TreePanel config
     * @private
     */
    rootVisible: false,

    /**
     * drag n drop
     */
    enableDrop: true,
    ddGroup: 'mailToTreeDDGroup',

    /**
     * @cfg
     */
    border: false,
    recordClass: Tine.Expressomail.Model.Account,
    filterMode: 'filterToolbar',

    /**
     * is needed by Tine.widgets.mainscreen.WestPanel to fake container tree panel
     */
    selectContainerPath: Ext.emptyFn,

    /**
     * init
     * @private
     */
    initComponent: function() {
        // get folder store
        this.folderStore = Tine.Tinebase.appMgr.get('Expressomail').getFolderStore();

        // init tree loader
        this.loader = new Tine.Expressomail.TreeLoader({
            folderStore: this.folderStore,
            app: this.app
        });

        // set the root node
        this.root = new Ext.tree.TreeNode({
            text: 'default',
            draggable: false,
            allowDrop: false,
            expanded: true,
            leaf: false,
            id: 'root'
        });

        // add account nodes
        this.initAccounts();

        // init drop zone
        this.dropConfig = {
            ddGroup: this.ddGroup || 'TreeDD',
            appendOnly: this.ddAppendOnly === true,
            notifyEnter : function() {this.isDropSensitive = true;}.createDelegate(this),
            notifyOut : function() {this.isDropSensitive = false;}.createDelegate(this),
            onNodeOver : function(n, dd, e, data) {
                var node = n.node;

                // auto node expand check (only for non-account nodes)
                if(!this.expandProcId && node.attributes.allowDrop && node.hasChildNodes() && !node.isExpanded()){
                    this.queueExpand(node);
                } else if (! node.attributes.allowDrop) {
                    this.cancelExpand();
                }
                return node.attributes.allowDrop ? 'tinebase-tree-drop-move' : false;
            },
            isValidDropPoint: function(n, dd, e, data){
                return n.node.attributes.allowDrop;
            }
        };

        // init selection model (multiselect)
        this.selModel = new Ext.tree.MultiSelectionModel({});

        // init context menu TODO use Ext.apply
        var initCtxMenu = Tine.Expressomail.setTreeContextMenus.createDelegate(this);
        initCtxMenu();

        // add listeners
        this.on('beforeclick', this.onBeforeClick, this);
        this.on('click', this.onClick, this);
        this.on('contextmenu', this.onContextMenu, this);
        this.on('beforenodedrop', this.onBeforenodedrop, this);
        this.on('append', this.onAppend, this);
        this.on('containeradd', this.onFolderAdd, this);
        this.on('containerrename', this.onFolderRename, this);
        this.on('beforecontainerrename', this.onBeforeFolderRename, this);
        this.on('containerdelete', this.onFolderDelete, this);
        this.selModel.on('selectionchange', this.onSelectionChange, this);
        this.folderStore.on('update', this.onUpdateFolderStore, this);

        // call parent::initComponent
        Tine.Expressomail.TreePanel.superclass.initComponent.call(this);
    },

    /**
     * add accounts from registry as nodes to root node
     * @private
     */
    initAccounts: function() {
        this.accountStore = this.app.getAccountStore();
        this.accountStore.each(this.addAccount, this);
        this.accountStore.on('update', this.onAccountUpdate, this);
    },

    /**
     * init extra tool tips
     */
    initToolTips: function() {
        this.folderTip = new Ext.ToolTip({
            target: this.getEl(),
            delegate: 'a.x-tree-node-anchor',
            renderTo: document.body,
            listeners: {beforeshow: this.updateFolderTip.createDelegate(this)}
        });

        this.folderProgressTip = new Ext.ToolTip({
            target: this.getEl(),
            delegate: '.expressomail-node-statusbox-progress',
            renderTo: document.body,
            listeners: {beforeshow: this.updateProgressTip.createDelegate(this)}
        });

        this.folderUnreadTip = new Ext.ToolTip({
            target: this.getEl(),
            delegate: '.expressomail-node-statusbox-unread',
            renderTo: document.body,
            listeners: {beforeshow: this.updateUnreadTip.createDelegate(this)}
        });
    },

    /**
     * called when tree selection changes
     *
     * @param {} sm
     * @param {} node
     */
    onSelectionChange: function(sm, nodes) {

        //Ignore non-selectable node
        if((typeof(nodes[0]) !== 'undefined')
            && !nodes[0].attributes.is_selectable
        ){
            return;
        }
        if (this.filterMode == 'gridFilter' && this.filterPlugin) {
            this.filterPlugin.onFilterChange();
        }
        if (this.filterMode == 'filterToolbar' && this.filterPlugin) {

            // get filterToolbar
            var ftb = this.filterPlugin.getGridPanel().filterToolbar;
            // in case of filterPanel
            ftb = ftb.activeFilterPanel ? ftb.activeFilterPanel : ftb;

            if (! ftb.rendered) {
                this.onSelectionChange.defer(150, this, [sm, nodes]);
                return;
            }

            // remove path filter
            var filter = this.getFilterPlugin().getFilter();
            var test = filter.value.length == 1 && filter.value[0].match(/^\/[A-z0-9]*$/);
            ftb.supressEvents = true;
            ftb.filterStore.each(function(filter) {
                if (!test) {
                    ftb.deleteFilter(filter);
                }
            }, this);
            ftb.supressEvents = false;

            // returns if account is selected
            if (filter.field == 'path' && (Ext.isEmpty(filter.value) || test))
            {
                return;
            }
            
            // set ftb filters according to tree selection
            ftb.addFilter(new ftb.record(filter));
            
            // Prevent empty path filters from trigger searchMessage
            if (Ext.isEmpty(filter.value)){
                ftb.supressEvents = true;
            }
            ftb.onFiltertrigger();
            ftb.supressEvents = false;

            // finally select the selected node, as filtertrigger clears all selections
            sm.suspendEvents();
            Ext.each(nodes, function(node) {
                sm.select(node, Ext.EventObject, true);
            }, this);
            sm.resumeEvents();
        }
    },

    onFilterChange: Tine.widgets.container.TreePanel.prototype.onFilterChange,

   /**
     * returns a filter plugin to be used in a grid
     * @private
     */
    getFilterPlugin: function() {
        if (!this.filterPlugin) {
            this.filterPlugin = new Tine.widgets.tree.FilterPlugin({
                treePanel: this,
                field: 'path',
                nodeAttributeField: 'path',
                singleNodeOperator: 'in'
            });
        }

        return this.filterPlugin;
    },

    /**
     * convert containerPath to treePath
     *
     * @param {String}  containerPath
     * @return {String} treePath
     */
    getTreePath: function(path) {
        return '/root' + path;
    },

    /**
     * @private
     *
     * expand default account and select INBOX
     */
    afterRender: function() {
        Tine.Expressomail.TreePanel.superclass.afterRender.call(this);
        this.initToolTips();
        this.selectInbox();

        if (this.filterMode == 'filterToolbar' && this.filterPlugin) {
            this.filterPlugin.getGridPanel().filterToolbar.on('change', this.onFilterChange, this);
        }
    },

    /**
     * select inbox of account
     *
     * @param {Record} account
     */
    selectInbox: function(account) {
        var accountId = (account) ? account.id : Tine.Expressomail.registry.get('preferences').get('defaultEmailAccount');

        this.expandPath('/root/' + accountId + '/', null, function(success, parentNode) {
            Ext.each(parentNode.childNodes, function(node) {
                if (Ext.util.Format.lowercase(node.attributes.localname) == 'inbox') {
                    node.select();
                    return false;
                }
            }, this);
        });
    },

    /**
     * called when an account record updates
     *
     * @param {Ext.data.JsonStore} store
     * @param {Tine.Expressomail.Model.Account} record
     * @param {String} action
     */
    onAccountUpdate: function(store, record, action) {
        if (action === Ext.data.Record.EDIT) {
            this.updateAccountStatus(record);
        }
    },

    /**
     * on append node
     *
     * render status box
     *
     * @param {Tine.Expressomail.TreePanel} tree
     * @param {Ext.Tree.TreeNode} node
     * @param {Ext.Tree.TreeNode} appendedNode
     * @param {Number} index
     */
    onAppend: function(tree, node, appendedNode, index) {
        appendedNode.ui.render = appendedNode.ui.render.createSequence(function() {
            var app = Tine.Tinebase.appMgr.get('Expressomail'),
                folder = app.getFolderStore().getById(appendedNode.id);

            if (folder) {
                app.getMainScreen().getTreePanel().addStatusboxesToNodeUi(this);
                app.getMainScreen().getTreePanel().updateFolderStatus(folder);
            }
        }, appendedNode.ui);
    },

    /**
     * add status boxes
     *
     * @param {Object} nodeUi
     */
    addStatusboxesToNodeUi: function(nodeUi) {
        if (nodeUi.elNode.lastChild.className!=="expressomail-node-statusbox") {
            Ext.DomHelper.insertAfter(nodeUi.elNode.lastChild, {tag: 'span', 'class': 'expressomail-node-statusbox', cn:[
                {'tag': 'img', 'src': Ext.BLANK_IMAGE_URL, 'class': 'expressomail-node-statusbox-progress'},
                {'tag': 'span', 'class': 'expressomail-node-statusbox-unread'}
            ]});
        }
    },

    /**
     * on before click handler
     * - accounts are not clickable because fetching all messages of account is too expensive
     * - skip event for folders that are not selectable
     *
     * @param {Ext.tree.AsyncTreeNode} node
     */
    onBeforeClick: function(node) {
        if (this.accountStore.getById(node.id) || ! this.app.getFolderStore().getById(node.id).get('is_selectable')) {
            return false;
        }
    },

    /**
     * on click handler
     *
     * - expand node
     * - update filter toolbar of grid
     * - start check mails delayed task
     *
     * @param {Ext.tree.AsyncTreeNode} node
     * @private
     */
    onClick: function(node) {
        if (node.expandable) {
            node.expand();
        }

        if (node.id && node.id != '/' && node.attributes.globalname != '') {
            var folder = this.app.getFolderStore().getById(node.id);
            this.app.checkMailsDelayedTask.delay(0);
        }
    },

    /**
     * show context menu for folder tree
     *
     * items:
     * - create folder
     * - rename folder
     * - delete folder
     * - ...
     *
     * @param {} node
     * @param {} event
     * @private
     */
    onContextMenu: function(node, event) {
        this.ctxNode = node;
        if (!node.ui || node.ui.elNode.className.search(/x-tree-node-loading/) >= 0) {
            // don't show context menu while node is still loading
            return;
        }

        var folder = this.app.getFolderStore().getById(node.id),
            account = folder ? this.accountStore.getById(folder.get('account_id')) :
                               this.accountStore.getById(node.id);

        if (! folder) {
            // edit/remove account
            if (account.get('ns_personal') !== 'default') {
                this.contextMenuAccount.items.each(function(item) {
                    // check account personal namespace -> disable 'add folder' if namespace is other than root
                    if (item.iconCls == 'action_add') {
                        item.setDisabled(account.get('ns_personal') != '');
                    }
                    // disable filter rules/vacation if no sieve hostname is set
                    if (item.iconCls == 'action_email_replyAll' || item.iconCls == 'action_email_forward') {
                        item.setDisabled(account.get('sieve_hostname') == null || account.get('sieve_hostname') == '');
                    }
                });

                this.contextMenuAccount.showAt(event.getXY());
            }
        } else {
            if (folder.get('globalname') === account.get('trash_folder')) {
                this.contextMenuTrash.showAt(event.getXY());
            } else if (! folder.get('is_selectable')){
                this.unselectableFolder.showAt(event.getXY());
            } else if (folder.get('system_folder')) {
                this.contextMenuSystemFolder.showAt(event.getXY());
            } else if(folder.get('has_children')){
                this.contextMenuUserFolderChildren.showAt(event.getXY());
            } else {
                this.contextMenuUserFolder.showAt(event.getXY());
            }
        }
    },

    /**
     * mail(s) got dropped on node
     *
     * @param {Object} dropEvent
     * @private
     */
    onBeforenodedrop: function(dropEvent) {
        var targetFolderId = dropEvent.target.attributes.folder_id,
            targetFolder = this.app.getFolderStore().getById(targetFolderId);

        this.app.getMainScreen().getCenterPanel().moveSelectedMessages(targetFolder, false);
        return true;
    },

    /**
     * cleanup on destruction
     */
    onDestroy: function() {
        this.folderStore.un('update', this.onUpdateFolderStore, this);
    },

    /**
     * folder store gets updated -> update tree nodes
     *
     * @param {Tine.Expressomail.FolderStore} store
     * @param {Tine.Expressomail.Model.Folder} record
     * @param {String} operation
     */
    onUpdateFolderStore: function(store, record, operation) {
        if (operation === Ext.data.Record.EDIT) {
            this.updateFolderStatus(record);
        }
    },

    /**
     * folders sort function
     *
     * @param {Tine.Expressomail.Model.Folder} node1
     * @param {Tine.Expressomail.Model.Folder} node2
     * @return {Number} 
     */
    sortFolders: function(node1, node2) {
        if (node1.attributes.system_folder && !node2.attributes.system_folder) {
            // system folders come first
            return -1;
        }
        if (node2.attributes.system_folder && !node1.attributes.system_folder) {
            // system folders come first
            return 1;
        }
        // sort folders
        if (node1.text.toUpperCase() > node2.text.toUpperCase()) {
            return 1;
        }
        else if (node1.text.toUpperCase() < node2.text.toUpperCase()) {
            return -1;
        }
        return 0;
    },

    /**
     * add new folder to the store
     *
     * @param {Object} folderData
     */
    onFolderAdd: function(folderData) {
        var recordData = Ext.copyTo({}, folderData, Tine.Expressomail.Model.Folder.getFieldNames());
        var newRecord = Tine.Expressomail.folderBackend.recordReader({responseText: Ext.util.JSON.encode(recordData)});

        var node = this.getNodeById(newRecord.id);
        node.ui.beforeLoad(newRecord); // show loading icon

        Tine.log.debug('Added new folder:' + newRecord.get('globalname'));

        this.folderStore.add([newRecord]);

        var parentId = newRecord.get('parent_path').split('/').pop(),
            parentNode = this.getNodeById(parentId);
        if (! (parentNode instanceof Ext.tree.AsyncTreeNode)) {
            this.app.getFolderStore().getById(parentId).set('has_children', true);
            parentNode.attributes.has_children = true;
            parentNode.attributes.leaf = false;
            parentNode.leaf = false;
            Ext.apply(parentNode,Ext.tree.AsyncTreeNode.prototype); // convert to AsyncTreeNode
            this.folderStore.asyncQuery('parent_path', parentNode.attributes.path, function(node){
                node.reload(); // reload parent node
            }, [parentNode], this, this.folderStore);
        }
        else {
            node.ui.afterLoad(newRecord); // hide loading icon
        }
        parentNode.sort(this.sortFolders); // sort childnodes from parent
        
        this.initNewFolderNode(newRecord);
    },

    /**
     * init new folder node
     *
     * @param {Tine.Expressomail.Model.Folder} newRecord
     */
    initNewFolderNode: function(newRecord) {
        // update paths in node
        var appendedNode = this.getNodeById(newRecord.id);

        if (! appendedNode) {
            // node is not yet rendered -> reload parent
            var parentId = newRecord.get('parent_path').split('/').pop(),
                parentNode = this.getNodeById(parentId);
            parentNode.reload(function() {
                this.initNewFolderNode(newRecord);
            }, this);
            return;
        }

        appendedNode.attributes.path = newRecord.get('path');
        appendedNode.attributes.parent_path = newRecord.get('parent_path');

        // add unreadcount/progress/tooltip
        this.addStatusboxesToNodeUi(appendedNode.ui);
        this.updateFolderStatus(newRecord);
    },
    
    /**
     * prepare folder node in tree panel
     *
     * @param {Object} node
     */
    onBeforeFolderRename: function(node){
        
        this.app.getFolderStore().getById(node.id).set('is_selectable',false);
        node.collapse();
        node.ui.beforeLoad(node);
        
        var selectedNode = this.getSelectionModel().getSelectedNode(),
            isSelectedNode = (selectedNode && node.id == selectedNode.id);
        
        if(isSelectedNode){
            this.app.getMainScreen().getCenterPanel().grid.getSelectionModel().clearSelections();
            this.app.getMainScreen().getCenterPanel().getStore().removeAll();
        }
    },

    /**
     * rename folder in the store
     *
     * @param {Object} folderData
     */
    onFolderRename: function(folderData) {
        
        if(folderData.recent === this.ctxNode.attributes.globalname){
            folderData.recent = '';
            var oldId = this.ctxNode.id;
            var node = this.getNodeById(oldId);

            node.ui.beforeLoad(node); // show loading icon

            this.folderStore.resetQueryAndRemoveRecords('parent_path',  node.attributes.path + node.account_id);

            var recordData = Ext.copyTo({}, folderData, Tine.Expressomail.Model.Folder.getFieldNames());
            var newRecord = Tine.Expressomail.folderBackend.recordReader({responseText: Ext.util.JSON.encode(recordData)});
            this.folderStore.add([newRecord]);
            this.folderStore.remove(this.folderStore.getById(oldId));

            node.setId(newRecord.get('id'));
            node.attributes.path = newRecord.get('path');
            node.attributes.parent_path = newRecord.get('parent_path');
            node.attributes.id = newRecord.get('id');
            node.attributes.folder_id = newRecord.get('id');
            node.attributes.globalname = newRecord.get('globalname');
            node.attributes.localname = newRecord.get('localname');
            node.attributes.text = newRecord.get('text');

            var sm = this.getSelectionModel();
            var selectedNode = sm.getSelectedNode();

            this.getSelectionModel().clearSelections();
            node.removeAll();
            
            var key = this.folderStore.getKey('parent_path', node.attributes.path);
            this.folderStore.queriesDone.remove(key);
            
            this.folderStore.asyncQuery('parent_path', node.attributes.path, function(node){
                if(!node.isLeaf()){
                    node.reload(); // reload node
                }
                else {
                    node.ui.afterLoad(node); // hide loading icon
                }
                var parentId = newRecord.get('parent_path').split('/').pop(),
                    parentNode = this.getNodeById(parentId);
                parentNode.sort(this.sortFolders); // sort childnodes from parent
                
               // if renamed node is the selectedNode, so select it and reload messages
               if(node.id == selectedNode.id){
                   sm.select(selectedNode, Ext.EventObject, true);
               }
                
            }, [node], this, this.folderStore);

            Tine.log.debug('Renamed folder:' + newRecord.get('globalname'));
        }
    },
    
    /**
     * remove deleted folder from the store
     *
     * @param {Object} folderData
     */
    onFolderDelete: function(folderData) {
        var parentId = folderData.parent_path.split('/').pop();
        var parentNode = this.app.getMainScreen().getTreePanel().getNodeById(parentId);
        parentNode.ui.beforeLoad(parentNode); // show loading icon
        
        // if we deleted account, remove it from account store
        if (folderData.record && folderData.record.modelName === 'Account') {
            this.accountStore.remove(this.accountStore.getById(folderData.id));
        }
        
        this.folderStore.remove(this.folderStore.getById(folderData.id));

        if(parentNode.childNodes.length == 0){
            this.app.getFolderStore().getById(parentId).set('has_children', false);
            parentNode.attributes.expandable = false;
            parentNode.attributes.has_children = false;
            parentNode.attributes.leaf = true;
            parentNode.leaf = true;
            this.folderStore.asyncQuery('parent_path', parentNode.attributes.path, function(node){
                node.reload(); // reload parent node
            }, [parentNode], this, this.folderStore);
        }
        else {
            parentNode.ui.afterLoad(parentNode); // hide loading icon
        }
    },

    /**
     * returns tree node id the given el is child of
     *
     * @param  {HTMLElement} el
     * @return {String}
     */
    getElsParentsNodeId: function(el) {
        return Ext.fly(el, '_treeEvents').up('div[class^=x-tree-node-el]').getAttribute('tree-node-id', 'ext');
    },

    /**
     * updates account status icon in this tree
     *
     * @param {Tine.Expressomail.Model.Account} account
     */
    updateAccountStatus: function(account) {
        var imapStatus = account.get('imap_status'),
            node = this.getNodeById(account.id),
            ui = node ? node.getUI() : null,
            nodeEl = ui ? ui.getEl() : null;

        Tine.log.info('Account ' + account.get('name') + ' updated with imap_status: ' + imapStatus);
        if (node && node.ui.rendered) {
            var statusEl = Ext.get(Ext.DomQuery.selectNode('span[class=expressomail-node-accountfailure]', nodeEl));
            if (! statusEl) {
                // create statusEl on the fly
                statusEl = Ext.DomHelper.insertAfter(ui.elNode.lastChild, {'tag': 'span', 'class': 'expressomail-node-accountfailure'}, true);
                statusEl.on('click', function() {
                    Tine.Expressomail.folderBackend.handleRequestException(account.getLastIMAPException());
                }, this);
            }

            statusEl.setVisible(imapStatus === 'failure');
        }
    },

    /**
     * updates folder status icons/info in this tree
     *
     * @param {Tine.Expressomail.Model.Folder} folder
     */
    updateFolderStatus: function(folder) {
        var unreadcount = folder.get('cache_unreadcount'),
            progress    = Math.round(folder.get('cache_job_actions_done') / folder.get('cache_job_actions_est') * 10) * 10,
            node        = this.getNodeById(folder.id),
            ui = node ? node.getUI() : null,
            nodeEl = ui ? ui.getEl() : null,
            cacheStatus = folder.get('cache_status'),
            lastCacheStatus = folder.modified ? folder.modified.cache_status : null,
            isSelected = folder.isCurrentSelection();

        this.setUnreadClass(folder.id);

        if (node && node.ui.rendered) {
            var domNode = Ext.DomQuery.selectNode('span[class=expressomail-node-statusbox-unread]', nodeEl);
            if (domNode) {

                // update unreadcount + visibity
                Ext.fly(domNode).update(unreadcount).setVisible(unreadcount > 0);

                // update progress
                var progressEl = Ext.get(Ext.DomQuery.selectNode('img[class^=expressomail-node-statusbox-progress]', nodeEl));
                progressEl.removeClass(['expressomail-node-statusbox-progress-pie', 'expressomail-node-statusbox-progress-loading']);
                if (! Ext.isNumber(progress)) {
                    progressEl.setStyle('background-position', 0 + 'px');
                    progressEl.addClass('expressomail-node-statusbox-progress-loading');
                } else {
                    progressEl.setStyle('background-position', progress + '%');
                    progressEl.addClass('expressomail-node-statusbox-progress-pie');
                }
                progressEl.setVisible(isSelected && cacheStatus !== 'complete' && cacheStatus !== 'disconnect' && progress !== 100 && lastCacheStatus !== 'complete');
            }
        }
    },

    /**
     * set unread class of folder node and parents
     *
     * @param {Tine.Expressomail.Model.Folder} folder
     *
     * TODO make it work correctly for parents (use events) and activate again
     */
    setUnreadClass: function(folderId) {
        var folder              = this.app.getFolderStore().getById(folderId),
            node                = this.getNodeById(folderId),
            isUnread            = folder.get('cache_unreadcount') > 0,
            hasUnreadChildren   = folder.get('unread_children').length > 0;

        if (node && node.ui.rendered) {
            var ui = node.getUI();
            ui[(isUnread || hasUnreadChildren) ? 'addClass' : 'removeClass']('expressomail-node-unread');
        }

        // get parent, update and call recursivly
//        var parentFolder = this.app.getFolderStore().getParent(folder);
//        if (parentFolder) {
//            // need to create a copy of the array here (and make sure it is unique)
//            var unreadChildren = Ext.unique(parentFolder.get('unread_children'));
//
//            if (isUnread || hasUnreadChildren) {
//                unreadChildren.push(folderId);
//            } else {
//                unreadChildren.remove(folderId);
//            }
//            parentFolder.set('unread_children', unreadChildren);
//            this.setUnreadClass(parentFolder.id);
//        }
    },

    /**
     * updates the given tip
     * @param {Ext.Tooltip} tip
     */
    updateFolderTip: function(tip) {
        var folderId = this.getElsParentsNodeId(tip.triggerElement),
            folder = this.app.getFolderStore().getById(folderId);
        if ((folder && !this.isDropSensitive) && (folder.get('client_access_time') != '')) {
            var limit = parseInt(folder.get('quota_limit'), 10),
                usage = parseInt(folder.get('quota_usage'), 10),
                left = limit - usage,
                percentage = String.format(this.app.i18n._('{0} %'), Ext.util.Format.number((left / limit * 100), '00.0'));
            var info = [
                '<table>',
                    '<tr>',
                        '<td>', this.app.i18n._('Total Messages:'), '</td>',
                        '<td>', folder.get('cache_totalcount'), '</td>',
                    '</tr>',
                    '<tr>',
                        '<td>', this.app.i18n._('Unread Messages:'), '</td>',
                        '<td>', folder.get('cache_unreadcount'), '</td>',
                    '</tr>',
                    '<tr>',
                        '<td>', this.app.i18n._('Name on Server:'), '</td>',
                        '<td>', folder.get('globalname'), '</td>',
                    '</tr>',
                    '<tr>',
                        '<td>', this.app.i18n._('Your quota'), ':</td>',
                        '<td>', isNaN(limit * 1024) ? "-" : Ext.util.Format.fileSize(limit * 1024), '</td>',
                    '</tr>',
                    '<tr>',
                        '<td>', this.app.i18n._('Available quota'), ' :</td>',
                        '<td>', isNaN(left * 1024) ? "-" : Ext.util.Format.fileSize(left * 1024),
                        ' (',  percentage,')</td>',
                    '</tr>',
                     '<tr>',
                        '<td>', this.app.i18n._('Last update:'), '</td>',
                        '<td>', Tine.Tinebase.common.dateTimeRenderer(folder.get('client_access_time')), '</td>',
                    '</tr>',
                '</table>'
            ];
            tip.body.dom.innerHTML = info.join('');
        } else {
            return false;
        }
    },

    /**
     * updates the given tip
     * @param {Ext.Tooltip} tip
     */
    updateProgressTip: function(tip) {
        var folderId = this.getElsParentsNodeId(tip.triggerElement),
            folder = this.app.getFolderStore().getById(folderId),
            progress = Math.round(folder.get('cache_job_actions_done') / folder.get('cache_job_actions_est') * 100);
        if (! this.isDropSensitive) {
            tip.body.dom.innerHTML = String.format(this.app.i18n._('Fetching messages... ({0}%% done)'), progress);
        } else {
            return false;
        }
    },

    /**
     * updates the given tip
     * @param {Ext.Tooltip} tip
     */
    updateUnreadTip: function(tip) {
        var folderId = this.getElsParentsNodeId(tip.triggerElement),
            folder = this.app.getFolderStore().getById(folderId),
            count = folder.get('cache_unreadcount');

        if (! this.isDropSensitive) {
            tip.body.dom.innerHTML = String.format(this.app.i18n.ngettext('{0} unread message', '{0} unread messages', count), count);
        } else {
            return false;
        }
    },

    /**
     * decrement unread count of currently selected folder
     */
    decrementCurrentUnreadCount: function() {
        var store  = Tine.Tinebase.appMgr.get('Expressomail').getFolderStore(),
            node   = this.getSelectionModel().getSelectedNode(),
            folder = node ? store.getById(node.id) : null;

        if (folder) {
            folder.set('cache_unreadcount', parseInt(folder.get('cache_unreadcount'), 10) -1);
            folder.commit();
        }
    },

    /**
     * add account record to root node
     *
     * @param {Tine.Expressomail.Model.Account} record
     */
    addAccount: function(record) {

        var node = new Ext.tree.AsyncTreeNode({
            id: record.data.id,
            path: '/' + record.data.id,
            record: record,
            globalname: '',
            draggable: false,
            allowDrop: false,
            expanded: false,
            text: Ext.util.Format.htmlEncode(record.get('name')),
            qtip: Tine.Tinebase.common.doubleEncode(record.get('host')),
            leaf: false,
            cls: 'expressomail-node-account',
            delimiter: record.get('delimiter'),
            ns_personal: record.get('ns_personal'),
            account_id: record.data.id,
            listeners: {
                scope: this,
                load: function(node) {
                    var account = this.accountStore.getById(node.id);
                    this.updateAccountStatus(account);
                }
            }
        });

        // we don't want appending folder effects
        this.suspendEvents();
        this.root.appendChild(node);
        this.resumeEvents();
    },

    /**
     * get active account by checking selected node
     * @return Tine.Expressomail.Model.Account
     */
    getActiveAccount: function() {
        var result = null;
        var node = this.getSelectionModel().getSelectedNode();
        if (node) {
            var accountId = node.attributes.account_id;
            result = this.accountStore.getById(accountId);
        }

        return result;
    }
});
/*
 * Tine 2.0
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @copyright   Copyright (c) 2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */
Ext.ns('Tine.Expressomail');

/**
 * display panel item registry per MIME type
 * 
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.MimeDisplayManager
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @singleton
 */
Tine.Expressomail.MimeDisplayManager = function() {
    var items = {},
        alternatives = {};
    
    return {
        /**
         * creates a new details panel for given mimeType
         * 
         * @param {String} mimeType
         * @return {Tine.widgets.grid.DisplayPanel}
         */
        create: function(mimeType, config) {
            var c = this.get(mimeType);
            
            if (! c) {
                throw new Ext.Error('No details panel registered for ' + mimeType);
            }
            
            return new c(config || {});
        },
        
        /**
         * returns basic form of MIME type if a display panel is registered for it
         * 
         * @param {String} mimeType
         * @return {String}
         */
        getMainType: function(mimeType) {
            var mainType = alternatives[mimeType];
            
            return items.hasOwnProperty(mainType) ? mainType : null;
        },
        
        /**
         * returns Display Panel for given MIME type
         * 
         * @param  {String} mimeType
         * @return {Function} consturctor function of a Tine.widgets.grid.DisplayPanel 
         */
        get: function(mimeType) {
            var mainType = this.getMainType(mimeType);
            
            return mainType ? items[mainType] : null;
        },
        
        /**
         * register Display Panel for given MIME type
         * 
         * @param {String/Array} mimeType
         * @param {Function} displayPanel consturctor function of a Tine.widgets.grid.DisplayPanel 
         * @param {Array} otherTypes
         */
        register: function(mimeType, displayPanel, otherTypes) {
            if (items.hasOwnProperty(mimeType)) {
                return;
                //throw new Ext.Error('There is already an registration for ' + mimeType);
            }
            
            items[mimeType] = displayPanel;
            alternatives[mimeType] = mimeType;
            
            if (Ext.isArray(otherTypes)) {
                Ext.each(otherTypes, function(otherType) {
                    if (alternatives.hasOwnProperty(otherType)) {
                        throw new Ext.Error(alternatives[otherType] + ' is already registered as alternative for ' + otherType);
                    }
                    alternatives[otherType] = mimeType;
                }, this);
            }
        }
    };
}();/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 */

Ext.ns('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.GridDetailsPanel
 * @extends     Tine.widgets.grid.DetailsPanel
 *
 * <p>Message Grid Details Panel</p>
 * <p>the details panel (shows message content)</p>
 *
 * TODO         replace telephone numbers in emails with 'call contact' link
 * TODO         make only text body scrollable (headers should be always visible)
 * TODO         show image attachments inline
 * TODO         add 'download all' button
 * TODO         'from' to contact: check for duplicates
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.GridDetailsPanel
 */
Tine.Expressomail.GridDetailsPanel = Ext.extend(Tine.widgets.grid.DetailsPanel, {

    /**
     * config
     * @private
     */
    defaultHeight: 350,
    currentId: null,
    record: null,
    app: null,
    i18n: null,
    isNavKey: false,
    isDecrypting: false,

    fetchBodyTransactionId: null,

    // model generics
    recordClass: Tine.Expressomail.Model.Rule,
    recordProxy: Tine.Expressomail.rulesBackend,

    /**
     * init
     * @private
     */
    initComponent: function() {
        this.initTemplate();
        this.initDefaultTemplate();
        //this.initTopToolbar();

        Tine.Expressomail.GridDetailsPanel.superclass.initComponent.call(this);
    },

    /**
     * use default Tpl for default and multi view
     */
    initDefaultTemplate: function() {
        this.defaultTpl = new Ext.XTemplate(
            '<div class="preview-panel-expressomail">',
                '<div class="preview-panel-expressomail-body">{[values ? values.msg : ""]}</div>',
            '</div>'
        );
    },

    /**
     * init bottom toolbar (needed for event invitations atm)
     *
     * TODO add buttons (show header, add to addressbook, create filter, show images ...) here
     */
//    initTopToolbar: function() {
//        this.tbar = new Ext.Toolbar({
//            hidden: true,
//            items: []
//        });
//    },

    /**
     * add on click event after render
     * @private
     */
    afterRender: function() {
        Tine.Expressomail.GridDetailsPanel.superclass.afterRender.apply(this, arguments);
        this.body.on('click', this.onClick, this);
    },

    fromApplet: function(data)
    {
        if (data) {
            // TODO: Change signedMessage to emlTosend
            this.record.set('eid', data.eid);
            this.record.set('body', data.body);
            this.record.set('attachments', data.attachments);
            this.record.set('decrypted', true);
            this.record.set('has_signature', data.has_signature);
            this.record.set('signature_info', data.signature_info);
            if (data.has_signature === true && !data.signature_info) {
                Tine.Tinebase.DigitalCertificateBackend.verifyCertificates(new Array(data.signatures[0].certificate),
                    {
                        callback: this.afterValidation.createDelegate(this)
                    });
            } else {
                this.updateDetails(this.record, '', true);
            }
        } else {
            Tine.log.debug('Operation Canceled!');
            this.loadMask.hide();
        }

    },

    afterValidation: function(request, success, response) {
        var data;
        if (success) {
            data = Ext.util.JSON.decode(response.responseText);
            // We should have only one certificate signing the message
            this.record.set('signature_info', {
                success: data.results[0].success,
                certificate: data.results[0].certificate,
                msgs: data.results[0].messages
            });
            this.fromApplet(this.record.data);
        } else {
            // TODO: Error message?
        }
    },

    /**
     * get panel for single record details
     *
     * @return {Ext.Panel}
     */
    getSingleRecordPanel: function() {
        if (! this.singleRecordPanel) {
            this.singleRecordPanel = new Ext.Panel({
                layout: 'vbox',
                layoutConfig: {
                    align:'stretch'
                },
                border: false,
                items: [
                    //this.getTopPanel(),
                    this.getMessageRecordPanel()
                    //this.getBottomPanel()
                ]
            });
        }
        return this.singleRecordPanel;
    },

    /**
     * get panel for single record details
     *
     * @return {Ext.Panel}
     */
    getMessageRecordPanel: function() {
        if (! this.messageRecordPanel) {
            this.messageRecordPanel = new Ext.Panel({
                border: false,
                autoScroll: true,
                flex: 1
            });
        }
        return this.messageRecordPanel;
    },

    /**
     * (on) update details
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {String} body
     * @private
     */
    updateDetails: function(record, body) {
        var encrypted = record.get('smimeEml') != '';
        var decrypted = record.get('decrypted') || false;
        if (record.id === this.currentId) {
            // nothing to do
        } else if (! record.bodyIsFetched()) {
            this.waitForContent(record, this.getMessageRecordPanel().body);
        } else if (encrypted) {
            if (!decrypted && !this.isNavKey) {
                this.isDecrypting = true;
                Tine.Expressomail.addSecurityApplet('SecurityApplet'); // Add applet if not already
                Tine.Expressomail.toSecurityApplet(this.id, Ext.util.JSON.encode(record.data), 'DECRYPT'); // call applet
            } else {
                // fromApplet

                // TODO: set signature, attachments, etc
                this.setTemplateContent(record, this.getMessageRecordPanel().body);
                this.fireEvent('dblwindow');
            }
            this.isNavKey = false;
        } else if (record === this.record) {
            this.setTemplateContent(record, this.getMessageRecordPanel().body);
            this.fireEvent('dblwindow');
        }
    },

    /**
     * wait for body content
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {String} body
     */
    waitForContent: function(record, body) {
        if (! this.grid || this.grid.getSelectionModel().getCount() == 1) {
            this.refetchBody(record, {
                success: this.updateDetails.createDelegate(this, [record, body]),
                failure: function (exception) {
                    Tine.log.debug(exception);
                    this.getLoadMask().hide();
                    if (exception.code == 404) {
                        this.defaultTpl.overwrite(body, {msg: this.app.i18n._('Message not available.')});
                    } else {
                        Tine.Expressomail.messageBackend.handleRequestException(exception);
                    }
                },
                scope: this
            });
            this.defaultTpl.overwrite(body, {msg: ''});
            this.getLoadMask().show();
            this.fireEvent('dblwindow');
        } else {
            this.getLoadMask().hide();
        }
    },

    /**
     * refetch message body
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {Function} callback
     */
    refetchBody: function(record, callback) {
        // cancel old request first
        if (this.fetchBodyTransactionId && ! Tine.Expressomail.messageBackend.isLoading(this.fetchBodyTransactionId)) {
            Tine.log.debug('Tine.Expressomail.GridDetailsPanel::refetchBody -> cancelling current fetchBody request.');
            Tine.Expressomail.messageBackend.abort(this.fetchBodyTransactionId);
        }
        Tine.log.debug('Tine.Expressomail.GridDetailsPanel::refetchBody -> calling fetchBody');
        this.fetchBodyTransactionId = Tine.Expressomail.messageBackend.fetchBody(record, callback);
    },

    /**
     * overwrite template with (body) content
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {String} body
     *
     * TODO allow other prepared parts than email invitations
     */
    setTemplateContent: function(record, body) {
        var encrypted = record.get('smimeEml') != '';
        this.currentId = record.id;
        this.getLoadMask().hide();

        this.doLayout();

        this.tpl.overwrite(body, record.data);
        this.getEl().down('div').down('div').scrollTo('top', 0, false);

        if (this.record.get('preparedParts') && this.record.get('preparedParts').length > 0) {
            Tine.log.debug('Tine.Expressomail.GridDetailsPanel::setTemplateContent about to handle preparedParts');
            this.handlePreparedParts(record);
        }

        if (!Tine.Tinebase.registry.get('preferences').get('windowtype')=='Browser') {
            this.app.mainScreen.GridPanel.reloadActionToolbar(encrypted);
        }
    },

    /**
     * init single message template (this.tpl)
     * @private
     */
    showErrorTemplate: function(error_message) {
        this.record.data.error_message = error_message;
        this.setTemplateContent(this.record, this.getMessageRecordPanel().body);
    },

    /**
     * handle invitation messages (show top + bottom panels)
     *
     * @param {Tine.Expressomail.Model.Message} record
     */
    handlePreparedParts: function(record) {
        var firstPreparedPart = this.record.get('preparedParts')[0],
            mimeType = String(firstPreparedPart.contentType).split(/[ ;]/)[0];

        // hack for "browser style" popups
        if(!Tine.Expressomail.MimeDisplayManager.getMainType(mimeType)){
            Tine.Expressomail.MimeDisplayManager.register('text/calendar', Tine.Calendar.iMIPDetailsPanel);
            Tine.Expressomail.MimeDisplayManager.register('text/readconf', Tine.Expressomail.ReadConfirmationDetailsPanel);
        }

        var mainType = Tine.Expressomail.MimeDisplayManager.getMainType(mimeType);

        if (! mainType) {
            Tine.log.info('Tine.Expressomail.GridDetailsPanel::handlePreparedParts nothing found to handle ' + mimeType);
            return;
        }

        var bodyEl = this.getMessageRecordPanel().getEl().query('div[class=preview-panel-expressomail-body]')[0],
            detailsPanel = Tine.Expressomail.MimeDisplayManager.create(mainType, {
                    preparedPart: firstPreparedPart
                });

        // quick hack till we have a card body here
        Ext.fly(bodyEl).update('');
        detailsPanel.render(bodyEl);
    },

    /**
     * init single message template (this.tpl)
     * @private
     */
    initTemplate: function() {

        this.tpl = new Ext.XTemplate(
            '<div class="preview-panel-expressomail">',
                '<div class="preview-panel-expressomail-headers {[(values.smimeEml ? "x-html-editor-encrypted" : "x-html-editor-decrypted")]}">',
                    '<b>' + this.i18n._('Subject') + ':</b> {[this.encode(values.subject)]}<br/>',
                    '<b>' + this.i18n._('From') + ':</b>',
                    ' {[this.showFrom(values.from_email, values.from_name, "' + this.i18n._('Add') + '", "'
                        + this.i18n._('Add contact to addressbook') + '")]}<br/>',
                    '<b>' + this.i18n._('Date') + ':</b> {[this.showDate(values.sent, values)]}',
                    '{[this.showRecipients(values.headers)]}',
                    '{[this.showHeaders("' + this.i18n._('Show or hide header information') + '")]}',
                '</div>',
                '<div class="preview-panel-felamimail-signature">{[this.showSignatureInfo(values.signature_info, values)]}</div>',
                '<div class="preview-panel-expressomail-attachments">{[this.showAttachments(values.attachments, values)]}</div>',
                '<div class="preview-panel-expressomail-body">{[this.showBody(values.body, values)]}</div>',
                '{[this.showErrorMessage(values.error_message, values)]}',
                '{[this.showDecryptButton(values)]}',
            '</div>',{
            app: this.app,
            panel: this,
            encode: function(value) {
                if (value) {
                    var encoded = Ext.util.Format.htmlEncode(value);
                    encoded = Ext.util.Format.nl2br(encoded);
                    // it should be enough to replace only 2 or more spaces
                    encoded = encoded.replace(/ /g, '&nbsp;');

                    return encoded;
                } else {
                    return '';
                }
                return value;
            },

            showDate: function(sent, messageData) {
                var date = (sent) ? sent : messageData.received;
                return Tine.Tinebase.common.dateTimeRenderer(date);
            },

            showFrom: function(email, name, addText, qtip) {
                if (name === null) {
                    return '';
                }

                var result = this.encode(name + ' <' + email + '>');

                // add link with 'add to contacts'
                var id = Ext.id() + ':' + email;

                var nameSplit = name.match(/^"*([^,^ ]+)(,*) *(.+)/i);
                var firstname = (nameSplit && nameSplit[1]) ? nameSplit[1] : '';
                var lastname = (nameSplit && nameSplit[3]) ? nameSplit[3] : '';
                if (nameSplit && nameSplit[2] == ',') {
                    firstname = lastname;
                    lastname = nameSplit[1];
                }

                id += Ext.util.Format.htmlEncode(':' + Ext.util.Format.trim(firstname) + ':' + Ext.util.Format.trim(lastname));
                result += ' <span ext:qtip="' + Tine.Tinebase.common.doubleEncode(qtip) + '" id="' + id + '" class="tinebase-addtocontacts-link">[+]</span>';
                                //if no sieve hostname is defined, block sender feature is not enabled
                var account = this.app.getActiveAccount(),
                    sieve_hostname = account.get('sieve_hostname');
                if( sieve_hostname && (sieve_hostname !== null || sieve_hostname !== '') )
                {
                    result += ' <span ext:qtip="Bloquear remetente" id="' + id + '" class="expressomail-addsievefilter-link">[x]</span>';
                }
                return result;
            },

            showBody: function(body, messageData) {
                body = body || '';
                if (body) {
                    var account = this.app.getActiveAccount();
                    if (account && (account.get('display_format') == 'plain' ||
                        (account.get('display_format') == 'content_type' && messageData.body_content_type == 'text/plain'))
                    ) {
                        var width = this.panel.body.getWidth()-25,
                            height = this.panel.body.getHeight()-90,
                            id = Ext.id();

                        if (height < 0) {
                            // sometimes the height is negative, fix this here
                            height = 500;
                        }

                        body = '<textarea ' +
                            'style="width: ' + width + 'px; height: ' + height + 'px; " ' +
                            'autocomplete="off" id="' + id + '" name="body" class="x-form-textarea x-form-field x-ux-display-background-border" readonly="" >' +
                            body + '</textarea>';
                    }
                }
                return body;
            },

            showHeaders: function(qtip) {
                var result = ' <span ext:qtip="' + Tine.Tinebase.common.doubleEncode(qtip) + '" id="' + Ext.id() + ':show" class="tinebase-showheaders-link">[...]</span>';
                return result;
            },

            showRecipients: function(value) {
                if (value) {
                    var i18n = Tine.Tinebase.appMgr.get('Expressomail').i18n,
                        result = '';
                    for (header in value) {
                        if (value.hasOwnProperty(header) && (header == 'to' || header == 'cc' || header == 'bcc')) {
                            result += '<br/><b>' + i18n._hidden(Ext.util.Format.capitalize(header)) + ':</b> '
                                + Ext.util.Format.htmlEncode(value[header]).replace(/,(?!\s)/g,', ');
                        }
                    }
                    return result;
                } else {
                    return '';
                }
            },

            showAttachments: function(attachments, messageData) {

                if(attachments.length == 0) return '';

                var result = '<b> ' + this.app.i18n._('Attachments') + ':</b> ';

                var encrypted = messageData.smimeEml != '';

                for(var i=0; i < attachments.length; i++) {
                    if(encrypted) {
                        result +=
                            '<a id="' + Ext.id() + ':' + i +
                                '" href="https://local.expressov3.serpro.gov.br:8998/download/' + attachments[i].eid + '" target="_blank"> ' +
                                '<i>' + attachments[i].filename + '</i> ' +
                                '(' + Ext.util.Format.fileSize(attachments[i].size) + ')</a>  ';
                    }
                    else {
                        result +=
                            '<span id="' + Ext.id() + ':' + i +
                                '" class="tinebase-download-link"> ' +
                                '<i>' + attachments[i].filename.replace(/[a-zA-Z]:[\\\/]fakepath[\\\/]/, '') + '</i> ' +
                                '(' + Ext.util.Format.fileSize(attachments[i].size) + ')</span>  ';
                    }
                }

                if(attachments.length > 1) {
                    if(encrypted) {
                        result +=
                            '<a ext:qtip="' + this.app.i18n._('Download all attachments') +
                                '" id="' + Ext.id() + ':A" ' +
                                'href="https://local.expressov3.serpro.gov.br:8998/downloadZip/' + messageData.eid + '" target="_blank"> ' +
                                '[' + this.app.i18n._('All attachments') + ']</a>';
                    }
                    else {
                        result +=
                            '<span ext:qtip="' + this.app.i18n._('Download all attachments') +
                                '" id="' + Ext.id() + ':A" ' +
                                'class="tinebase-download-link"> ' +
                                '[' + this.app.i18n._('All attachments') + ']</span>';
                    }
                }
                else if(attachments.length == 1 && attachments[0].filename.substring(attachments[0].filename.length-4).toUpperCase() == '.EML') {
                    result +=
                        '<span ext:qtip="' + this.app.i18n._('Download attachment') +
                            '" id="' + Ext.id() + ':A" ' +
                            'class="tinebase-download-link">' +
                            '[' + this.app.i18n._('Download attachment') + ']</span>';
                }

                return result;
            },

            showSignatureInfo: function(signature_info, messageData) {
                var result = signature_info ? '<b>' + Ext.util.Format.htmlEncode(this.app.i18n._('Digital Signature')) + ':</b> ' : '';

                if (signature_info)
                {
                    if (signature_info.success)
                    {
                        result += Ext.util.Format.htmlEncode(this.app.i18n._('Verification Successful!'));

                    } else {
                        result += Ext.util.Format.htmlEncode(this.app.i18n._('Verification Failed!'));
                    }

                    result += ' <a ext:qtip="'+
                        Ext.util.Format.htmlEncode(this.app.i18n._('show details of digitial signature verification')) +
                        '" class="expressomail-smime-details">[...]</a>';
                }

                return result;
            },

            showErrorMessage: function(error_message, messageData) {
                var div = '<div class="preview-panel-error-message x-html-editor-encrypted-noicon">';
                var result = error_message ? div + '<p><b>' + Ext.util.Format.htmlEncode(error_message) + '</b></p></div>' : '';

                return result;
            },

            showDecryptButton: function(messageData) {
                if (messageData.smimeEml && !messageData.decrypted) {
                    result = '<div class="preview-panel-error-message x-html-editor-encrypted-noicon"><p><b>' +
                             '<a href="#" class="expressomail-decrypt-message">' + this.app.i18n._('Click here or press ENTER to decypher this message.') + '</a></b></p></div>';
                }
                else {
                    result = '';
                }

                return result;
            }
        });
    },

    reloadDetails: function() {
        // force the reload of currently selected message
        this.getLoadMask().show();
        this.currentId = null;
        this.updateDetails(this.record, this.getSingleRecordPanel().body);
    },

    blockSender: function(){

        var transResponse = Ext.util.JSON.decode(this.recordProxy.transId.conn.response);

        if(typeof(transResponse == 'undefined')) //IE 9 native type handler
            transResponse = Ext.util.JSON.decode(this.recordProxy.transId.conn.responseText);

        var resultCount = transResponse.result.totalcount;

        var defaultRule = [],
            result = [],
            condition,
            rules = [];

        for (i = 0; i < resultCount; i++ )
        {
            var idTmp = i + 1;
            transResponse.result.results[i].id = String(idTmp);
            rules.push(transResponse.result.results[i]);

//            rules[i].set('id', String(idTmp));

        }

        var defaultRule = new Tine.Expressomail.Model.Rule( Tine.Expressomail.Model.Rule.getDefaultData(),Ext.id() );

        defaultRule.set('action_type', 'discard');

        defaultRule.set('account_id', '');
        defaultRule.set('container_id', '');
        defaultRule.set('created_by', '');
        defaultRule.set('creation_time', '');
        defaultRule.set('deleted_by', '');
        defaultRule.set('deleted_time', '');

        var id = resultCount + 1;
        defaultRule.set('id', String(id));
        defaultRule.set('last_modified_by', '');
        defaultRule.set('last_modified_time', '');

        //var result = [], condition;
        condition = {
            test: 'address',
            header: 'from',
            comperator: 'contains',
            key: this.record.json.from_email
        };

        result.push(condition);

        defaultRule.set('conditions', result);

        //var rules = [];
        rules.push(defaultRule.data);

        Tine.Expressomail.rulesBackend.saveRules(this.record.json.account_id, rules, {
            scope: this,
            success: function(record) {
                    this.purgeListeners();
            },
            failure: Tine.Expressomail.handleRequestException.createSequence(function() {
                this.loadMask.hide();
            }, this),
            timeout: 150000
        });
    },

    certificateDetails: function(certificate) {

    },

    /**
     * on click for attachment download / compose dlg / edit contact dlg
     *
     * @param {} e
     * @private
     */
    onClick: function(e) {
        var selectors = [
            'a[class=expressomail-decrypt-message]',
            'span[class=tinebase-download-link]',
            'a[class=tinebase-email-link]',
            'a[class=expressomail-smime-details]',
            'span[class=tinebase-addtocontacts-link]',
            'span[class=tinebase-showheaders-link]',
            'span[class=expressomail-addsievefilter-link]'
        ];

        // find the correct target
        for (var i=0, target=null, selector=''; i < selectors.length; i++) {
            target = e.getTarget(selectors[i]);
            if (target) {
                selector = selectors[i];
                break;
            }
        }

        Tine.log.debug('Tine.Expressomail.GridDetailsPanel::onClick found target:"' + selector + '".');

        switch (selector) {
            case 'a[class=expressomail-decrypt-message]':
                this.reloadDetails();
                break;

            case 'span[class=tinebase-download-link]':
                var idx = target.id.split(':')[1],
                    attachment = this.record.get('attachments')[idx];

                if (! this.record.bodyIsFetched()) {
                    // sometimes there is bad timing and we do not have the attachments available -> refetch body
                    this.refetchBody(this.record, this.onClick.createDelegate(this, [e]));
                    return;
                }

                // remove part id if set (that is the case in message/rfc822 attachments)
                var messageId;

                if(idx == 'A'){
                    messageId = this.record.id;
                    new Ext.ux.file.Download({
                        params: {
                            requestType: 'HTTP',
                            method: 'Expressomail.downloadAttachment',
                            messageId: messageId,
                            partId: 'A',
                            getAsJson: false
                        }
                    }).start();
                } else {
                    messageId = (this.record.id.match(/_/)) ? this.record.id.split('_')[0] : this.record.id;
                    if (attachment['content-type'] === 'message/rfc822') {

                        Tine.log.debug('Tine.Expressomail.GridDetailsPanel::onClick openWindow for:"' + messageId + '_' + attachment.partId + '".');
                        // display message
                        Tine.Expressomail.MessageDisplayDialog.openWindow({
                            record: new Tine.Expressomail.Model.Message({
                                id: messageId + '_' + attachment.partId
                            })
                        });

                    } else {
                        // download attachment
                        new Ext.ux.file.Download({
                            params: {
                                requestType: 'HTTP',
                                method: 'Expressomail.downloadAttachment',
                                messageId: messageId,
                                partId: attachment.partId,
                                getAsJson: false
                            }
                        }).start();
                    }
                }
                break;

            case 'a[class=expressomail-smime-details]':
                var sigInfo = this.record.get('signature_info');
                var msg = '';

                if (typeof(sigInfo.certificate) == 'object') {
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Serial Number')+': </span> ' +
                            '<span>'+sigInfo.certificate.serialNumber+'</span><br />';
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Issuer')+': </span> ' +
                            '<span>'+sigInfo.certificate.issuerCn+'</span><br />';
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Owner')+': </span> ' +
                            '<span>'+sigInfo.certificate.cn+'</span><br />';
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Email')+': </span> ' +
                            '<span>'+sigInfo.certificate.email+'</span><br />';
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Valid From')+': </span> ' +
                            '<span>'+sigInfo.certificate.validFrom+'</span><br />';
                    msg += '<span style="float:left;width:10em;text-align:left;font-weight:bold">'+
                            this.app.i18n._('Valid To')+': </span> ' +
                            '<span>'+sigInfo.certificate.validTo+'</span><br />';
                }

                if (!sigInfo.success)  {
                    msg += '<br /><span style="font-weight:bold">'+this.app.i18n._('Errors')+': </span><br />';
                    for (var j = 0; j < sigInfo.msgs.length; j++) {
                        msg += '<span style="padding-left:6em;color:#FF0000">'+this.app.i18n._(sigInfo.msgs[j])
                            +'</span><br />';
                    }
                }

                Ext.MessageBox.show({
                    title:      this.app.i18n._('Verification Details'),
                    msg:        msg,
                    buttons:    Ext.Msg.OK,
                    width:      500
                });
                break;

            case 'a[class=tinebase-email-link]':
                // open compose dlg
                // support RFC 6068
                var mailto = target.id.split('123:')[1],
                    to = typeof mailto.split('?')[0] != 'undefined' ? mailto.split('?')[0].split(',') : [],
                    fields = typeof mailto.split('?')[1] != 'undefined' ? mailto.split('?')[1].split('&') : [],
                    subject = '', body = '', cc = [], bcc = [];

                var defaults = Tine.Expressomail.Model.Message.getDefaultData();

                Ext.each(fields, function(field){
                    var test = field.split('=');
                    switch (Ext.util.Format.lowercase(test[0]))
                    {
                        case 'subject':
                            subject = decodeURIComponent(test[1]);
                            break;
                        case 'body':
                            test[1] = test[1].replace(/%0A/g, '<br />'); // adding line breaks
                            body = decodeURIComponent(test[1]);
                            break;
                        case 'to':
                            to = Ext.isEmpty(to) ? test[1].split[','] : to;
                        case 'cc':
                            cc = Ext.isEmpty(cc) ? test[1].split[','] : cc;
                            break;
                        case 'bcc':
                            bcc = Ext.isEmpty(bcc) ? test[1].split[','] : bcc;
                            break;
                    }
                });

                defaults.to = to;
                defaults.cc = cc;
                defaults.bcc = bcc;
                defaults.subject = subject;
                defaults.body = body + Tine.Expressomail.getSignature();

                var record = new Tine.Expressomail.Model.Message(defaults, 0);
                var popupWindow = Tine.Expressomail.MessageEditDialog.openWindow({
                    record: record
                });
                break;

            case 'span[class=tinebase-addtocontacts-link]':
                // open edit contact dlg

                // check if addressbook app is available
                if (! Tine.Addressbook || ! Tine.Tinebase.common.hasRight('run', 'Addressbook')) {
                    return;
                }

                var id = Ext.util.Format.htmlDecode(target.id);
                var parts = id.split(':');

                var popupWindow = Tine.Addressbook.ContactEditDialog.openWindow({
                    listeners: {
                        scope: this,
                        'load': function(editdlg) {
                            editdlg.record.set('email', parts[1]);
                            editdlg.record.set('n_given', parts[2]);
                            editdlg.record.set('n_family', parts[3]);
                        }
                    }
                });

                break;

               case 'span[class=expressomail-addsievefilter-link]':

                var id = Ext.util.Format.htmlDecode(target.id);
                var parts = id.split(':');

                var account = this.app.getActiveAccount();

                //var id = Ext.id() + ':' + email;
                Ext.MessageBox.confirm(
                    this.app.i18n._('Block Sender'),
                    this.app.i18n._('Block incoming messages from this sender?'),
                    function (button) {
                        if (button == 'yes') {
                            Tine.Expressomail.rulesBackend.getRules(account.id, {
                                scope: this,
                                success: function(record) {
                                        this.blockSender();
                                        this.purgeListeners();
                                },
                                failure: Tine.Expressomail.handleRequestException.createSequence(function() {
                                    this.loadMask.hide();
                                }, this),
                                timeout: 6000,
                                parts: parts[1]

                            });
                        }
                    },
                    this
                );
                break;

            case 'span[class=tinebase-showheaders-link]':
                // show headers

                var parts = target.id.split(':');
                var targetId = parts[0];
                var action = parts[1];

                var html = '';
                if (action == 'show') {
                    var recordHeaders = this.record.get('headers');

                    for (header in recordHeaders) {
                        if (recordHeaders.hasOwnProperty(header) && (header != 'to' || header != 'cc' || header != 'bcc')) {
                            html += '<br/><b>' + header + ':</b> '
                                + Ext.util.Format.htmlEncode(recordHeaders[header]);
                        }
                    }

                    target.id = targetId + ':' + 'hide';

                } else {
                    html = ' <span ext:qtip="' + Ext.util.Format.htmlEncode(this.i18n._('Show or hide header information')) + '" id="'
                        + Ext.id() + ':show" class="tinebase-showheaders-link">[...]</span>'
                }

                target.innerHTML = html;

                break;
        }
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */

Ext.namespace('Tine.Expressomail');

/**
 * Message grid panel
 *
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.GridPanel
 * @extends     Tine.widgets.grid.GridPanel
 *
 * <p>Message Grid Panel</p>
 * <p><pre>
 * </pre></p>
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.GridPanel
 */
Tine.Expressomail.GridPanel = Ext.extend(Tine.widgets.grid.GridPanel, {
    /**
     * record class
     * @cfg {Tine.Expressomail.Model.Message} recordClass
     */
    recordClass: Tine.Expressomail.Model.Message,

    /**
     * message detail panel
     *
     * @type Tine.Expressomail.GridDetailsPanel
     * @property detailsPanel
     */
    detailsPanel: null,

    /**
     * transaction id of current delete message request
     * @type Number
     */
    deleteTransactionId: null,

    /**
     * this is true if messages are moved/deleted
     *
     * @type Boolean
     */
    movingOrDeleting: false,

    manualRefresh: false,

    /**
     * @private model cfg
     */
    evalGrants: false,
    filterSelectionDelete: true,
    // autoRefresh is done via onUpdateFolderStore
    autoRefreshInterval: false,
    
    fetchtry: 0,

    saveOnDestroy: {},

    /**
     * @private grid cfg
     */
    defaultSortInfo: {field: 'received', direction: 'DESC'},
    gridConfig: {
        autoExpandColumn: 'subject',
        // drag n dropfrom
        enableDragDrop: true,
        ddGroup: 'mailToTreeDDGroup'
    },
    // we don't want to update the preview panel on context menu
    updateDetailsPanelOnCtxMenu: true,

    /**
     * Return CSS class to apply to rows depending upon flags
     * - checks Flagged, Deleted and Seen
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {Integer} index
     * @return {String}
     */
    getViewRowClass: function(record, index) {
        var className = '';
        if (record.isImportant()) {
            className += ' importance_high';
        }
        if (record.hasFlag('\\Flagged')) {
            className += ' flag_flagged';
        }
        if (record.hasFlag('\\Deleted')) {
            className += ' flag_deleted';
        }
        if (! record.hasFlag('\\Seen')) {
            className += ' flag_unread';
        }

        return className;
    },

    /**
     * init message grid
     * @private
     */
    initComponent: function() {

        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        this.i18nEmptyText = this.app.i18n._('No Messages found or the cache is empty.');

        this.recordProxy = Tine.Expressomail.messageBackend;

        this.gridConfig.columns = this.getColumns();
        this.initFilterToolbar();
        this.initDetailsPanel();

        this.plugins = this.plugins || [];
        this.plugins.push(this.filterToolbar);

        this.pagingConfig = {
            doRefresh: this.doRefresh.createDelegate(this)
        };

        this.defaultPaging.limit = parseInt(Tine.Expressomail.registry.get('preferences').get('emailsPerPage'));

        Tine.Expressomail.GridPanel.superclass.initComponent.call(this)

        this.grid.getSelectionModel().on('rowselect', this.onRowSelection, this);
        this.app.getFolderStore().on('update', this.onUpdateFolderStore, this);

        this.initPagingToolbar();

        this.saveOnDestroy = {};
    },
    
    /**
     * get load mask
     * 
     * @return {Ext.LoadMask}
     */
    getLoadMask: function() {
        if (! this.loadMask) {
            this.loadMask = new Ext.LoadMask(this.el);
        }
        
        return this.loadMask;
    },

    /**
     * add quota bar to paging toolbar
     */
    initPagingToolbar: function() {
        Ext.QuickTips.init();

        this.quotaBar = new Ext.ProgressBar({
            width: 120,
            height: 16,
            style: {
                marginTop: '1px'
            },
            text: this.app.i18n._('Quota usage')
        });
        this.pagingToolbar.insert(12, new Ext.Toolbar.Separator());
        this.pagingToolbar.insert(12, this.quotaBar);

        // NOTE: the Ext.progessbar has an ugly bug: it does not layout correctly when hidden
        //       so we need to listen when we get activated to relayout the progessbar
        Tine.Tinebase.appMgr.on('activate', function(app) {
            if (app.appName === 'Expressomail') {
                this.quotaBar.syncProgressBar();
                this.quotaBar.setWidth(this.quotaBar.getWidth());
            }
        }, this);

        this.pagingToolbar.pageSize = parseInt(Tine.Expressomail.registry.get('preferences').get('emailsPerPage'));
    },

    /**
     * cleanup on destruction
     */
    onDestroy: function() {
        this.app.getFolderStore().un('update', this.onUpdateFolderStore, this);
    },

    /**
     * folder store gets updated -> refresh grid if new messages arrived or messages have been removed
     *
     * @param {Tine.Expressomail.FolderStore} store
     * @param {Tine.Expressomail.Model.Folder} record
     * @param {String} operation
     */
    onUpdateFolderStore: function(store, record, operation) {
        if (operation === Ext.data.Record.EDIT && record.isModified('cache_totalcount') && (this.saveOnDestroy ? !this.saveOnDestroy.confirm : true)) {
            var tree = this.app.getMainScreen().getTreePanel(),
                selectedNodes = (tree) ? tree.getSelectionModel().getSelectedNodes() : [];

            // only refresh if 1 or no messages are selected
            if (this.getGrid().getSelectionModel().getCount() <= 1) {
                var refresh = false;
                for (var i = 0; i < selectedNodes.length; i++) {
                    if (selectedNodes[i].id == record.id) {
                        refresh = true;
                        break;
                    }
                }

                // check if folder is in filter or allinboxes are selected and updated folder is an inbox
                if (! refresh) {
                    var filters = this.filterToolbar.getValue();
                    filters = filters.filters ? filter.filters : filters;

                    for (var i = 0; i < filters.length; i++) {
                        if (filters[i].field == 'path' && filters[i].operator == 'in') {
                            if (filters[i].value.indexOf(record.get('path')) !== -1 || (filters[i].value.indexOf('/allinboxes') !== -1 && record.isInbox())) {
                                refresh = true;
                                break;
                            }
                        }
                    }
                }

                if (refresh && this.noDeleteRequestInProgress()) {
                    Tine.log.debug('Refresh grid because of folder update.');
                    this.loadGridData({
                        removeStrategy: 'keepBuffered',
                        autoRefresh: true
                    });
                }
            }
        }
    },

    /**
     * skip initial till we know the INBOX id
     */
    initialLoad: function() {
        var account = this.app.getActiveAccount(),
            accountId = account ? account.id : null,
            inbox = accountId ? this.app.getFolderStore().queryBy(function(record) {
                return record.get('account_id') === accountId && record.get('localname').match(/^inbox$/i);
            }, this).first() : null;

        if (! inbox) {
            this.initialLoad.defer(100, this, arguments);
            return;
        }

        return Tine.Expressomail.GridPanel.superclass.initialLoad.apply(this, arguments);
    },

    /**
     * init actions with actionToolbar, contextMenu and actionUpdater
     *
     * @private
     */
    initActions: function() {
        var encrypted_text = ' ('+this.app.i18n._('encrypted') + ')';

        this.action_write = new Ext.Action({
            requiredGrant: 'addGrant',
            actionType: 'add',
            text: this.app.i18n._('Compose'),
            handler: this.onMessageCompose.createDelegate(this),
            disabled: ! this.app.getActiveAccount(),
            iconCls: this.app.appName + 'IconCls'
        });

        // all actions reply
        this.action_reply = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'reply',
            text: this.app.i18n._('Reply'),
            handler: this.onMessageReplyTo.createDelegate(this, [false]),
            iconCls: 'action_email_reply',
            disabled: true
        });

        this.action_reply_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'reply',
            text: this.app.i18n._('Reply') + encrypted_text,
            handler: this.onMessageReplyTo.createDelegate(this, [false,false,true]),
            iconCls: 'action_email_reply',
            disabled: true
        });

        this.action_split_reply = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Reply'),
            handler: this.onMessageReplyTo.createDelegate(this, [false,false,false]),
            disabled: true,
            iconCls: 'action_email_reply',
            scope: this,
            menu:{
                items:[
                    this.action_reply_encrypted
                ]
            }
        });
        
        this.action_split_reply_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Reply') + encrypted_text,
            handler: this.onMessageReplyTo.createDelegate(this, [false,false,true]),
            disabled: true,
            iconCls: 'action_email_reply',
            scope: this,
            menu:{
                items:[
                    this.action_reply
                ]
            }
        });
        
        // all actions replyAll
        this.action_replyAll = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'replyAll',
            text: this.app.i18n._('Reply To All'),
            handler: this.onMessageReplyTo.createDelegate(this, [true]),
            iconCls: 'action_email_replyAll',
            disabled: true
        });

        this.action_replyAll_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'replyAll',
            text: this.app.i18n._('Reply To All') + encrypted_text,
            handler: this.onMessageReplyTo.createDelegate(this, [true,false,true]),
            iconCls: 'action_email_replyAll',
            disabled: true
        });
        
        this.action_split_replyAll = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Reply To All'),
            handler: this.onMessageReplyTo.createDelegate(this, [true,false,false]),
            disabled: true,
            iconCls: 'action_email_replyAll',
            scope: this,
            menu:{
                items:[
                    this.action_replyAll_encrypted
                ]
            }
        });
        
        this.action_split_replyAll_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Reply To All') + encrypted_text,
            handler: this.onMessageReplyTo.createDelegate(this, [true,false,true]),
            disabled: true,
            iconCls: 'action_email_replyAll',
            scope: this,
            menu:{
                items:[
                    this.action_replyAll
                ]
            }
        });
        
        // all actions forward
        this.action_forward = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'forward',
            text: this.app.i18n._('Forward'),
            handler: this.onMessageForward.createDelegate(this, []),
            iconCls: 'action_email_forward',
            disabled: true
        });

        this.action_forward_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            actionType: 'forward',
            text: this.app.i18n._('Forward') + encrypted_text,
            handler: this.onMessageForward.createDelegate(this, [false,true]),
            iconCls: 'action_email_forward',
            disabled: true
        });
        
        this.action_split_forward = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Forward'),
            handler: this.onMessageForward.createDelegate(this, [false,false]),
            disabled: true,
            iconCls: 'action_email_forward',
            scope: this,
            menu:{
                items:[
                    this.action_forward_encrypted
                ]
            }
        });
        
        this.action_split_forward_encrypted = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Forward') + encrypted_text,
            handler: this.onMessageForward.createDelegate(this, [false,true]),
            disabled: true,
            iconCls: 'action_email_forward',
            scope: this,
            menu:{
                items:[
                    this.action_forward
                ]
            }
        });

        this.action_flag = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Toggle highlighting'),
            handler: this.onToggleFlag.createDelegate(this, ['\\Flagged'], true),
            iconCls: 'action_email_flag',
            allowMultiple: true,
            disabled: true
        });

        this.action_markUnread = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Mark read/unread'),
            handler: this.onToggleFlag.createDelegate(this, ['\\Seen'], true),
            iconCls: 'action_mark_read',
            allowMultiple: true,
            disabled: true
        });

        this.action_deleteRecord = new Ext.Action({
            requiredGrant: 'deleteGrant',
            allowMultiple: true,
            singularText: this.app.i18n._('Delete'),
            pluralText: this.app.i18n._('Delete'),
            translationObject: this.i18nDeleteActionText ? this.app.i18n : Tine.Tinebase.translation,
            text: this.app.i18n._('Delete'),
            handler: this.onDeleteRecords,
            disabled: true,
            iconCls: 'action_delete',
            scope: this
        });

        this.action_addAccount = new Ext.Action({
            text: this.app.i18n._('Add Account'),
            handler: this.onAddAccount,
            iconCls: 'action_add',
            scope: this,
            disabled: ! Tine.Tinebase.common.hasRight('add_accounts', 'Expressomail')
        });
        this.action_exportMsg = new Ext.Action({
            requiredGrant: 'readGrant',
            allowMultiple: true,
            text: this.app.i18n._('Export'),
            handler: this.onExportMsgs,
            iconCls: 'action_exportMsg',
            scope: this
        });
        this.action_printPreview = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Print Preview'),
            handler: this.onPrintPreview.createDelegate(this, []),
            disabled:true,
            iconCls:'action_printPreview',
            scope:this
        });
        this.action_print = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Print Message'),
            handler: this.onPrint.createDelegate(this, []),
            disabled:true,
            iconCls:'action_print',
            scope:this,
            menu:{
                items:[
                    this.action_printPreview
                ]
            }
        });
        this.action_reportPhishing = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.app.i18n._('Report phishing'),
            handler: this.onReportPhishing,
            iconCls: 'action_reportPhishing',
            allowMultiple: true,
            disabled: true,
            scope: this
        });
        this.actionUpdater.addActions([
//            this.action_write,
            this.action_deleteRecord,
            this.action_reply,
            this.action_replyAll,
            this.action_forward,
            this.action_flag,
            this.action_markUnread,
            this.action_addAccount,
            this.action_print,
            this.action_printPreview,
            this.action_reportPhishing
        ]);

        if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1'
            && Tine.Tinebase.registry.get('preferences').get('windowtype')== 'Ext') {
            this.actionUpdater.addActions([
                this.action_reply_encrypted,
                this.action_replyAll_encrypted,
                this.action_forward_encrypted,
                this.action_split_reply,
                this.action_split_replyAll,
                this.action_split_forward,
                this.action_split_reply_encrypted,
                this.action_split_replyAll_encrypted,
                this.action_split_forward_encrypted
            ]);
            var menu_items = [
                    this.action_reply,
                    this.action_reply_encrypted,
                    '-',
                    this.action_replyAll,
                    this.action_replyAll_encrypted,
                    '-',
                    this.action_forward,
                    this.action_forward_encrypted,
                    '-',
                    this.action_flag,
                    this.action_markUnread,
                    this.action_exportMsg,
                    this.action_deleteRecord
            ];
        }
        else {
            var menu_items = [
                    this.action_reply,
                    this.action_replyAll,
                    this.action_forward,
                    this.action_flag,
                    this.action_markUnread,
                    this.action_exportMsg,
                    this.action_deleteRecord
            ];
        }

        if (Tine.Expressomail.registry.get("reportPhishingEmail") && Tine.Expressomail.registry.get("reportPhishingEmail") != '') {
            // just show this menu item if report phishing email is configured
            menu_items.push(this.action_reportPhishing);
        }

        this.contextMenu = new Ext.menu.Menu({
            items: menu_items
        });

    },

    /**
     * delete messages handler
     *
     * @return {void}
     */
    onReportPhishing: function() {
        var email = Tine.Expressomail.registry.get("reportPhishingEmail");
        var subjects = [];
        var msgs = this.selectionModel.getSelections();
        var msgsIds = [];
        Ext.each(msgs, function(item) {
            subjects.push(item.get('subject'));
            msgsIds.push(item.id);
        });
        var title = this.app.i18n._('Report phishing');
        var text_init = this.app.i18n._("Phishing are messages with the intention of getting personal data like:<br/>passwords, finantial data like credit card numbers and so on.");
        var text_end = this.app.i18n._("Report listed messages as phishing?");
        var text = text_init
                 + "<ul class='mail-phishing-subjects-list'>"
                 + "<li class='mail-phishing-subject-item'>"
                 + subjects.join("</li><li class='mail-phishing-subject-item'>")
                 + "</li>"
                 + "</ul>"
                 + text_end;

        Ext.MessageBox.confirm(title, text, function(btn) {
            Ext.MessageBox.updateText(""); // this is a workaround to the MessageBox width issue in Chrome
            if(btn === 'yes') {
                this.reportPhishingMask = new Ext.LoadMask(Ext.getBody(), {msg: this.app.i18n._('Sending phishing report...')});
                this.reportPhishingMask.show();
                var account = this.app.getActiveAccount(),
                    accountId = account ? account.id : null;
                var message = new Tine.Expressomail.Model.Message(Tine.Expressomail.Model.Message.getDefaultData());
                message.set('account_id', accountId);
                message.set('to', [email]);
                message.set('cc', []);
                message.set('bcc', []);
                var subject_string = 'Phishing - {0} message',
                    subject_string_plural = 'Phishing - {0} messages',
                    subject = String.format(this.app.i18n.ngettext(subject_string, subject_string_plural, subjects.length), subjects.length);
                message.set('subject', subject);
                var body_string = "At {0}, user {1} reported attached message as phishing:",
                    body_string_plural = "At {0}, user {1} reported attached messages as phishing:";
                // the first param of the translated string is not replaced in js, the php will replace it with server datetime
                var body = String.format(this.app.i18n.ngettext(body_string, body_string_plural, subjects.length), "{0}", account.get('from'));
                message.set("body",
                            "<p>" + body + "</p>"
                          + "<ul class='mail-phishing-subjects-list'>"
                          + "<li class='mail-phishing-subject-item'>"
                          + subjects.join("</li><li class='mail-phishing-subject-item'>")
                          + "</li>"
                          + "</ul>");
                Tine.Expressomail.messageBackend.reportPhishing(message, msgsIds, {
                    scope: this,
                    success: function(record){
                        var account = this.app.getActiveAccount(),
                            trashId = (account) ? account.getTrashFolderId() : null,
                            trash = trashId ? this.app.getFolderStore().getById(trashId) : null,
                            trashConfigured = (account.get('trash_folder'));

                        if ( (Tine.Expressomail.registry.get('preferences').get('confirmUseTrash') == '1' && trash && ! trash.isCurrentSelection())
                           || (! trash && trashConfigured) ) {
                            this.moveSelectedMessages(trash, true);
                        } else {
                            this.deleteSelectedMessages();
                        }
                        this.reportPhishingMask.hide();
                    },
                    failure: function(record){
                        Ext.MessageBox.show({
                            title: this.app.i18n._('Failed to send phishing report!'),
                            msg: this.app.i18n._('Your phishing report could not be send.'),
                            buttons: Ext.MessageBox.OK,
                            icon: Ext.MessageBox.WARNING
                        });
                        this.reportPhishingMask.hide();
                    },
                    timeout: 300000 // 5 minutes
                });
            }
        }, this);

    },

    /**
     * initialises filter toolbar
     *
     * @private
     */
    initFilterToolbar: function() {
        this.filterToolbar = this.getFilterToolbar({
            dontRefreshOnDeleteFilter: true
        });
        this.filterToolbar.criteriaIgnores = [
            {field: 'query',     operator: 'contains',     value: ''},
            {field: 'id'},
            {field: 'path'}
        ];
    },

    /**
     * the details panel (shows message content)
     *
     * @private
     */
    initDetailsPanel: function() {
        this.detailsPanel = new Tine.Expressomail.GridDetailsPanel({
            gridpanel: this,
            grid: this,
            app: this.app,
            i18n: this.app.i18n
        });
    },

    /**
     * reload action toolbar
     *
     * @param encrypted reload toolbar for an encrypted message
     * @return {void}
     */
    reloadActionToolbar: function(encrypted) {
        this.actionToolbar = null;
        this.encrypted_message = encrypted;
        this.app.mainScreen['ActionToolbar'] = null;
        this.app.mainScreen.showNorthPanel();
    },

    /**
     * get action toolbar
     *
     * @return {Ext.Toolbar}
     */
    getActionToolbar: function() {
        if (! this.actionToolbar) {
            var button_reply, button_replyAll, button_forward;
            if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1'
                && Tine.Tinebase.registry.get('preferences').get('windowtype')== 'Ext') {
                if (this.encrypted_message) {
                    button_reply = new Ext.SplitButton(this.action_split_reply_encrypted);
                    button_replyAll = new Ext.SplitButton(this.action_split_replyAll_encrypted);
                    button_forward = new Ext.SplitButton(this.action_split_forward_encrypted);
                }
                else {
                    button_reply = new Ext.SplitButton(this.action_split_reply);
                    button_replyAll = new Ext.SplitButton(this.action_split_replyAll);
                    button_forward = new Ext.SplitButton(this.action_split_forward);
                }
            }
            else {
                button_reply = new Ext.Button(this.action_reply);
                button_replyAll = new Ext.Button(this.action_replyAll);
                button_forward = new Ext.Button(this.action_forward);
            }
            this.actionToolbar = new Ext.Toolbar({
                defaults: {height: 55},
                items: [{
                    xtype: 'buttongroup',
                    columns: 8,
                    items: [
                        Ext.apply(new Ext.SplitButton(this.action_write), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top',
                            arrowAlign:'right',
                            menu: new Ext.menu.Menu({
                                items: [],
                                plugins: [{
                                    ptype: 'ux.itemregistry',
                                    key:   'Tine.widgets.grid.GridPanel.addButton'
                                }]
                            })
                        }),
                        Ext.apply(new Ext.Button(this.action_deleteRecord), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(button_reply, {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign:'top',
                            arrowAlign:'right'
                        }),
                        Ext.apply(button_replyAll, {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(button_forward, {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.SplitButton(this.action_print), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign:'top',
                            arrowAlign:'right'
                        }),
                        this.action_flag,
                        this.action_addAccount,
                        this.action_markUnread
                    ]
                }, this.getActionToolbarItems()]
            });

            if (this.filterToolbar && typeof this.filterToolbar.getQuickFilterField == 'function') {
                this.actionToolbar.add('->', this.filterToolbar.getQuickFilterField());
            }
        }

        return this.actionToolbar;
    },

    /**
     * returns cm
     *
     * @private
     */
    getColumns: function(){
        return [{
            id: 'id',
            header: this.app.i18n._("Id"),
            width: 100,
            sortable: true,
            dataIndex: 'id',
            hidden: true
        }, {
            id: 'smime',
            header: this.app.i18n._("Security"),
            width: 12,
            sortable: true,
            dataIndex: 'smime',
            renderer: this.smimeRenderer
        }, {
            id: 'content_type',
            header: this.app.i18n._("Attachments"),
            width: 12,
            sortable: true,
            dataIndex: 'has_attachment',
            renderer: this.attachmentRenderer
        }, {
            id: 'flags',
            header: this.app.i18n._("Flags"),
            width: 24,
            sortable: true,
            dataIndex: 'flags',
            renderer: this.flagRenderer
        },{
            id: 'subject',
            header: this.app.i18n._("Subject"),
            width: 300,
            sortable: true,
            dataIndex: 'subject'
        },{
            id: 'from_email',
            header: this.app.i18n._("From (Email)"),
            width: 100,
            sortable: true,
            dataIndex: 'from_email'
        },{
            id: 'from_name',
            header: this.app.i18n._("From (Name)"),
            width: 100,
            sortable: true,
            dataIndex: 'from_name'
        },{
            id: 'sender',
            header: this.app.i18n._("Sender"),
            width: 100,
            sortable: true,
            dataIndex: 'sender',
            hidden: true
        },{
            id: 'to',
            header: this.app.i18n._("To"),
            width: 150,
            sortable: true,
            dataIndex: 'to',
            hidden: true
        },{
            id: 'sent',
            header: this.app.i18n._("Sent"),
            width: 100,
            sortable: true,
            dataIndex: 'sent',
            hidden: true,
            renderer: Tine.Tinebase.common.dateTimeRenderer
        },{
            id: 'received',
            header: this.app.i18n._("Received"),
            width: 100,
            sortable: true,
            dataIndex: 'received',
            renderer: Tine.Tinebase.common.dateTimeRenderer
        },{
            id: 'folder_id',
            header: this.app.i18n._("Folder"),
            width: 100,
            sortable: true,
            dataIndex: 'folder_id',
            hidden: true,
            renderer: this.accountAndFolderRenderer.createDelegate(this)
        },{
            id: 'size',
            header: this.app.i18n._("Size"),
            width: 80,
            sortable: true,
            dataIndex: 'size',
            hidden: true,
            renderer: Ext.util.Format.fileSize
        }];
    },

    /**
     * Smime renderer
     *
     * @param {String} value
     * @return {String}
     * @private
     */
    smimeRenderer: function(value) {
        var icons = [], result = '';
        value = new String(value);

        if (value && value.match(/1/))  
        {
            icons.push({src: 'images/oxygen/16x16/mimetypes/application-pkcs7-signature.png', qtip: _('Assinado')});  // _('Signed')
        }
        if (value && value.match(/2/))
        { 
            icons.push({src: 'images/oxygen/16x16/actions/encrypted.png', qtip: _('Criptografado')});   //_('Encrypted')
        }
        if (value && value.match(/3/))
        {
            icons.push({src: 'images/oxygen/16x16/mimetypes/application-zip.png', qtip: _('Compactado')});  //_('Compressed')
        }
        if (value && value.match(/4/))
        {
            icons.push({src: 'images/oxygen/16x16/mimetypes/application-pkcs7-mime.png', qtip: _('Apenas Certificados')});   //_('Certs Only')
        }
        if (value && value.match(/5/))
        {
            icons.push({src: 'images/oxygen/16x16/mimetypes/application-pkcs7-mime.png', qtip: _('pkcs7-mime')});
        }

        Ext.each(icons, function(icon) {
            result += '<img class="FelamimailFlagIcon" src="' + icon.src + '" ext:qtip="' + icon.qtip + '">';
        }, this);

        return result;
    },

    /**
     * attachment column renderer
     *
     * @param {String} value
     * @return {String}
     * @private
     */
    attachmentRenderer: function(value, metadata, record) {
        var result = '';

        if (value == 1) {
            result = '<img class="ExpressomailFlagIcon" src="images/oxygen/16x16/actions/attach.png">';
        }

        return result;
    },

    /**
     * get flag icon
     *
     * @param {String} flags
     * @return {String}
     * @private
     *
     * TODO  use spacer if first flag(s) is/are not set?
     */
    flagRenderer: function(value, metadata, record) {
        var icons = [],
            result = '';
       
        if (record.get('reading_conf')) {
            icons.push({src: 'images/oxygen/16x16/actions/mail-mark-task.png', qtip: Ext.util.Format.htmlEncode(_('Confirmação de Leitura'))});  //_('Reading Confirmation')
        }

        if (record.hasFlag('\\Answered')) {
            icons.push({src: 'images/oxygen/16x16/actions/mail-reply-sender.png', qtip: Ext.util.Format.htmlEncode(_('Respondida'))}); //_('Answered')
        }
        if (record.hasFlag('Passed')) {
            icons.push({src: 'images/oxygen/16x16/actions/mail-forward.png', qtip: Ext.util.Format.htmlEncode(_('Encaminhada'))}); //_('Forwarded')
        }
//        if (record.hasFlag('\\Recent')) {
//            icons.push({src: 'images/oxygen/16x16/actions/knewstuff.png', qtip: _('Recent')});
//        }

        Ext.each(icons, function(icon) {
            result += '<img class="ExpressomailFlagIcon" src="' + icon.src + '" ext:qtip="' + Tine.Tinebase.common.doubleEncode(icon.qtip) + '">';
        }, this);

        return result;
    },

    /**
     * returns account and folder globalname
     *
     * @param {String} folderId
     * @param {Object} metadata
     * @param {Folder|Account} record
     * @return {String}
     */
    accountAndFolderRenderer: function(folderId, metadata, record) {
        var folderStore = this.app.getFolderStore(),
            account = this.app.getAccountStore().getById(record.get('account_id')),
            result = (account) ? account.get('name') : record.get('account_id'),
            folder = folderStore.getById(folderId);

        if (! folder) {
            folder = folderStore.getById(record.id);
            if (! folder) {
                // only account
                return (result) ? result : record.get('name');
            }
        }

        result += '/';
        if (folder) {
            result += folder.get('globalname');
        } else {
            result += folderId;
        }

        return result;
    },

    /**
     * executed when user clicks refresh btn
     */
    doRefresh: function() {
        var folder = this.getCurrentFolderFromTree(),
            refresh = this.pagingToolbar.refresh;

        // refresh is explicit
        this.editBuffer = [];
        this.manualRefresh = true;

        if (folder) {
            refresh.disable();
            Tine.log.info('User forced mail check for folder "' + folder.get('localname') + '"');
            this.app.checkMails(folder, function() {
                refresh.enable();
                this.manualRefresh = false;
            });
        } else {
            this.filterToolbar.onFilterChange();
        }
    },

        /**
     * Export messages handler
     *
     * @return {void}
     */
    onExportMsgs: function() {
        var sm = this.getGrid().getSelectionModel(),
            filter = sm.getSelectionFilter(),
            msgsIds = '';
        if (sm.isFilterSelect) {
            var msgs = this.getStore(),
                nextRecord = null;
        } else {
            var msgs = sm.getSelectionsCollection(),
                nextRecord = this.getNextMessage(msgs);
        }
        var increaseUnreadCountInTargetFolder = 0;
        msgs.each(function(msg) {
            if(msgsIds.length==0) {
                msgsIds = msgsIds + msg.id;
            } else {
                msgsIds = msgsIds + ',' + msg.id;
            }
        },  this);
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressomail.downloadMessage',
                requestType: 'HTTP',
                messageId: msgsIds
            }
        }).start();

    },

    /**
     * get currently selected folder from tree
     * @return {Tine.Expressomail.Model.Folder}
     */
    getCurrentFolderFromTree: function() {
        var tree = this.app.getMainScreen().getTreePanel(),
            node = tree ? tree.getSelectionModel().getSelectedNode() : null,
            folder = node ? this.app.getFolderStore().getById(node.id) : null;

        return folder;
    },

    /**
     * delete messages handler
     *
     * @return {void}
     */
    onDeleteRecords: function() {
        var account = this.app.getActiveAccount(),
            trashId = (account) ? account.getTrashFolderId() : null,
            trash = trashId ? this.app.getFolderStore().getById(trashId) : null,
            trashConfigured = (account.get('trash_folder'));

        if(Tine.Expressomail.registry.get('preferences').get('confirmDelete') == '1')
        {
            Ext.MessageBox.confirm('', this.app.i18n._('Confirm Delete') + ' ?', function(btn) {
                if(btn == 'yes') {
                    return (Tine.Expressomail.registry.get('preferences').get('confirmUseTrash') == '1' && (trash && ! trash.isCurrentSelection()) || (! trash && trashConfigured)) ? this.moveSelectedMessages(trash, true) : this.deleteSelectedMessages();
                }
                this.focusSelectedMessage();
            }, this);
        }
        else
            {
                return (Tine.Expressomail.registry.get('preferences').get('confirmUseTrash') == '1' && (trash && ! trash.isCurrentSelection()) || (! trash && trashConfigured)) ? this.moveSelectedMessages(trash, true) : this.deleteSelectedMessages();
            }
    },

    focusSelectedMessage: function(){

        // Return focus to grid
        var record = this.getGrid().getSelectionModel().getSelected();
        if (record) {
            this.getGrid().getView().focusRow(this.getGrid().store.indexOf(record));
        }
    },

    /**
     * permanently delete selected messages
     */
    deleteSelectedMessages: function() {
        this.moveOrDeleteMessages(null);
    },

    /**
     * move selected messages to given folder
     *
     * @param {Tine.Expressomail.Model.Folder} folder
     * @param {Boolean} toTrash
     */
    moveSelectedMessages: function(folder, toTrash) {
        if (folder && folder.isCurrentSelection()) {
            // nothing to do ;-)
            return;
        }

        this.moveOrDeleteMessages(folder, toTrash);
    },

    /**
     * move (folder !== null) or delete selected messages
     *
     * @param {Tine.Expressomail.Model.Folder} folder
     * @param {Boolean} toTrash
     */
    moveOrDeleteMessages: function(folder, toTrash) {

        // this is needed to prevent grid reloads while messages are moved or deleted
        this.movingOrDeleting = true;

        var sm = this.getGrid().getSelectionModel(),
            filter = sm.getSelectionFilter(),
            msgsIds = [],
            foldersNeedUpdate = false;

        if (sm.isFilterSelect) {
            var msgs = this.getStore(),
                nextRecord = null;
        } else {
            var msgs = sm.getSelectionsCollection(),
                nextRecord = this.getNextMessage(msgs);
        }

        var folderStore = this.app.getFolderStore(),
            account = this.app.getActiveAccount(),
            dRegexp = new RegExp('\\'+account.get('delimiter')+'$'),
            inboxName = account.get('ns_personal').replace(dRegexp, '').toUpperCase(),
            inboxRecord = folderStore.getAt(folderStore.findExact('globalname', inboxName)),
            totalSize = 0,
            quotaLimit = parseInt(inboxRecord.get('quota_limit'), 10)*1024,
            quotaUsage = parseInt(inboxRecord.get('quota_usage'), 10)*1024,
            trashId = (account) ? account.getTrashFolderId() : null,
            trashConfigured = (account.get('trash_folder')),
            trashRecord = trashId ?
                folderStore.getById(trashId) :
                folderStore.find('globalname', trashConfigured);

        msgs.each(function (msg){
            totalSize += parseInt(msg.get('size'));
        }, this);


        if (isNaN(totalSize) || isNaN(quotaLimit) || isNaN(quotaUsage)){
            Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - isNaN value ');
            return;
        }

        if (toTrash && quotaUsage + totalSize > quotaLimit){
            var config = {
                title: this.app.i18n._('Not Enough Space to Move Message to Trash Folder.',
                    msgs.length),
                msg: this.app.i18n._('YOU CAN DELETE MESSAGE PERMANENTLY.', msgs.length)
                    + '<br />' + this.app.i18n._("This operation can't be undone!"),
                buttons: {
                    // Changed order of buttons
                    cancel: this.app.i18n._('Cancel'),
                    yes:    this.app.i18n._('Delete Permanently')
                },
                fn: function(button) {
                    if (button === 'yes') {
                        this.moveOrDeleteMessages(null);
                    }
                },
                icon: Ext.MessageBox.WARNING,
                scope: this
            };
            // Don't have enough space to move, see if trash folder is not empty
            if (trashRecord.get('cache_totalcount') <= 0) {
                // Ask to delete permanently.
                Ext.MessageBox.show(config);
            } else {
                config.msg = this.app.i18n._('Your trash folder is not empty!') + '<br />'
                    + this.app.i18n._('You can CANCEL this operation and clean trash before try again.') + '<br />'
                    + this.app.i18n._('or,') + '<br />' + config.msg;
                Ext.MessageBox.show(config);
//                Ext.MessageBox.show({
//                    title: this.app.i18n._('Not Enough Space to Move Message to Trash Folder.',
//                        msgs.length),
//                    msg: this.app.i18n._('Do you want to clean trash folder before move message?',
//                        msgs.length),
//                    buttons: {
//                        cancel: this.app.i18n._('Cancel'),
//                        yes:    this.app.i18n._('Clean Trash Folder')
//                    },
//                    fn: function(button) {
//                        if (button === 'yes') {
//                            // Clean Trash Folder
//                            var treePanel = this.app.getMainScreen().getTreePanel(),
//                                trashNode = treePanel.getNodeById(trashId);
//                            if (trashNode){
//                                trashNode.getUI().addClass("x-tree-node-loading");
//                            }
//                            Ext.Ajax.request({
//                                params: {
//                                    method: 'Expressomail.emptyFolder',
//                                    folderId: trashId
//                                },
//                                scope: this,
//                                //success: function(result, request, folder, toTrash, treePanel, trashId, trashNode){
//                                success: function(result, request){
//                                    var selectedNode = treePanel.getSelectionModel().getSelectedNode(),
//                                        isTrashSelected = (trashNode && selectedNode.id === trashNode.id),
//                                        responseJson = Ext.util.JSON.decode(result.responseText);
//
//                                    if (isTrashSelected) {
//                                        var folder = Tine.Expressomail.folderBackend.recordReader(result);
//                                        folder.set('cache_unreadcount', 0);
//                                        folder.set('cache_totalcount', 0);
//                                        folder.set('quota_usage', responseJson.quota_usage);
//                                        inboxRecord.set('quota_usage', responseJson.quota_usage);
//                                        this.app.getFolderStore().updateFolder(folder);
//                                    } else {
//                                        var folder = this.app.getFolderStore().getById(trashId);
//                                        folder.set('cache_unreadcount', 0);
//                                        folder.set('cache_totalcount', 0);
//                                        folder.set('quota_usage', responseJson.quota_usage);
//                                        inboxRecord.set('quota_usage', responseJson.quota_usage);
//                                    }
//                                    if (trashNode){
//                                        trashNode.getUI().removeClass("x-tree-node-loading");
//                                        trashNode.removeAll();
//                                    }
//                                    // Trash folder cleaned successfully try to move messages again.
//                                    this.moveOrDeleteMessages(folder, toTrash);
//                                }.createDelegate(this),
//                                failure: function() {
//                                    if (trashNode){
//                                        trashNode.getUI().removeClass("x-tree-node-loading");
//                                    }
//                                }.createDelegate(this),
//                                timeout: 120000 // 2 minutes
//                            });
//                        }
//                        // canceled
//                    },
//                    icon: Ext.MessageBox.QUESTION,
//                    scope: this
//                 });
            }
            return;
        }

        var increaseUnreadCountInTargetFolder = 0;
        msgs.each(function(msg) {
            var isSeen = msg.hasFlag('\\Seen'),
                currFolder = this.app.getFolderStore().getById(msg.get('folder_id')),
                diff = isSeen ? 0 : 1;

            if (currFolder) {
                currFolder.set('cache_unreadcount', currFolder.get('cache_unreadcount') - diff);
                currFolder.set('cache_totalcount', currFolder.get('cache_totalcount') - 1);
                if (sm.isFilterSelect && sm.getCount() > 50 && currFolder.get('cache_status') !== 'pending') {
                    Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - Set cache status to pending for folder ' + currFolder.get('globalname'));
                    currFolder.set('cache_status', 'pending');
                    foldersNeedUpdate = true;
                }
                currFolder.commit();
            }
            if (folder) {
                increaseUnreadCountInTargetFolder += diff;
            }

            msgsIds.push(msg.id);
            this.getStore().remove(msg);
        },  this);

        if (folder) {
            if(increaseUnreadCountInTargetFolder > 0){
                // update unread count of target folder (only when moving)
                folder.set('cache_unreadcount', folder.get('cache_unreadcount') + increaseUnreadCountInTargetFolder);
                if (foldersNeedUpdate) {
                    Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - Set cache status to pending for target folder ' + folder.get('globalname'));
                    folder.set('cache_status', 'pending');
                }
            }
            folder.set('cache_totalcount', folder.get('cache_totalcount') + msgsIds.length);
            folder.commit();
        }

        if (foldersNeedUpdate) {
            Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - update message cache for "pending" folders');
            this.app.checkMailsDelayedTask.delay(1000);
        }

        this.deleteQueue = this.deleteQueue.concat(msgsIds);
        this.pagingToolbar.refresh.disable();
        if (nextRecord !== null) {
            sm.selectRecords([nextRecord]);
        }
        this.waitForReload = true;
        var callbackFn = this.onAfterDelete.createDelegate(this, [msgsIds]);

        if (folder !== null || toTrash) {
            // move
            var targetFolderId = (toTrash) ? '_trash_' : folder.id;
            this.deleteTransactionId = Tine.Expressomail.messageBackend.moveMessages(filter, targetFolderId, {
                callback: callbackFn
            });
        } else {
            // delete
            this.deleteTransactionId = Tine.Expressomail.messageBackend.addFlags(filter, '\\Deleted', {
                callback: callbackFn
            });
        }
    },

    /**
     * get next message in grid
     *
     * @param {Ext.util.MixedCollection} msgs
     * @return Tine.Expressomail.Model.Message
     */
    getNextMessage: function(msgs) {

        var nextRecord = null;

        if (msgs.getCount() == 1 && this.getStore().getCount() > 1) {
            // select next message (or previous if it was the last or BACKSPACE)
            var lastIdx = this.getStore().indexOf(msgs.last()),
                direction = Ext.EventObject.getKey() == Ext.EventObject.BACKSPACE ? -1 : +1;

            nextRecord = this.getStore().getAt(lastIdx + 1 * direction);
            if (! nextRecord) {
                nextRecord = this.getStore().getAt(lastIdx + (-1) * direction);
            }
        }

        return nextRecord;
    },
    
    onEditClose: function(contact) {
        Tine.log.debug('Tine.Expressomail.GridPanel::onEditClose / arguments:' + contact);
        
        var toAdd = new Tine.Addressbook.Model.EmailAddress(Ext.util.JSON.decode(contact));
        var mailStore = Tine.Expressomail.getMailStore();
        mailStore.addSorted(toAdd);
        //mailStore.sort('email', 'ASC');
        mailStore.commitChanges();
        
    },

    /**
     * executed after a msg compose
     *
     * @param {String} composedMsg
     * @param {String} action
     * @param {Array}  [affectedMsgs]  messages affected
     * @param {String} [mode]
     */
    onAfterCompose: function(composedMsg, action, affectedMsgs, mode) {
        Tine.log.debug('Tine.Expressomail.GridPanel::onAfterCompose / arguments:');
        Tine.log.debug(arguments);

        // mark send folders cache status incomplete
        composedMsg = Ext.isString(composedMsg) ? new this.recordClass(Ext.decode(composedMsg)) : composedMsg;

        // NOTE: if affected messages is decoded, we need to fetch the originals out of our store
        if (Ext.isString(affectedMsgs)) {
            var msgs = [],
                store = this.getStore();
            Ext.each(Ext.decode(affectedMsgs), function(msgData) {
                var msg = store.getById(msgData.id);
                if (msg) {
                    msgs.push(msg);
                }
            }, this);
            affectedMsgs = msgs;
        }

        var composerAccount = this.app.getAccountStore().getById(composedMsg.get('account_id')),
            sendFolderId = composerAccount ? composerAccount.getSendFolderId() : null,
            sendFolder = sendFolderId ? this.app.getFolderStore().getById(sendFolderId) : null;

        if (sendFolder) {
            sendFolder.set('cache_status', 'incomplete');
        }

        if (Ext.isArray(affectedMsgs)) {
            Ext.each(affectedMsgs, function(msg) {
                if (['reply', 'forward'].indexOf(action) !== -1) {
                    msg.addFlag(action === 'reply' ? '\\Answered' : 'Passed');
                } else if (action == 'senddraft') {
                    this.deleteTransactionId = Tine.Expressomail.messageBackend.addFlags(msg.id, '\\Deleted', {
                        callback: this.onAfterDelete.createDelegate(this, [[msg.id]])
                    });
                }
            }, this);
        }
    },

    /**
     * executed after msg delete
     *
     * @param {Array} [ids]
     */
    onAfterDelete: function(ids) {
        this.deleteQueue = this.deleteQueue.diff(ids);
        this.editBuffer = this.editBuffer.diff(ids);

        this.movingOrDeleting = false;

        // if cursor is out of messages ranges then redirect to last page
        if((this.pagingToolbar.store.getTotalCount() > 0)
            && (this.pagingToolbar.cursor >= this.pagingToolbar.store.getTotalCount())) {

            Tine.log.debug('Tine.Expressomail.GridPanel::onAfterDelete() -> Redirecting to the last page after delete.');
            this.pagingToolbar.moveLast();
            this.focusSelectedMessage();
        }
        else {
            Tine.log.debug('Tine.Expressomail.GridPanel::onAfterDelete() -> Loading grid data after delete.');
            this.loadGridData({
                scope: this,
                callback: function(){
                    this.waitForReload = false;
                    this.focusSelectedMessage();
                },
                removeStrategy: 'keepBuffered',
                autoRefresh: true
            });
        }
    },

    /**
     * check if delete/move action is running atm
     *
     * @return {Boolean}
     */
    noDeleteRequestInProgress: function() {
        return (
            ! this.movingOrDeleting &&
            (! this.deleteTransactionId || ! Tine.Expressomail.messageBackend.isLoading(this.deleteTransactionId))
        );
    },

    /**
     * compose new message handler
     * @param encrypted open as encrypted message composer
     */
    onMessageCompose: function(encrypted) {
        var activeAccount = Tine.Tinebase.appMgr.get('Expressomail').getActiveAccount();
        
        var win = Tine.Expressomail.MessageEditDialog.openWindow({
            encrypted: encrypted === true?true:false,
            mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
            accountId: activeAccount ? activeAccount.id : null,
            editCloseCallback: this.onEditClose.createDelegate(this), // workaround for this to work properly
            listeners: {
                'update': this.onAfterCompose.createDelegate(this, ['compose', []], 1)
            }
        });
    },

    /**
     * forward message(s) handler
     * 
     * @param {bool} fetch
     * @param {bool} encrypted open as encrypted message composer
     */
    onMessageForward: function(fetch,encrypted) {
        var sm = this.getGrid().getSelectionModel(),
            msgs = sm.getSelections(),
            msgsData = [];
        
        var bodyIsFetched = true;
        Ext.each(msgs, function(msg) {
            msgsData.push(msg.data)
            if (!msg.bodyIsFetched()) bodyIsFetched = false;
        }, this);
        
        if (sm.getCount() > 0) {
            if (!bodyIsFetched) {
                    this.detailsPanel.getLoadMask().hide();
                    if(this.fetchtry < 4){
                        this.fetchtry++;
                        this.getLoadMask().show();
                        if (fetch !== false) sm.selectRecords(msgs);
                        this.dblClickDelayedTask = new Ext.util.DelayedTask(this.onMessageForward, this, [false,encrypted]);
                        this.dblClickDelayedTask.delay(1000);
                        return;
                    }
                }
            this.fetchtry =0;    
            var win = Tine.Expressomail.MessageEditDialog.openWindow({
                encrypted: encrypted || false,
                mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
                forwardMsgs : Ext.encode(msgsData),
                listeners: {
                    'update': this.onAfterCompose.createDelegate(this, ['forward', msgs], 1),
                    'addcontact': this.onEditClose.createDelegate(this)
                }
            });
            this.getLoadMask().hide();
        }
    },
            
    /**
     * reply message handler
     *
     * @param {bool} toAll
     * @param {bool} fetch
     * @param {bool} encrypted open as encrypted message composer
     */
    onMessageReplyTo: function(toAll, fetch, encrypted) {
        var sm = this.getGrid().getSelectionModel(),
            msg = sm.getSelected();

        if (msg) {
            if (!msg.bodyIsFetched()) {
                   this.detailsPanel.getLoadMask().hide();
                    if(this.fetchtry < 4){
                        this.fetchtry++;
                        this.getLoadMask().show();
                        if (fetch !== false) sm.selectRecords([msg]);
                        this.dblClickDelayedTask = new Ext.util.DelayedTask(this.onMessageReplyTo, this, [toAll,false,encrypted]);
                        this.dblClickDelayedTask.delay(1000);
                        return;
                    }
                }
            this.fetchtry =0;     
            var win = Tine.Expressomail.MessageEditDialog.openWindow({
                encrypted: encrypted || false,
                mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
                replyTo : Ext.encode(msg.data),
                replyToAll: toAll,
                listeners: {
                    update: this.onAfterCompose.createDelegate(this, ['reply', [msg]], 1),
                    addcontact: this.onEditClose.createDelegate(this)
                }
            });
        }
        this.getLoadMask().hide();
    },

    /**
     * called when a row gets selected
     *
     * @param {SelectionModel} sm
     * @param {Number} rowIndex
     * @param {Tine.Expressomail.Model.Message} record
     * @param {Number} retryCount
     *
     * TODO find a better way to check if body is fetched, this does not work correctly if a message is removed
     *       and the next one is selected automatically
     */
    onRowSelection: function(sm, rowIndex, record, retryCount) {
        if (sm.getCount() == 1 && ((! retryCount || retryCount < 5) && ! record.bodyIsFetched() || this.waitForReload)) {
            Tine.log.debug('Tine.Expressomail.GridPanel::onRowSelection() -> Deferring onRowSelection');
            retryCount = (retryCount) ? retryCount++ : 1;
            return this.onRowSelection.defer(250, this, [sm, rowIndex, record, retryCount+1]);
        }

        if (sm.getCount() == 1 && sm.isIdSelected(record.id) && !record.hasFlag('\\Seen')) {
            Tine.log.debug('Tine.Expressomail.GridPanel::onRowSelection() -> Selected unread message');
            Tine.log.debug(record);

            record.addFlag('\\Seen');
            record.mtime = new Date().getTime();
            Tine.Expressomail.messageBackend.addFlags(record.id, '\\Seen');
            this.app.getMainScreen().getTreePanel().decrementCurrentUnreadCount();

//            if (record.get('headers')['disposition-notification-to']) {
//                Ext.Msg.confirm(
//                    this.app.i18n._('Send Reading Confirmation'),
//                    this.app.i18n._('Do you want to send a reading confirmation message?'),
//                    function(btn) {
//                        if (btn == 'yes'){
//                            Tine.Expressomail.sendReadingConfirmation(record.id);
//                        }
//                    },
//                    this
//                );
//            }
        }
    },

    /**
     * row doubleclick handler
     *
     * - opens message edit dialog (if draft/template)
     * - opens message display dialog (everything else)
     *
     * @param {Tine.Expressomail.GridPanel} grid
     * @param {Row} row
     * @param {Event} e
     */
    onRowDblClick: function(grid, row, e) {
        if (this.isRowOutdated(row)) {
            // cannot open messages marked as outdated
            return;
        }
        var encrypted = this.detailsPanel.record.get('smimeEml') != '';
        var decrypted = this.detailsPanel.record.get('decrypted') || false;
        if (encrypted && !decrypted) {
            if (!this.detailsPanel.isDecrypting) {
                this.detailsPanel.reloadDetails();
            }
            this.detailsPanel.getLoadMask().hide();
            this.getLoadMask().show();
            this.detailsPanel.addListener('dblwindow', this.openDblClickWindow, this);
        }
        else {
            if(this.detailsPanel.record.bodyIsFetched()){
                this.openDblClickWindow(grid, row, e);
            }else{
                this.detailsPanel.getLoadMask().hide();
                this.getLoadMask().show();
                this.detailsPanel.addListener('dblwindow', this.openDblClickWindow, this);
            }
        }
    },
    
    openDblClickWindow: function(grid, row, e){
        this.detailsPanel.un('dblwindow',this.openDblClickWindow, this);
        this.getLoadMask().hide();
                var record = this.grid.getSelectionModel().getSelected(),
            folder = this.app.getFolderStore().getById(record.get('folder_id')),
            account = this.app.getAccountStore().getById(folder.get('account_id')),
            action = (folder.get('globalname') == account.get('drafts_folder')) ? 'senddraft' :
                      folder.get('globalname') == account.get('templates_folder') ? 'sendtemplate' : null,
            win;
        
        var sm = this.getGrid().getSelectionModel(),
            msg = sm.getSelected();

        if (!record.bodyIsFetched()) {
            if (!msg.bodyIsFetched()) {
                   this.detailsPanel.getLoadMask().hide();
                    if(this.fetchtry < 4){
                        this.fetchtry++;
                        this.getLoadMask().show();
                        this.dblClickDelayedTask = new Ext.util.DelayedTask(this.onRowDblClick, this, [grid,row,e]);
                        this.dblClickDelayedTask.delay(1000);
                        return;
                    }
                }
         }     
        this.fetchtry =0;   
        // check folder to determine if mail should be opened in compose dlg
        if (action !== null) {
            win = Tine.Expressomail.MessageEditDialog.openWindow({
                mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
                draftOrTemplate: Ext.encode(record.data),
                listeners: {
                    scope: this,
                    'update': this.onAfterCompose.createDelegate(this, [action, [record]], 1),
                    'addcontact': this.onEditClose.createDelegate(this)
                    }
            });
        } else {
            win = Tine.Expressomail.MessageDisplayDialog.openWindow({
                mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
                record: Ext.encode(record.data),
                listeners: {
                    scope: this,
                    'update': this.onAfterCompose.createDelegate(this, ['compose', []], 1),
                    'remove': this.onRemoveInDisplayDialog,
                    'addcontact': this.onEditClose.createDelegate(this)
                    }
            });
        }
    },
    
    /**
     * When message edit window is closed cancel further autosave
     *
     */
    onMessageEditWindowDestroy: function() {
        this.saveDraftsInterval = 0; // prevent more save draft calls
        this.unloading = true;
        var grid = this.window.ref.getMainScreen().getCenterPanel();
        if (!this.saving) { // just check and save draft if it is not already doing it
            if (this.autoSaveDraftsEnabled) {
                this.checkDraftChanges();
            }
            else {
                if (this.checkDraftChanges(grid.saveOnDestroy.forced)) {
                    grid.confirmSaveDraft();
                }
            }
        }
    },

    confirmSaveDraft: function() {
        this.saveOnDestroy.confirm = true;
        Ext.MessageBox.confirm('', this.app.i18n._('Keep as a draft?'),
            function(btn) {
                if(btn === 'yes') {
                    if (this.isDraftFolderSelected && this.saveOnDestroy.record) {
                        this.updateGridPanel(this.saveOnDestroy.record);
                    }
                    else {
                        if (this.saveOnDestroy.error) {
                            this.showSaveDraftErrorMessage();
                        }
                    }
                    this.saveOnDestroy.save = true;
                }
                else {
                    if (this.saveOnDestroy.record) {
                        this.removeDraft(this.saveOnDestroy.record);
                    }
                    else {
                        this.saveOnDestroy.save = false;
                    }
                }
                this.saveOnDestroy.confirm = false;
                this.saveOnDestroy.started = true;
            },
        this);
    },

    /**
     * call recordproxy saveDraft and set callback functions
     *
     * @param {Tine.Tinebase.data.Record} record
     * @param {String} folderName
     * @param {Tine.Expressomail.MessageEditDialog} editwindow
     * @param {Boolean} forced
     */
    callSaveDraft: function(record, folderName, editwindow, forced) {
        // this is in GridPanel instead of MessageEditWindow
        // because of 'Browser' style window related issues in IE
        this.saveOnDestroy = {
            confirm: false, // true if confirmSaveDraft is in progress
            started: false, // true if either confirmSaveDraft or onSaveDraftSuccess are completed
            save: false, // true if user selected YES on confirmSaveDraft
            error: false, // true if onSaveDraftFailure has been called,
            forced: forced || false, // true if draft saving was called by clicking 'Save Draft' button
            record: null // message record
        };
        if (editwindow.isDraftFolderSelected) {
            this.isDraftFolderSelected = true;
            this.markRowOutdated(record.get('original_id'));
        }
        else {
            this.isDraftFolderSelected = false;
        }
        this.recordProxy.saveDraft(record, folderName, {
            scope: this,
            success: function(record) { this.onSaveDraftSuccess(record, editwindow); },
            failure: function(record) { this.onSaveDraftFailure(record, editwindow); },
            timeout: 60000 // 1 minute
        });
    },

    /**
     * process the result when save draft is successful
     *
     * @param {Tine.Tinebase.data.Record} record
     * @param {Tine.Expressomail.MessageEditDialog} editwindow
     */
    onSaveDraftSuccess: function(record, editwindow) {
        var unloading;
        try { // because editwindow could be closed yet
            unloading = editwindow.unloading;
        } catch (e) { // editwindow already closed, unloading must be true
            unloading = true;
        }
        if (this.isDraftFolderSelected) {
            if (this.autoSaveDraftsEnabled || this.saveOnDestroy.save || (this.saveOnDestroy.forced && !unloading)) {
                this.updateGridPanel(record);
            }
            else {
                this.saveOnDestroy.record = record;
            }
        }
        if (!this.autoSaveDraftsEnabled) {
            if (this.saveOnDestroy.started && !this.saveOnDestroy.save) {
                this.removeDraft(record);
            }
            else {
                this.saveOnDestroy.started = true;
            }
        }
        try { // because editwindow could be closed yet
            editwindow.record.commit(true);
            editwindow.record.set('draft_id', record.get('original_id'));
            editwindow.setMessageOnTitle(editwindow.notifyClear);
            editwindow.setSaveDraftsDelayedTask();
            editwindow.saving = false;
        }
        catch (e) { } // editwindow already closed, ignore and go on
    },

    /**
     * process the result when save draft has failed
     *
     * @param {Tine.Tinebase.data.Record} record
     * @param {Tine.Expressomail.MessageEditDialog} editwindow
     */
    onSaveDraftFailure: function(record, editwindow) {
        if (!this.autoSaveDraftsEnabled) {
            this.saveOnDestroy.error = true;
            if (this.saveOnDestroy.started && this.saveOnDestroy.save) {
                this.showSaveDraftErrorMessage();
            }
        }
        try { // because editwindow could be closed yet
            editwindow.setMessageOnTitle(editwindow.notifySaveDraftFailure);
            editwindow.setSaveDraftsDelayedTask();
            editwindow.saving = false;
        }
        catch (e) { } // editwindow already closed, ignore and go on
    },

    showSaveDraftErrorMessage: function() {
        Ext.MessageBox.show({
            title: this.app.i18n._('Failed to save draft!'),
            msg: this.app.i18n._('A draft could not be saved. Maybe the connection to the server was lost.'),
            buttons: Ext.MessageBox.OK,
            icon: Ext.MessageBox.WARNING
        });
    },

    /**
     * remove draft (setting flag to '\Deleted')
     *
     * @param {Tine.Tinebase.data.Record} record
     */
    removeDraft: function(record) {
        var id = record.get('original_id');
        if (id) {
            if (Ext.isObject(id)) {
                id = id.id;
            }
            this.deleteTransactionId = Tine.Expressomail.messageBackend.addFlags(id, '\\Deleted', {
                callback: function(msgId){
                    this.movingOrDeleting = false;
                }
            });
            if (this.saveOnDestroy.forced) {
                // draft in grid is outdated
                id = record.get('draft_id');
            }
            var msg = this.getStore().getById(id);
            this.getStore().remove(msg);
        }
    },

    /**
     * mark grid row as outdated
     *
     * @param {String} id
     */
    markRowOutdated: function(id) {
        if (id) {
            var msg = this.getStore().getById(id);
            msg.markDirty();
            var ix = this.getStore().indexOf(msg);
            this.getView().addRowClass(ix, 'x-grid-outdated-row');
        }
    },

    /**
     * returns true if grid row is marked as outdated
     *
     * @param {Integer} row
     * @returns {Boolean} true or false
     */
    isRowOutdated : function(row) {
        var r = this.getView().getRow(row);
        return (this.getView().fly(r).hasClass('x-grid-outdated-row'));
    },

    /**
     * update grid panel with record changes
     *
     * @param {Tine.Tinebase.data.Record} record
     */
    updateGridPanel: function(record) {
        var id = record.get('draft_id');
        if (id) {
            if (Ext.isObject(id)) {
                id = id.id;
            }
            var old_msg = this.getStore().getById(id);
            this.getStore().remove(old_msg);
        }
        var msg = record.copy();
        msg.set('id',record.get('original_id'));
        msg.id = record.get('original_id');
        this.addRecordToStore(msg);
    },

    /**
     * add a record to the grid store
     *
     * @param {Tine.Tinebase.data.Record} record
     */
    addRecordToStore: function(record) {
        var store = this.getStore();
        store.suspendEvents();
        var data = store.data.clone();
        store.data.add(record);
        store.sortData(store.sortInfo.field, store.sortInfo.direction);
        var index = store.data.indexOf(record);
        store.data = data;
        store.resumeEvents();
        store.insert(index, record);
    },

     /**
     * Opens the required EditDialog
     * @param {Object} actionButton the button the action was called from
     * @param {Tine.Tinebase.data.Record} record the record to display/edit in the dialog
     * @param {Array} plugins the plugins used for the edit dialog
     * @param {Object} additionalConfig plain Object, which will be applied to the edit dialog on initComponent
     * @return {Boolean}
     */
    onEditInNewWindow: function(button, record, plugins, additionalConfig) {
        if (! record) {
            if (button.actionType == 'edit' || button.actionType == 'copy') {
                if (! this.action_editInNewWindow || this.action_editInNewWindow.isDisabled()) {
                    // if edit action is disabled or not available, we also don't open a new window
                    return false;
                }
                var selectedRows = this.grid.getSelectionModel().getSelections();
                record = selectedRows[0];
            } else {
                record = new this.recordClass(this.recordClass.getDefaultData(), 0);
            }
        }

        // plugins to add to edit dialog
        var plugins = plugins ? plugins : [];
        
        var totalcount = this.selectionModel.getCount(),
            selectedRecords = [],
            fixedFields = (button.hasOwnProperty('fixedFields') && Ext.isObject(button.fixedFields)) ? Ext.encode(button.fixedFields) : null,
            editDialogClass = this.editDialogClass || Tine[this.app.appName][this.recordClass.getMeta('modelName') + 'EditDialog'],
            additionalConfig = additionalConfig ? additionalConfig : {};
        
        // add "multiple_edit_dialog" plugin to dialog, if required
        if (((totalcount > 1) && (this.multipleEdit) && (button.actionType == 'edit'))) {
            Ext.each(this.selectionModel.getSelections(), function(record) {
                selectedRecords.push(record.data);
            }, this );
            
            plugins.push({
                ptype: 'multiple_edit_dialog', 
                selectedRecords: selectedRecords,
                selectionFilter: this.selectionModel.getSelectionFilter(),
                isFilterSelect: this.selectionModel.isFilterSelect,
                totalRecordCount: totalcount
            });
        }
        
        var popupWindow = editDialogClass.openWindow(Ext.copyTo(
            this.editDialogConfig || {}, {
                encrypted: button.encrypted,
                mailStoreData: Tine.Expressomail.getMailStoreData(Tine.Expressomail.getMailStore()),
                plugins: plugins ? Ext.encode(plugins) : null,
                fixedFields: fixedFields,
                additionalConfig: Ext.encode(additionalConfig),
                record: editDialogClass.prototype.mode == 'local' ? Ext.encode(record.data) : record,
                copyRecord: (button.actionType == 'copy'),
                listeners: {
                        scope: this,
                    'update': ((this.selectionModel.getCount() > 1) && (this.multipleEdit)) ? this.onUpdateMultipleRecords : this.onUpdateRecord,
                    'addcontact': this.onEditClose.createDelegate(this)
                    }
            }, 'encrypted,mailStoreData,record,listeners,fixedFields,copyRecord,plugins,additionalConfig')
        );
        return true;
    },

    /**
     * message got removed in display dialog
     *
     * @param {} msgData
     */
    onRemoveInDisplayDialog: function (msgData) {
        var msg = this.getStore().getById(Ext.decode(msgData).id),
            folderId = msg ? msg.get('folder_id') : null,
            folder = folderId ? this.app.getFolderStore().getById(folderId) : null,
            accountId = folder ? folder.get('account_id') : null,
            account = accountId ? this.app.getAccountStore().getById(accountId) : null;

        this.getStore().remove(msg);
        this.onAfterDelete(null);
    },

    /**
     * called when the store gets updated
     *
     * NOTE: we only allow updateing flags BUT the actual updating is done
     *       directly from the UI fn's to support IMAP optimised bulk actions
     */
    onStoreUpdate: function(store, record, operation) {
        if (operation === Ext.data.Record.EDIT && record.isModified('flags')) {
            record.commit()
        }
    },

    /**
     * key down handler
     *
     * @param {Event} e
     */
    onKeyDown: function(e) {
        if (e.ctrlKey) {
            switch (e.getKey()) {
                case e.N:
                case e.M:
                    this.onMessageCompose();
                    e.preventDefault();
                    break;
                case e.R:
                    this.onMessageReplyTo();
                    e.preventDefault();
                    break;
                case e.L:
                    this.onMessageForward();
                    e.preventDefault();
                    break;
            }
        }
        this.detailsPanel.isNavKey = (e.getKey()==38 || e.getKey()==40);
        if (e.getKey()==e.ENTER) {
            // load/reload currently selected message
            this.detailsPanel.reloadDetails();
        }

        Tine.Expressomail.GridPanel.superclass.onKeyDown.call(this, e);
    },

    /**
     * toggle flagged status of mail(s)
     * - Flagged/Seen
     *
     * @param {Button} button
     * @param {Event} event
     * @param {String} flag
     */
    onToggleFlag: function(btn, e, flag) {
        var sm = this.getGrid().getSelectionModel(),
            filter = sm.getSelectionFilter(),
            msgs = sm.isFilterSelect ? this.getStore() : sm.getSelectionsCollection(),
            flagCount = 0;

        // switch all msgs to one state -> toogle most of them
        msgs.each(function(msg) {
            flagCount += msg.hasFlag(flag) ? 1 : 0;
        });
        var action = flagCount >= Math.round(msgs.getCount()/2) ? 'clear' : 'add';

        // mark messages in UI and add to edit buffer
        msgs.each(function(msg) {
            // update unreadcount
            if (flag === '\\Seen') {
                var isSeen = msg.hasFlag('\\Seen'),
                    folder = this.app.getFolderStore().getById(msg.get('folder_id')),
                    diff = (action === 'clear' && isSeen) ? 1 :
                           (action === 'add' && ! isSeen) ? -1 : 0;

                if (folder) {
                    folder.set('cache_unreadcount', folder.get('cache_unreadcount') + diff);
                    if (sm.isFilterSelect && sm.getCount() > 50 && folder.get('cache_status') !== 'pending') {
                        Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - Set cache status to pending for folder ' + folder.get('globalname'));
                        folder.set('cache_status', 'pending');
                    }
                    folder.commit();
                }
            }

            msg[action + 'Flag'](flag);

            this.addToEditBuffer(msg);
        }, this);

        if (sm.isFilterSelect && sm.getCount() > 50) {
            Tine.log.debug('Tine.Expressomail.GridPanel::moveOrDeleteMessages - Update message cache for "pending" folders');
            this.app.checkMailsDelayedTask.delay(1000);
        }

        // do request
        Tine.Expressomail.messageBackend[action+ 'Flags'](filter, flag);
    },

    /**
     * called before store queries for data
     */
    onStoreBeforeload: function(store, options) {
        this.supr().onStoreBeforeload.apply(this, arguments);

        if (! Ext.isEmpty(this.deleteQueue)) {
            options.params.filter.push({field: 'id', operator: 'notin', value: this.deleteQueue});
        }
    },

    /**
     *  called after a new set of Records has been loaded
     *
     * @param  {Ext.data.Store} this.store
     * @param  {Array}          loaded records
     * @param  {Array}          load options
     * @return {Void}
     */
    onStoreLoad: function(store, records, options) {
        
        if (this.detailsPanel.getLoadMask()) {
            this.detailsPanel.getLoadMask().hide();
        }
        
        this.supr().onStoreLoad.apply(this, arguments);

        Tine.log.debug('Tine.Expressomail.GridPanel::onStoreLoad(): store loaded new records.');

        var folder = this.getCurrentFolderFromTree();
        if (folder && records.length < folder.get('cache_totalcount')) {
            Tine.log.debug('Tine.Expressomail.GridPanel::onStoreLoad() - Count mismatch: got ' + records.length + ' records for folder ' + folder.get('globalname'));
            Tine.log.debug(folder);
            folder.set('cache_status', 'pending');
            folder.commit();
            this.app.checkMailsDelayedTask.delay(1000);
        }

        this.updateQuotaBar();
    },

    /**
     * update quotaBar / only do it if we have a path filter with a single account id
     *
     * @param {Record} accountInbox
     */
    updateQuotaBar: function(accountInbox) {
        var accountId = this.extractAccountIdFromFilter();

        if (accountId === null) {
            Tine.log.debug('No or multiple account ids in filter. Resetting quota bar.');
            this.quotaBar.hide();
            return;
        }
        var treex = this.app.getMainScreen().getTreePanel(),
                selectedNodes = (treex) ? treex.getSelectionModel().getSelectedNodes() : null;
        if(selectedNodes) {
            var accountInbox = this.app.getFolderStore().queryBy(function(folder) {
                if(folder.get('id') == selectedNodes[0].id) return folder;
            }, this).first();
            if (!parseInt(accountInbox.get('quota_limit'), 10) || !parseInt(accountInbox.get('quota_usage'), 10)) {
                accountInbox = false;
            }
        }

        if (! accountInbox) {
            var accountInbox = this.app.getFolderStore().queryBy(function(folder) {
                return folder.isInbox() && (folder.get('account_id') == accountId);
            }, this).first();
        }
        if (accountInbox && parseInt(accountInbox.get('quota_limit'), 10) && accountId == accountInbox.get('account_id')) {
            Tine.log.debug('Showing quota info.');

            var limit = parseInt(accountInbox.get('quota_limit'), 10),
                usage = parseInt(accountInbox.get('quota_usage'), 10),
                left = limit - usage,
                text = String.format(this.app.i18n._('{0} %'), Ext.util.Format.number((usage/limit * 100), '00.0'));
            this.quotaBar.show();
            this.quotaBar.setWidth(75);
            this.quotaBar.updateProgress(usage/limit, text);

            Ext.QuickTips.register({
                target:  this.quotaBar,
                dismissDelay: 30000,
                title: this.app.i18n._('Your quota'),
                text: String.format(this.app.i18n._('{0} available (total: {1})'),
                    Ext.util.Format.fileSize(left * 1024),
                    Ext.util.Format.fileSize(limit * 1024)
                ),
                width: 200
            });
        } else {
            Tine.log.debug('No account inbox found or no quota info found.');
            this.quotaBar.hide();
        }
    },

    /**
     * get account id from filter (only returns the id if a single account id was found)
     *
     * @param {Array} filter
     * @return {String}
     */
    extractAccountIdFromFilter: function(filter) {
        if (! filter) {
            filter = this.filterToolbar.getValue();
        }

        // use first OR panel in case of filterPanel
        Ext.each(filter, function(filterData) {
            if (filterData.condition && filterData.condition == 'OR') {
                filter = filterData.filters[0].filters;
                return false;
            }
        }, this);

        // condition from filterPanel
        while (filter.filters || (Ext.isArray(filter) && filter.length > 0 && filter[0].filters)) {
            filter = (filter.filters) ? filter.filters : filter[0].filters;
        }

        var accountId = null,
            filterAccountId = null,
            accountIdMatch = null;

        for (var i = 0; i < filter.length; i++) {
            if (filter[i].field == 'path' && filter[i].operator == 'in') {
                for (var j = 0; j < filter[i].value.length; j++) {
                    accountIdMatch = filter[i].value[j].match(/^\/(?!allinboxes)([a-z0-9]*)/i);
                    if (accountIdMatch) {
                        filterAccountId = accountIdMatch[1];
                        if (accountId && accountId != filterAccountId) {
                            // multiple different account ids found!
                            return null;
                        } else {
                            accountId = filterAccountId;
                        }
                    }
                }
            }
        }

        return accountId;
    },

    /**
     * add new account button
     *
     * @param {Button} button
     * @param {Event} event
     */
    onAddAccount: function(button, event) {
        var popupWindow = Tine.Expressomail.AccountEditDialog.openWindow({
            record: null,
            listeners: {
                scope: this,
                'update': function(record) {
                    var account = new Tine.Expressomail.Model.Account(Ext.util.JSON.decode(record));

                    // add to registry
                    Tine.Expressomail.registry.get('preferences').replace('defaultEmailAccount', account.id);
                    // need to do this because store could be unitialized yet
                    var registryAccounts = Tine.Expressomail.registry.get('accounts');
                    registryAccounts.results.push(account.data);
                    registryAccounts.totalcount++;
                    Tine.Expressomail.registry.replace('accounts', registryAccounts);

                    // add to tree / store
                    var treePanel = this.app.getMainScreen().getTreePanel();
                    treePanel.addAccount(account);
                    treePanel.accountStore.add([account]);
                }
            }
        });
    },

    /**
     * print handler
     *
     * @todo move this to Ext.ux.Printer as iframe driver
     * @param {Tine.Expressomail.GridDetailsPanel} details panel [optional]
     */
    onPrint: function(detailsPanel) {
        var id = Ext.id(),
            doc = document,
            frame = doc.createElement('iframe');

        Ext.fly(frame).set({
            id: id,
            name: id,
            style: {
                position: 'absolute',
                width: '210mm',
                height: '297mm',
                top: '-10000px',
                left: '-10000px'
            }
        });

        doc.body.appendChild(frame);

        Ext.fly(frame).set({
           src : Ext.SSL_SECURE_URL
        });

        var doc = frame.contentWindow.document || frame.contentDocument || WINDOW.frames[id].document,
            content = this.getDetailsPanelContentForPrinting(detailsPanel || this.detailsPanel);

        doc.open();
        doc.write(content);
        doc.close();

        frame.contentWindow.focus();
        frame.contentWindow.print();

        // removeing frame chrashes chrome
//        setTimeout(function(){Ext.removeNode(frame);}, 100);
    },

    /**
     * get detail panel content
     *
     * @param {Tine.Expressomail.GridDetailsPanel} details panel
     * @return {String}
     */
    getDetailsPanelContentForPrinting: function(detailsPanel) {
        // TODO somehow we have two <div class="preview-panel-expressomail"> -> we need to fix that and get the first element found
        var detailsPanels = detailsPanel.getEl().query('.preview-panel-expressomail');
        var detailsPanelContent = (detailsPanels.length > 1) ? detailsPanels[1].innerHTML : detailsPanels[0].innerHTML;

        var buffer = '<html><head>';
        buffer += '<title>' + this.app.i18n._('Print Preview') + '</title>';
        buffer += '</head><body>';
        buffer += detailsPanelContent;
        buffer += '</body></html>';

        return buffer;
    },

    /**
     * print preview handler
     *
     * @param {Tine.Expressomail.GridDetailsPanel} details panel [optional]
     */
    onPrintPreview: function(detailsPanel) {
        var content = this.getDetailsPanelContentForPrinting(detailsPanel || this.detailsPanel);

        var win = window.open('about:blank','_blank','width=500,height=500,scrollbars=yes,toolbar=yes,status=yes,menubar=yes');
        win.document.open()
        win.document.write(content);
        win.document.close();
        win.focus();
    },

    /**
     * format headers
     *
     * @param {Object} headers
     * @param {Bool} ellipsis
     * @param {Bool} onlyImportant
     * @return {String}
     */
    formatHeaders: function(headers, ellipsis, onlyImportant) {
        var result = '';
        for (header in headers) {
            if (headers.hasOwnProperty(header) &&
                    (! onlyImportant || header == 'from' || header == 'to' || header == 'cc' || header == 'subject' || header == 'date'))
            {
                if (header == 'date') headers[header] = new Date(headers[header]).toLocaleString();
                result += '<b>' + _(header) + ':</b> '
                    + Ext.util.Format.htmlEncode(
                        (ellipsis)
                            ? Ext.util.Format.ellipsis(headers[header], 40)
                            : headers[header]
                    ) + '<br/>';
            }
        }
        return result;
    }
});
/*
 * Tine 2.0
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2013 Metaways Infosystems GmbH (http://www.metaways.de)
 */

Ext.ns('Tine.Expressomail')


Tine.Expressomail.MessageDisplayDialog = Ext.extend(Tine.Expressomail.GridDetailsPanel ,{
    /**
     * @cfg {Tine.Expressomail.Model.Message}
     */
    record: null,
    
    mailStoreData: null,

    autoScroll: false,
    
    encrypted: false,

    initComponent: function() {
        if (Ext.isString(this.record)) {
            this.record = Tine.Expressomail.messageBackend.recordReader({responseText: this.record});
        }

        this.encrypted = (this.record.get('smimeEml') != '') && (typeof(this.record.get('smimeEml')) != "undefined");
        
        this.addEvents('remove',
            /**
            * @event addcontact
            * @desc  Fired when contact is added
            * @param {Json String} data data of the contact
            */
            'addcontact'
        );

        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        this.i18n = this.app.i18n;

        // trick onPrint/onPrintPreview
        this.detailsPanel = this;

        this.initActions();
        this.initToolbar();

        Tine.log.debug('Tine.Expressomail.MessageDisplayDialog::initComponent() -> message record:');
        Tine.log.debug(this.record);

        this.supr().initComponent.apply(this, arguments);
    },

    /**
     * init actions
     */
    initActions: function() {
        this.action_deleteRecord = new Ext.Action({
            text: this.app.i18n._('Delete'),
            handler: this.onMessageDelete.createDelegate(this, [false]),
            iconCls: 'action_delete',
            disabled: this.record.id.match(/_/)
        });

        if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1'
            && Tine.Tinebase.registry.get('preferences').get('windowtype')== 'Ext') {
            
            var encrypted_option_text = ' ('+this.app.i18n._('encrypted') + ')';
            var encrypted_text = (this.encrypted ? encrypted_option_text : '');
            var encrypted_text_alternate = (this.encrypted ? '' : encrypted_option_text);
            
            this.action_reply = new Ext.Action({
                text: this.app.i18n._('Reply') + encrypted_text,
                handler: this.onMessageReplyTo.createDelegate(this, [false,this.encrypted]),
                iconCls: 'action_email_reply',
                menu: {
                    items: [new Ext.Action({
                            text: this.app.i18n._('Reply') + encrypted_text_alternate,
                            handler: this.onMessageReplyTo.createDelegate(this, [false,!this.encrypted]),
                            iconCls:'action_email_reply'
                        })
                    ]
                }
            });

            this.action_replyAll = new Ext.Action({
                text: this.app.i18n._('Reply To All') + encrypted_text,
                handler: this.onMessageReplyTo.createDelegate(this, [true,this.encrypted]),
                iconCls: 'action_email_replyAll',
                menu: {
                    items: [
                        new Ext.Action({
                            text: this.app.i18n._('Reply To All') + encrypted_text_alternate,
                            handler: this.onMessageReplyTo.createDelegate(this, [true,!this.encrypted]),
                            iconCls:'action_email_replyAll'
                        })
                    ]
                }
            });

            this.action_forward = new Ext.Action({
                text: this.app.i18n._('Forward') + encrypted_text,
                handler: this.onMessageForward.createDelegate(this,[this.encrypted]),
                iconCls: 'action_email_forward',
                menu: {
                    items: [
                        new Ext.Action({
                            text: this.app.i18n._('Forward') + encrypted_text_alternate,
                            handler: this.onMessageForward.createDelegate(this,[!this.encrypted]),
                            iconCls:'action_email_forward'
                        })
                    ]
                }
            });
        }
        else {
            this.action_reply = new Ext.Action({
                text: this.app.i18n._('Reply'),
                handler: this.onMessageReplyTo.createDelegate(this, [false,this.encrypted]),
                iconCls: 'action_email_reply'
            });

            this.action_replyAll = new Ext.Action({
                text: this.app.i18n._('Reply To All'),
                handler: this.onMessageReplyTo.createDelegate(this, [true,this.encrypted]),
                iconCls: 'action_email_replyAll'
            });

            this.action_forward = new Ext.Action({
                text: this.app.i18n._('Forward'),
                handler: this.onMessageForward.createDelegate(this,[this.encrypted]),
                iconCls: 'action_email_forward'
            });
        }
        
        this.action_download = new Ext.Action({
            text: this.app.i18n._('Save'),
            handler: this.onMessageDownload.createDelegate(this),
            iconCls: 'action_email_download',
            disabled: this.record.id.match(/_/)
        });

        this.action_print = new Ext.Action({
            text: this.app.i18n._('Print Message'),
            handler: this.onMessagePrint.createDelegate(this.app.getMainScreen().getCenterPanel(), [this]),
            iconCls:'action_print',
            menu:{
                items:[
                    new Ext.Action({
                        text: this.app.i18n._('Print Preview'),
                        handler: this.onMessagePrintPreview.createDelegate(this.app.getMainScreen().getCenterPanel(), [this]),
                        iconCls:'action_printPreview'
                    })
                ]
            }
        });

    },

    /**
     * init toolbar
     */
    initToolbar: function() {
        if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1'
            && Tine.Tinebase.registry.get('preferences').get('windowtype')== 'Ext') {
            var button_type = Ext.SplitButton;
        }
        else {
            var button_type = Ext.Button;
        }
        // use toolbar from gridPanel
        this.tbar = new Ext.Toolbar({
            defaults: {height: 55},
            items: [{
                xtype: 'buttongroup',
                columns: 5,
                items: [
                    Ext.apply(new Ext.Button(this.action_deleteRecord), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top'
                    }),
                    Ext.apply(new button_type(this.action_reply), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top'
                    }),
                    Ext.apply(new button_type(this.action_replyAll), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top'
                    }),
                    Ext.apply(new button_type(this.action_forward), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top'
                    }),
                    Ext.apply(new Ext.SplitButton(this.action_print), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign:'top',
                        arrowAlign:'right'
                    }),
                    Ext.apply(new Ext.Button(this.action_download), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign:'top',
                        arrowAlign:'right'
                    })
                ]
            }]
        });

    },

    /**
     * after render
     */
    afterRender: function() {
        this.supr().afterRender.apply(this, arguments);
        this.showMessage();

        var title = this.record.get('subject');
        if (title !== undefined) {
            // TODO make this work for attachment mails
            this.window.setTitle(title);
        }
    },

    /**
     * show message
     */
    showMessage: function() {
        this.layout.setActiveItem(this.getSingleRecordPanel());
        this.record.set('decrypted',true);
        this.updateDetails(this.record, this.getSingleRecordPanel().body);
    },
    
    onEditClose: function(contact) {
        // Relaying the contact to mainWindow
        Tine.log.debug('Tine.Expressomail.GridPanel::onEditClose / arguments:' + contact);
        this.fireEvent('addcontact', contact);
        
    },

    onDestroy: function() {
        this.supr().onDestroy.apply(this, arguments);
        this.app.mainScreen.GridPanel.focusSelectedMessage();
    },

    /**
     * executed after a msg compose
     *
     * @param {String} composedMsg
     * @param {String} action
     * @param {Array}  [affectedMsgs]  messages affected
     *
     */
    onAfterCompose: function(composedMsg, action, affectedMsgs) {
        this.fireEvent('update', composedMsg, action, affectedMsgs);
        this.app.mainScreen.GridPanel.onAfterCompose(composedMsg, action, affectedMsgs);
    },

    /**
     * executed after deletion of this message
     */
    onAfterDelete: function() {
                 this.fireEvent('remove', Ext.encode(this.record.data));
                 this.window.close();
    },

    /**
     * download message
     */
    onMessageDownload: function() {
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressomail.downloadMessage',
                requestType: 'HTTP',
                messageId: this.record.id,
                filter : ''
            }
        }).start();
    },

    /**
     * delete message handler
     */
      onMessageDelete: function(force) {
        var mainApp = Ext.ux.PopupWindowMgr.getMainWindow().Tine.Tinebase.appMgr.get('Expressomail'),
            folderId = this.record.get('folder_id'),
            folder = mainApp.getFolderStore().getById(folderId),
            accountId = folder ? folder.get('account_id') : null,
            account = mainApp.getAccountStore().getById(accountId),
            trashId = account ? account.getTrashFolderId() : null;
        
        this.loadMask.show();
        if(Tine.Expressomail.registry.get('preferences').get('confirmDelete') == '1')
        {
            Ext.MessageBox.confirm('', this.app.i18n._('Confirm Delete') + ' ?', function(btn) {
                if(btn == 'yes') { 
                    this.moveOrDeleteMessage(trashId);
                }
                else {
                    this.loadMask.hide();
                }
            }, this);
            
        }
        else
        {
             this.moveOrDeleteMessage(trashId);
        }
    },
            
         
    /**
     * Do the actual delete or move
     */
    
    moveOrDeleteMessage: function(trashId){
        if (trashId) {
            var filter = [{field: 'id', operator: 'equals', value: this.record.id}];

            Tine.Expressomail.messageBackend.moveMessages(filter, trashId, { 
                callback: this.onAfterDelete.createDelegate(this, ['move'])
            });
        } else {
            Tine.Expressomail.messageBackend.addFlags(this.record.id, '\\Deleted', { 
                callback: this.onAfterDelete.createDelegate(this, ['flag'])
            });
        }
    },       

    /**
     * reply message handler
     */
    onMessageReplyTo: function(toAll,encrypted) {
        Tine.Expressomail.MessageEditDialog.openWindow({
            encrypted: encrypted || false,
            mailStoreData: this.mailStoreData,
            replyTo : Ext.encode(this.record.data),
            replyToAll: toAll,
            listeners: {
                'update': this.onAfterCompose.createDelegate(this, ['reply', Ext.encode([this.record.data])], 1),
                'addcontact': this.onEditClose.createDelegate(this)
                }
        });
    },

    /**
     * forward message handler
     */
    onMessageForward: function(encrypted) {
        Tine.Expressomail.MessageEditDialog.openWindow({
            encrypted: encrypted || false,
            mailStoreData: this.mailStoreData,
            forwardMsgs : Ext.encode([this.record.data]),
            listeners: {
                'update': this.onAfterCompose.createDelegate(this, ['forward', Ext.encode([this.record.data])], 1),
                'addcontact': this.onEditClose.createDelegate(this)
                }
        });
    },

    /**
     * (on) update details
     *
     * @param {Tine.Expressomail.Model.Message} record
     * @param {String} body
     * @private
     */
    updateDetails: function(record, body) {
        this.supr().updateDetails.apply(this, arguments);
        var title = this.record.get('subject');
        if (title !== undefined) {
            this.window.setTitle(title);
        }
    },

    onMessagePrint: Tine.Expressomail.GridPanel.prototype.onPrint,
    onMessagePrintPreview: Tine.Expressomail.GridPanel.prototype.onPrintPreview
});

Tine.Expressomail.MessageDisplayDialog.openWindow = function (config) {
    var record = (Ext.isString(config.record)) ? Ext.util.JSON.decode(config.record) : config.record,
        id = (record && record.id) ? record.id : 0,
        window = Tine.WindowFactory.getWindow({
            width: 800,
            height: 700,
            name: 'TineExpressomailMessageDisplayDialog_' + id,
            contentPanelConstructor: 'Tine.Expressomail.MessageDisplayDialog',
            contentPanelConstructorConfig: config
        });
    return window;
};
Ext.ns('Tine.Expressomail');

Tine.Expressomail.FileUploadGrid = Ext.extend(Tine.widgets.grid.FileUploadGrid, {
    
    parent: null,
    messageRecord: null,

    initComponent : function() {
        Tine.Expressomail.FileUploadGrid.superclass.initComponent.call(this);
        this.store.on('update', this.onUpdateStore, this);
    },

    onFilesSelect: function (fileSelector, e) {
        
        var files = fileSelector.getFileList();
        Ext.each(files, function (file) {

            var upload = new Ext.ux.file.Upload({
                isLocal: this.parent.encrypted,
                eid: this.parent.eid,
                file: file,
                fileSelector: fileSelector
            });

            var uploadKey = Tine.Tinebase.uploadManager.queueUpload(upload);
            var fileRecord = Tine.Tinebase.uploadManager.upload(uploadKey);
            
            fileRecord.fileUploadGrid = this;
            
            upload.on('uploadfailure', this.onUploadFail, this);
            upload.on('uploadcomplete', this.onUploadComplete, fileRecord);
            upload.on('uploadstart', Tine.Tinebase.uploadManager.onUploadStart, this);

            if(fileRecord.get('status') !== 'failure' ) {
                this.store.add(fileRecord);
            }

            
        }, this);
        
    },

    onUploadComplete: function(upload, fileRecord) {
    
        fileRecord.beginEdit();
        fileRecord.set('status', 'complete');
        fileRecord.set('progress', 100);
        fileRecord.commit(false);
        Tine.Tinebase.uploadManager.onUploadComplete();
        
        // check if the last attachment violates maximum size, remove it it if it does
        if (!upload.isLocal) {
            fileRecord.fileUploadGrid.validateLastAttachment(fileRecord.fileUploadGrid);
        }
    },
    
    /**
     * store has been updated
     *
     * @param {} store
     * @param {} record
     * @param {} operation
     * @private
     */
    onUpdateStore: function(store, record, operation) {
        this.setModifiedFlag();
    },

    /**
     * set flag to indicate that the message has changed
     *
     * @private
     */
    setModifiedFlag: function() {
        if (this.messageRecord) {
            if (!this.messageRecord.modified) {
                this.messageRecord.modified = {};
            }
            this.messageRecord.modified['attachments'] = true;
        }
    },

    validateLastAttachment: function(that) {
        var messageContent = new Tine.Expressomail.Model.MessageContent(
            Tine.Expressomail.Model.MessageContent.getDefaultData(), 0);
        
        var idx = that.store.data.items.length-1;
        var att = that.store.data.items[idx];
        var attachments = [];
        attachments.push({
            tempFile: Ext.ux.file.Upload.file.getFileData(att).tempFile
        });
        messageContent.data.attachments = attachments;
        
        that.showMask(that.parent);
        Ext.Ajax.request({
            params: {
                method: 'Expressomail.calcMessageSize',
                recordData: messageContent
            },
            success: function (result) {
                var response = JSON.parse(result.responseText);
                if (response.maxMessageSize <= 0) {
                    // parameter maxMessageSize is not defined in config.inc.php
                    that.hideMask(that.parent);
                    return;
                }
                var totalSize = response.attachmentSize;
                var limitExceeded = totalSize > response.maxMessageSize;

                var maxMB = Ext.util.Format.number(response.maxMessageSize/(1024*1024), '0.00');
                var usedMB = Ext.util.Format.number(totalSize/(1024*1024), '0.00');

                if (limitExceeded) {
                    Ext.MessageBox.show({
                        title: that.parent.app.i18n._('Message Size Limit Exceeded'),
                        msg: String.format(that.parent.app.i18n._('Maximum allowed message size is {0} Mb.'), maxMB)
                                + '<br>' + 
                             String.format(that.parent.app.i18n._('The last attachment was removed.'), maxMB),
                        buttons: Ext.MessageBox.OK,
                        icon: Ext.MessageBox.INFO,
                        fn: function(btn) {
                            that.validateAllAttachment(that);
                        }
                    });
                    that.selModel.selectRow(idx);
                    that.onRemove();
                    that.hideMask(that.parent);
                }
                else {
                    that.hideMask(that.parent);
                    // if last file size passed, calculates the total attachment size
                    that.validateAllAttachment(that);
                }
            },
            failure: function (err, details) {
                Ext.MessageBox.alert(that.parent.app.i18n._('Failed'),
                    that.parent.app.i18n._('File size validation step failed.'));
                that.hideMask(that.parent);
            }
        });
    },
    
    validateAllAttachment: function(that) {
        var messageContent = new Tine.Expressomail.Model.MessageContent(
            Tine.Expressomail.Model.MessageContent.getDefaultData(), 0);
            
        var attachments = [];
        that.store.each(function(attachment) {
            attachments.push({
                tempFile: Ext.ux.file.Upload.file.getFileData(attachment).tempFile
            });
        }, this);
        messageContent.data.attachments = attachments;
        
        that.showMask(that.parent);
        Ext.Ajax.request({
            params: {
                method: 'Expressomail.calcMessageSize',
                recordData: messageContent
            },
            success: function (result) {
                var response = JSON.parse(result.responseText);
                if (response.maxMessageSize <= 0) {
                    // parameter maxMessageSize is not defined in config.inc.php
                    that.hideMask(that.parent);
                    return;
                }
                var totalSize = response.attachmentSize;
                var limitExceeded = totalSize > response.maxMessageSize;

                var maxMB = Ext.util.Format.number(response.maxMessageSize/(1024*1024), '0.00');
                var usedMB = Ext.util.Format.number(totalSize/(1024*1024), '0.00');

                if (limitExceeded) {
                    Ext.MessageBox.show({
                        title: that.parent.app.i18n._('Message Size Limit Exceeded'),
                        msg: String.format(that.parent.app.i18n._('Maximum allowed message size is {0} Mb.'), maxMB)
                                + '<br>' + 
                             String.format(that.parent.app.i18n._('This message total attachment size is {0} Mb.'), usedMB),
                        buttons: Ext.MessageBox.OK,
                        icon: Ext.MessageBox.INFO
                    });
                }
                that.hideMask(that.parent);
            },
            failure: function (err, details) {
                Ext.MessageBox.alert(that.parent.app.i18n._('Failed'),
                    that.parent.app.i18n._('File size validation step failed.'));
                that.hideMask(that.parent);
            }
        });
    },
    
    showMask: function(that) {
        try {
            that.contactsCheckMask = new Ext.LoadMask(that.ownerCt.body, {msg: that.app.i18n._('Calculating message size...')});
        }
        catch (e) {
            that.contactsCheckMask = new Ext.LoadMask(Ext.getBody(), {msg: that.app.i18n._('Calculating message size...')});
        }
        that.contactsCheckMask.show();
    },
    
    hideMask: function(that) {
        that.contactsCheckMask.hide();
    },
    
    onRemove: function (button, event) {
        var selectedRows = this.getSelectionModel().getSelections();
        var attachments = [];
        var messageAttachments = new Tine.Expressomail.Model.MessageAttachments(
            Tine.Expressomail.Model.MessageAttachments.getDefaultData(), 0)
        for (var i = 0; i < selectedRows.length; i += 1) {
            this.store.remove(selectedRows[i]);
            var upload = Tine.Tinebase.uploadManager.getUpload(selectedRows[i].get('uploadKey'));
            if (typeof(upload) != "undefined") {
                upload.setPaused(true);
            }
            attachments.push({
                tempFile: selectedRows[i].data.tempFile
            });
        }
        messageAttachments.data.attachments = attachments;
        this.setModifiedFlag();
        Ext.Ajax.request({
            params: {
                method: 'Expressomail.removeTempFiles',
                recordData: messageAttachments
            },
            success: function (result) {
                var response = JSON.parse(result.responseText);
                console.log('attachments removed');
                console.log(response);
            },
            failure: function (err, details) {
                console.log(err);
                console.log(details);
            }
        });
    }
    
});
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.MessageEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Message Compose Dialog</p>
 * <p>This dialog is for composing emails with recipients, body and attachments.
 * you can choose from which account you want to send the mail.</p>
 * <p>
 * TODO         make email note editable
 * </p>
 *
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new MessageEditDialog
 */
Tine.Expressomail.MessageEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {
    /**
     * @cfg {Array/String} bcc
     * initial config for bcc
     */
    bcc: null,

    /**
     * @cfg {String} body
     */
    msgBody: '',

    /**
     * @cfg {Array/String} cc
     * initial config for cc
     */
    cc: null,

    /**
     * @cfg {Array} of Tine.Expressomail.Model.Message (optionally encoded)
     * messages to forward
     */
    forwardMsgs: null,

    /**
     * @cfg {String} accountId
     * the accout id this message is sent from
     */
    accountId: null,

    /**
     * @cfg {Tine.Expressomail.Model.Message} (optionally encoded)
     * message to reply to
     */
    replyTo: null,

    /**
     * @cfg {Tine.Expressomail.Model.Message} (optionally encoded)
     * message to use as draft/template
     */
    draftOrTemplate: null,

    /**
     * @cfg {Boolean} (defaults to false)
     */
    replyToAll: false,

    /**
     * @cfg {String} subject
     */
    subject: '',

    /**
     * @cfg {Array/String} to
     * initial config for to
     */
    to: null,

    /**
     * validation error message
     * @type String
     */
    validationErrorMessage: '',

    /**
     * array with e-mail-addresses used as recipients
     * @type {Array}
     */
    mailAddresses: null,
    /**
     * json-encoded selection filter
     * @type {String} selectionFilter
     */
    selectionFilter: null,
    
    /**
     * holds default values for the record
     * @type {Object}
     */
    recordDefaults: null,

    /**
     * dynamic contacts checking message mask
     * @type {Object}
     */
    contactsCheckMask: null,
    
    /*** 
     *  Certificate store
     */
    certificatesStore: null,
    
    recipientsCertificates:  [],
    
    recipientsWithoutCertificates: [],
    /**
     * dialog is currently sending email
     * @type Boolean
     */
    sending: false,
    /**
     * dialog is currently checking email size
     * @type Boolean
     */
    checking: false,
    
    /**
     * @private
     */
    windowNamePrefix: 'MessageEditWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Message,
    recordProxy: Tine.Expressomail.messageBackend,
    loadRecord: false,
    evalGrants: false,

    mailStoreData: null,
    bodyStyle:'padding:0px',
    encrypted: false,
    eid: null,
    windowTitle: 'Compose email:',

    /**
     * overwrite update toolbars function (we don't have record grants)
     * @private
     */
    updateToolbars: Ext.emptyFn,
    editCloseCallback: Ext.emptyFn,
    keyEscrowCertificates: [],
    saveDraftsInterval: 0,
    isDraftFolderSelected: false,
    notifyClear: 0,
    notifySaveDraftOperation: 1,
    notifySaveDraftFailure: 2,
    isBodyLoaded: false,
    unloading: false,
    autoSaveDraftsEnabled: false,

    //private
    initComponent: function() {
        this.eid = this.rndEid();
        this.addEvents(
            /**
            * @event addcontact
            * @desc  Fired when contact is added
            * @param {Json String} data data of the contact
            */
            'addcontact'
        );
        Tine.Expressomail.MessageEditDialog.superclass.initComponent.call(this);
        // Allows new contacts to be available for immediate use (i.e. added in the cache)
        // except for IE<=8 that will be added when the cache is reloaded
        if(!Ext.isIE){
            this.on('addcontact', this.editCloseCallback);
        }
        this.on('save', this.onSave, this);
        this.on('show', this.onShow, this);
        if (!this.encrypted) {
            // will call the onMessageEditWindowDestroy on unload event even if autosave is disabled (interval=0)
            if (Tine.Tinebase.registry.get('preferences').get('windowtype')==='Browser') {
                this.window.popup.addEventListener('unload', this.window.ref.getMainScreen().getCenterPanel().onMessageEditWindowDestroy.createDelegate(this));
            }
            else {
                this.on('removed', this.window.ref.getMainScreen().getCenterPanel().onMessageEditWindowDestroy, this);
            }
        }
        //Disable autosave for encrypted messages
        if(this.encrypted){
            this.autoSaveDraftsEnabled = false;
        }
        else{
            if (Tine.Expressomail.registry.get("autoSaveDraftsInterval")) {
                // autosave is enabled (interval>0)
                this.saveDraftsInterval = Tine.Expressomail.registry.get("autoSaveDraftsInterval") * 1000; // convert s to ms
                this.autoSaveDraftsEnabled = true;
                this.window.ref.getMainScreen().getCenterPanel().autoSaveDraftsEnabled = true;
                this.saveDraftsIntervaledTask = new Ext.util.DelayedTask(this.checkDraftChanges, this);
                this.setSaveDraftsDelayedTask();
            } else {
                // autosave is enabled (interval=0)
                this.saveDraftsInterval = 0;
                this.autoSaveDraftsEnabled = false;
            }
        }
    },

    /**
     * initialize required information for autosave
     */
    initDraft: function() {
        this.isDraftFolderSelected = false;
        try {
            var selNode = this.window.ref.getMainScreen().getWestPanel().TreePanel.getSelectionModel().getSelectedNode();
            if (selNode) {
                var account = this.app.getAccountStore().getById(this.record.get('account_id'));
                if (selNode.attributes.globalname === account.get('drafts_folder')) {
                    this.isDraftFolderSelected = true;
                    if (this.draftOrTemplate) {
                        this.record.set('draft_id', this.draftOrTemplate.get('id'));
                    }
                }
            }
        }
        catch (e) {
        }
    },

    /**
     * delay the save draft task
     *
     * @private
     */
    setSaveDraftsDelayedTask: function() {
        if (this.saveDraftsInterval>0) {
            this.saveDraftsIntervaledTask.delay(this.saveDraftsInterval);
        }
    },

    /**
     * remove draft from grid panel
     * @param {String} id
     *
     * @private
     */
    removeDraftFromGridPanel: function(id) {
        if (id && this.isDraftFolderSelected) {
            var grid = this.window.ref.getMainScreen().getCenterPanel();
            if (Ext.isObject(id)) {
                id = id.id;
            }
            var msg = grid.getStore().getById(id);
            grid.getStore().remove(msg);
        }
    },

    /**
     * set autosave message on title
     * @param {Integer} notify
     *
     * @private
     */
    setMessageOnTitle: function(notify) {
        if (this.saveDraftsInterval>0) {
            var msg;
            var notify_type = '';
            switch (notify) {
                case this.notifySaveDraftOperation:
                    msg = this.app.i18n._('Saving as a draft...');
                    break;
                case this.notifySaveDraftFailure:
                    msg = this.app.i18n._('Failed to save draft!');
                    notify_type = 'error';
                    break;
                case this.notifyClear:
                default:
                    msg = '';
            }
            if (Tine.Tinebase.registry.get('preferences').get('windowtype')==='Browser') {
                var title = this.app.i18n._(this.windowTitle) + msg;
                if (this.record.get('subject')) {
                    title = title + ' ' + this.record.get('subject');
                }
                this.window.setTitle(title);
            }
            else {
                // use notification tool
                this.window.setNotify(msg, notify_type);
            }
        }
    },

    /**
     * check changes to save draft in autosave
     * @param {Boolean} forced
     *
     * @private
     */
    checkDraftChanges: function(forced) {
        if (this.autoSaveDraftsEnabled) {
            this.saveDraftsIntervaledTask.cancel();
        }

        this.onRecordUpdate();

        var hasRecipient = new Array().concat(this.returnArrayContacts('to')).concat(this.returnArrayContacts('cc')).concat(this.returnArrayContacts('bcc')).length>0;
        var hasSubject = this.record.get('subject').length>0;
        var hasContent = this.htmlEditor.getEditorBody().innerHTML.replace(/\u200B/g,"").length>0;
        var hasAttachment = (this.record.get('attachments'))||false;
        var modified = this.record.modified;
        var wasModified = (modified && (modified.to!==undefined || modified.cc!==undefined || modified.cco!==undefined || modified.subject!==undefined || modified.body!==undefined || modified.attachments!==undefined));

        if (wasModified
            && !this.sending
            && !this.checking
            && !this.attachmentGrid.isUploading()
            && (hasRecipient || hasSubject || hasContent || hasAttachment)
        ) {
            if (this.isDraftFolderSelected) {
                this.window.ref.getMainScreen().getCenterPanel().markRowOutdated(this.record.get('draft_id'));
            }
            this.callSaveDraft(forced);
            return true;
        }
        else {
            this.setSaveDraftsDelayedTask();
            return false;
        }
    },

    /**
     * call save draft in gridpanel for autosave
     * @param {Boolean} forced
     *
     * @private
     */
    callSaveDraft: function(forced) {
        var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(this.record.get('account_id')),
            folderName = account.get('drafts_folder');
        this.saving = true; // this will reset at GridPanel.callSaveDraft callbacks (success, failure)
        this.setMessageOnTitle(this.notifySaveDraftOperation);
        try {
            // draft saving must be executed in main window grid panel due to IE issues in browser window mode
            this.window.ref.getMainScreen().getCenterPanel().callSaveDraft(this.record, folderName, this, forced);
        }
        catch(e) {
            this.setMessageOnTitle(this.notifySaveDraftFailure);
        }
    },

    /**
     * init buttons
     */
    initButtons: function() {
        this.fbar = [];

        this.action_send = new Ext.Action({
            text: this.app.i18n._('Send'),
            handler: this.encrypted ? this.onSaveAndClose : this.checkMessageSize,
            iconCls: 'ExpressomailIconCls',
            disabled: false,
            scope: this
        });

        this.action_searchContacts = new Ext.Action({
            text: this.app.i18n._('Search Recipients'),
            handler: this.onSearchContacts,
            iconCls: 'AddressbookIconCls',
            disabled: false,
            scope: this
        });

        this.action_saveAsDraft = new Ext.Action({
            text: this.app.i18n._('Save As Draft'),
            handler: this.onSaveInFolder.createDelegate(this, ['drafts_folder']),
            iconCls: 'action_saveAsDraft',
            disabled: (this.encrypted),
            scope: this
        });

        this.action_saveAsTemplate = new Ext.Action({
            text: this.app.i18n._('Save As Template'),
            handler: this.onSaveInFolder.createDelegate(this, ['templates_folder']),
            iconCls: 'action_saveAsTemplate',
            disabled: (this.encrypted),
            scope: this
        });

        // TODO think about changing icon onToggle
        this.action_saveEmailNote = new Ext.Action({
            text: this.app.i18n._('Save Email Note'),
            handler: this.onToggleSaveNote,
            iconCls: 'notes_noteIcon',
            disabled: (this.encrypted),
            scope: this,
            enableToggle: true
        });
        this.button_saveEmailNote = Ext.apply(new Ext.Button(this.action_saveEmailNote), {
            tooltip: this.app.i18n._('Activate this toggle button to save the email text as a note attached to the recipient(s) contact(s).')
        });

        this.action_toggleReadingConfirmation = new Ext.Action({
            text: this.app.i18n._('Reading Confirmation'),
            handler: this.onToggleReadingConfirmation,
            iconCls: 'expressomail-action-reading-confirmation',
            disabled: false,
            scope: this,
            enableToggle: true
        });
        this.button_toggleReadingConfirmation = Ext.apply(new Ext.Button(this.action_toggleReadingConfirmation), {
            tooltip: this.app.i18n._('Activate this toggle button to receive a reading confirmation.')
        });
        this.action_toggleMarkAsImportant = new Ext.Action({
            text: this.app.i18n._('Mark as Important'),
            handler: this.onToggleMarkAsImportant,
            iconCls: 'emblems_emblemImportant',
            disabled: false,
            scope: this,
            enableToggle: true
        });
        this.button_toggleMarkAsImportant = Ext.apply(new Ext.Button(this.action_toggleMarkAsImportant), {
            tooltip: this.app.i18n._('Activate this toggle button to mark this message as important.')
        });

        this.action_toggleSendAsPlain = new Ext.Action({
            text: this.app.i18n._('Send As Plain'),
            handler: this.onToggleSendAsPlain,
            iconCls: 'notes_noteIcon',
            disabled: false,
            scope: this,
            enableToggle: true
        });
        this.button_toggleSendAsPlain = Ext.apply(new Ext.Button(this.action_toggleSendAsPlain), {
            tooltip: this.app.i18n._('Activate this toggle button to send the message as text/plain')
        });
        
        this.action_toggleSignMail = new Ext.Action({
            text: this.app.i18n._('Digitally Sign Mail'),
            handler: this.onToggleSignMail,
            iconCls: 'notes_noteIcon', // TODO: change Icon
            disabled: false,
            scope: this,
            enableToggle: true,
            pressed: Tine.Expressomail.registry.get('preferences').get('alwaysSign') == "1" ? true : false
        });
        this.button_toggleSignMail = Ext.apply(new Ext.Button(this.action_toggleSignMail), {
            tooltip: this.app.i18n._('Activate this toggle button to sign a message on send')
        });

        this.action_toggleAddUnknownContacts = new Ext.Action({
            text: this.app.i18n._('Add Contacts'),
            handler: this.onToggleAddContacts,
            iconCls: 'expressomail-action-add-unknown-contacts',
            disabled: false,
            scope: this,
            pressed: true,
            enableToggle: true
        });
        this.button_toggleAddUnknownContacts = Ext.apply(new Ext.Button(this.action_toggleAddUnknownContacts), {
            tooltip: this.app.i18n._('Activate this toggle button to add unknown recipient to contacts.')
        });
        
        this.tbar = new Ext.Toolbar({
            defaults: {height: 55},
            items: [{
                xtype: 'buttongroup',
                columns: 7,
                items: [
                    Ext.apply(new Ext.Button(this.action_send), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top'
                    }),
                    Ext.apply(new Ext.Button(this.action_searchContacts), {
                        scale: 'medium',
                        rowspan: 2,
                        iconAlign: 'top',
                        tooltip: this.app.i18n._('Click to search for and add recipients from the Addressbook.')
                    }),
                    this.action_saveAsDraft,
                    this.button_saveEmailNote,
                    this.action_saveAsTemplate,
                    this.button_toggleAddUnknownContacts,
                    this.button_toggleReadingConfirmation,
                    this.button_toggleSendAsPlain,
                    this.button_toggleMarkAsImportant,
                    this.button_toggleSignMail
                ]
            }]
        });
    },
    
    checkMessageSize: function() {
        this.checking = true;
        try {
            this.contactsCheckMask = new Ext.LoadMask(this.ownerCt.body, {msg: this.app.i18n._('Calculating message size...')});
        }
        catch (e) {
            this.contactsCheckMask = new Ext.LoadMask(Ext.getBody(), {msg: this.app.i18n._('Calculating message size...')});
        }
        this.contactsCheckMask.show();
        
        var messageContent = new Tine.Expressomail.Model.MessageContent(
            Tine.Expressomail.Model.MessageContent.getDefaultData(), 0);
        
        messageContent.data.body = this.record.data.body ? this.record.data.body : this.htmlEditor.getValue();
        if (this.record.data.embedded_images) {
            var images = [];
            var image;
            for (var i=0; i<this.record.data.embedded_images.length; i++) {
                image = this.record.data.embedded_images[i];
                if (messageContent.data.body.indexOf(image.id) != -1) {
                    images.push({
                        id: image.id
                    });
                }
            }
            messageContent.data.embedded_images = images;
        }
        
        if (this.record.data.attachments) {
            messageContent.data.attachments = this.record.data.attachments;
        } else if (this.attachmentGrid) {
            var attachments = [];
            this.attachmentGrid.store.each(function(attachment) {
                attachments.push({
                    tempFile: Ext.ux.file.Upload.file.getFileData(attachment).tempFile
                });
            }, this);
            messageContent.data.attachments = attachments;
        }

        var that = this;
        
        Ext.Ajax.request({
            params: {
                method: 'Expressomail.calcMessageSize',
                recordData: messageContent
            },
            success: function (result) {
                var response = JSON.parse(result.responseText);
                
                if (response.maxMessageSize <= 0) {
                    // parameter maxMessageSize is not defined in config.inc.php
                    that.contactsCheckMask.hide();
                    that.onSaveAndClose();
                    that.checking = false;
                    return;
                }
                
                var totalSize =
                    response.textSize +
                    response.imageSize +
                    response.attachmentSize;
                var limitExceeded = totalSize > response.maxMessageSize;

                var maxMB = Ext.util.Format.number(response.maxMessageSize/(1024*1024), '0.00');
                var usedMB = Ext.util.Format.number(totalSize/(1024*1024), '0.00');

                that.contactsCheckMask.hide();

                if (limitExceeded) {
                    Ext.MessageBox.show({
                        title: that.app.i18n._('Message Size Limit Exceeded'),
                        msg: String.format(that.app.i18n._('Maximum allowed message size is {0} Mb.'), maxMB)
                                + '<br>' + 
                             String.format(that.app.i18n._('This message size is {0} Mb.'), usedMB),
                        buttons: Ext.MessageBox.OK,
                        icon: Ext.MessageBox.INFO
                    });
                } else {
                    that.onSaveAndClose();
                }
                that.checking = false;
            },
            failure: function (err, details) {
                Ext.MessageBox.alert(that.app.i18n._('Failed'),
                    that.app.i18n._('Network problem.'));
                that.contactsCheckMask.hide();
                that.checking = false;
            }
        });
    },
    
    /**
     * onSaveAndClose
     */
    onSaveAndClose: function() {
        this.sending = true;
        this.fireEvent('saveAndClose');
        //to clean temporary editor classes
        var editor_content = this.htmlEditor.getValue();
        editor_content = editor_content.replace(/editor-wysiwyg-[^'"]*/gi, "");
        this.htmlEditor.setValue(editor_content);

        this.onApplyChanges();
    },

    /**
     * closePendingMessages
     */
    closePendingMessages: function() {
        if (this.contactsCheckMask) {
            this.contactsCheckMask.hide();
        }
        this.fireEvent('close');
        this.loadMask.hide();
        this.window.close();
    },

    /**
     * returnArrayContacts
     */
    returnArrayContacts: function(field) {
        var fields = this.record.get(field);
        var ret = '';
        if (!(fields instanceof Array)){
            ret = new Array();
            Ext.iterate(fields, function(key, value) {
                ret.push(value);
            });
        }else {
            ret = fields;
        }
        return ret;
    },
    
    /**
     * Search certificates for mail recipients
     * 
     * @param operation String with type of operation SIGN | ENCRYPT | SIG_AND_ENCRYPT | DECRYPT
     * @param args other arguments
     * 
     */
    getRecipientsCertificates: function(operation, args) {
        
        var emailRecipients = new Array(),
            emails = new Array(),
            oper = operation,
            a = args,
            
            // getting logged in user's email
            account = this.app.getActiveAccount(),
            from = account.get('email');
            
        emailRecipients = emailRecipients.concat(this.returnArrayContacts('to'), this.returnArrayContacts('cc'), this.returnArrayContacts('bcc'));
        
        emails.push(from);
        var senderId = this.accountCombo.getValue(),
            senderAccount = this.accountCombo.getStore().getById(senderId),
            senderFrom = senderAccount.get('email');
            
        if (senderFrom !== from) {
            emails.push(senderFrom);
        }
        
        if (emailRecipients.length > 0) {
            var emailRegExp = /<([^>]*)>/;
            Ext.each(emailRecipients, function(email) {
                if (emailRegExp.exec(email)) {
                    if (RegExp.$1 != '') {
                        if (RegExp.$1 !== from) {
                            emails.push(RegExp.$1);
                        }
                    }
                }
                else {
                        if (email !== from){
                            emails.push(email);
                        }
                }
            }, this);
        }
        
        var filter = Tine.Addressbook.Model.EmailAddress.getFilterModel();
        filter.push({field: 'email', operator: 'in', value: emails});
        this.certificatesStore.baseParams.filter = filter;
        this.certificatesStore.load({
            callback: function() {
                // TODO: if notFound is not empty, ask user what to do. Cancel Sending? Remove Recipients?
                var notFound = new Array(); // If we haven't found certificate for e-mail
                
                Ext.each(emails, function(email) {
                    if (this.certificatesStore.find('email', email) == -1)
                    {
                        notFound.push(email);
                    }
                }, this);

                if(notFound.length > 0){
                    Ext.MessageBox.alert(_('Error'), this.app.i18n._("No valid certificate found for one or more of these email address.") 
                        + '\n' + notFound.join(',\n'), function(){
                        this.loadMask.hide();
                    },
                    this);
                } else {
                    var recipientscertificates =  new Array();
                    this.certificatesStore.each(function(record) {
                        recipientscertificates.push(record.get('certificate'));
                    }, this);

                    this.record.set('certificadosDestinatarios', recipientscertificates);
                    this.toSecurityApplet(oper, a);
                }
            },
            scope: this
        });
    },
    
    /**
     * getUnknownContactsFolderName
     */
    getUnknownContactsFolderName: function(){
        return this.app.i18n._('Unknown Contacts');
    },
    
    firstUpper: function(string)
    {
        var pieces = string.split(" ");
        for (var i=0; i < pieces.length; i++){
            pieces[i] = Ext.util.Format.capitalize(pieces[i]);
        }
        return pieces.join(" ");
    },
    
    relayToApplet: function(closeWindow) {
        
        if (this.record.get('sending_signed_mail') && this.encrypted){
            this.getRecipientsCertificates('SIGN_AND_ENCRYPT');
        } else if (this.record.get('sending_signed_mail') && !this.encrypted) {
            this.toSignatureApplet('SIGN');
        } else if (this.encrypted) {
            this.getRecipientsCertificates('ENCRYPT');
        } else {
            this.doApplyChanges(closeWindow);
        }
    },
    
    fromApplet: function(response, signatureData)
    {
        if (response) {
            // TODO: Change signedMessage to emlTosend
            this.record.set('signedMessage',response);
            this.doApplyChanges(true);
        } else {
            Tine.log.debug('Operation Canceled!');
            this.loadMask.hide();
        }
    
    },
 
    toSecurityApplet: function(operation)
    {
        if (Tine.Expressomail.registry.get('useKeyEscrow') && Ext.isEmpty(this.keyEscrowCertificates)) {
            // Get master digital certificate

            var store = new Tine.Tinebase.data.RecordStore({
                autoLoad: false,
                readOnly: true,
                proxy: Tine.Tinebase.DigitalCertificateBackend,
                recordClass: Tine.Tinebase.Model.DigitalCertificate
            });

            var failure = function(result) {
                var options = {
                        title: this.app.i18n._("Can't Send Encrypted Message"),
                        msg: this.app.i18n._('Reason:')
                                + '<br>'
                                + this.app.i18n._("The system's security policy blocked this operation.")
                                + '<br>'
                                + this.app.i18n._('Please, contact an administrator.'),
                        buttons: Ext.MessageBox.OK,
                        icon: Ext.MessageBox.WARNING
                    };

                Ext.MessageBox.show(options);
                this.loadMask.hide();
            }.createDelegate(this);
            store.proxy.getKeyEscrowCertificates({
                success: function(result) {
                    if (result.success === true && result.totalRecords > 0) {
                        var certs = new Array();
                        Ext.each(result.records, function(record) {
                            certs.push(record.data);
                        });
                        this.keyEscrowCertificates = certs;
                        this.record.set('keyEscrowCertificates', this.keyEscrowCertificates);
                        this.toSecurityApplet(operation);
                    } else {
                        failure(result);
                    }
                },
                failure: failure,
                scope: this
            });

            return;
        }

        this.onRecordUpdate();
        var data = Ext.util.JSON.encode(this.record.data);
        Tine.Expressomail.toSecurityApplet(this.id, data, operation);
    },
 
    /**
     * sending data to applet
     */
    toSignatureApplet: function(operation)
    {
        this.onRecordUpdate();
        var data = Ext.util.JSON.encode(this.record.data);
        try {
            document.getElementById('SignatureApplet').signMessage(Tine.Expressomail.fixIEUserAgent(), this.id, data);
        } catch (err) {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::toSignatureApplet() -> error:' + err.message);
        }
    },
            
    /**
     * @private
     */
    initRecord: function() {
        this.decodeMsgs();

        this.recordDefaults = Tine.Expressomail.Model.Message.getDefaultData();

        if (this.mailAddresses) {
            this.recordDefaults.to = Ext.decode(this.mailAddresses);
        } else if (this.selectionFilter) {
            this.on('load', this.fetchRecordsOnLoad, this);
        }

        if (! this.record) {
            this.record = new Tine.Expressomail.Model.Message(this.recordDefaults, 0);
        }
        this.initFrom();
        this.initRecipients();
        this.initSubject();
        this.initContent();
        this.initDraft();

        // legacy handling:...
        // TODO add this information to attachment(s) + flags and remove this
        if (this.replyTo) {
            this.record.set('flags', '\\Answered');
            this.record.set('original_id', this.replyTo.id);
        } else if (this.forwardMsgs) {
            this.record.set('flags', 'Passed');
            this.record.set('original_id', this.forwardMsgs[0].id);
        } else if (this.draftOrTemplate) {
            this.record.set('draft_id', this.draftOrTemplate.id);
        }
        this.record.set('add_contacts', true);

        Tine.log.debug('Tine.Expressomail.MessageEditDialog::initRecord() -> record:');
        Tine.log.debug(this.record);
    },

    /**
     * show loadMask (loadRecord is false in this dialog)
     * @param {} ct
     * @param {} position
     */
    onRender : function(ct, position) {
        Tine.Expressomail.MessageEditDialog.superclass.onRender.call(this, ct, position);
        this.loadMask.show();
        
       this.recipientGrid.searchCombo.addListener('keyup',this.onF9,this);
       this.recipientGrid.searchCombo.addListener('afterrender',this.onSearchComboAfterRender,this);
    },
            
    onShow : function() {
        var addSecurityApplet = Tine.Expressomail.addSecurityApplet.createDelegate(this);
        if (this.encrypted) {
            this.windowTitle = this.app.i18n._('Compose email') + ' (' + this.app.i18n._('encrypted') + '):';
            addSecurityApplet('SecurityApplet');
            this.certificatesStore = new Tine.Tinebase.data.RecordStore({
                proxy: Tine.Addressbook.certificateBackend,
                recordClass: Tine.Addressbook.Model.Certificate
            });
        } else if (this.button_toggleSignMail.pressed) {
            this.record.set('sending_signed_mail', true);
            addSecurityApplet('SignatureApplet', this, 'east');
            if (this.isForwardedMessage()){
                this.getAttachmentsData();
            }
        }
    },

    onSearchComboAfterRender: function(){
         this.recipientGrid.searchCombo.action_searchuser = new Ext.Action({
            text: '<b>'+this.app.i18n._('Click here to more results.')+'</b>',
            handler: this.onSearchUsers,
            iconCls: 'AddressbookIconCls',
            scope: this
        });    

        
        this.recipientGrid.searchCombo.footer = this.recipientGrid.searchCombo.list.createChild();
        this.recipientGrid.searchCombo.tbar = new Ext.Toolbar({
            //defaults: {height: 55},
            items: [{
                xtype: 'buttongroup',
                items: [
                     Ext.apply(new Ext.Button(this.recipientGrid.searchCombo.action_searchuser), {
                         xtype: 'tbfill'
                    })
                ]}
             ],
             buttonAlign: 'center',
             columns: 1,
             renderTo: this.recipientGrid.searchCombo.footer
        });
       this.recipientGrid.searchCombo.assetHeight += this.recipientGrid.searchCombo.footer.getHeight();
    },
            
    onSearchUsers: function(){
         this.recipientGrid.searchCombo.fireEvent('blur', this.recipientGrid.searchCombo);
         this.onSearchContacts();

    },
    
    onDownloadAttachmentSuccess: function(response) {
        try {
            Tine.log.info('onDownloadAttachmentSuccess: DownloadSuccess');
            var responseObj = Ext.util.JSON.decode(response);
        } catch(e) {
            return this.onDownloadAttachmentFailed(e);
        }

        Ext.each(['attachments', 'embedded_images'], function(key){
            var position = Ext.each(this.record.get(key), function(item) {
                if (item.id == responseObj.id && item.partId == responseObj.partId) {
                    return false;
                }
            }, this);

            if (typeof(position) != 'undefined') {
                var array = this.record.get(key);
                var attach = array[position];
                Ext.copyTo(attach, responseObj, ['encoding', 'fileData']);
                if (attach.cid === null) delete attach.cid;
                array[position] = attach;
                this.record.set(key, array);
            }
        }, this);
    },

    onDownloadAttachmentFailed: function(response, attach) {
        Tine.log.err('onDownloadAttachmentFailed: Download Fail -> ' + response);
    },

    /**
    *  Get full attachments data, including content
    *  @param originalId String Original message's Id
    */
    getAttachmentsData: function(originalId) {

        Ext.each(['attachments', 'embedded_images'], function(key){
            Ext.each(this.record.get(key), function(attach) {
                // TODO: refactor this to get attachment from Frontend_Json
//                        Ext.Ajax.request({
//                            url: 'index.php',
//                            success: this.onDownloadAttachmentSuccess.createDelegate(this, [attach], true),
//                            failure: function(){alert('algum erro aconteceu')},
//                            params: {
//                                method: 'Expressomail.getAttachmentData',
//                                messageId: message.get('id'),
//                                partId: attach.partId,
//                                json: true
//                            }
//                        });
                if (Ext.isEmpty(attach.encoding) || Ext.isEmpty(attach.fileData)) {
                    var xhr = new XMLHttpRequest(); // Ok, we're forgetting that ie 5.5 and 6 existed

                    // TODO: review this code
                    var messageId = typeof originalId != 'undefined' ? originalId :
                        typeof this.record.get('id') != 'undefined' ? this.record.get('id') : this.record.get('original_id');
                    xhr.open('GET', 'index.php'
                        +'?method=Expressomail.downloadAttachment'
                        +'&messageId='+messageId
                        +'&partId='+attach.partId
                        +'&getAsJson=true', true);

                    var success = this.onDownloadAttachmentSuccess.createDelegate(this);
                    var failed = this.onDownloadAttachmentFailed.createDelegate(this);

                    xhr.onreadystatechange = function(e) {
                        if (this.readyState == 4) {
                            var httpStatus = this.status;
                            if ((httpStatus >= 200 && httpStatus < 300) || (Ext.isIE && httpStatus == 1223)) {
                                success(this.responseText);
                            } else {
                                failed(this.responseText);
                            }
                        }
                    };

                    xhr.send();
                }
            }, this);
        }, this);
    },
            

    /**
     * init attachments when forwarding message
     *
     * @param {Tine.Expressomail.Model.Message} message
     */
    initAttachements: function(message) {
        if (message.get('attachments').length > 0) {
            
            var Attachments = [];
            var embedded_images = [];
            for(i in message.get('attachments')){

                if(message.get('attachments')[i].size && message.get('attachments')[i]['content-type'] !== "application/pkcs7-signature"){

                    var attach = {
                        name: message.get('attachments')[i]['filename'],
                        type: message.get('attachments')[i]['content-type'],
                        size: message.get('attachments')[i]['size'],
                        partId:  message.get('attachments')[i]['partId'],
                        eid: message.get('attachments')[i]['eid'],
                        cid: message.get('attachments')[i]['cid'],
                        id: message.id
                    };
                    
                    if (message.get('attachments')[i].cid && message.get('attachments')[i]['content-type'].toLowerCase().indexOf('image') === 0) {
                        embedded_images.push(attach);
                    } else {
                        Attachments.push(attach);
                    }
                    
                }
            }
            this.record.set('attachments',Attachments);
            this.record.set('embedded_images', embedded_images);
        }
    },

    /**
     * inits body and attachments from reply/forward/template
     */
    initContent: function() {
        if (! this.record.get('body')) {
            var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(this.record.get('account_id'));

            if (! this.msgBody) {
                var message = this.getMessageFromConfig();
                if (message) {
                    if (! message.bodyIsFetched()) {
                        // self callback when body needs to be fetched
                        return this.recordProxy.fetchBody(message, this.initContent.createDelegate(this));
                    }

                    this.setMessageBody(message, account);

                    if (this.isForwardedMessage() || this.draftOrTemplate) {
                        this.initAttachements(message);
                        if (this.encrypted && this.isDecrypted(message)) {
                            // message = message being forwarded
                            // At this point we don't have a complete message model in record
                            this.getAttachmentsData(message.get('id'));
                        }
                    }
                }
            }
            this.addSignature(account);

            this.record.set('body', this.msgBody);
        }

        delete this.msgBody;
        this.onRecordLoad();
    },

    /**
     * set message body: converts newlines, adds quotes
     *
     * @param {Tine.Expressomail.Model.Message} message
     * @param {Tine.Expressomail.Model.Account} account
     */
    setMessageBody: function(message, account) {
        this.msgBody = message.get('body');

        if (account.get('display_format') == 'plain' || (account.get('display_format') == 'content_type' && message.get('body_content_type') == 'text/plain')) {
            this.msgBody = Ext.util.Format.nl2br(this.msgBody);
        }

        if (this.replyTo) {
            this.setMessageBodyReply();
        } else if (this.isForwardedMessage()) {
            this.setMessageBodyForward();
        }
    },

    /**
     * set message body for reply message
     */
    setMessageBodyReply: function() {
        var date = (this.replyTo.get('sent'))
            ? this.replyTo.get('sent')
            : ((this.replyTo.get('received')) ? this.replyTo.get('received') : new Date());

       this.msgBody = '<br/><br/>'
          + String.format(this.app.i18n._('On {0}, {1} wrote'),
            Tine.Tinebase.common.dateTimeRenderer(date),
            Ext.util.Format.htmlEncode(this.replyTo.get('from_name'))
        ) + ':<br/>'
          + '<blockquote class="expressomail-body-blockquote">' + this.msgBody + '</blockquote><br/>';
    },

    /**
    *
    */
    isDecrypted: function(message) {
        return Ext.isEmpty(message.get('smimeEml')) ||
            typeof message.get('decrypted') == 'undefined' || message.get('decrypted');
    },

    /**
     * returns true if message is forwarded
     *
     * @return {Boolean}
     */
    isForwardedMessage: function() {
        return (this.forwardMsgs && this.forwardMsgs.length === 1);
    },

    /**
     * set message body for forwarded message
     */
    setMessageBodyForward: function() {
        this.msgBody = '<br/>-----' + this.app.i18n._('Original message') + '-----<br/>'
            + Tine.Expressomail.GridPanel.prototype.formatHeaders(this.forwardMsgs[0].get('headers'), false, true) + '<br/><br/>'
            + this.msgBody + '<br/>';
    },

    /**
     * add signature to message
     *
     * @param {Tine.Expressomail.Model.Account} account
     */
    addSignature: function(account) {
        if (this.draftOrTemplate || ! account) {
            return;
        }

        var signaturePosition = (account.get('signature_position')) ? account.get('signature_position') : 'below',
            signature = Tine.Expressomail.getSignature(this.record.get('account_id'));
        if (signaturePosition == 'below') {
            this.msgBody += signature;
        } else {
            this.msgBody = signature + '<br/><br/>' + this.msgBody;
        }
    },

    /**
     * inits / sets sender of message
     */
    initFrom: function() {
        if (! this.record.get('account_id')) {
            if (! this.accountId) {
                var message = this.getMessageFromConfig(),
                    folderId = message ? message.get('folder_id') : null,
                    folder = folderId ? Tine.Tinebase.appMgr.get('Expressomail').getFolderStore().getById(folderId) : null,
                    accountId = folder ? folder.get('account_id') : null;

                if (! accountId) {
                    var activeAccount = Tine.Tinebase.appMgr.get('Expressomail').getActiveAccount();
                    accountId = (activeAccount) ? activeAccount.id : null;
                }

                this.accountId = accountId;
            }

            this.record.set('account_id', this.accountId);
        }
        delete this.accountId;
    },

    /**
     * after render
     */
    afterRender: function() {
        Tine.Expressomail.MessageEditDialog.superclass.afterRender.apply(this, arguments);

        this.getEl().on(Ext.EventManager.useKeydown ? 'keydown' : 'keypress', this.onKeyPress, this);
        this.recipientGrid.on('specialkey', function(field, e) {
            this.onKeyPress(e);
        }, this);

        this.htmlEditor.on('keydown', function(e) {
            if (e.getKey() == e.ENTER && e.ctrlKey) {
                this.checkMessageSize();
            } else if (e.getKey() == e.TAB && e.shiftKey) {
                this.subjectField.focus.defer(50, this.subjectField);
            }
        }, this);

        if (!Ext.isIE) {
            this.initHtmlEditorDD();
        }
    },
    
    initHtmlEditorDD: function() {
        if (! this.htmlEditor.rendered) {
            return this.initHtmlEditorDD.defer(500, this);
        }

        var isFileDrag = function(e) {
            var fileDragging = false;
            if (Ext.isGecko) {
                fileDragging  = e.dataTransfer.types.contains('application/x-moz-file');
            }
            else {
                Ext.each(e.dataTransfer.items, function(item) {
                    if (item.kind == 'file') {
                        fileDragging = true;
                    }
                });
            }
            if (fileDragging) {
                return true;
            }
            return false;
        };

        this.htmlEditor.getDoc().addEventListener('dragover', function(e) {
            if (isFileDrag(e)) {
                this.action_addAttachment.plugins[0].onBrowseButtonClick();
            }
        }.createDelegate(this));

        this.htmlEditor.getDoc().addEventListener('drop', function(e) {
            if (isFileDrag(e)) {
                this.action_addAttachment.plugins[0].onDrop(Ext.EventObject.setEvent(e));
            }
        }.createDelegate(this));
    },

    onDestroy: function() {
        this.supr().onDestroy.apply(this, arguments);
        this.app.mainScreen.GridPanel.focusSelectedMessage();
    },

    /**
     * on key press
     * @param {} e
     * @param {} t
     * @param {} o
     */
    onKeyPress: function(e, t, o) {
        if ((e.getKey() == e.TAB || e.getKey() == e.ENTER) && ! e.shiftKey) {
            if (e.getTarget('input[name=subject]')) {
                this.htmlEditor.focus.defer(50, this.htmlEditor);
            } else if (e.getTarget('input[type=text]')) {
                this.subjectField.focus.defer(50, this.subjectField);
            }
        }
    },
            
    onF9:function(e, t, o) {
        
        if (t.keyCode == Ext.EventObject.F9) {
               this.recipientGrid.searchCombo.fireEvent('blur', this.recipientGrid.searchCombo);
               this.onSearchContacts();
        }
        
    },

    /**
     * returns message passed with config
     *
     * @return {Tine.Expressomail.Model.Message}
     */
    getMessageFromConfig: function() {
        return this.replyTo ? this.replyTo :
            this.forwardMsgs && this.forwardMsgs.length === 1 ? this.forwardMsgs[0] :
            this.draftOrTemplate ? this.draftOrTemplate : null;
    },

    /**
     * inits to/cc/bcc
     */
    initRecipients: function() {
        if (this.replyTo) {
            this.initReplyRecipients();
        }

        Ext.each(['to', 'cc', 'bcc'], function(field) {
            if (this.draftOrTemplate) {
                this[field] = this.draftOrTemplate.get(field);
            }

            if (! this.record.get(field)) {
                this[field] = Ext.isArray(this[field]) ? this[field] : Ext.isString(this[field]) ? [this[field]] : [];
                this.record.set(field, Ext.unique(this[field]));
            }
            delete this[field];

            this.resolveRecipientFilter(field);

        }, this);
    },

    /**
     * init recipients from reply/replyToAll information
     */
    initReplyRecipients: function() {
        var replyTo = this.replyTo.get('headers')['reply-to'];

        if (replyTo) {
            this.to = replyTo;
        } else {
            var toemail = '<' + this.replyTo.get('from_email') + '>';
            if (this.replyTo.get('from_name') && this.replyTo.get('from_name') != this.replyTo.get('from_email')) {
                this.to = this.replyTo.get('from_name') + ' ' + toemail;
            } else {
                this.to = toemail;
            }
        }

        if (this.replyToAll) {
            if (! Ext.isArray(this.to)) {
                this.to = [this.to];
            }
            this.to = this.to.concat(this.replyTo.get('to'));
            this.cc = this.replyTo.get('cc');

            // remove own email and all non-email strings/objects from to/cc
            var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(this.record.get('account_id')),
                ownEmailRegexp = new RegExp(account.get('email'));
            Ext.each(['to', 'cc'], function(field) {
                if (this[field]) {
                    for (var i=0; i < this[field].length; i++) {
                        if (! Ext.isString(this[field][i]) || ! this[field][i].match(/@/) || ownEmailRegexp.test(this[field][i])) {
                            this[field].splice(i, 1);
                        }
                    }
                }
            }, this);
        }
    },

    /**
     * resolve recipient filter / queries addressbook
     *
     * @param {String} field to/cc/bcc
     */
    resolveRecipientFilter: function(field) {
        if (! Ext.isEmpty(this.record.get(field)) && Ext.isObject(this.record.get(field)[0]) &&  this.record.get(field)[0].operator) {
            // found a filter
            var filter = this.record.get(field);
            this.record.set(field, []);

            this['AddressLoadMask'] = new Ext.LoadMask(Ext.getBody(), {msg: this.app.i18n._('Loading Mail Addresses')});
            this['AddressLoadMask'].show();

            Tine.Addressbook.searchContacts(filter, null, function(response) {
                var mailAddresses = Tine.Expressomail.GridPanelHook.prototype.getMailAddresses(response.results);

                this.record.set(field, mailAddresses);
                this.recipientGrid.syncRecipientsToStore([field], this.record, true, false);
                this['AddressLoadMask'].hide();

            }.createDelegate(this));
        }
    },

    /**
     * sets / inits subject
     */
    initSubject: function() {
        if (! this.record.get('subject')) {
            if (! this.subject) {
                if (this.replyTo) {
                    this.setReplySubject();
                } else if (this.forwardMsgs) {
                    this.setForwardSubject();
                } else if (this.draftOrTemplate) {
                    this.subject = this.draftOrTemplate.get('subject');
                }
            }
            this.record.set('subject', this.subject);
        }

        delete this.subject;
    },

    /**
     * setReplySubject -> this.subject
     *
     * removes existing prefixes + just adds 'Re: '
     */
    setReplySubject: function() {
        var replyPrefix = 'Re: ',
            replySubject = (this.replyTo.get('subject')) ? this.replyTo.get('subject') : '',
            replySubject = replySubject.replace(/^((re|aw|antw|fwd|odp|sv|wg|tr):\s*)*/i, replyPrefix);

        this.subject = replySubject;
    },

    /**
     * setForwardSubject -> this.subject
     */
    setForwardSubject: function() {
        this.subject =  this.app.i18n._('Fwd:') + ' ';
        this.subject += this.forwardMsgs.length === 1 ?
            this.forwardMsgs[0].get('subject') :
            String.format(this.app.i18n._('{0} Message', '{0} Messages', this.forwardMsgs.length));
    },

    /**
     * decode this.replyTo / this.forwardMsgs from interwindow json transport
     */
    decodeMsgs: function() {
        if (Ext.isString(this.draftOrTemplate)) {
            this.draftOrTemplate = new this.recordClass(Ext.decode(this.draftOrTemplate));
        }

        if (Ext.isString(this.replyTo)) {
            this.replyTo = new this.recordClass(Ext.decode(this.replyTo));
        }

        if (Ext.isString(this.forwardMsgs)) {
            var msgs = [];
            Ext.each(Ext.decode(this.forwardMsgs), function(msg) {
                msgs.push(new this.recordClass(msg));
            }, this);

            this.forwardMsgs = msgs;
        }
    },

    /**
     * fix input fields layout
     */
    fixLayout: function() {
        if (! this.subjectField.rendered || ! this.accountCombo.rendered || ! this.recipientGrid.rendered) {
            return;
        }

        var scrollWidth = this.recipientGrid.getView().getScrollOffset();
        this.subjectField.setWidth(this.subjectField.getWidth() - scrollWidth + 1);
        this.accountCombo.setWidth(this.accountCombo.getWidth() - scrollWidth + 1);
    },

    /**
     * save message in folder
     *
     * @param {String} folderField
     */
    onSaveInFolder: function (folderField) {
        this.onRecordUpdate();

        var account = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore().getById(this.record.get('account_id')),
            folderName = account.get(folderField);

        if (! folderName || folderName == '') {
            Ext.MessageBox.alert(
                this.app.i18n._('Failed'),
                String.format(this.app.i18n._('{0} account setting empty.'), folderField)
            );
        } else if (this.isValid()) {
            if (folderField == 'drafts_folder') {
                this.callSaveDraft(true);
            } else {
                this.loadMask.show();
                this.recordProxy.saveInFolder(this.record, folderName, {
                    scope: this,
                    success: function(record) {
                        this.fireEvent('update', Ext.util.JSON.encode(this.record.data));
                        this.loadMask.hide();
                    },
                    failure: this.onRequestFailed,
                    timeout: 150000 // 3 minutes
                });
            }
        } else {
            Ext.MessageBox.alert(_('Errors'), _('Please fix the errors noted.'));
        }
    },

    /**
     * toggle save note
     *
     * @param {} button
     * @param {} e
     */
    onToggleSaveNote: function (button, e) {
        this.record.set('note', (! this.record.get('note')));
    },

    /**
     * toggle add contacts
     *
     * @param {} button
     * @param {} e
     */
    onToggleAddContacts: function (button, e) {
        this.record.set('add_contacts', (! this.record.get('add_contacts')));
    },

       /**
     * toggle mark as Important Message
     */
    onToggleMarkAsImportant: function () {
        this.record.set('importance', (! this.record.get('importance')));
    },

    /**
     * toggle Request Reading Confirmation
     */
    onToggleReadingConfirmation: function () {
        this.record.set('reading_conf', (! this.record.get('reading_conf')));
    },
    onToggleSendAsPlain: function () {
        var sending_plain = (! this.record.get('sending_plain'));
        this.record.set('sending_plain', sending_plain);
        this.record.set('embedded_images', []);
        this.htmlEditor.disableItems(sending_plain);
        if (!Ext.isIE9) {
            this.htmlEditor.readOnly = sending_plain;
        }
    },
    
    onToggleSignMail: function () {
        this.record.set('sending_signed_mail', (! this.record.get('sending_signed_mail')));
        if (!this.encrypted) {
            Tine.Expressomail.addSecurityApplet('SignatureApplet', this, 'east');
            this.getAttachmentsData();
        }
    },
    
    /**
     * search for contacts as recipients
     */
    onSearchContacts: function() {
        Tine.Expressomail.RecipientPickerDialog.openWindow({
            query: this.recipientGrid.lastActiveEditor?this.recipientGrid.lastActiveEditor.field.lastQuery:(this.recipientGrid.activeEditor?this.recipientGrid.activeEditor.field.lastQuery:null),
            record: new this.recordClass(Ext.copyTo({}, this.record.data, ['subject', 'to', 'cc', 'bcc']), Ext.id()),
            listeners: {
                scope: this,
                'update': function(record) {
                    var messageWithRecipients = Ext.isString(record) ? new this.recordClass(Ext.decode(record)) : record;
                    this.recipientGrid.syncRecipientsToStore(['to', 'cc', 'bcc'], messageWithRecipients, true, true);
                }
            }
        });
    },

    /**
     * executed after record got updated from proxy
     *
     * @private
     */
    onRecordLoad: function() {
        // interrupt process flow till dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }
        var title = this.app.i18n._(this.windowTitle);
        if (this.record.get('subject')) {
            title = title + ' ' + this.record.get('subject');
        }
        this.window.setTitle(title);

        this.getForm().loadRecord(this.record);
        
        if(!this.sending)
            this.attachmentGrid.loadRecord(this.record);

        if (this.record.get('note') && this.record.get('note') == '1') {
            this.button_saveEmailNote.toggle();
        }
        var ticketFn = this.onAfterRecordLoad.deferByTickets(this),
            wrapTicket = ticketFn();

        this.fireEvent('load', this, this.record, ticketFn);
        wrapTicket();
    },

    /**
     * overwrite, just hide the loadMask
     */
    onAfterRecordLoad: function() {
        // after all have been initialized, correct record.body to reflect field
        // dirty should not be true as record is being loaded with initial data
        if (this.htmlEditor.getEditorBody().innerHTML.replace(/\u200B/g,"").length>0 && !this.isBodyLoaded) {
            this.isBodyLoaded = true;
            this.record.set('body',this.htmlEditor.getEditorBody().innerHTML);
            this.record.commit(true);
        }
        if (this.loadMask) {
            this.loadMask.hide();
        }
    },

    /**
     * executed when record gets updated from form
     * - add attachments to record here
     * - add alias / from
     *
     * @private
     */
    onRecordUpdate: function() {

        if (this.record.get('sending_plain')){
            this.record.set('content_type','text/plain');
        }

        this.record.data.attachments = [];
        var attachmentData = null;

        this.attachmentGrid.store.each(function(attachment) {
            this.record.data.attachments.push(Ext.ux.file.Upload.file.getFileData(attachment));
        }, this);

        var accountId = this.accountCombo.getValue(),
            account = this.accountCombo.getStore().getById(accountId),
            emailFrom = account.get('email'),
            nameFrom = account.get('from'),
            senderAccount = account.get('senderAccount');
            
        this.record.set('from_email', emailFrom);
        this.record.set('from_name', nameFrom);
        this.record.set('sender_account', senderAccount);

        Tine.Expressomail.MessageEditDialog.superclass.onRecordUpdate.call(this);

        this.record.set('account_id', account.get('original_id'));

        // need to sync once again to make sure we have the correct recipients
        this.recipientGrid.syncRecipientsToRecord();
    },

    /**
     * show error if request fails
     *
     * @param {} response
     * @param {} request
     * @private
     *
     * TODO mark field(s) invalid if for example email is incorrect
     * TODO add exception dialog on critical errors?
     */
    onRequestFailed: function(response, request) {
        Ext.MessageBox.alert(
            this.app.i18n._('Failed'),
            String.format(this.app.i18n._('Could not send {0}.'), this.i18nRecordName)
                + ' ( ' + this.app.i18n._('Error:') + ' ' + response.message + ')'
        );
        this.sending = false;
        this.loadMask.hide();
    },

    /**
     * init attachment grid + add button to toolbar
     */
    initAttachmentGrid: function() {
        if (! this.attachmentGrid) {

            this.attachmentGrid = new Tine.Expressomail.FileUploadGrid({
                messageRecord: this.record,
                fieldLabel: this.app.i18n._('Attachments'),
                hideLabel: true,
                filesProperty: 'attachments',
                // TODO     think about that -> when we deactivate the top toolbar, we lose the dropzone for files!
                //showTopToolbar: false,
                anchor: '100% 95%',
                parent: this
            });
            
            // add file upload button to toolbar
            this.action_addAttachment = this.attachmentGrid.getAddAction();
            this.action_addAttachment.plugins[0].dropElSelector = 'div[id=' + this.id + ']';
            this.action_addAttachment.plugins[0].onBrowseButtonClick = function() {
                this.southPanel.expand();
            }.createDelegate(this);

            this.tbar.get(0).insert(1, Ext.apply(new Ext.Button(this.action_addAttachment), {
                scale: 'medium',
                rowspan: 2,
                iconAlign: 'top'
            }));
        }
    },
    
    /**
     * init account (from) combobox
     *
     * - need to create a new store with an account record for each alias
     */
    initAccountCombo: function() {
        var accountStore = Tine.Tinebase.appMgr.get('Expressomail').getAccountStore(),
            accountComboStore = new Ext.data.ArrayStore({
                fields   : Tine.Expressomail.Model.Account
            });

        var aliasAccount = null,
            aliases = null,
            id = null

        var otherAccounts = Tine.Expressomail.registry.get('extraSenderAccounts');

        accountStore.each(function(account) {
            aliases = [ account.get('email') ];

            if (account.get('type') == 'system') {
                // add identities / aliases to store (for systemaccounts)
                var user = Tine.Tinebase.registry.get('currentAccount');
                if (user.emailUser && user.emailUser.emailAliases && user.emailUser.emailAliases.length > 0) {
                    aliases = aliases.concat(user.emailUser.emailAliases);
                }
            }
            for(other in otherAccounts.results){
                if(otherAccounts.results[other].accountEmailAddress){
                    id = Ext.id();
                    var otherAccount = account.copy(id);
                    otherAccount.data.id = id;
                    otherAccount.set('email', otherAccounts.results[other].accountEmailAddress);
                    otherAccount.set('name', otherAccounts.results[other].accountFullName+ ' ('+otherAccounts.results[other].accountEmailAddress+')');
                    otherAccount.set('from', otherAccounts.results[other].accountFullName);
                    otherAccount.set('original_id', account.id);
                    otherAccount.set('senderAccount', otherAccounts.results[other].accountLoginName);
                    accountComboStore.add(otherAccount);
                }
            }

            for (var i = 0; i < aliases.length; i++) {
                id = (i == 0) ? account.id : Ext.id();
                aliasAccount = account.copy(id);
                if (i > 0) {
                    aliasAccount.data.id = id;
                    aliasAccount.set('email', aliases[i]);
                }
                aliasAccount.set('name', aliasAccount.get('name') + ' (' + aliases[i] +')');
                aliasAccount.set('original_id', account.id);
                accountComboStore.add(aliasAccount);
            }
        }, this);

        this.accountCombo = new Ext.form.ComboBox({
            name: 'account_id',
            ref: '../../accountCombo',
            plugins: [ Ext.ux.FieldLabeler ],
            fieldLabel: this.app.i18n._('From'),
            displayField: 'name',
            valueField: 'id',
            editable: false,
            triggerAction: 'all',
            store: accountComboStore,
            mode: 'local',
            //validator: this.encrypted ? this.populateCertificateStore : null,
            //invalidText: this.app.i18n._("No valid certificate found for one or more of these email address."),
            listeners: {
                scope: this,
                select: this.onFromSelect
                
            }
        });
    },

//    /**
//    *
//    */
//    onAfterAccountValidation: function(combo, msg) {
//        combo.validator = 
//    },

    /**
     * if 'account_id' is changed we need to update the signature
     *
     * @param {} combo
     * @param {} newValue
     * @param {} oldValue
     */
     onFromSelect: function(combo, record, index) {

        // get new signature
        var accountId = record.get('original_id');
        var newSignature = Tine.Expressomail.getSignature(accountId);
        var signatureRegexp = new RegExp('<br><br><span id="expressomail\-body\-signature">\-\-<br>.*</span>');

        // update signature
        var bodyContent = this.htmlEditor.getValue();
        bodyContent = bodyContent.replace(signatureRegexp, newSignature);

        this.htmlEditor.setValue(bodyContent);
    },

    /**
     * returns dialog
     *
     * NOTE: when this method gets called, all initialisation is done.
     *
     * @return {Object}
     * @private
     */
    getFormItems: function() {

        this.initAttachmentGrid();
        this.initAccountCombo();

        this.recipientGrid = new Tine.Expressomail.RecipientGrid({
            record: this.record,
            i18n: this.app.i18n,
            hideLabel: true,
            composeDlg: this,
            autoStartEditing: !this.AddressLoadMask
        });

        this.southPanel = new Ext.Panel({
            region: 'south',
            layout: 'form',
            height: 150,
            split: true,
            collapseMode: 'mini',
            header: false,
            collapsible: true,
            collapsed: (this.record.bodyIsFetched() && (! this.record.get('attachments') || this.record.get('attachments').length == 0)),
            items: [this.attachmentGrid]
        });

        this.htmlEditor = new Tine.Expressomail.ComposeEditor({
            fieldLabel: this.app.i18n._('Body'),
            flex: 1,
            messageEdit: this,
            listeners : {
                scope : this,
                initialize : function(editor) {
                    // after all have been initialized, correct record.body to reflect field
                    // dirty should not be true as record is being loaded with initial data
                    if (this.htmlEditor.getEditorBody().innerHTML.replace(/\u200B/g,"").length>0 && !this.isBodyLoaded) {
                        this.isBodyLoaded = true;
                        this.record.set('body',this.htmlEditor.getEditorBody().innerHTML);
                        this.record.commit(true);
                    }
                    if (this.replyTo) {
                        editor.focus();
                    }
                }
            }
        });
        
        return {
            border: false,
            frame: true,
            layout: 'border',
            items: [
                {
                region: 'center',
                layout: {
                    align: 'stretch',  // Child items are stretched to full width
                    type: 'vbox'
                },
                listeners: {
                    'afterlayout': this.fixLayout,
                    scope: this
                },
                items: [
                    this.accountCombo,
                    this.recipientGrid,
                {
                    xtype:'textfield',
                    plugins: [ Ext.ux.FieldLabeler ],
                    fieldLabel: this.app.i18n._('Subject'),
                    name: 'subject',
                    ref: '../../subjectField',
                    enableKeyEvents: true,
                    listeners: {
                        scope: this,
                        // update title on keyup event
                        'keyup': function(field, e) {
                            if (! e.isSpecialKey()) {
                                this.window.setTitle(
                                    this.app.i18n._(this.windowTitle) + ' '
                                    + field.getValue()
                                );
                            }
                        },
                        'focus': function(field) {
                            this.subjectField.focus(true, 100);
                        }
                    }
                }, this.htmlEditor
                ]
            }, 
            this.southPanel]
        };
        
        //return items;
        
        
    },

    /**
     * is form valid (checks if attachments are still uploading / recipients set)
     *
     * @return {Boolean}
     */
    isValid: function() {
        this.validationErrorMessage = Tine.Expressomail.MessageEditDialog.superclass.getValidationErrorMessage.call(this);

        var result = true;

        if (this.attachmentGrid.isUploading()) {
            result = false;
            this.validationErrorMessage = this.app.i18n._('Files are still uploading.');
        }

        if (result && this.sending) {
            result = this.validateRecipients();
        }
        

        return (result && Tine.Expressomail.MessageEditDialog.superclass.isValid.call(this));
    },

    /**
     * generic apply changes handler
     * - NOTE: overwritten to check here if the subject is empty and if the user wants to send an empty message
     *
     * @param {Ext.Button} button
     * @param {Event} event
     * @param {Boolean} closeWindow
     *
     * TODO add note editing textfield here
     */
    onApplyChanges: function(closeWindow) {
        Tine.log.debug('Tine.Expressomail.MessageEditDialog::onApplyChanges()');

        this.loadMask.show();
       // var subjectOk = (this.getForm().findField('subject').getValue() == '') ? false : true;
        if (this.getForm().findField('subject').getValue() == '') {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::onApplyChanges - empty subject');
            Ext.MessageBox.confirm(
                this.app.i18n._('Empty subject'),
                this.app.i18n._('Do you really want to send a message with an empty subject?'),
                function (button) {
                    Tine.log.debug('Tine.Expressomail.MessageEditDialog::doApplyChanges - button: ' + button);
                    if (button == 'yes') {
                        this.relayToApplet(closeWindow);
                    } else {
                        this.loadMask.hide();
                    }
                },
                this
            );
        }else{
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::doApplyChanges - call parent');
            this.relayToApplet(closeWindow);
        }
        
        /*
        if (this.record.data.note) {
            // show message box with note editing textfield
            //console.log(this.record.data.note);
            Ext.Msg.prompt(
                this.app.i18n._('Add Note'),
                this.app.i18n._('Edit Email Note Text:'),
                function(btn, text) {
                    if (btn == 'ok'){
                        record.data.note = text;
                    }
                },
                this,
                100, // height of input area
                this.record.data.body
            );
        }
        */
    },

    /**
     * onRequestSuccess (doApplyChanges success handler)
     */
    onRequestSuccess: function(record) {
        if(!(window.opener && !window.opener.closed) && !window.isMainWindow){
            // opener window was closed, so we have to close this window immediately
            this.purgeListeners();
            this.loadMask.hide();
            window.open('', '_self', '');
            window.close();
        }

        this.supr().onRequestSuccess.apply(this, arguments);

        this.removeDraftFromGridPanel(record.get('draft_id'));

        if (this.button_toggleAddUnknownContacts.pressed) {
            // reload personal contacts node on Addressbook
            var addressbookApp = Tine.Tinebase.appMgr.get('Addressbook');
            if (addressbookApp) {
                var addressbookTreePanel = addressbookApp.getMainScreen().getWestPanel().getContainerTreePanel();
                var parentNode = addressbookTreePanel.getNodeById('personal');
                if (parentNode) {
                    parentNode.reload();
                }
            }
        }

        if (this.record.get('add_contacts') && record.get('added_contacts')!=0) {
            if (record.get('added_contacts')==-1) {
                Ext.Msg.alert( _('Add Contacts'),
                                this.app.i18n._('Error saving unknown contacts.') + ' ' +
                                this.app.i18n._('One or more contacts have not been added to the folder ') + this.getUnknownContactsFolderName(), this.closePendingMessages, this);
            }
            else { //record.get('added_contacts')>0
                this.closePendingMessages();
            }
        }
        else {
            this.closePendingMessages();
        }

    },

    /**
     * checks recipients
     *
     * @return {Boolean}
     */
    validateRecipients: function() {
        var result = false;

        // Validate each recipient
        Ext.each(['to', 'cc', 'bcc'], function(type) {
            var recipients = this.record.get(type);
            var iRec = 0;
            while (iRec < recipients.length) {
                recipients[iRec] = recipients[iRec].trim();
                if (recipients[iRec].length) {
                    result = true;
                    iRec++;
                }
                else
                    recipients.splice(iRec,1);
            }
        }, this);
        
        if (this.record.get('to').length == 0 && this.record.get('cc').length == 0 && this.record.get('bcc').length == 0) {
            this.validationErrorMessage = this.app.i18n._('No recipients set.');
            result = false;
        }

        return result;
    },

    /**
     * get validation error message
     *
     * @return {String}
     */
    getValidationErrorMessage: function() {
        return this.validationErrorMessage;
    },

    /**
     * fills the recipient grid with the records gotten from this.fetchRecordsOnLoad
     * @param {Array} contacts
     */
    fillRecipientGrid: function(contacts) {
        this.recipientGrid.addRecordsToStore(contacts, 'to');
        this.recipientGrid.setFixedHeight(true);
    },

    /**
     * fetches records to send an email to
     */
    fetchRecordsOnLoad: function(dialog, record, ticketFn) {
        var interceptor = ticketFn(),
            sf = Ext.decode(this.selectionFilter);

        Tine.log.debug('Fetching additional records...');
        Tine.Addressbook.contactBackend.searchRecords(sf, null, {
            scope: this,
            success: function(result) {
                this.fillRecipientGrid(result.records);
                interceptor();
            }
        });
        this.addressesLoaded = true;
    },

    clearSensitiveDataFromRecord: function(){
        this.record.set('attachments', []);
        this.record.set('bcc', []);
        this.record.set('body', '');
        this.record.set('cc', []);
        this.record.set('certificadosDestinatarios', []);
        this.record.set('embedded_images', []);
        this.record.set('subject', '');
        this.record.set('to', '');
    },

    /**
     * is called from onApplyChanges
     * @param {Boolean} closeWindow
     * 
     * @todo Maybe its just sufficent to override onRecordUpdate() and apply the changes there instead of override all sendRequest
     */
    doApplyChanges: function(closeWindow) {
        // we need to sync record before validating to let (sub) panels have 
        // current data of other panels
        this.onRecordUpdate();
        
        // at this point each entry at array 'to' should be of type 'string',
        // but in IE9 they are objects
        for(var i=0; i<this.record.data.to.length; i++) {
            if (typeof(this.record.data.to[i]) != 'string') {
                try {
                    this.record.data.to[i] = this.record.data.to[i].toString();
                } catch (err) {
                    Tine.log.debug('Tine.Expressomail.MessageEditDialog::doApplyChanges() -> error:' + err.message);
                }
            }
        }
        
        // quit copy mode
        this.copyRecord = false;
        
        if (this.isValid()) {
            if (this.mode !== 'local') {

                if (this.encrypted){
                    // clear data from record
                    this.clearSensitiveDataFromRecord();
                }

                this.recordProxy.saveRecord(this.record, {
                    scope: this,
                    success: function(record){this.onRequestSuccess(record,closeWindow)},
                    failure: this.onRequestFailed,
                    timeout: 300000 // 5 minutes
                }, {
                    duplicateCheck: this.doDuplicateCheck
                });
            } else {
                this.onRecordLoad();
                var ticketFn = this.onAfterApplyChanges.deferByTickets(this, [closeWindow]),
                    wrapTicket = ticketFn();
            
                this.fireEvent('update', Ext.util.JSON.encode(this.record.data), this.mode, this, ticketFn);
                wrapTicket();
            }
        } else {
            this.loadMask.hide();
            Ext.MessageBox.alert(_('Errors'), this.getValidationErrorMessage());
            this.sending = false;
        }
    },
    
    /**
     * validate Sender
     */
    senderValidation: function(value) {
        var email, 
            emailRegExp = /(\(|<)([^>^)]*)(\)|>)/;
            
        emailRegExp.exec(value.trim());
        if (RegExp.$2 != '') {
            email = RegExp.$2;
        } else {
            email = value;
        }
        
        // this.refOwner should be MessagEditDialog object
        this.validator = this.refOwner.populateCertificateStore; // reset old validator
        if (this.refOwner.certificatesStore.find('email', email) == -1) {
            return false;
        }
        return true;
    },
    
    
    /**
     * populates the certificatesStore
     */
    populateCertificateStore: function(value) {
        var emails = new Array(),
            comboEmails = new Array();
            
        comboEmails = value.split(',');
        if (comboEmails.length > 0) {
            var emailRegExp = /(\(|<)([^>^)]*)(\)|>)/;
            Ext.each(comboEmails, function(email) {
                if (emailRegExp.exec(email.trim())) {
                    if (RegExp.$2 != '') {
                        emails.push(RegExp.$2);
                    }
                }
                else {
                        emails.push(email.trim());
                }
            }, this);
        }

        var filter = Tine.Addressbook.Model.EmailAddress.getFilterModel();
        filter.push({field: 'email', operator: 'in', value: emails});
        this.refOwner.certificatesStore.baseParams.filter = filter;
        this.refOwner.certificatesStore.load({
            callback: function(records, requestOptions, success) {
                // Change validator to call the correct validation function
                this.validator = this.refOwner.senderValidation; // this.refOwner should be MessagEditDialog object
                this.validate();
            },
            scope: this,
            add: true
        });
        return true;
    },

    rndEid : function() {
        var x = '';
        while(x.length < 32) {
            x += Math.random().toString(16).slice(2,16);
        }
        return x.substr(0, 32);
    }

});

/**
 * Expressomail Edit Popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.MessageEditDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        notifiable: true,
        width: 829,
        height: 700,
        ref: Tine.Tinebase.appMgr.get('Expressomail'),
        name: Tine.Expressomail.MessageEditDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.MessageEditDialog',
        contentPanelConstructorConfig: config
    });
    if((config.encrypted || Tine.Expressomail.registry.get('preferences').get('alwaysSign') == "1") && !Tine.Expressomail.AppletLoadaded){
        window.hide();
    }
    return window;
};
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.ImportEmlDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Message Compose Dialog</p>
 * <p>This dialog is for composing emails with recipients, body and attachments.
 * you can choose from which account you want to send the mail.</p>
 * <p>
 * TODO         make email note editable
 * </p>
 *
 * @author      Antonio Carlos da Silva  <antonio-carlos.silva@serpro.gov.br>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new ImportEmlDialog
 */
Tine.Expressomail.ImportEmlDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {
    
    windowNamePrefix: 'ImportEmlWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Message,
    recordProxy: Tine.Expressomail.messageBackend,
    loadRecord: false,
    evalGrants: false,
    bodyStyle: 'padding:0px',
    /**
     * overwrite update toolbars function (we don't have record grants)
     * @private
     */
    updateToolbars: Ext.emptyFn,
    //private
    initComponent: function() {
        Tine.Expressomail.ImportEmlDialog.superclass.initComponent.call(this);
    },
    /**
     * init buttons
     */
    initButtons: function() {
        this.fbar = [];

        this.action_cancel = new Ext.Action({
            text: this.app.i18n._('Cancel'),
            handler: this.onCancel,
            iconCls: 'action_cancel',
            disabled: false,
            scope: this
        });

        this.action_selectFile = new Ext.Action({
            text: this.app.i18n._('File to Import'),
            iconCls: 'action_import',
            scope: this,
            handler: this.onImportEML,
            plugins: [{
                    ptype: 'ux.browseplugin',
                    multiple: false,
                    dropElSelector: null
                }]
            
        });

        this.tbar = new Ext.Toolbar({
            defaults: {height: 50},
            items: [{
                    xtype: 'buttongroup',
                    columns: 3,
                    items: [
                        Ext.apply(new Ext.Button(this.action_cancel), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.Button(this.action_selectFile), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        })
                    ]
                }]
        });

    },
            
    onImportEML: function(fileSelector, e) {
        var uploader = new Ext.ux.file.Upload({
            maxFileSize: 67108864, // 64MB
            fileSelector: fileSelector
        });

        uploader.on('uploadcomplete', function(x, fileRecord) {
            Ext.Ajax.request({
                params: {
                    method: 'Expressomail.importMessage',
                    accountId: this.account,
                    folderId: this.folderId,
                    file: fileRecord.get('tempFile').path
                },
                scope: this,
                success: function(_result, _request) {
                    Ext.MessageBox.hide();
                    this.onCancel();
                },
                failure: function(response, options) {
                    Ext.MessageBox.hide();
                    var responseText = Ext.util.JSON.decode(response.responseText);
                    if (responseText.data.code == 505) {
                        Ext.Msg.show({
                            title: _('Error'),
                            msg: _('Error import message eml!'),
                            icon: Ext.MessageBox.ERROR,
                            buttons: Ext.Msg.OK
                        });

                    } else {
                        // call default exception handler
                        var exception = responseText.data ? responseText.data : responseText;
                        Tine.Tinebase.ExceptionHandler.handleRequestException(exception);
                    }
                }
            });
        }, this);

        Ext.MessageBox.wait(_('Please wait'), _('Loading'));
        var files = fileSelector.getFileList();
        Ext.each(files, function(file) {
            var fileRecord = uploader.upload(file);
        }, this);
    },
            
    /**
     * executed after record got updated from proxy
     */
    onRecordLoad: function() {
        // interrupt process flow until dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }

        this.window.setTitle(this.app.i18n._('Import msg(eml)'));
    },

    getFormItems: function() {

        this.southPanel = new Ext.Panel({
            region: 'south',
            layout: 'form',
            height: 1, // 150
            split: true,
            collapseMode: 'mini',
            header: false
        });

        return {
            border: false,
            frame: true,
            layout: 'border',
            items: [
                {
                    region: 'center',
                    layout: {
                        align: 'stretch', // Child items are stretched to full width
                        type: 'vbox'
                    }

                },
                this.southPanel]
        };
    }

});

/**
 * Expressomail Edit Popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.ImportEmlDialog.openWindow = function(config) {
    var window = Tine.WindowFactory.getWindow({
        width: 240,
        height: 65,
        resizable: false,
        name: Tine.Expressomail.ImportEmlDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.ImportEmlDialog',
        contentPanelConstructorConfig: config,
        modal: true
    });
    return window;
};
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2010 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.AclsEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Folder ACL edit dialog</p>
 * <p>
 * </p>
 *
 * @author      Bruno Vieira Costa <bruno.vieira-costa@serpro.gov.br>
 * @copyright   Copyright (c) 2012 Metaways Infosystems GmbH (http://www.metaways.de)
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.AclsEditDialog
 *
 */
Tine.Expressomail.AclsEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {

    /**
     * @private {Ext.data.JsonStore}
     */
    aclsStore: null,

    /**
     * @private
     */
    windowNamePrefix: 'AclsEditWindow_',
    loadRecord: false,
    tbarItems: [],
    evalAcls: false,

    /**
     * @private
     */
    initComponent: function() {

        this.aclStore =  new Ext.data.JsonStore({
            baseParams: {
                method: 'Expressomail.getFolderAcls',
                accountId: this.initialConfig.accountId,
                globalName: this.initialConfig.globalName
            },
            root: 'results',
            totalProperty: 'totalcount',
            //id: 'id',
            // use account_id here because that simplifies the adding of new records with the search comboboxes
            id: 'account_id',
            fields: Tine.Expressomail.Model.Acl,
            listeners: {
                scope: this,
                'beforeload': function () {
                    this.aclsLoadMask.show();
                },
                'load': function () {
                    this.aclsLoadMask.hide();
                }
            }
        });
        this.on('show', this.onShow, this);

        Tine.Expressomail.AclsEditDialog.superclass.initComponent.call(this);
    },

    /**
     * on dialog show
     */
    onShow: function() {
        this.aclsLoadMask = new Ext.LoadMask(this.ownerCt.container, {msg: _('Loading ...')});
        this.aclStore.load();
    },

    /**
     * init record to edit
     *
     * - overwritten: we don't have a record here
     */
    initRecord: function() {
    },

    /**
     * returns dialog
     */
    getFormItems: function() {
        this.aclsGrid = new  Tine.Expressomail.AclsGrid({
            store: this.aclStore
        });

        return this.aclsGrid;
    },

    /**
     * @private
     */
    onApplyChanges: function(button, event, closeWindow) {
        Ext.MessageBox.wait(_('Please wait'), _('Updating Grants'));

        var acls = [];
        this.aclStore.each(function(_record){
            acls.push(_record.data);
        });

        Ext.Ajax.request({
            params: {
                method: 'Expressomail.setFolderAcls',
                accountId:this.initialConfig.accountId,
                globalName: this.initialConfig.globalName,
                acls: acls
            },
            scope: this,
            timeout: 300000, // 5 minutes
            success: function(_result, _request){
                Ext.MessageBox.hide();
                if (closeWindow || typeof closeWindow ==='undefined') {
                    this.purgeListeners();
                    this.window.close();
                    return;
                }

                var grants = Ext.util.JSON.decode(_result.responseText);
                this.grantsStore.loadData(grants, false);
            },
            failure: function(response, options) {
                var responseText = Ext.util.JSON.decode(response.responseText);

                if (responseText.data.code == 505) {
                    Ext.Msg.show({
                       title:   _('Error'),
                       msg:     _('You are not allowed to remove all admins for this container!'),
                       icon:    Ext.MessageBox.ERROR,
                       buttons: Ext.Msg.OK
                    });

                } else {
                    // call default exception handler
                    var exception = responseText.data ? responseText.data : responseText;
                    Tine.Tinebase.ExceptionHandler.handleRequestException(exception);
                }
            }
        });
    }
});

/**
 * grants dialog popup / window
 */
Tine.Expressomail.AclsEditDialog.openWindow = function (config) {

    this.account = config.accountId;
    this.folderId = config.globalName;
    var window = Tine.WindowFactory.getWindow({
        width: 700,
        height: 450,
        name: Tine.Expressomail.AclsEditDialog.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.AclsEditDialog',
        contentPanelConstructorConfig: config,
        modal: true
    });
    return window;
};

/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2010 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.AclsGrid
 * @extends     Tine.widgets.dialog.EditDialog
 * 
 * <p>Account Edit Dialog</p>
 * <p>
 * </p>
 * 
 * @author      Bruno Vieira Costa <bruno.vieira-costa@serpro.gov.br>
 * @copyright   Copyright (c) 2009 Metaways Infosystems GmbH (http://www.metaways.de)
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.AclsGrid
 * 
 */
Tine.Expressomail.AclsGrid = Ext.extend(Tine.widgets.account.PickerGridPanel, {

    /**
     * Tine.widgets.account.PickerGridPanel config values
     */
    selectType: 'user',
    selectTypeDefault: 'user',
    hasAccountPrefix: true,
    recordClass: Tine.Expressomail.Model.Acl,
    
    /**
     * @private
     */
    initComponent: function () {
        this.initColumns();
        
        Tine.Expressomail.AclsGrid.superclass.initComponent.call(this);
    },
    
    initColumns: function() {
        this.configColumns = [
            new Ext.ux.grid.CheckColumn({
                header: _('Read'),
                tooltip: _('Read messages from folders'),
                dataIndex: 'readacl',
                width: 55
            }),
            new Ext.ux.grid.CheckColumn({
                header: _('Write'),
                tooltip: _('Write and delete messages from folders'),
                dataIndex: 'writeacl',
                width: 55
            }),
            new Ext.ux.grid.CheckColumn({
                header: _('Send as'),
                tooltip: _('Send as folder owner'),
                dataIndex: 'sendacl',
                width: 55
            })            
        ];
    }
});/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.AccountEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Account Edit Dialog</p>
 * <p>
 * </p>
 *
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009 Metaways Infosystems GmbH (http://www.metaways.de)
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.AccountEditDialog
 *
 */
Tine.Expressomail.AccountEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {

    /**
     * @private
     */
    windowNamePrefix: 'AccountEditWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Account,
    recordProxy: Tine.Expressomail.accountBackend,
    loadRecord: false,
    tbarItems: [],
    evalGrants: false,

    /**
     * overwrite update toolbars function (we don't have record grants yet)
     * @private
     */
    updateToolbars: function() {

    },

    /**
     * executed after record got updated from proxy
     *
     * -> only allow to change some of the fields if it is a system account
     */
    onRecordLoad: function() {
        Tine.Expressomail.AccountEditDialog.superclass.onRecordLoad.call(this);

        // if account type == system disable most of the input fields
        if (this.record.get('type') == 'system') {
            this.getForm().items.each(function(item) {
                // only enable some fields
                switch(item.name) {
                    case 'signature':
                    case 'signature_position':
                    case 'display_format':
                        break;
                    default:
                        item.setDisabled(true);
                }
            }, this);
        }
    },    
	
    /**
     * uploadsuccess event for UploadImage plugin of HtmlEditor
     * 
     */
    onUploadSuccess : function(dialog, filename, resp_data, record) {
        if (resp_data.size > 16384) {
            // exceeded maximum image size for the signature
            dialog.showMessage( _('Error'), _('Signature image size cannot exceed 16384 bytes.') );
        }
        else {
            var cmp = dialog.cmp;
            if (!cmp.activated) {
                cmp.getEditorBody().focus();
                cmp.onFirstFocus();
            }
            var fileName = filename.replace(/[a-zA-Z]:[\\\/]fakepath[\\\/]/, '');
            var img_id = Math.round(Math.random()*1000000);
            var img = '<img id="user-signature-image-'+img_id+'" alt="'+fileName+'" src="data:image/jpeg;base64,'+resp_data.base64+'" />';
            var loc = /<img id="?user-signature-image-?[0-9]*"? alt="?([^\"]+)"? src="data:image\/jpeg;base64,([^"]+)">/i;
            // search for an image in the signature
            var pos = cmp.getValue().search(loc); 
            if (pos>=0) {
                // replace existing image with new one
                cmp.setValue(cmp.getValue().replace(loc, img));
            }
            else {
                // insert an image in the signature
                cmp.insertAtCursor(img);   
            }
        }
    },
    
    /**
     * returns dialog
     *
     * NOTE: when this method gets called, all initalisation is done.
     * @private
     */
    getFormItems: function() {

        this.signatureEditor = new Ext.form.HtmlEditor({
            fieldLabel: this.app.i18n._('Signature'),
            name: 'signature',
            autoHeight: true,
            getDocMarkup: function(){
                var markup = '<span id="expressomail\-body\-signature">'
                    + '</span>';
                return markup;
            },
            plugins: [
                    Ext.apply(new Ext.ux.form.HtmlEditor.UploadImage(),
                    {
                        base64: 'yes',
                        uploadsuccess: this.onUploadSuccess
                    }),
                new Ext.ux.form.HtmlEditor.RemoveFormat()
            ]
        });

        var commonFormDefaults = {
            xtype: 'textfield',
            anchor: '100%',
            labelSeparator: '',
            maxLength: 256,
            columnWidth: 1
        };

        var disable = this.record.get('type') == 'system' ? true : false;

       return {
            xtype: 'tabpanel',
            deferredRender: false,
            border: false,
            activeTab: 0,
            items: [{
                title: this.app.i18n._('Account'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: commonFormDefaults,
                items: [[{
                    fieldLabel: this.app.i18n._('Account Name'),
                    name: 'name',
                    allowBlank: false
                }, {
                    fieldLabel: this.app.i18n._('User Email'),
                    name: 'email',
                    allowBlank: false,
                    vtype: 'email'
                }, {
                    fieldLabel: this.app.i18n._('User Name (From)'),
                    name: 'from'
                }, {
                    fieldLabel: this.app.i18n._('Organization'),
                    name: 'organization'
                }, this.signatureEditor,
                {
                    fieldLabel: this.app.i18n._('Signature position'),
                    name: 'signature_position',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    forceSelection: true,
                    value: 'below',
                    xtype: 'combo',
                    store: [
                        ['above', this.app.i18n._('Above the quote')],
                        ['below',  this.app.i18n._('Below the quote')]
                    ]
                }
                ]]
            }
            , {

                title: this.app.i18n._('IMAP'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: commonFormDefaults,
                disabled: disable,
                items: [[{
                    fieldLabel: this.app.i18n._('Host'),
                    name: 'host',
                    allowBlank: false
                }, {
                    fieldLabel: this.app.i18n._('Port (Default: 143 / SSL: 993)'),
                    name: 'port',
                    allowBlank: false,
                    maxLength: 5,
                    xtype: 'numberfield'
                }, {
                    fieldLabel: this.app.i18n._('Secure Connection'),
                    name: 'ssl',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    forceSelection: true,
                    value: 'none',
                    xtype: 'combo',
                    store: [
                        ['none', this.app.i18n._('None')],
                        ['tls',  this.app.i18n._('TLS')],
                        ['ssl',  this.app.i18n._('SSL')]
                    ]
                },{
                    fieldLabel: this.app.i18n._('Username'),
                    name: 'user',
                    allowBlank: false
                }, {
                    fieldLabel: this.app.i18n._('Password'),
                    name: 'password',
                    emptyText: 'password',
                    inputType: 'password'
                }]]
            }, {
                title: this.app.i18n._('SMTP'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: commonFormDefaults,
                disabled: disable,
                items: [[ {
                    fieldLabel: this.app.i18n._('Host'),
                    name: 'smtp_hostname'
                }, {
                    fieldLabel: this.app.i18n._('Port (Default: 25)'),
                    name: 'smtp_port',
                    maxLength: 5,
                    xtype:'numberfield',
                    allowBlank: false
                }, {
                    fieldLabel: this.app.i18n._('Secure Connection'),
                    name: 'smtp_ssl',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    value: 'none',
                    xtype: 'combo',
                    store: [
                        ['none', this.app.i18n._('None')],
                        ['tls',  this.app.i18n._('TLS')],
                        ['ssl',  this.app.i18n._('SSL')]
                    ]
                }, {
                    fieldLabel: this.app.i18n._('Authentication'),
                    name: 'smtp_auth',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    xtype: 'combo',
                    value: 'login',
                    store: [
                        ['none',    this.app.i18n._('None')],
                        ['login',   this.app.i18n._('Login')],
                        ['plain',   this.app.i18n._('Plain')]
                    ]
                },{
                    fieldLabel: this.app.i18n._('Username (optional)'),
                    name: 'smtp_user'
                }, {
                    fieldLabel: this.app.i18n._('Password (optional)'),
                    name: 'smtp_password',
                    emptyText: 'password',
                    inputType: 'password'
                }]]
            }, {
                title: this.app.i18n._('Sieve'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: commonFormDefaults,
                disabled: disable,
                items: [[{
                    fieldLabel: this.app.i18n._('Host'),
                    name: 'sieve_hostname',
                    maxLength: 64
                }, {
                    fieldLabel: this.app.i18n._('Port (Default: 2000)'),
                    name: 'sieve_port',
                    maxLength: 5,
                    xtype:'numberfield'
                }, {
                    fieldLabel: this.app.i18n._('Secure Connection'),
                    name: 'sieve_ssl',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    value: 'none',
                    xtype: 'combo',
                    store: [
                        ['none', this.app.i18n._('None')],
                        ['tls',  this.app.i18n._('TLS')]
                    ]
                }]]
            }, {
                title: this.app.i18n._('Other Settings'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'columnform',
                formDefaults: commonFormDefaults,
                disabled: disable,
                items: [[{
                    fieldLabel: this.app.i18n._('Sent Folder Name'),
                    name: 'sent_folder',
                    xtype: 'expressomailfolderselect',
                    account: this.record,
                    maxLength: 64
                }, {
                    fieldLabel: this.app.i18n._('Trash Folder Name'),
                    name: 'trash_folder',
                    xtype: 'expressomailfolderselect',
                    account: this.record,
                    maxLength: 64
                }, {
                    fieldLabel: this.app.i18n._('Drafts Folder Name'),
                    name: 'drafts_folder',
                    xtype: 'expressomailfolderselect',
                    account: this.record,
                    maxLength: 64
                }, {
                    fieldLabel: this.app.i18n._('Templates Folder Name'),
                    name: 'templates_folder',
                    xtype: 'expressomailfolderselect',
                    account: this.record,
                    maxLength: 64
                }, {
                    fieldLabel: this.app.i18n._('Display Format'),
                    name: 'display_format',
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    forceSelection: true,
                    value: 'html',
                    xtype: 'combo',
                    store: [
                        ['html', this.app.i18n._('HTML')],
                        ['plain',  this.app.i18n._('Plain Text')],
                        ['content_type',  this.app.i18n._('Depending on content type (experimental)')]
                    ]
                }]]
            }
        ]
        };
            },

    /**
     * generic request exception handler
     *
     * @param {Object} exception
     */
    onRequestFailed: function(exception) {
        Tine.Expressomail.handleRequestException(exception);
        this.loadMask.hide();
    },
    
    onAfterApplyChanges: function(closeWindow) {
        this.window.rename(this.windowNamePrefix + this.record.id);
        this.loadMask.hide();
        this.saving = false;
        
        if (closeWindow) {
            this.window.fireEvent('saveAndClose');
            this.purgeListeners();
            this.window.close();
            if (Tine.Tinebase.registry.get('preferences').get('windowtype')=='Browser') {
                // reload mainscreen to make sure registry gets updated
                this.window.windowManager.getMainWindow().location = this.window.windowManager.getMainWindow().location.href.replace(/#+.*/, '');
            }
        }
    }
});

/**
 * Expressomail Account Edit Popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
 Tine.Expressomail.AccountEditDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 620,
        height: 550,
        name: Tine.Expressomail.AccountEditDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.AccountEditDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 * contacts combo box and store
 * 
 * @package     Addressbook
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.ns('Tine.Addressbook');

/**
 * contact selection combo box
 * 
 * @namespace   Tine.Addressbook
 * @class       Tine.Addressbook.SearchCombo
 * @extends     Ext.form.ComboBox
 * 
 * <p>Contact Search Combobox</p>
 * <p><pre>
 * TODO         make this a twin trigger field with 'clear' button?
 * TODO         add switch to filter for expired/enabled/disabled user accounts
 * </pre></p>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Addressbook.SearchCombo
 * 
 * TODO         add     forceSelection: true ?
 */
Tine.Addressbook.SearchCombo = Ext.extend(Tine.Tinebase.widgets.form.RecordPickerComboBox, {
    
    /**
     * @cfg {Boolean} userOnly
     */
    userOnly: false,
    
    /**
     * use account objects/records in get/setValue
     * 
     * @cfg {Boolean} legacy
     * @legacy
     * 
     * TODO remove this later
     */
    useAccountRecord: false,
    allowBlank: false,
    
    itemSelector: 'div.search-item',
    minListWidth: 350,
    
    //private
    initComponent: function(){
        
        if (this.recordClass === null) {
            this.recordClass = Tine.Addressbook.Model.Contact;
            this.recordProxy = Tine.Addressbook.contactBackend;
        }

        this.initTemplate();
        Tine.Addressbook.SearchCombo.superclass.initComponent.call(this);
    },
    
    /**
     * is called in accountMode to reset the value
     * @param value
     */
    processValue: function(value) {
        if (this.useAccountRecord) {
            if (value == '') {
                this.accountId = null;
                this.selectedRecord = null;
            }
        }
        return Tine.Addressbook.SearchCombo.superclass.processValue.call(this, value);
    },

    /**
     * use beforequery to set query filter
     * 
     * @param {Event} qevent
     */
    onBeforeQuery: function(qevent){
        Tine.Addressbook.SearchCombo.superclass.onBeforeQuery.apply(this, arguments);
        
        if (this.userOnly) {
            this.store.baseParams.filter.push({field: 'type', operator: 'equals', value: 'user'});
        }
    },
    
    /**
     * init template
     * @private
     */
    initTemplate: function() {
        // Custom rendering Template
        if (! this.tpl) {
            this.tpl = new Ext.XTemplate(
                '<tpl for="."><div class="search-item addressbook-search-combo">',
                    '<table>',
                        '<tr>',
                            '<td style="min-width: 20px;">{[Tine.Addressbook.ContactGridPanel.contactTypeRenderer(null, null, values)]}</td>',
                            '<td width="30%"><b>{[Tine.Addressbook.ContactGridPanel.displayNameRenderer(values.n_fileas)]}</b><br/>,' +
                                '{[this.encode(values.org_name)]}</td>',
                            '<td width="25%">{[this.encode(values.adr_one_street)]}<br/>',
                                '{[this.encode(values.adr_one_postalcode)]} {[this.encode(values.adr_one_locality)]}</td>',
                            '<td width="25%">{[this.encode(values.tel_work)]}<br/>{[this.encode(values.tel_cell)]}</td>',
                            '<td width="50px">',
                                '<img width="45px" height="39px" src="{jpegphoto}" />',
                            '</td>',
                        '</tr>',
                    '</table>',
                '</div></tpl>'
             );
        }
    },
    
    getValue: function() {
        if (this.useAccountRecord) {
            if (this.selectedRecord) {
                return this.selectedRecord.get('account_id');
            } else {
                return this.accountId;
            }
        } else {
            return Tine.Addressbook.SearchCombo.superclass.getValue.call(this);
        }
    },

    setValue: function (value) {
        if (this.useAccountRecord) {
            if (value) {
                if(value.accountId) {
                    // account object
                    this.accountId = value.accountId;
                    value = value.accountDisplayName;
                } else if (typeof(value.get) == 'function') {
                    // account record
                    this.accountId = value.get('id');
                    value = value.get('name');
                }
            } else {
                this.accountId = null;
                this.selectedRecord = null;
            }
        }
        return Tine.Addressbook.SearchCombo.superclass.setValue.call(this, value);
    }

});

Ext.reg('addressbookcontactpicker', Tine.Addressbook.SearchCombo);
Tine.widgets.form.RecordPickerManager.register('Addressbook', 'Contact', Tine.Addressbook.SearchCombo);
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.ContactSearchCombo
 * @extends     Tine.Addressbook.SearchCombo
 *
 * <p>Email Search ComboBox</p>
 * <p></p>
 * <pre></pre>
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.ContactSearchCombo
 */
Tine.Expressomail.ContactSearchCombo = Ext.extend(Tine.Addressbook.SearchCombo, {

    /**
     * @cfg {Boolean} forceSelection
     */
    forceSelection: false,

    /**
     * @cfg {Boolean} onlyContacts
     */
    onlyContacts: false,

    /**
     * @private
     */
    valueIsList: false,
    
    /**
     *@cfg {Integer} queryDelay
     */
    queryDelay : 100,
    
    triggerAction: 'query',
    
    pageSize: 0,
    
    minChars: 1,
    
    filterFields: null,

    lazyInit: false,

    
    /**
     * @private
     */
    initComponent: function() {
        // add additional filter to show only contacts with email addresses
        this.additionalFilters = [{field: 'email_query', operator: 'contains', value: '@' }];
              
        this.tpl = new Ext.XTemplate(
             '<tpl for="."><div class="search-item">',
                '{[this.encode(values.n_fn)]}',
                ' (<b>{[this.encode(values.email, values.email_home)]}</b>)',
                '  {[this.encode(values.org_unit)]}',
            '</div></tpl>',
            {
                encode: function(email, email_home, emails) {
                    if (email) {
                        return this.shorten(Ext.util.Format.htmlEncode(email));
                    } else if (email_home) {
                        return Ext.util.Format.htmlEncode(email_home);
                    } else if (emails) {
                        return Ext.util.Format.htmlEncode(emails);
                    } else {
                        return '';
                    }
                },
                shorten: function(text) {
                    if (text) {
                        if (text.length < 50) {
                            return text;
                        } else {
                            return text.substr(0,50) + "...";
                        }
                    } else {
                        return "";
                    }
                }
            }
        );

        Tine.Expressomail.ContactSearchCombo.superclass.initComponent.call(this);

        this.store.on('load', this.onStoreLoad, this);
        
        this.listEmptyText = '<b>'+Tine.Tinebase.appMgr.get('Expressomail').i18n._('No result found in yours personal catalogues.')+'</b>';
    },

    /**
     * override default onSelect
     * - set email/name as value
     *
     * @param {} record
     * @private
     */
    onSelect: function(record, index) {
        if (!record.get("emails")) {
            var value = Tine.Expressomail.getEmailStringFromContact(record);
            this.setValue(value);
            this.valueIsList = false;
        } else {
            this.setValue(record.get("emails"));
            this.valueIsList = true;
        }

        this.collapse();
        this.fireEvent('blur', this);
        this.fireEvent('select', this, record, index);
    },

    /**
     * @return bool
     */
    getValueIsList: function() {
        return this.valueIsList;
    },

    /**
     * always return raw value
     *
     * @return String
     */
    getValue: function() {
        return this.getRawValue();
    },

    /**
     * always set valueIsList to false
     *
     * @param String value
     */
    setValue: function(value) {
       this.valueIsList = false;
       if(value !== undefined)
           Tine.Expressomail.ContactSearchCombo.superclass.setValue.call(this, value);
    },
 
    /**
     * on load handler of combo store
     * -> add additional record if contact has multiple email addresses
     *
     * @param {} store
     * @param {} records
     * @param {} options
     */
    onStoreLoad: function(store, records, options) {
        this.addAlternativeEmail(store, records);
        this.removeDuplicates(store);
    },
            
    /**
     * add alternative email addresses
     *
     * @param {} store
     * @param {} records
     */
    addAlternativeEmail: function(store, records) {
        var index = 0,
            newRecord,
            recordData;

        Ext.each(records, function(record) {
            if (record.get('email') && record.get('email_home') && record.get('email') !== record.get('email_home')) {
                index++;
                recordData = Ext.copyTo({}, record.data, ['email_home', 'n_fn']);
                newRecord = Tine.Addressbook.contactBackend.recordReader({responseText: Ext.util.JSON.encode(recordData)});
                newRecord.id = Ext.id();

                Tine.log.debug('add alternative: ' + Tine.Expressomail.getEmailStringFromContact(newRecord));
                store.insert(index, [newRecord]);
            }
            index++;
        });
    },

    /**
     * remove duplicate contacts
     *
     * @param {} store
     */
    removeDuplicates: function(store) {
        var duplicates = null;

        store.each(function(record) {
            duplicates = store.queryBy(function(contact) {
                return record.id !== contact.id && Tine.Expressomail.getEmailStringFromContact(record) == Tine.Expressomail.getEmailStringFromContact(contact);
            });
            if (duplicates.getCount() > 0) {
                Tine.log.debug('remove duplicate: ' + Tine.Expressomail.getEmailStringFromContact(record));
                store.remove(record);
            }
        });
    },
    
    /**
     * Execute a query to filter the dropdown list.  Fires the {@link #beforequery} event prior to performing the
     * query allowing the query action to be canceled if needed.
     * @param {String} q query The SQL query to execute
     * @param {Boolean} forceAll <tt>true</tt> to force the query to execute even if there are currently fewer
     * characters in the field than the minimum specified by the <tt>{@link #minChars}</tt> config option.  It
     * also clears any filter previously saved in the current store (defaults to <tt>false</tt>)
     */
    doQuery : function (q, forceAll) {
        q = Ext.isEmpty(q) ? '' : q;
        var qe = {
            query: q,
            forceAll: forceAll,
            combo: this,
            cancel:false
        };
        if(this.fireEvent('beforequery', qe)===false || qe.cancel){
            return false;
        }
        q = qe.query;
        forceAll = qe.forceAll;
        if(forceAll === true || (q.length >= this.minChars)){
            if(this.lastQuery !== q){
                this.lastQuery = q;
                if(this.mode == 'local'){
                    this.selectedIndex = -1;
                    if(forceAll){
                        this.store.clearFilter();
                    }else{
                        this.store.filter(this.filterFields, q, true);
                    }
                    this.onLoad();
                }else{
                    this.store.baseParams[this.queryParam] = q;
                    this.store.load({
                        params: this.getParams(q)
                    });
                    this.expand();
                }
            }else{
                this.selectedIndex = -1;
                this.onLoad();
            }
        }
    },
    
    /**
     * use beforequery to set query filter
     * 
     * @param {Object} qevent
     */
    onBeforeQuery: function (qevent) {}
});
Ext.reg('expressomailcontactcombo', Tine.Expressomail.ContactSearchCombo);
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2013 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.RecipientGrid
 * @extends     Ext.grid.EditorGridPanel
 * 
 * <p>Recipient Grid Panel</p>
 * <p>grid panel for to/cc/bcc recipients</p>
 * <pre>
 * </pre>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.RecipientGrid
 */
Tine.Expressomail.RecipientGrid = Ext.extend(Ext.grid.EditorGridPanel, {
    
    /**
     * @private
     */
    cls: 'expressomail-recipient-grid',
    
    /**
     * the message record
     * @type Tine.Expressomail.Model.Message
     * @property record
     */
    record: null,
    
    /**
     * message compose dlg
     * @type Tine.Expressomail.MessageEditDialog
     */
    composeDlg: null,
    
    /**
     * @type Ext.Menu
     * @property contextMenu
     */
    contextMenu: null,
    
    /**
     * @type Ext.data.SimpleStore
     * @property store
     */
    store: null,
    
    /**
     * @cfg {Boolean} autoStartEditing
     */
    autoStartEditing: true,
    
    /**
     * @cfg {String} autoExpandColumn
     * auto expand column of grid
     */
    autoExpandColumn: 'address',
    
    /**
     * @cfg {Number} clicksToEdit
     * clicks to edit for editor grid panel
     */
    clicksToEdit:1,
    
    /**
     * @cfg {Number} numberOfRecordsForFixedHeight
     */
    numberOfRecordsForFixedHeight: 6,

    /**
     * @cfg {Boolean} header
     * show header
     */
    header: false,
    
    /**
     * @cfg {Boolean} border
     * show border
     */
    border: false,
    
    /**
     * @cfg {Boolean} deferredRender
     * deferred rendering
     */
    deferredRender: false,
    
    forceValidation: true,
    
    enableDrop: true,
    ddGroup: 'recipientDDGroup',
    
    /**
     * options are saved in onAfterEdit
     * 
     * @type Ext.data.Record
     */
    lastEditedRecord: null,
    backToFieldWithError: false,
    certificatesStore: null,
    lastRawValue: null,
    
    /**
     * @private
     */
    initComponent: function() {
        
        this.initStore();
        this.initColumnModel();
        this.initActions();
        this.sm = new Ext.grid.RowSelectionModel();
        
        Tine.Expressomail.RecipientGrid.superclass.initComponent.call(this);
        
        this.on('rowcontextmenu', this.onCtxMenu, this);
        // this is relayed by the contact search combo
        this.on('contextmenu', this.onCtxMenu.createDelegate(this, [this, null], 0), this);
        
        this.on('beforeedit', this.onBeforeEdit, this);
        this.on('afteredit', this.onAfterEdit, this);
    },
    
    /**
     * show context menu
     * 
     * @param {Tine.Expressomail.RecipientGrid} grid
     * @param {Number} row
     * @param {Event} e
     */
    onCtxMenu: function(grid, row, e) {
        var activeRow = (row === null) ? ((this.activeEditor) ? this.activeEditor.row : 0) : row;
        
        e.stopEvent();
        var selModel = grid.getSelectionModel();
        if (! selModel.isSelected(activeRow)) {
            selModel.selectRow(activeRow);
        }
        
        var record = this.store.getAt(activeRow);
        if (record) {
            this.action_remove.setDisabled(record.get('address') == '');
            this.contextMenu.showAt(e.getXY());
        }
    },
    
    /**
     * init store
     * @private
     */
    initStore: function() {
        
        if(!this.record) {
            this.initStore.defer(200, this);
            return false;
        }
        
        this.store = new Ext.data.SimpleStore({
            fields   : ['type', 'address']
        });
        
        // init recipients (on reply/reply to all)
        this.syncRecipientsToStore(['to', 'cc', 'bcc']);
        
        this.store.add(new Ext.data.Record({type: 'to', 'address': ''}));
        
        this.store.on('update', this.onUpdateStore, this);
        this.store.on('add', this.onAddStore, this);
    },

    onValidate: function(value) {
        if (Ext.util.Format.trim(value).length == 0) {
            this.searchCombo.markInvalid(this.searchCombo.invalidText);
            return false;
        }
        if (this.backToFieldWithError) {
            this.backToFieldWithError = false;
            return true;
        }
        var emailRecipients = new Array(),
            emails = new Array(),
            emailRecipients = value.split(',');

        if (emailRecipients.length > 0) {
            var emailRegExp = /<([^>]*)>/;
            Ext.each(emailRecipients, function(email) {
                if (emailRegExp.exec(email.trim())) {
                    if (RegExp.$1 != '') {
                        emails.push(RegExp.$1);
                    }
                }
                else {
                        emails.push(email.trim());
                }
            }, this);
        }

        var filter = Tine.Addressbook.Model.EmailAddress.getFilterModel();
        filter.push({field: 'email', operator: 'in', value: emails});
        this.certificatesStore.baseParams.filter = filter;
        this.certificatesStore.load({
            callback: function(records) {
                var notFound = new Array();
                var found = new Array();
                for(i=0;i<emails.length;i++) {
                    if (this.certificatesStore.find('email', emails[i]) == -1) {
                        notFound.push(emails[i]);
                    } else {
                        found.push(emails[i]);
                    }
                };
                if(notFound.length > 0) {
                    this.backToFieldWithError = true;
                    var invalidText = this.searchCombo.invalidText;
                    this.searchCombo.markInvalid(this.i18n._("No valid certificate found for one or more of these email address."));
                    this.searchCombo.invalidText = invalidText;
                }
            }, 
            scope: this,
            add: true
        });
        return true;
    },
    
    /**
     * init cm
     * @private
     */
    initColumnModel: function() {
        
        this.certificatesStore = new Tine.Tinebase.data.RecordStore({
            proxy: Tine.Addressbook.certificateBackend,
            recordClass: Tine.Addressbook.Model.Certificate
        });

        var app = Tine.Tinebase.appMgr.get('Expressomail');
        
        var mailStoreData = this.composeDlg.mailStoreData;
        
        this.searchCombo = new Tine.Expressomail.ContactSearchCombo({
            
            filterFields: ['email','n_fn','org_unit'],
            store: Tine.Expressomail.createMailStore(mailStoreData),
            mode: 'local',
            pageSize: 0,
            //validator: this.onValidate.createDelegate(this),
            //certificatesStore: this.certificatesStore,
            listeners: {
                scope: this,
                specialkey: this.onSearchComboSpecialkey,
                select: this.onSearchComboSelect,
                blur: function(combo) {
                    Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboBlur()');
                    this.getView().el.select('.x-grid3-td-address-editing').removeClass('x-grid3-td-address-editing');
                    // need to update record because it might not be updated otherwise (for example: delete value, click into next row or subject)
                    if (this.activeEditor) {
                        var value = combo.getRawValue();
                        Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboBlur() -> current value: ' + value);
                        if (value !== null && this.activeEditor.record.get('address') != value) {
                            this.activeEditor.record.set('address', value);
                        }
                    }
                    this.stopEditing();
                }
            }
        });
        
        this.cm = new Ext.grid.ColumnModel([
            {
                resizable: true,
                id: 'type',
                dataIndex: 'type',
                width: 104,
                menuDisabled: true,
                header: 'type',
                renderer: function(value) {
                    var result = '',
                        qtip = Ext.util.Format.htmlEncode(app.i18n._('Click here to set To/CC/BCC.'));

                    switch(value) {
                        case 'to':
                            result = Ext.util.Format.htmlEncode(app.i18n._('To:'));
                            break;
                        case 'cc':
                            result = Ext.util.Format.htmlEncode(app.i18n._('Cc:'));
                            break;
                        case 'bcc':
                            result = Ext.util.Format.htmlEncode(app.i18n._('Bcc:'));
                            break;
                    }
                    
                    result = Tine.Tinebase.common.cellEditorHintRenderer(result);
                    
                    return '<div qtip="' + qtip +'">' + result + '</div>';
                },
                editor: new Ext.form.ComboBox({
                    typeAhead     : false,
                    triggerAction : 'all',
                    lazyRender    : true,
                    editable      : false,
                    mode          : 'local',
                    value         : null,
                    forceSelection: true,
                    lazyInit      : false,
                    store         : [
                        ['to',  app.i18n._('To:')],
                        ['cc',  app.i18n._('Cc:')],
                        ['bcc', app.i18n._('Bcc:')]
                    ],
                    listeners: {
                        focus: function(combo) {
                            combo.onTriggerClick();
                        }
                    }
                })
            },{
                resizable: true,
                menuDisabled: true,
                id: 'address',
                dataIndex: 'address',
                header: 'address',
                editor: this.searchCombo
            }
        ]);
    },
    
    /**
     * specialkey is pressed in search combo
     * 
     * @param {Combo} combo
     * @param {Event} e
     */
    onSearchComboSpecialkey: function(combo, e) {
        var value = combo.getValue();
        Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboSpecialkey() -> current value: ' + value);
        
        if (e.getKey() == e.ENTER) {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboSpecialkey() -> ENTER');
            if (this.activeEditor && value !== null && this.activeEditor.record.get('address') != value) {
                this.activeEditor.record.set('address', value);
            }
            if (!this.searchCombo.getValueIsList()) {
                this.onSearchComboSelect(combo);
            }
        } else if (this.activeEditor && e.getKey() == e.BACKSPACE) {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboSpecialkey() -> BACKSPACE');
            // remove row on backspace if we have more than 1 rows in grid
            if (value == '' && this.store.getCount() > 1 && this.activeEditor.row > 0) {
                this.store.remove(this.activeEditor.record);
                this.activeEditor.row -= 1;
                this.setFixedHeight(false);
                this.ownerCt.doLayout();
                this.startEditing.defer(50, this, [this.activeEditor.row, this.activeEditor.col]);
                return true;
            }
        } else if (this.activeEditor && e.getKey() == e.ESC) {
            // TODO should ESC close the compose window if search combo is already empty?
//            if (value == '') {
//                this.fireEvent('specialkey', this, e);
//            }
            this.startEditing.defer(50, this, [this.activeEditor.row, this.activeEditor.col]);
            return true;
        }

        // jump to subject if we are in the last row and it is empty OR TAB was pressed
        if (this.activeEditor && e.getKey() == e.TAB || (e.getKey() == e.ENTER && value == '' && this.store.getCount() == this.activeEditor.row + 1)) {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboSpecialkey() -> TAB');
            if (value !== null && this.activeEditor.record.get('address') != value) {
                this.activeEditor.record.set('address', value);
            }
            
            this.fireEvent('specialkey', combo, e);
            this.getView().el.select('.x-grid3-td-address-editing').removeClass('x-grid3-td-address-editing');
            this.stopEditing();
            return false;
        }
    },
    
    onSearchComboSelect: function(combo) {
        Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSearchComboSelect()');
        
        var value = combo.getValue();
        if (combo.getValueIsList()) {
            var emails = value.split(";");
            emails.sort();
            var editor = (this.activeEditor ? this.activeEditor : this.lastActiveEditor);
            var recipient_type = (editor ? this.store.data.items[editor.row].get('type') : 'to');
            this._addRecipients(emails, recipient_type);
            this.setFixedHeight(false);
            this.ownerCt.doLayout();
            this.store.remove(this.activeEditor ? this.activeEditor.record : this.lastEditedRecord);
            this.addRowAndDoLayout(this.activeEditor ? this.activeEditor.record : this.lastEditedRecord);
        } else {
            if (value !== '') {
                this.addRowAndDoLayout(this.activeEditor ? this.activeEditor.record : this.lastEditedRecord);
            }
        }
    },
    
    /**
     * adds row and adjusts layout
     * 
     * @param {} oldRecord
     */
    addRowAndDoLayout: function(oldRecord) {
        var emptyRecord = this.store.getAt(this.store.findExact('address', ''));
        if (! emptyRecord) {
            emptyRecord = new Ext.data.Record({type: oldRecord.data.type, 'address': ''});
            this.store.add(emptyRecord);
            this.store.commitChanges();
            this.setFixedHeight(false);
            this.ownerCt.doLayout();
        }
        
        this.startEditing.defer(50, this, [this.store.indexOf(emptyRecord), 1]);
    },
    
    /**
     * start editing (check if message compose dlg is saving/sending first)
     * 
     * @param {} row
     * @param {} col
     */
    startEditing: function(row, col) {
        if (! this.composeDlg || ! this.composeDlg.saving) {
            Tine.Expressomail.RecipientGrid.superclass.startEditing.apply(this, arguments);
        }
    },
    
    /**
     * init actions / ctx menu
     * @private
     */
    initActions: function() {
        this.action_remove = new Ext.Action({
            text: _('Remove'),
            handler: this.onDelete,
            iconCls: 'action_delete',
            scope: this
        });
        
        this.contextMenu = new Ext.menu.Menu({
            items:  this.action_remove
        });
    },
    
    /**
     * start editing after render
     * @private
     */
    afterRender: function() {
        Tine.Expressomail.RecipientGrid.superclass.afterRender.call(this);
        
        // kill x-scrollers
        this.el.child('div[class=x-grid3-scroller]').setStyle('overflow-x', 'hidden');
        
        if (this.autoStartEditing && this.store.getCount() == 1) {
            this.startEditing.defer(200, this, [0, 1]);
        }
        
        this.setFixedHeight(true);
        
        this.initDropTarget();
    },
    
    /**
     * init drop target with notifyDrop fn 
     * - adds new records from drag data to the recipient store
     */
    initDropTarget: function() {
        var dropTargetEl = this.getView().scroller.dom;
        var dropTarget = new Ext.dd.DropTarget(dropTargetEl, {
            ddGroup    : 'recipientDDGroup',
            notifyDrop : function(ddSource, e, data) {
                this.grid.addRecordsToStore(ddSource.dragData.selections);
                return true;
            },
            grid: this
        });
    },
    
    /**
     * add records to recipient store
     * 
     * @param {Array} records
     * @param {String} type
     */
    addRecordsToStore: function(records, type) {
        if (! type) {
            var emptyRecord = this.store.getAt(this.store.findExact('address', '')),
                type = (emptyRecord) ? emptyRecord.get('type') : 'to';
        }
        
        var hasEmail = false,
            added = false;
        
        Ext.each(records, function(record) {
            if (record.hasEmail()) {
                this.store.add(new Ext.data.Record({type: type, 'address': Tine.Expressomail.getEmailStringFromContact(record)}));
                added = true;
            }
        }, this);
    },
    
    /**
     * set grid to fixed height if it has more than X records
     *  
     * @param {} doLayout
     */
    setFixedHeight: function (doLayout) {
        if (this.store.getCount() > this.numberOfRecordsForFixedHeight) {
            this.setHeight(155);
        } else {
            this.setHeight(this.store.getCount()*24 + 1);
        }

        if (doLayout && doLayout === true) {
            this.ownerCt.doLayout();
        }
    },
    
    /**
     * store has been updated
     * 
     * @param {} store
     * @param {} record
     * @param {} operation
     * @private
     */
    onUpdateStore: function(store, record, operation) {
        if (typeof record != 'undefined') {
            if(!this.record.modified){
                this.record.modified = {};
            }
            this.record.modified[record.get('type')] = record.get('address');
        }
        this.syncRecipientsToRecord();
    },
    
    /**
     * on add event of store
     * 
     * @param {} store
     * @param {} records
     * @param {} index
     */
    onAddStore: function(store, records, index) {
        this.syncRecipientsToRecord();
    },
    
    /**
     * sync grid with record
     * -> update record to/cc/bcc
     */
    syncRecipientsToRecord: function() {
        // update record recipient fields
        this.record.data.to = [];
        this.record.data.cc = [];
        this.record.data.bcc = [];
        this.store.each(function(recipient){
            if (recipient.data.address != '') {
                this.record.data[recipient.data.type].push(recipient.data.address);
            }
        }, this);
    },

    /**
     * sync grid with record
     * -> update store
     * 
     * @param {Array} fields
     * @param {Tine.Expressomail.Model.Message} record
     * @param {Boolean} setHeight
     * @param {Boolean} clearStore
     */
    syncRecipientsToStore: function(fields, record, setHeight, clearStore) {
        if (clearStore) {
            this.store.removeAll(true);
        }
        
        record = record || this.record;
        
        Ext.each(fields, function(field) {
            this._addRecipients(record.get(field), field);
        }, this);
        this.store.sort('address');
        
        if (clearStore) {
            this.store.add(new Ext.data.Record({type: 'to', 'address': ''}));
        }
        
        if (setHeight && setHeight === true) {
            this.setFixedHeight(true);
        }
    },
    
    /**
     * after edit
     * 
     * @param {} o
     */
    onAfterEdit: function(o) {
        if (o.field == 'address') {
            Tine.log.debug('Tine.Expressomail.MessageEditDialog::onAfterEdit()');
            Tine.log.debug(o);
            
            Ext.fly(this.getView().getCell(o.row, o.column)).removeClass('x-grid3-td-address-editing');
            
            this.lastEditedRecord = o.record;
        }
    },
    
    /**
     * delete handler
     */
    onDelete: function(btn, e) {
        var sm = this.getSelectionModel();
        var records = sm.getSelections();
        Ext.each(records, function(record) {
            if (record.get('address') != '' && this.store.getCount() > 1) {
                this.store.remove(record);
                this.store.fireEvent('update', this.store);
            }
        }, this);
        
        this.setFixedHeight(true);
    },
    
    /**
     * on before edit
     * 
     * @param {} o
     */
    onBeforeEdit: function(o) {
        this.getView().el.select('.x-grid3-td-address-editing').removeClass('x-grid3-td-address-editing');
        Ext.fly(this.getView().getCell(o.row, o.column)).addClass('x-grid3-td-address-editing');
    },
    
    /**
     * add recipients to grid store
     * 
     * @param {Array} recipients
     * @param {String} type
     * @private
     */
    _addRecipients: function(recipients, type) {
        if (recipients) {
            recipients = Ext.unique(recipients);
            for (var i=0; i < recipients.length; i++) {
                this.store.add(new Ext.data.Record({type: type, 'address': recipients[i]}));
            }
        }
    }
});

Ext.reg('expressomailrecipientgrid', Tine.Expressomail.RecipientGrid);

// overridden to process the TAB key
Ext.override(Ext.Editor, {
    // private
    onSpecialKey : function(field, e){
        var key = e.getKey(),
            move = key === e.TAB,
            complete = this.completeOnEnter && key === e.ENTER,
            cancel = this.cancelOnEsc && key === e.ESC;
        if(complete || cancel){
            e.stopEvent();
            if(complete){
                this.completeEdit();
            }else{
                this.cancelEdit();
            }
            if(field.triggerBlur){
                field.triggerBlur();
            }
        }
        this.fireEvent('specialkey', field, e);
        if(move){
            if(field.triggerBlur){
                field.triggerBlur();
            }
        }
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2009-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.ns('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.Application
 * @extends     Tine.Tinebase.Application
 *
 * <p>Expressomail application obj</p>
 * <p>
 * </p>
 *
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 *
 * @param       {Object} config
 *
 * @constructor
 * Create a new  Tine.Expressomail.Application
 */
Tine.Expressomail.Application = Ext.extend(Tine.Tinebase.Application, {

    /**
     * auto hook text _('New Mail')
     */
    addButtonText: 'New Mail',

    /**
     * @property checkMailsDelayedTask
     * @type Ext.util.DelayedTask
     */
    checkMailsDelayedTask: null,

    /**
     * @property defaultAccount
     * @type Tine.Expressomail.Model.Account
     */
    defaultAccount: null,

    /**
     * @type Ext.data.JsonStore
     */
    folderStore: null,

    /**
     * @type Ext.data.JsonStore
     */
    accountStore: null,

    /**
     * @property updateInterval user defined update interval (milliseconds)
     * @type Number
     */
    updateInterval: null,

    /**
    * @property cleanTrash already run.
    * @type Number
    */
    cleanTrash: null,
    
    /**
     * @property mailStore store with contacts for quick search
     * @type Tine.Tinebase.data.RecordStore
     */
    mailStore: null,

    /**
     * transaction id of current update message cache request
     * @type Number
     */
    updateMessageCacheTransactionId: null,

    getFolderStatusTransactionInProgress: false,

    /**
     * unreadcount in default account inbox
     * @type Number
     */
    unreadcountInDefaultInbox: 0,

    /**
     * returns title (Email)
     *
     * @return {String}
     */
    getTitle: function() {
        return this.i18n._('Email');
    },


   /**
   * Verify trash folder for clean ...
   */
    verifyTrashToClean: function(account_id) {
        if(!account_id) return;
        if(!Tine.Expressomail.registry.get('preferences').get('deleteFromTrash')) return;
        Ext.MessageBox.wait(_('Please wait'), _('Verifying trash folder'));
        var params = {
          method: 'Expressomail.deleteMsgsBeforeDate'
        };
        params.accountId = account_id;
        Ext.Ajax.request({
            params: params,
            scope: this,
            timeout: 150000, // 2 minutes
            success: function(result, request){
                //alert(result.msgs + ' Removed from trash folder');
                Ext.MessageBox.hide();
            }
        });
        return;
    },

    /**
     * start delayed task to init folder store / updateFolderStore
     */
    init: function() {
        Tine.log.info('initialising app');
        Tine.Expressomail.MimeDisplayManager.register('text/readconf', Tine.Expressomail.ReadConfirmationDetailsPanel);
        this.checkMailsDelayedTask = new Ext.util.DelayedTask(this.checkMails, this);

        this.updateInterval = parseInt(Tine.Expressomail.registry.get('preferences').get('updateInterval')) * 60000;
        Tine.log.debug('user defined update interval is "' + this.updateInterval/1000 + '" seconds');

        this.defaultAccount = Tine.Expressomail.registry.get('preferences').get('defaultEmailAccount');
        Tine.log.debug('default account is "' + this.defaultAccount);

        if (window.isMainWindow) {
            if (Tine.Tinebase.appMgr.getActive() != this && this.updateInterval) {
                var delayTime = this.updateInterval/20;
                Tine.log.debug('start preloading mails in "' + delayTime/1000 + '" seconds');
                this.checkMailsDelayedTask.delay(delayTime);
            }

            this.showActiveVacation();
            var adbHook1 = new Tine.Expressomail.GridPanelHook({
                app: this,
                foreignAppName: 'Addressbook',
                recordTypeName: 'Contact'
            });
            var adbHook2 = new Tine.Expressomail.GridPanelHook({
                app: this,
                foreignAppName: 'Addressbook',
                recordTypeName: 'List'
            });

            this.mailStore = Tine.Expressomail.getMailStore();
            this.mailStore.load();
        }
        
        Tine.Expressomail.registry.get('preferences').on('replace', this.onPreferenceChange.createDelegate());
    },

    /**
     * init app auto hooks 
     */
    initAutoHooks: function() {
        Ext.ux.ItemRegistry.registerItem('Tine.widgets.grid.GridPanel.addButton', {
            text: this.i18n._hidden(this.addButtonText), 
            iconCls: this.getIconCls(),
            scope: this,
            handler: function(button) {
                var ms = this.getMainScreen(),
                    cp = ms.getCenterPanel();

                cp.onEditInNewWindow.call(cp, {encrypted:false});
            }
        });
        
        if (Tine.Expressomail.registry.get('preferences').get('enableEncryptedMessage') == '1' && Tine.Tinebase.registry.get('preferences').get('windowtype')=='Ext') {
            Ext.ux.ItemRegistry.registerItem('Tine.widgets.grid.GridPanel.addButton', {
                text: this.i18n._hidden('New Mail (encrypted)'), 
                iconCls: this.getIconCls(),
                scope: this,
                handler: function(button) {
                    var ms = this.getMainScreen(),
                        cp = ms.getCenterPanel();

                    cp.onEditInNewWindow.call(cp, {encrypted:true});
                }
            });
        }

    },

    /**
     * show notification with active vacation information
     */
    showActiveVacation: function () {
        var accountsWithActiveVacation = this.getAccountStore().query('sieve_vacation_active', true);
        accountsWithActiveVacation.each(function(item) {
                Ext.MessageBox.alert(this.i18n._('Active Vacation Message'), String.format(this.i18n._('Email account "{0}" has an active vacation message.'), item.get('name')));
        }, this);
    },

    /**
     * check mails
     *
     * if no folder is given, we find next folder to update ourself
     *
     * @param {Tine.Expressomail.Model.Folder} [folder]
     * @param {Function} [callback]
     */
    checkMails: function(folder, callback) {
        this.checkMailsDelayedTask.cancel();
        
        if (! this.getFolderStore().getCount() && this.defaultAccount) {
            this.fetchSubfolders('/' + this.defaultAccount);
            return;
        }

        Tine.log.info('checking mails' + (folder ? ' for folder ' + folder.get('localname') : '') + ' now: ' + new Date());

        // if no folder is given, see if there is a folder to check in the folderstore
        if (! folder) {
            folder = this.getNextFolderToUpdate();
        }

        // for testing purposes
        //console.log('update disabled');
        //return;

        if (folder) {
            var executionTime = folder.isCurrentSelection() ? 10 : Math.min(this.updateInterval/1000, 120);

            if (this.updateMessageCacheTransactionId && Tine.Expressomail.folderBackend.isLoading(this.updateMessageCacheTransactionId)) {
                var currentRequestFolder = this.folderStore.query('cache_status', 'pending').first(),
                    expectedResponseIn = Math.floor((this.updateMessageCacheTransactionExpectedResponse.getTime() - new Date().getTime())/1000);

                if (currentRequestFolder && (currentRequestFolder !== folder || expectedResponseIn > executionTime)) {
                    Tine.log.debug('aborting current update message request (expected response in ' + expectedResponseIn + ' seconds)');
                    Tine.Expressomail.folderBackend.abort(this.updateMessageCacheTransactionId);
                    currentRequestFolder.set('cache_status', 'incomplete');
                    currentRequestFolder.commit();
                } else {
                    Tine.log.debug('a request updating message cache for folder "' + folder.get('localname') + '" is in progress -> wait for request to return');
                    return;
                }
            }

            Tine.log.debug('Updating message cache for folder "' + folder.get('localname') + '" of account ' + folder.get('account_id'));
            Tine.log.debug('Max execution time: ' + executionTime + ' seconds');

            this.updateMessageCacheTransactionExpectedResponse = new Date().add(Date.SECOND, executionTime);
            folder.set('cache_status', 'pending');
            folder.commit();

            this.updateMessageCacheTransactionId = Tine.Expressomail.folderBackend.updateMessageCache(folder.id, executionTime, {
                scope: this,
                callback: callback,
                failure: this.onBackgroundRequestFail,
                success: function(folder) {
                    this.getAccountStore().getById(folder.get('account_id')).setLastIMAPException(null);
                    this.getFolderStore().updateFolder(folder);

                    if (folder.get('cache_status') === 'updating') {
                        Tine.log.debug('updating message cache for folder "' + folder.get('localname') + '" is in progress on the server (folder is locked)');
                        return this.checkMailsDelayedTask.delay(this.updateInterval);
                    }
                    this.checkMailsDelayedTask.delay(0);
                }
            });
        } else {
            var allFoldersFetched = this.fetchSubfolders();

            if (allFoldersFetched) {
                Tine.log.info('nothing more to do -> will check mails again in "' + this.updateInterval/1000 + '" seconds');
                if (this.updateInterval > 0) {
                    this.checkMailsDelayedTask.delay(this.updateInterval);
                }
            } else {
                this.checkMailsDelayedTask.delay(20000);
            }
        }
    },
//    
//    /**
//     * Get mail Store
//     */
//    getMailStore: function() {
//        
//        if (!this.mailStore) {
//            // Load mailStore
//            this.mailStore = Tine.Expressomail.getMailStore();
//        }
//        return this.mailStore;
//    },

    /**
     * fetch subfolders by parent path
     * - if parentPath param is empty, it loops all accounts and account folders to find the next folders to fetch
     *
     * @param {String} parentPath
     * @return {Boolean} true if all folders of all accounts have been fetched
     */
    fetchSubfolders: function(parentPath) {
        var folderStore = this.getFolderStore(),
            accountStore = this.getAccountStore(),
            doQuery = true,
            allFoldersFetched = false;

        var index = accountStore.findExact('all_folders_fetched', false),
        account = accountStore.getAt(index);

        if(!this.cleanTrash){
            this.verifyTrashToClean(account.id);
            this.cleanTrash = 1;
        }

        if (! parentPath) {
            // find first account that has unfetched folders
            var index = accountStore.findExact('all_folders_fetched', false),
                account = accountStore.getAt(index);

            if (account) {
                // determine the next level of folders that is not fetched
                parentPath = '/' + account.id;

                var recordsOfAccount = folderStore.query('account_id', account.id);
                if (recordsOfAccount.getCount() > 0) {
                    // loop account folders and find the next folder path that hasn't been queried and has children
                    var path, found = false;
                    recordsOfAccount.each(function(record) {
                        // Get parent_path from folder record
                        parentPath = record.get('parent_path');
                        path = parentPath + '/' + record.id;
                        if (! folderStore.isLoadedOrLoading('parent_path', path) && (! account.get('has_children_support') || record.get('has_children'))) {
                            parentPath = path;
                            found = true;
                            Tine.log.debug('fetching next level of subfolders for ' + record.get('globalname'));
                            return false;
                        }
                        return true;
                    }, this);

                    if (! found) {
                        Tine.log.debug('all folders of account ' + account.get('name') + ' have been fetched ...');
                        account.set('all_folders_fetched', true);
                        return false;
                    }
                } else {
                    Tine.log.debug('fetching first level of folders for account ' + account.get('name'));
                }

            } else {
                Tine.log.debug('all folders of all accounts have been fetched ...');
                return true;
            }
        } else {
            Tine.log.debug('no folders in store yet, fetching first level ...');
        }

        if (! folderStore.queriesPending || folderStore.queriesPending.length == 0) {
            folderStore.asyncQuery('parent_path', parentPath, this.checkMails.createDelegate(this, []), [], this, folderStore);
        } else {
            this.checkMailsDelayedTask.delay(0);
        }

        return false;
    },

    /**
     * get folder store
     *
     * @return {Tine.Expressomail.FolderStore}
     */
    getFolderStore: function() {
        if (! this.folderStore) {
            Tine.log.debug('creating folder store');
            this.folderStore = new Tine.Expressomail.FolderStore({
                listeners: {
                    scope: this,
                    update: this.onUpdateFolder
                }
            });
        }

        return this.folderStore;
    },

    /**
     * gets next folder which needs to be checked for mails
     *
     * @return {Model.Folder/null}
     */
    getNextFolderToUpdate: function() {
        var currNode = this.getMainScreen().getTreePanel().getSelectionModel().getSelectedNode(),
            currFolder = currNode ? this.getFolderStore().getById(currNode.id) : null;

        // current selection has highest prio!
        if (currFolder && currFolder.needsUpdate(this.updateInterval)) {
            return currFolder;
        }

        // check if inboxes need updates
        var inboxes = this.folderStore.queryBy(function(folder) {
            return folder.isInbox() && folder.needsUpdate(this.updateInterval);
        }, this);
        if (inboxes.getCount() > 0) {
            return inboxes.first();
        }

        // check for incompletes
        var incompletes = this.folderStore.queryBy(function(folder) {
            return (['complete', 'updating', 'disconnect'].indexOf(folder.get('cache_status')) === -1 && folder.get('is_selectable'));
        }, this);
        if (incompletes.getCount() > 0) {
            Tine.log.debug('Got ' + incompletes.getCount() + ' incomplete folders.');
            var firstIncomplete = incompletes.first();
            Tine.log.debug('First ' + firstIncomplete.get('cache_status') + ' folder to check: ' + firstIncomplete.get('globalname'));
            return firstIncomplete;
        }

        // check for outdated
        if (! this.getFolderStatusTransactionInProgress) {
            this.getStatusOfOutdatedFolders();
        } else {
            Tine.log.debug('getFolderStatus() already running ... wait a little more.');
        }

        // nothing to update
        return null;
    },

    /**
     * collects outdated folders and calls getFolderStatus on server to fetch all folders that need to be updated
     */
    getStatusOfOutdatedFolders: function() {
        var outdated = this.folderStore.queryBy(function(folder) {
            if (! folder.get('is_selectable')) {
                return false;
            }

            var timestamp = folder.get('client_access_time');
            if (! Ext.isDate(timestamp)) {
                return true;
            }
            // update inboxes more often than other folders
            if (timestamp.getElapsed() > this.updateInterval) {
                return true;
            }
            return false;
        }, this);

        if (outdated.getCount() > 0) {
            Tine.log.debug('Still got ' + outdated.getCount() + ' outdated folders to update');

            // call Expressomail.getFolderStatus() with ids of outdated folders -> update folder store on success
            // get only max 50 folders at once
            var rangeOfFolders = (outdated.getCount() > 50) ? outdated.getRange(0, 49) : outdated.getRange(),
                ids = [],
                now = new Date();
            Ext.each(rangeOfFolders, function(folder) {
                folder.set('client_access_time', now);
                ids.push(folder.id);
            });

            var filter = [{field: 'id', operator: 'in', value: ids}];
            Tine.log.debug('Requesting folder status of ' + rangeOfFolders.length + ' folders ...');
            Tine.Expressomail.getFolderStatus(filter, this.onGetFolderStatusSuccess.createDelegate(this));
            this.getFolderStatusTransactionInProgress = true;
        }
    },

    /**
     * get folder status returned -> set folders that need an update to pending status
     *
     * @param {Array} response
     */
    onGetFolderStatusSuccess: function(response) {
        this.getFolderStatusTransactionInProgress = false;
        Tine.log.debug('Tine.Expressomail.Application::onGetFolderStatusSuccess() -> Folder status update successful.');
        Tine.log.debug(response);

        if (response && response.length > 0) {
            Tine.log.debug('Tine.Expressomail.Application::onGetFolderStatusSuccess() -> Got ' + response.length + ' folders that need an update.');

            Ext.each(response, function(folder) {
                var folderToUpdate = this.folderStore.getById(folder.id);
                folderToUpdate.set('cache_status', 'pending');
            }, this);

            this.checkMailsDelayedTask.delay(1000);
        } else {
            Tine.log.debug('Tine.Expressomail.Application::onGetFolderStatusSuccess() -> No folders for update found.');
        }
    },

    /**
     * executed when updateMessageCache requests fail
     *
     * NOTE: We show the credential error dlg and this only for the first error
     *
     * @param {Object} exception
     */
    onBackgroundRequestFail: function(exception) {
        var currentRequestFolder = this.folderStore.query('cache_status', 'pending').first();
        var accountId     = currentRequestFolder.get('account_id'),
            account       = accountId ? this.getAccountStore().getById(accountId): null,
            imapStatus    = account ? account.get('imap_status') : null,
            grid          = this.getMainScreen().getCenterPanel(),
            manualRefresh = grid && grid.manualRefresh;

        if (manualRefresh) {
            grid.manualRefresh = false;
            grid.pagingToolbar.refresh.enable();
        }

        if (exception.code == 913) {
            // folder not found -> remove folder from store and tree panel
            var treePanel = this.getMainScreen().getTreePanel(),
                node = treePanel.getNodeById(currentRequestFolder.id);
            if (node) {
                node.remove();
            }
            this.getFolderStore().remove(currentRequestFolder);
        } else if (account && (manualRefresh ||
            //  do not show exclamation mark for timeouts and connection losses
            (exception.code !== 520 && exception.code !== 510))
        ) {
            account.setLastIMAPException(exception);

            this.getFolderStore().each(function(folder) {
                if (folder.get('account_id') === accountId) {
                    folder.set('cache_status', 'disconnect');
                    folder.commit();
                }
            }, this);

            if (exception.code == 912 && imapStatus !== 'failure' && Tine.Tinebase.appMgr.getActive() === this) {
                Tine.Expressomail.folderBackend.handleRequestException(exception);
            }
        }

        Tine.log.info((manualRefresh ? 'Manual' : 'Background') + ' update failed (' + exception.message
            + ') for folder ' + currentRequestFolder.get('globalname')
            + ' -> will check mails again in "' + this.updateInterval/1000 + '" seconds');
        Tine.log.debug(exception);
        this.checkMailsDelayedTask.delay(this.updateInterval);
    },

    /**
     * executed right before this app gets activated
     */
    onBeforeActivate: function() {
        Tine.log.info('activating expressomail now');
        // abort preloading/old actions and force fresh fetch
        this.checkMailsDelayedTask.delay(0);
    },

    /**
     * on update folder
     *
     * @param {Tine.Expressomail.FolderStore} store
     * @param {Tine.Expressomail.Model.Folder} record
     * @param {String} operation
     */
    onUpdateFolder: function(store, record, operation) {
        if (operation === Ext.data.Record.EDIT) {
            if (record.isModified('cache_status')) {
                Tine.log.info('Tine.Expressomail.Application::onUpdateFolder(): Folder "' + record.get('localname') + '" updated with cache_status: ' + record.get('cache_status'));

                // as soon as we get a folder with status != complete we need to trigger checkmail soon!
                if (['complete', 'pending'].indexOf(record.get('cache_status')) === -1) {
                    this.checkMailsDelayedTask.delay(1000);
                }

                if (record.isInbox() && record.isModified('cache_unreadcount')) {
                    this.showNewMessageNotification(record);
                }
            }

            if (record.isInbox()) {
                if (this.isDefaultAccountId(record.get('account_id'))) {
                    if (record.isModified('cache_unreadcount') || record.get('cache_unreadcount') != this.unreadcountInDefaultInbox) {
                        this.setTitleWithUnreadcount(record.get('cache_unreadcount'));
                    }
                }

                if (record.isModified('quota_usage') || record.isModified('quota_limit')) {
                    this.onUpdateFolderQuota(record);
                }
            }
        }
    },

    /**
     * executed when a value in Expressomail registry/preferences changed
     * @param {string} key
     * @param {value} oldValue
     * @param {value} newValue
     */
    onPreferenceChange: function (key, oldValue, newValue) {
        switch (key) {
            case 'enableEncryptedMessage':
                // reload mainscreen
                var reload = new Ext.util.DelayedTask(function(){window.location.reload(false);},this);
                reload.delay(500);
                
                break;
        }
    },
    
    /**
     * checks default account id
     *
     * @param {String} accountId
     * @return {Boolean}
     */
    isDefaultAccountId: function(accountId) {
        return accountId == Tine.Expressomail.registry.get('preferences').get('defaultEmailAccount');
    },

    /**
     * show notification for new messages
     *
     * @param {Tine.Expressomail.Model.Folder} record
     */
    showNewMessageNotification: function(record) {
        var recents = (record.get('cache_unreadcount') - record.modified.cache_unreadcount),
            account = this.getAccountStore().getById(record.get('account_id'));

        if (recents > 0 ) {
            Tine.log.info('Show notification: ' + recents + ' new mails.');
            var title = this.i18n._('New mails'),
                message = String.format(this.i18n._('You got {0} new mail(s) in folder {1} ({2}).'), recents, record.get('localname'), account.get('name'));

            if (record.isCurrentSelection()) {
                // need to defer the notification because the new messages are not shown yet
                // -> improve this with a callback fn or something like that / unread count should be updated when the messages become visible, too
                Ext.ux.Notification.show.defer(3500, this, [title, message]);
            } else {
                Ext.ux.Notification.show(title, message);
            }
        }
    },

    /**
     * write number of unread messages in all accounts into title
     *
     * @param {Number} unreadcount
     */
    setTitleWithUnreadcount: function(unreadcount) {
        if (! window.isMainWindow) {
            return;
        }

        this.unreadcountInDefaultInbox = unreadcount;
        if (this.unreadcountInDefaultInbox < 0) {
            this.unreadcountInDefaultInbox = 0;
        }

        Tine.log.info('Updating title with new unreadcount: ' + this.unreadcountInDefaultInbox);
        var currentTitle = document.title,
            unreadString = (this.unreadcountInDefaultInbox != 0) ? '(' + this.unreadcountInDefaultInbox + ') ' : '';

        if (currentTitle.match(/^\([0-9]+\) /)) {
            document.title = document.title.replace(/^\([0-9]+\) /, unreadString);
        } else {
            document.title = unreadString + currentTitle;
        }
    },

    /**
     * folder quota is updated
     *
     * @param {Tine.Expressomail.Model.Folder} record
     */
    onUpdateFolderQuota: function(record) {
        if (record.get('quota_usage')) {
            Tine.log.info('Folder "' + record.get('localname') + '" updated with quota values: '
                + record.get('quota_usage') + ' / ' + record.get('quota_limit'));

            this.getMainScreen().getCenterPanel().updateQuotaBar(record);
        }
    },

    /**
     * get active account
     * @return {Tine.Expressomail.Model.Account}
     */
    getActiveAccount: function() {
        var account = null;

        var treePanel = this.getMainScreen().getTreePanel();
        if (treePanel && treePanel.rendered) {
            account = treePanel.getActiveAccount();
        }

        if (account === null) {
            account = this.getAccountStore().getById(Tine.Expressomail.registry.get('preferences').get('defaultEmailAccount'));
        }

        if (account === null) {
            // try to get first account in store
            account = this.getAccountStore().getAt(0);
        }

        return account;
    },

    /**
     * get account store
     *
     * @return {Ext.data.JsonStore}
     */
    getAccountStore: function() {
        if (! this.accountStore) {
            Tine.log.debug('creating account store');

            // create store (get from initial data)
            this.accountStore = new Ext.data.JsonStore({
                fields: Tine.Expressomail.Model.Account,
                data: Tine.Expressomail.registry.get('accounts'),
                autoLoad: true,
                id: 'id',
                root: 'results',
                totalProperty: 'totalcount',
                proxy: Tine.Expressomail.accountBackend,
                reader: Tine.Expressomail.accountBackend.getReader(),
                listeners: {
                    scope: this,
                    'add': function (store, records) {
                        Tine.log.info('Account added: ' + records[0].get(Tine.Expressomail.Model.Account.getMeta('titleProperty')));
                        this.getMainScreen().getCenterPanel().action_write.setDisabled(! this.getActiveAccount());
                    },
                    'remove': function (store, record) {
                        Tine.log.info('Account removed: ' + record.get(Tine.Expressomail.Model.Account.getMeta('titleProperty')));
                        this.getMainScreen().getCenterPanel().action_write.setDisabled(! this.getActiveAccount());
                    }
                }
            });
        }

        return this.accountStore;
    },

    /**
     * show expressomail credentials dialog
     *
     * @param {Tine.Expressomail.Model.Account} account
     * @param {String} username [optional]
     */
    showCredentialsDialog: function(account, username) {
        Tine.Expressomail.credentialsDialog = Tine.widgets.dialog.CredentialsDialog.openWindow({
            windowTitle: String.format(this.i18n._('IMAP Credentials for {0}'), account.get('name')),
            appName: 'Expressomail',
            credentialsId: account.id,
            i18nRecordName: this.i18n._('Credentials'),
            recordClass: Tine.Tinebase.Model.Credentials,
            record: new Tine.Tinebase.Model.Credentials({
                id: account.id,
                username: username ? username : ''
            }),
            listeners: {
                scope: this,
                'update': function(data) {
                    var folderStore = this.getFolderStore();
                    if (folderStore.queriesPending.length > 0) {
                        // reload all folders of account and try to select inbox
                        var accountId = folderStore.queriesPending[0].substring(16, 56),
                            account = this.getAccountStore().getById(accountId),
                            accountNode = this.getMainScreen().getTreePanel().getNodeById(accountId);

                        folderStore.resetQueryAndRemoveRecords('parent_path', '/' + accountId);
                        account.set('all_folders_fetched', true);

                        accountNode.loading = false;
                        accountNode.reload(function(callback) {
                            Ext.each(accountNode.childNodes, function(node) {
                                if (Ext.util.Format.lowercase(node.attributes.localname) == 'inbox') {
                                    node.select();
                                    return false;
                                }
                            }, this);
                        });
                    } else {
                        this.checkMailsDelayedTask.delay(0);
                    }
                }
            }
        });
    }
});

/**
 * get flags store
 *
 * @return {Ext.data.JsonStore}
 */
Tine.Expressomail.getMailStore = function() {
    Tine.log.debug('Tine.Expressomail.getMailStore() - getting mail store');
    var store = Ext.StoreMgr.get('ExpressomailMailStore');

    if (!store) {
        // create store (get from initial registry data)
        store = new Tine.Tinebase.data.RecordStore({
            autoLoad: false,
            readOnly: false,
            proxy: Tine.Addressbook.emailaddressBackend,
            recordClass: Tine.Addressbook.Model.EmailAddress,
            sortInfo: {field: 'email', direction: 'ASC'}
        });
        
        var filter = Tine.Addressbook.Model.EmailAddress.getFilterModel();
        filter.push({field: 'email_query', operator: 'contains', value: '@'});
        filter.push({field: 'container_id', operator: 'equals', value: '/personal/'
                + Tine.Tinebase.registry.get('currentAccount').accountId});
        
        store.baseParams.filter = filter;
        // TODO: acertar o modelo de Contatos / Listas para ser possível ordar por nome
        //store.baseParams.sort = 'n_fn';
        store.baseParams.sort = 'email';
        store.baseParams.dir = 'ASC';
        
        Ext.StoreMgr.add('ExpressomailMailStore', store);
    }

    return store;
};

/**
 * @namespace Tine.Expressomail
 * @class Tine.Expressomail.MainScreen
 * @extends Tine.widgets.MainScreen
 *
 * MainScreen Definition
 */
Tine.Expressomail.MainScreen = Ext.extend(Tine.widgets.MainScreen, {
    /**
     * adapter fn to get folder tree panel
     *
     * @return {Ext.tree.TreePanel}
     */
    getTreePanel: function() {
        return this.getWestPanel().getContainerTreePanel();
    }
});

/**
 * get flags store
 *
 * @param {Boolean} reload
 * @return {Ext.data.JsonStore}
 */
Tine.Expressomail.loadFlagsStore = function(reload) {

    var store = Ext.StoreMgr.get('ExpressomailFlagsStore');

    if (!store) {
        // create store (get from initial registry data)
        store = new Ext.data.JsonStore({
            fields: Tine.Expressomail.Model.Flag,
            data: Tine.Expressomail.registry.get('supportedFlags'),
            autoLoad: true,
            id: 'id',
            root: 'results',
            totalProperty: 'totalcount'
        });

        Ext.StoreMgr.add('ExpressomailFlagsStore', store);
    }

    return store;
};

Tine.Expressomail.getMailStoreData = function(store) {
    
    var items = [];
    Ext.each(store.data.items, function(item){
        items.push(item.data);
    });
    
    return Ext.util.JSON.encode(items);
    
}

/**
 * Create a new Store from records
 */
Tine.Expressomail.createMailStore = function(json) {
    
    var items = Ext.util.JSON.decode(json);
    var records = [];
    Ext.each(items, function(item){
        records.push(new Tine.Addressbook.Model.EmailAddress(item));
    });
    
    var store = new Ext.data.SimpleStore({
        autoLoad: false,
        //data: items,
        readOnly: true,
        recordClass: Tine.Addressbook.Model.EmailAddress
    });
    
    store.add(records);
    
    Ext.apply(store, {
        createFilterFn: function(property, value, anyMatch, caseSensitive){
            if(Ext.isEmpty(value, false)){
                return false;
            }
            value = this.data.createValueMatcher(value, anyMatch, caseSensitive);
            return function(r){
                if (Ext.isArray(property)) {
                    var test = false;
                    Ext.each(property, function(p) {
                        test = value.test(r.data[p]) || test;
                    });
                    return test;
                } else {
                    return value.test(r.data[property]);
                }
            };
        } //,
//        filter : function(property, value, anyMatch, caseSensitive){
//            var fn = this.createFilterFn(property, value, anyMatch, caseSensitive);
//            return fn ? this.filterBy(fn) : this.clearFilter();
//        }
    });
    
    
    return store;
}

/**
 * add signature (get it from default account settings)
 *
 * @param {String} id
 * @return {String}
 */
Tine.Expressomail.getSignature = function(id) {

    var result = '',
        app = Tine.Tinebase.appMgr.get('Expressomail'),
        activeAccount = app.getMainScreen().getTreePanel().getActiveAccount();

    id = id || (activeAccount ? activeAccount.id : 'default');

    if (id === 'default') {
        id = Tine.Expressomail.registry.get('preferences').get('defaultEmailAccount');
    }

    var defaultAccount = app.getAccountStore().getById(id);
    var signature = (defaultAccount) ? defaultAccount.get('signature') : '';
    if (signature && signature != '') {
        // NOTE: signature is always in html, nl2br here would cause duplicate linebreaks!
        result = '<br><br><span id="expressomail-body-signature">-- <br>' + signature + '</span>';
    }

    return result;
};

/**
 * get email string (n_fn <email@host.tld>) from contact
 *
 * @param {Tine.Addressbook.Model.Contact} contact
 * @return {String}
 */
Tine.Expressomail.getEmailStringFromContact = function(contact) {
    Tine.log.debug('Tine.Expressomail.getEmailStringFromContact() - getting contact email');
    Tine.log.debug(contact);

    var result = contact.get('n_fn') + ' <';
    result += contact.getPreferredEmail();
    result += '>';

   if (contact.get('org_unit') != '' && contact.get('org_unit') != null ) {
        result += '  ' + contact.get('org_unit');
    }

    return result;
};

/**
 * generic exception handler for expressomail (used by folder and message backends and updateMessageCache)
 *
 * TODO move all 902 exception handling here!
 * TODO invent requery on 902 with cred. dialog
 *
 * @param {Tine.Exception|Object} exception
 */
Tine.Expressomail.handleRequestException = function(exception) {
    if (! exception.code && exception.responseText) {
        // we need to decode the exception first
        var response = Ext.util.JSON.decode(exception.responseText);
        exception = response.data;
    }

    Tine.log.warn('Request exception :');
    Tine.log.warn(exception);

    var app = Tine.Tinebase.appMgr.get('Expressomail');

    switch(exception.code) {
        case 910: // Expressomail_Exception_IMAP
        case 911: // Expressomail_Exception_IMAPServiceUnavailable
            Ext.Msg.show({
               title:   app.i18n._('IMAP Error'),
               msg:     exception.message ? exception.message : app.i18n._('No connection to IMAP server.'),
               icon:    Ext.MessageBox.ERROR,
               buttons: Ext.Msg.OK
            });
            break;

        case 912: // Expressomail_Exception_IMAPInvalidCredentials
            var accountId   = exception.account && exception.account.id ? exception.account.id : '',
                account     = accountId ? app.getAccountStore().getById(accountId): null,
                imapStatus  = account ? account.get('imap_status') : null;

            if (account) {
                account.set('all_folders_fetched', true);
                if (account.get('type') == 'system') {
                    // just show message box for system accounts
                    Ext.Msg.show({
                       title:   app.i18n._('IMAP Credentials Error'),
                       msg:     app.i18n._('Your email credentials are wrong. Please contact your administrator'),
                       icon:    Ext.MessageBox.ERROR,
                       buttons: Ext.Msg.OK
                    });
                } else {
                    app.showCredentialsDialog(account, exception.username);
                }
            } else {
                exception.code = 910;
                return this.handleRequestException(exception);
            }
            break;

        case 913: // Expressomail_Exception_IMAPFolderNotFound
            Ext.Msg.show({
               title:   app.i18n._('IMAP Error'),
               msg:     app.i18n._('One of your folders was deleted or renamed by another client. Please update the folder list of this account.'),
               icon:    Ext.MessageBox.ERROR,
               buttons: Ext.Msg.OK
            });
            // TODO reload account root node
            break;

        case 914: // Expressomail_Exception_IMAPMessageNotFound
            Tine.log.notice('Message was deleted by another client.');

            // remove message from store and select next message
            var requestParams = Ext.util.JSON.decode(exception.request).params,
                centerPanel = app.getMainScreen().getCenterPanel(),
                msg = centerPanel.getStore().getById(requestParams.id);

            if (msg) {
                var sm = centerPanel.getGrid().getSelectionModel(),
                    selectedMsgs = sm.getSelectionsCollection(),
                    nextMessage = centerPanel.getNextMessage(selectedMsgs);

                centerPanel.getStore().remove(msg);
                if (nextMessage) {
                    sm.selectRecords([nextMessage]);
                }
            }
            break;

        case 920: // Expressomail_Exception_SMTP
            Ext.Msg.show({
               title:   app.i18n._('SMTP Error'),
               msg:     exception.message ? exception.message : app.i18n._('No connection to SMTP server.'),
               icon:    Ext.MessageBox.ERROR,
               buttons: Ext.Msg.OK
            });
            break;

        case 930: // Expressomail_Exception_Sieve
            Ext.Msg.show({
               title:   app.i18n._('Sieve Error'),
               msg:     exception.message ? exception.message : app.i18n._('No connection to Sieve server.'),
               icon:    Ext.MessageBox.ERROR,
               buttons: Ext.Msg.OK
            });
            break;

        case 931: // Expressomail_Exception_SievePutScriptFail
            Ext.Msg.show({
               title:   app.i18n._('Save Sieve Script Error'),
               msg:     app.i18n._('Could not save script on Sieve server.') + (exception.message ? ' (' + exception.message + ')' : ''),
               icon:    Ext.MessageBox.ERROR,
               buttons: Ext.Msg.OK
            });
            break;

        case 932: // Expressomail_Exception_IMAPCacheTooMuchResults
            Ext.Msg.show({
               title:   app.i18n._('IMAP Backend Warning'),
               msg:     app.i18n._(exception.message),
               icon:    Ext.MessageBox.WARNING,
               buttons: Ext.Msg.OK
            });
            break;

        case 933: // Expressomail_Exception_IMAPFolderDuplicated
            Ext.Msg.show({
               title:   app.i18n._('IMAP Backend Warning'),
               msg:     app.i18n._(exception.message),
               icon:    Ext.MessageBox.WARNING,
               buttons: Ext.Msg.OK
            });
        break;

        default:
            Tine.Tinebase.ExceptionHandler.handleRequestException(exception);
            break;
    }
};


Tine.Expressomail.fixIEUserAgent = function()
{
    if(!Ext.isNewIE){
        return navigator.userAgent;

    }else{
        var re = /\(.*\)/,
            result = re.exec(navigator.userAgent)[0],
            original = result.substring(1, result.length -1),
            elements = original.split('; '),
            newUserAgent = '';
        for(var i = 0; i < 4; i++){
            newUserAgent = newUserAgent+elements[i]+'; ';
        }
        return navigator.userAgent.replace(original,newUserAgent.substr(0,newUserAgent.length -2));
    }
}

Tine.Expressomail.onSecurityAppletError = function () {
    // hides all pending messagebox shown on screen
    Ext.MessageBox.hide(); // 'loading criptography components' messagebox
    var app = Tine.Tinebase.appMgr.get('Expressomail');
    app.mainScreen.GridPanel.detailsPanel.isDecrypting = false; // clear decrypting flag
    app.mainScreen.GridPanel.getLoadMask().hide(); // 'loading' messagebox on GridPanel
    app.mainScreen.GridPanel.detailsPanel.getLoadMask().hide(); // 'loading' messagebox on GridDetailsPanel
    // purge all pending event listeners
    app.mainScreen.GridPanel.detailsPanel.purgeListeners();
    // show error message
    if (app.mainScreen.GridPanel.detailsPanel.record) {
        app.mainScreen.GridPanel.detailsPanel.showErrorTemplate(app.i18n._('Failure decrypting the message, please try again.'));
    }
    // focus selected message on grid
    app.mainScreen.GridPanel.focusSelectedMessage();
}

/**
 * send data to Security Applet
 * 
 * @todo test if applet is ready() if exceptions occurs try again in a few miliseconds
 */
Tine.Expressomail.toSecurityApplet = function (id, data, operation, tries){
    var MAXTRIES = 5;
    tries = tries || 1;
    try {
        var applet = document.getElementById('SecurityApplet');
        try {
            applet.isActive();
        } catch (err) {
            Tine.log.debug('Tine.Expressomail.toSecurityApplet() -> Applet not Active: try ' + tries);
            if (tries < MAXTRIES) {
                Tine.Expressomail.toSecurityApplet.defer(500, this, [id, data, operation, ++tries]);
            } else {
                Tine.Expressomail.onSecurityAppletError();
                return false;
            }
        }
        
        switch (operation) {
            case 'SIGN_AND_ENCRYPT' :
                applet.signAndEncryptMessage(Tine.Expressomail.fixIEUserAgent(), id, data);
                break;
            case 'ENCRYPT' :
                applet.encryptMessage(Tine.Expressomail.fixIEUserAgent(), id, data);
                break;
            case 'DECRYPT' :
                // Slicing data to send to applet. Workaround to liveConnect issue
                // 1MB
                for (var chunkLength = 0x100000, i = 0; i < data.length; i += chunkLength) {
                    if (i+chunkLength < data.length) {
                        applet.decryptMessage(Tine.Expressomail.fixIEUserAgent(), id, data.slice(i, i+chunkLength), false);
                    } else {
                        applet.decryptMessage(Tine.Expressomail.fixIEUserAgent(), id, data.slice(i), true);
                    }
                }
                break;
        }
        
    } catch (err) {
        Tine.log.debug('Tine.Expressomail.MessageEditDialog::onSecurityApplet() -> error:' + err.message);
        return false;
    }
    
    return true;
    
};

/**
 * Add applet to component or center panel
 * 
 */
Tine.Expressomail.addSecurityApplet = function (id, panel, region) {
    var app = Tine.Tinebase.appMgr.get('Expressomail');
    var centerPanel = Tine.Tinebase.appMgr.get('Expressomail').getMainScreen().getCenterPanel();
    if (typeof(panel) === 'undefined') {
        region = 'east';
        panel = centerPanel.getLayout()[region].panel;
    }
    
    if (panel && typeof(panel.securityApplet) === 'undefined') {
        Tine.Expressomail.AppletLoadaded = false;
        Ext.MessageBox.wait(app.i18n._('Loading Criptography Components...'), _('Please wait!'));
        
        Tine.Expressomail.afterAppletLoad = function(){
            Tine.Expressomail.AppletLoadaded = true;
            if (typeof(this.window) != 'undefined' && Ext.isFunction(this.window.show)) {
                this.window.show();
            }
            //Tine.WindowFactory.windowManager.each(function(data){data.show()});
            Ext.MessageBox.hide();
        }.createDelegate(this);
        
        //var testAppletLoad = Tine.Expressomail.testAppletLoad.createDelegate(this);
        //testAppletLoad.defer(30000, this, [id,0,panel,loadindAppletMessageBox]);
        panel.securityApplet = Tine.Expressomail.getSecurityApplet(id, region);
        panel.add(panel.securityApplet);
        panel.doLayout();
        panel.securityApplet.doLayout();
        panel.securityApplet.show();
    }
};

//Tine.Expressomail.testAppletLoad = function(id,tries,panel,msgbox){
//    tries = tries || 1;
//    try{
//        var applet = document.getElementById(id);
//        applet.isActive();
//        Tine.Expressomail.AppletLoadaded = true;
//        msgbox.hide();
//        if (typeof(this.window) != 'undefined' && Ext.isFunction(this.window.show)) {
//            this.window.show();
//        }
//    }catch(e){
//        Tine.log.debug(e);
//        if(tries < 11){
//             Tine.log.debug('Tine.Expressomail.testAppletLoad() -> Applet not Active: try ' + tries);
//             Tine.Expressomail.testAppletLoad.defer(5000,this,[id,++tries,panel,msgbox]);
//            
//        }
//        else{
//            var app = Tine.Tinebase.appMgr.get('Expressomail');
//            msgbox.hide();
//            panel.remove(panel.securityApplet,true); 
//            delete(panel.securityApplet);
//            panel.doLayout();
//            Tine.log.debug('Tine.Expressomail.testAppletLoad() -> Applet not Active: Max tries ');
//            Ext.MessageBox.alert(app.i18n._('Warning'),app.i18n._('Failure loading criptography components, please try again.'));
//        }
//    }
//    
//}

/**
 * Get applet's html
 *
 */
Tine.Expressomail.getSecurityApplet = function(id, region, params) {
/*
    var archive =
        'httpclient-4.2.6.jar, '+
        'httpcore-4.2.5.jar, '+
        'commons-codec-1.6.jar, '+
        'commons-logging-1.1.1.jar, '+
        'commons-lang3-3.1.jar, '+
        'guava-15.0.jar, '+
        'jackson-core-asl-1.9.13.jar, '+
        'jackson-mapper-asl-1.9.13.jar, '+
//        'javax.mail-1.5.1.jar, '+
        'mail.jar, '+
        'bcprov-jdk15on-1.49.jar, '+
        'bcpkix-jdk15on-1.49.jar, '+
        'bcmail-jdk15on-1.49.jar, '+
        'jericho-html-3.3.jar, '+
        'apache-mime4j-dom-0.7.2.jar, '+
        'apache-mime4j-core-0.7.2.jar, '+
        'picocontainer-2.14.3.jar, '+
        'expressocert-1.0.35.jar, '+
        'ExpressoCertMail.jar';
*/
    var archive = 'ExpressoCertMail-all.jar';


    var ieHtml = '<object id="'+ id +'" ' +
            'name="' + id + '" ' +
            'classid="clsid:CAFEEFAC-0016-0000-FFFF-ABCDEFFEDCBA">' +
            '<param name="codebase" value="/Expressomail/Java/"/>' +
            '<param name="archive" value="'+archive+'"/>' +
            '<param name="code" value="br.gov.serpro.expresso.security.applet.SmimeApplet"/>' +
//            '<param name="java_version" value="1.6*">' +
            '<param name="mayscript" value="true"/>' +
        '</object>';
    var ffHtml = '<embed id="' + id + '" ' +
            'name="' + id + '" ' +
            'codebase="/Expressomail/Java/" ' +
            'archive="'+archive+'" ' +
            'code="br.gov.serpro.expresso.security.applet.SmimeApplet" ' +
            'type="application/x-java-applet" ' +
//            'java_version="1.6*">' +
            'mayscript ' +
        '</embed>';

    return new Tine.Expressomail.SignatureAppletPanel({
        record: this.record,
        width:  0,
        height: 0,
        region: region,
        layout: 'fit',
        border: false,
//        autoShow: true,
        html: Ext.isIE ? ieHtml : ffHtml
    });
};

/**
 * Stub method to receive data from applet and relay it to the rigth ext component
 * Highly experimental
 * 
 * @todo buffer inside ext component. Use id to get component (var extObj = Ext.ComponentMgr.get(id);)
 */
var appletStub = function(last, id, response) {
    if (typeof(appletBuffer) === 'undefined') {
        appletBuffer = new Array();
    }
    
    if (typeof(appletBuffer.id) === 'undefined') {
        appletBuffer.id = new Array();
    }
    var data = '';
    if (last === 'false') {
        appletBuffer.id.push(response);
        return;
    } else {
        appletBuffer.id.push(response);
        data = Ext.util.JSON.decode(appletBuffer.id.join('').trim());
        delete appletBuffer.id;
    }
    
    var extObj = Ext.ComponentMgr.get(id);
    if (data.success) {
        var app = Tine.Tinebase.appMgr.get('Expressomail');
        if (app.mainScreen.GridPanel.detailsPanel.record != null) {
            app.mainScreen.GridPanel.detailsPanel.record.data.error_message = ''; //
        }

        if (data.signerCertificate != null){
            // Verify
            var store = new Tine.Tinebase.data.RecordStore({
                autoLoad: false,
                readOnly: true,
                proxy: Tine.Tinebase.DigitalCertificateBackend,
                recordClass: Tine.Tinebase.Model.DigitalCertificate
            });

            var verifyFailed = function (result, component){
                var msgOptions = {
                    title:      component.app.i18n._("Error Verifying Signer's Digital Certificate!"),
                    msg:        '',
                    buttons:    Ext.Msg.OK,
                    width:      500
                }
                if (result.success && result.totalRecords == 1){
                    if (result.records && result.records[0]){
                        msgOptions.msg = '<br /><span style="font-weight:bold">'+
                            component.app.i18n._('Details:')+': </span><br />';
                        Ext.each(result.records[0].get('messages'), function(msg){
                            msgOptions.msg += '<span style="padding-left:2em;color:#FF0000">'+
                                component.app.i18n._(msg)+'</span><br />';
                        });
                    } else {
                        msgOptions.msg = '<span style="padding-left:2em;color:#FF0000">'+
                            component.app.i18n._("Unknown Error. Please notify an Administrator")+
                            '</span><br />';
                    }
                } else {
                    msgOptions.msg = '<span style="padding-left:2em;color:#FF0000">'+
                        component.app.i18n._("Unknown Error. Please notify an Administrator")+
                        '</span><br />';
                }

                Ext.MessageBox.show(msgOptions);
                component.loadMask.hide();
            };
            var verifySuccess = function (result, appletData, component){
                var msg = appletData.mimeMsg ? appletData.mimeMsg : appletData.exprMsg;
                if (result.totalRecords == 1 && result.records && result.records[0].get('success')) {
                    component.fromApplet(msg, result.records[0]);
                } else {
                    verifyFailed(result, component);
                }
            };

            store.proxy.verifyCertificates([data.signerCertificate], {
                success: verifySuccess.createDelegate(this, [data, extObj], true),
                failure: verifyFailed.createDelegate(this, [extObj], true)
            });
            
            return;
        }
// TODO: This can change in the future.
        extObj.fromApplet(data.mimeMsg ? data.mimeMsg : data.exprMsg);
    } 
    else {
        Tine.Expressomail.onSecurityAppletError();
        var exception = data.exception;
        switch (exception.message) {
            case 'OPERATION_CANCELLED' :
                Tine.log.debug(exception);
                break;
            default :
                var title;
                switch (exception.type) {
                    case 'SIGN': 
                        title = extObj.app.i18n._('Error Signing Message!');
                        break;
                    case 'ENCRYPT' :
                    case 'SIGN_AND_ENCRYPT' :
                        title = extObj.app.i18n._('Error Encrypting Message!');
                        break;
                    case 'DECRYPT' : 
                        title = extObj.app.i18n._('Error Decrypting Message!');
                        break;
                }
                
                Tine.log.error(exception.message);
                Ext.MessageBox.show({
                    title: title,
                    msg: exception.message,
                    buttons: Ext.Msg.OK
                });
        }
        
        extObj.loadMask.hide();
    }
};
/*!
 * @author Fernando Lages
 * Some changes to Ext.form.HtmlEditor
 */

Tine.Expressomail.HtmlEditor = Ext.extend(Ext.form.HtmlEditor, {
    /**
     * @cfg {Array} fontFamilies An array of available font families
     */
    fontFamilies : {
        'Arial': true,
        'Courier New': true,
        'Tahoma': false,
        'Times New Roman': true,
        'Verdana': true,
        'Calibri': false,
        'Cambria': false,
        'Franklin Gothic book': false,
        'Myriad Pro': false,
        'Spranq Eco Sans': false,
        'Tekton Pro Ext': false
    },
    defaultFont: 'Tahoma',
    /**
     * @cfg {Array} fontSizes An array of available font sizes
     */
    fontSizes : [
        '8', '9', '10', '11', '12', '13', '14', '16', '18', '20', '24', '28', '32', '40', '48'
    ],
    // kept for IE8 compatibility
    fontSizesIE : [
        '10', '13', '16', '18', '24', '32', '48'
    ],
    defaultSize: '16',
    // character for unidentified styles
    unidentified: '\u200B',

    // private
    /**
     * Shortcut for translation on Expressomail's scope
     * @param {String} msgid - string to be translated
     * @returns {String} translated string
     */
    _ : function(msgid) {
        return this.messageEdit.app.i18n._(msgid);
    },

    // private
    /**
     * check if the font is installed
     * @param {String} fontName A string with the font to be checked
     * @return {Boolean} true if the font is installed
     */
    doesFontExist : function(fontName) {
        try {
            var canvas = document.createElement("canvas");
            var context = canvas.getContext("2d");
            var text = "abcdefghijklmnopqrstuvwxyz0123456789";
            context.font = "180px monospace";
            var baselineSize = context.measureText(text).width;
            context.font = "180px '" + fontName + "', monospace";
            var newSize = context.measureText(text).width;
            delete canvas;
            return (newSize !== baselineSize);
        }
        catch (e) {
            return (true);
        }
    },

    /**
     * get the font used at the selection
     * @param {Object} target The selection
     * @return {String} the font name
     */
    getFontFromSelection : function(target) {
        return this.getNodeStyle('fontFamily', target);
    },

    /**
     * get the first existent font from a list of fonts
     * @param {String} fontlist a list of fonts (comma sepparated)
     * @return {String} the name from the applied font
     */
    getAppliedFont : function(fontlist) {
        var font;
        var fonts = fontlist.replace(/,\s/g,",").split(",");
        if (fonts.length===1) {
            return fonts[0];
        }
        else {
            for(var i = 0, len = fonts.length; i< len; i++){
                font = fonts[i];
                if (this.fontFamilies[font]||this.doesFontExist(font)||font===this.unidentified) {
                    return font;
                }
            }
            return this.defaultFont;
        }
    },

    /**
     * get the style of the node
     * @param {String} stylename A string with the css style name
     * @param {Object} target The target node
     * @return {String} the definition for that style
     */
    getNodeStyle : function(stylename, target) {
        var selection = this.win.getSelection();
        var node = selection.anchorNode;
        var styles = [];
        var nodeStyle, focusStyle;
        if (target) {
            nodeStyle = this.getStyleFromNode(stylename, target);
            if (!this.isEmptySelection(selection)) {
                styles = this.getStylesFromSelection(stylename, selection, nodeStyle);
                if ((styles.length>1)) {
                    return this.unidentified;
                }
            }
            node = target;
            if (styles.length>0) {
                nodeStyle = nodeStyle||styles[0];
            }
        }
        else {
            nodeStyle = this.getStyleFromNode(stylename, node);
        }
        focusStyle = this.getStyleFromNode(stylename, selection.focusNode);
        if (!styles || (styles[0] && nodeStyle!=='' && styles[0]!==nodeStyle) ||
            (selection.focusOffset>0 && nodeStyle!=='' && focusStyle!=='' && focusStyle!==nodeStyle)) {
            return this.unidentified;
        }
        else {
            return nodeStyle||focusStyle;
        }
    },

    /**
     * get all styles applied to a selection
     * @param {String} stylename A string with the css style name
     * @param {Object} selection The selection
     * @param {String} docstyle The main document style
     * @return {String} the styles from the selection
     */
    getStylesFromSelection : function(stylename, selection, docstyle) {
        var style;
        var childNode;
        var node = selection.getRangeAt(0).cloneContents();
        var styles = [];
        for (var i=0; i<node.childNodes.length;i++) {
            childNode = node.childNodes[i];
            if (childNode.textContent.replace(/\u200B/g,"").length>0) {
                style = this.getStyleFromNode(stylename, childNode);
                if (style!=="" && styles.indexOf(style)<0) {
                    styles.push(style);
                }
                else {
                    styles.push(docstyle);
                }
            }
        }
        return styles;
    },

    /**
     * get all styles applied to a selection
     * @param {String} stylename A string with the css style name
     * @param {Object} node the node to check
     * @return {String} the style from the node
     */
    getStyleFromNode : function(stylename, node) {
        var value;
        var fontstyle;
        if (stylename==='fontFamily') {
            fontstyle = 'face';
        }
        else if (stylename==='fontSize') {
            fontstyle = 'size';
        }
        if (node) {
            if (!value && node.style && (!node.tagName ||
                 (node.tagName && node.tagName.toLowerCase()!=='html' && node.tagName.toLowerCase()!=='body'))) {
                value = node.style[stylename].replace(/'/g,""); // clean single quotes in style
            }
            while (!value && node.parentNode && (!node.tagName ||
                    (node.tagName && node.tagName.toLowerCase()!=='html' && node.tagName.toLowerCase()!=='body'))) {
                node = node.parentNode;
                if (node.style) {
                    value = node.style[stylename].replace(/'/g,""); // clean single quotes in style
                }
                if (!value && node.attributes && node.attributes[fontstyle]) {
                    var size = node.attributes[fontstyle].value;
                    if (size) {
                        value = this.fontSizesIE[size-1]; // applies even though it is not IE
                    }
                }
            }
        }
        else {
            value = this.win.document.body.style[stylename];
        }
        if (value) {
            return this.getAppliedFont(value);
        }
        else {
            return "";
        }
    },

    /**
     * get if the selection point is empty
     * @param {Object} selection The selection
     * @return {Boolean} true if it is empty
     */
    isEmptySelection : function(selection) {
        return (selection.anchorOffset===selection.focusOffset && selection.anchorNode===selection.focusNode);
    },

    // private
    createFontFamilyOptions : function(){
        if (!Ext.isIE && !Ext.isNewIE) {
            var menutpl = new Ext.XTemplate(
                '<a id="{itemId}" class="x-menu-item x-menu-fontfamily-noicon">',
                    '<tpl if="true">',
                        '<span class="x-edit-fontfamily-text" style="font-family:{text}">{text}</span>',
                    '</tpl>',
                '</a>'
            );
            menutpl.compile();
            var ffsHandler = function(e) {
                var font = e.text;
                this.focus();
                this.relayCmd('stylewithcss', true);
                this.relayCmd('fontname',font);
                this.deferFocus();
            };
            var ffsToggle = function(state) {
                state = state === undefined ? !this.pressed : !!state;
                if(state !== this.pressed) {
                    if(this.rendered){
                        this.el[state ? 'addClass' : 'removeClass']('x-btn-pressed');
                        this.el[state ? 'addClass' : 'removeClass']('x-menu-item-selected');
                        this.pressed = state;
                    }
                    this.fireEvent('toggle', this, state);
                    if(this.toggleHandler){
                        this.toggleHandler.call(this.scope || this, this, state);
                    }
                }
                return this;
            };
            var buf = [], fs = this.fontFamilies, ff, lc;
            for (var ff in fs) {
                lc = ff.toLowerCase();
                buf.push({
                    itemId : 'fontfamily'+ff,
                    enableToggle: true,
                    handler: ffsHandler,
                    disabled: !(fs[ff]||this.doesFontExist(ff)),
                    tooltip: ff,
                    text: ff,
                    hideLabel: true,
                    textCls: 'x-edit-fontfamily-text',
                    group: 'fontfamily',
                    toggle: ffsToggle,
                    itemTpl: menutpl,
                    scope: this
                });
            }
            return buf;
        }
        else {
            var buf = [], fs = this.fontFamilies, ff, lc;
            for (var ff in fs) {
                lc = ff.toLowerCase();
                buf.push(
                    '<option value="',lc,'" style="font-family:',ff,';"',
                        (this.defaultFont === lc ? ' selected="true">' : '>'),
                        ff,
                    '</option>'
                );
            }
            return buf.join('');
        }
    },

    createFontSizeOptions : function(){
        if (!Ext.isIE && !Ext.isNewIE) {
            var menutpl = new Ext.XTemplate(
                '<a id="{itemId}" class="x-menu-item x-menu-fontsize-noicon">',
                    '<tpl if="true">',
                        '<span class="{iconCls}">a</span>',
                        '<span class="x-edit-fontsize-text">{text}</span>',
                    '</tpl>',
                    '<hr class="separator">',
                '</a>'
            );
            menutpl.compile();
            var fssHandler = function(e) {
                var size = e.text;
                var selection = this.win.getSelection();
                var node = selection.focusNode;
                var sentence;
                if (this.isEmptySelection(selection)) {
                    sentence = '&nbsp;';
                }
                else {
                    if (selection.rangeCount) {
                        var container = document.createElement("div");
                        for (var i = 0, len = selection.rangeCount; i < len; ++i) {
                            container.appendChild(selection.getRangeAt(i).cloneContents());
                        }
                        sentence = container.innerHTML;
                        sentence = sentence.replace(/font-size:.[^;"']*/g,""); // clean inner font-size settings
                        sentence = sentence.replace(/<(.[^>])*><\/(.[^>])*>/g,""); // clean empty tags
                    } else {
                        sentence = selection.toString();
                    }
                }
                this.insertAtCursor("<span style='font-size:"+size+"px'>" + sentence + "</span>");
                selection.extend (selection.focusNode, 0);
                this.deferFocus();
            };
            var fssToggle = function(state) {
                state = state === undefined ? !this.pressed : !!state;
                if(state !== this.pressed) {
                    if(this.rendered){
                        this.el[state ? 'addClass' : 'removeClass']('x-btn-pressed');
                        this.el[state ? 'addClass' : 'removeClass']('x-menu-item-selected');
                        this.pressed = state;
                    }
                    this.fireEvent('toggle', this, state);
                    if(this.toggleHandler){
                        this.toggleHandler.call(this.scope || this, this, state);
                    }
                }
                return this;
            };
            var buf = [], fs = this.fontSizes, ff, lc;
            for(var i = 0, len = fs.length; i< len; i++){
                ff = fs[i];
                lc = ff.toLowerCase();
                buf.push({
                    itemId : 'fontsize'+ff,
                    enableToggle: true,
                    handler: fssHandler,
                    clickEvent:'mousedown',
                    tooltip: ff,
                    text: ff,
                    hideLabel: true,
                    iconCls: 'x-edit-fontsize'+ff,
                    textCls: 'x-edit-fontsize-text',
                    itemCls: 'x-menu-item',
                    group: 'fontsize',
                    toggle: fssToggle,
                    itemTpl: menutpl,
                    scope: this
                });
            }
            return buf;
        }
        else {
            var buf = [], fs = this.fontSizesIE, ff, lc;
            for(var i = 0, len = fs.length; i< len; i++){
                ff = fs[i];
                lc = ff.toLowerCase();
                buf.push(
                    '<option value="',lc,'" style="font-size:',ff,';"',
                        (this.defaultSize === lc ? ' selected="true">' : '>'),
                        ff,
                    '</option>'
                );
            }
            return buf.join('');
        }
    },

    /*
     * Protected method that will not generally be called directly. It
     * is called when the editor creates its toolbar. Override this method if you need to
     * add custom toolbar buttons.
     * @param {HtmlEditor} editor
     */
    createToolbar : function(editor){
        var items = [];
        var tipsEnabled = Ext.QuickTips && Ext.QuickTips.isEnabled();


        function btn(id, toggle, handler){
            return {
                itemId : id,
                cls : 'x-btn-icon',
                iconCls: 'x-edit-'+id,
                enableToggle:toggle !== false,
                scope: editor,
                handler:handler||editor.relayBtnCmd,
                clickEvent:'mousedown',
                tooltip: tipsEnabled ? editor.supr().buttonTips[id] || undefined : undefined,
                overflowText: editor.supr().buttonTips[id].title || undefined,
                tabIndex:-1
            };
        }


        if(this.enableFont && !Ext.isSafari2){
            if (!Ext.isIE && !Ext.isNewIE) {
                var ffsToggleHandler = function (e) {
                    var target;
                    if (e) {
                        var target = e.target;
                    }
                    else {
                        var target = editor.win.getSelection();
                    }
                    var font = editor.getFontFromSelection(target);
                    if (font) {
                        this.btnEl.dom.className = 'x-btn-text no-icon x-menu-fontfamily-hideicon';
                        this.btnEl.dom.innerHTML = font.replace(/'/g,"");
                    }
                    Ext.each(this.menu.items, function(b, i){
                        b = this.menu.items.get(i);
                        b.toggle(font===b.text);
                    }, this);
                };
                this.fontSelectItem = new Ext.Toolbar.Button({
                    xtype: 'tbsplit',
                    id: 'fontfamily_button',
                    itemId: 'fontfamily',
                    cls: 'x-btn-icon x-menu-layout-fontfamily',
                    iconCls: 'no-icon x-menu-fontsize-hideicon',
                    tooltip: this._('Select font name'),
                    enableToggle: true,
                    menu: {
                        showSeparator: false,
                        items: this.createFontFamilyOptions()
                    },
                    toggle: ffsToggleHandler,
                    scope: this
                });
            }
            else {
                this.fontSelectItem = new Ext.Toolbar.Item({
                   autoEl: {
                        tag:'select',
                        cls:'x-font-select',
                        html: this.createFontFamilyOptions()
                   }
                });
            }

            items.push(
                this.fontSelectItem,
                '-'
            );
        }

        if(this.enableFontSize){
            if (!Ext.isIE && !Ext.isNewIE) {
                var fssToggleHandler = function (e) {
                    var target;
                    if (e) {
                        var target = e.target;
                    }
                    var size = editor.getNodeStyle('fontSize',target)||editor.defaultSize;
                    if (size) {
                        size = size.replace('px','');
                        if (!parseInt(size)) {
                            size = editor.unidentified;
                        }
                        this.btnEl.dom.className = 'x-btn-text no-icon x-menu-fontsize-hideicon';
                        this.btnEl.dom.innerHTML = size;
                    }
                    Ext.each(this.menu.items, function(b, i){
                        b = this.menu.items.get(i);
                        b.toggle(size===b.text.replace('px',''));
                    }, this);
                };

                this.fontSelectSize = new Ext.Toolbar.Button({
                    xtype: 'tbsplit',
                    id: 'fontsize_button',
                    itemId: 'fontsize',
                    cls: 'x-btn-icon x-menu-layout-fontsize',
                    iconCls: 'no-icon x-menu-fontsize-hideicon',
                    tooltip: this._('Select font size'),
                    enableToggle: true,
                    menu: {
                        hideLabel: true,
                        showSeparator: true,
                        items: this.createFontSizeOptions(),
                        layout: 'menu',
                        cls: 'x-menu-layout-fontsize'
                    },
                    showSeparator: true,
                    toggle: fssToggleHandler,
                    scope: this
                });
            }
            else {
                this.fontSelectSize = new Ext.Toolbar.Item({
                   autoEl: {
                        tag:'select',
                        cls:'x-font-select',
                        html: this.createFontSizeOptions()
                   }
                });
            }

            items.push(
                this.fontSelectSize,
                '-'
            );
        }

        if(this.enableFormat){
            items.push(
                btn('bold'),
                btn('italic'),
                btn('underline')
            );
        }

        if(this.enableColors){
            items.push(
                '-', {
                    itemId:'forecolor',
                    cls:'x-btn-icon',
                    iconCls: 'x-edit-forecolor',
                    clickEvent:'mousedown',
                    tooltip: tipsEnabled ? editor.buttonTips.forecolor || undefined : undefined,
                    tabIndex:-1,
                    menu : new Ext.menu.ColorMenu({
                        allowReselect: true,
                        focus: Ext.emptyFn,
                        value:'000000',
                        plain:true,
                        listeners: {
                            scope: this,
                            select: function(cp, color){
                                this.execCmd('forecolor', Ext.isWebKit || Ext.isIE ? '#'+color : color);
                                this.deferFocus();
                            }
                        },
                        clickEvent:'mousedown'
                    })
                }, {
                    itemId:'backcolor',
                    cls:'x-btn-icon',
                    iconCls: 'x-edit-backcolor',
                    clickEvent:'mousedown',
                    tooltip: tipsEnabled ? editor.buttonTips.backcolor || undefined : undefined,
                    tabIndex:-1,
                    menu : new Ext.menu.ColorMenu({
                        focus: Ext.emptyFn,
                        value:'FFFFFF',
                        plain:true,
                        allowReselect: true,
                        listeners: {
                            scope: this,
                            select: function(cp, color){
                                if(Ext.isGecko){
                                    this.execCmd('useCSS', false);
                                    this.execCmd('hilitecolor', color);
                                    this.execCmd('useCSS', true);
                                    this.deferFocus();
                                }else{
                                    this.execCmd(Ext.isOpera ? 'hilitecolor' : 'backcolor', Ext.isWebKit || Ext.isIE ? '#'+color : color);
                                    this.deferFocus();
                                }
                            }
                        },
                        clickEvent:'mousedown'
                    })
                }
            );
        }

        if(!Ext.isSafari2){
            if(this.enableLinks){
                items.push(
                    '-',
                    btn('createlink', false, this.createLink)
                );
            }

            if(this.enableLists){
                items.push(
                    '-',
                    btn('insertorderedlist'),
                    btn('insertunorderedlist')
                );
            }
            if(this.enableSourceEdit){
                items.push(
                    '-',
                    btn('sourceedit', true, function(btn){
                        this.toggleSourceEdit(!this.sourceEditMode);
                    })
                );
            }
        }

        // build the toolbar
        var tb = new Ext.Toolbar({
            renderTo: this.wrap.dom.firstChild,
            items: items
        });

        if (Ext.isIE || Ext.isNewIE) {
            if (this.fontSelectItem) {
                this.fontSelect = this.fontSelectItem.el;

                this.mon(this.fontSelect, 'change', function(){
                    var font = this.fontSelect.dom.value;
                    this.relayCmd('fontname', font);
                    this.deferFocus();
                }, this);
            }
            if (this.fontSelectSize) {
                this.fontSelectS = this.fontSelectSize.el;

                this.mon(this.fontSelectS, 'change', function(){
                    var size = this.fontSelectS.dom.value;
                    this.relayCmd('fontsize', this.fontSizesIE.indexOf(size)+1);
                    this.deferFocus();
                }, this);
            }
        }

        // stop form submits
        this.mon(tb.el, 'click', function(e){
            e.preventDefault();
        });

        this.tb = tb;
    },

    // private
    initEditor : function(){
        //Destroying the component during/before initEditor can cause issues.
        try{
            var dbody = this.getEditorBody(),
                ss = this.el.getStyles('font-size', 'font-family','background-image', 'background-repeat'),
                doc,
                fn;

            ss['font-family'] = 'Tahoma, Arial, Helvetica, Sans-Serif'; // w3c
            ss['background-attachment'] = 'fixed'; // w3c
            dbody.bgProperties = 'fixed'; // ie

            Ext.DomHelper.applyStyles(dbody, ss);

            doc = this.getDoc();

            if(doc){
                try{
                    Ext.EventManager.removeAll(doc);
                }catch(e){}
            }

            /*
             * We need to use createDelegate here, because when using buffer, the delayed task is added
             * as a property to the function. When the listener is removed, the task is deleted from the function.
             * Since onEditorEvent is shared on the prototype, if we have multiple html editors, the first time one of the editors
             * is destroyed, it causes the fn to be deleted from the prototype, which causes errors. Essentially, we're just anonymizing the function.
             */
            fn = this.onEditorEvent.createDelegate(this);
            Ext.EventManager.on(doc, {
                mousedown: fn,
                mouseup: fn,
                dblclick: fn,
                click: fn,
                keyup: fn,
                buffer:100
            });

            if(Ext.isGecko){
                Ext.EventManager.on(doc, 'keypress', this.applyCommand, this);
            }
            if(Ext.isIE || Ext.isWebKit || Ext.isOpera){
                Ext.EventManager.on(doc, 'keydown', this.fixKeys, this);
            }
            doc.editorInitialized = true;
            this.initialized = true;
            this.pushValue();
            this.setReadOnly(this.readOnly);
            this.fireEvent('initialize', this);
            if (!Ext.isIE && !Ext.isNewIE) {
                var font = this.getFontFromSelection();
                if (font) {
                    this.defaultFont = font;
                    doc.body.style.fontFamily = font;
                    this.fontSelectItem.toggle();
                }
                var size = this.getNodeStyle('fontSize');
                if (size) {
                    doc.body.style.fontSize = size;
                    this.fontSelectSize.toggle();
                }
            }
        }catch(e){}
    },

    /**
     * Protected method that will not generally be called directly. It triggers
     * a toolbar update by reading the markup state of the current selection in the editor.
     */
    updateToolbar: function(e){

        if(this.readOnly){
            return;
        }

        if(!this.activated){
            this.onFirstFocus();
            return;
        }

        var btns = this.tb.items.map,
            doc = this.getDoc();

        if(this.enableFont && !Ext.isSafari2){
            if (!Ext.isIE && !Ext.isNewIE) {
                this.fontSelectItem.toggle(e);
            }
            else {
                var name = (doc.queryCommandValue('FontName')||this.defaultFont).toLowerCase();
                if(name != this.fontSelect.dom.value){
                    this.fontSelect.dom.value = name;
                }
            }
        }
        if(this.enableFontSize && !Ext.isSafari2){
            if (!Ext.isIE && !Ext.isNewIE) {
                this.fontSelectSize.toggle(e);
            }
            else {
                var size = (doc.queryCommandValue('FontSize')||this.defaultSize);
                if(size != this.fontSelectS.dom.value){
                    this.fontSelectS.dom.value = this.fontSizesIE[size-1];
                }
            }
        }
        if(this.enableFormat){
            btns.bold.toggle(doc.queryCommandState('bold'));
            btns.italic.toggle(doc.queryCommandState('italic'));
            btns.underline.toggle(doc.queryCommandState('underline'));
        }
        if(!Ext.isSafari2 && this.enableLists){
            btns.insertorderedlist.toggle(doc.queryCommandState('insertorderedlist'));
            btns.insertunorderedlist.toggle(doc.queryCommandState('insertunorderedlist'));
        }

        Ext.menu.MenuMgr.hideAll();

        this.syncValue();
    }

});
Ext.reg('htmleditor', Ext.form.HtmlEditor);/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

Ext.override(Tine.Expressomail.HtmlEditor, {
    onEditorEvent: function(e) {
        if (Ext.isIE || Ext.isNewIE) {
            this.currentRange = this.getDoc().selection.createRange();
        }
        this.updateToolbar(e);
    },
    insertAtCursor: function(text) {
        if (Ext.isIE || Ext.isNewIE) {
            this.win.focus();
            var r = this.currentRange;
            if (r) {
                //r.expand("word");
                r.collapse(true);
            }
            else {
                r = this.getDoc().selection.createRange();
                if (r) {
                    r.collapse(true);
                }
            }
            if (r) {
                r.pasteHTML(text);
                this.syncValue();
                this.deferFocus();
            }

        } else if (Ext.isSafari) {
            this.execCmd('InsertText', text);
            this.deferFocus();
        } else { //if (Ext.isGecko || Ext.isOpera) {
            this.win.focus();
            this.execCmd('InsertHTML', text);
            this.deferFocus();
        }
    }
});

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.ComposeEditor
 * @extends     Ext.form.HtmlEditor
 *
 * <p>Compose HTML Editor</p>
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.ComposeEditor
 */
Tine.Expressomail.ComposeEditor = Ext.extend(Tine.Expressomail.HtmlEditor, {

    cls: 'felamimail-message-body-html',
    name: 'body',
    allowBlank: true,

    getDocMarkup: function(){
        var markup = '<html>'
            + '<head>'
            + '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">'
            + '<title></title>'
            + '<style type="text/css">'
                // standard css reset
                + "html,body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,form,fieldset,input,p,th,td{margin:0;padding:0;}img,body,html{border:0;}address,caption,cite,code,dfn,th,var{font-style:normal;font-weight:normal;}ol,ul {list-style:none;}caption,th {text-align:left;}h1,h2,h3,h4,h5,h6{font-size:100%;}q:before,q:after{content:'';}"
                // small forms
                + "html,body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,form,fieldset,input,p,blockquote,th,td{font-size: 100%;}"
                // standard blockquotes
                + "blockquote{margin-top:0;margin-bottom:0;}"
                // lists
                + "ul {list-style:circle outside; margin-left: 20px;}"
                + "ol {list-style:decimal outside; margin-left: 20px;}"
                // fmail special
                + '.felamimail-body-blockquote {'
                    + 'margin: 5px 10px 0 3px;'
                    + 'padding-left: 10px;'
                    + 'border-left: 2px solid #000088;'
                + '} '
            + '.editor-body-panel { font-size: 100% !important; font-family: Tahoma,Arial,Helvetica,Sans-Serif !important; }'
            + 'td.editor-wysiwyg-table { border: dotted 1px #ccc !important; }'
            + '</style>'
            + '</head>'
            + '<body class="editor-body-panel" style="padding: 5px 0px 0px 5px; margin: 0px">'
            + '</body></html>';

        return markup;
    },

    getContainer: function() {
            return this.iframe;
    },

    /**
     * @private
     */
    initComponent: function() {

        this.plugins = [
            new Ext.ux.form.HtmlEditor.UploadImage(),
            new Ext.ux.form.HtmlEditor.Table(),
            new Ext.ux.form.HtmlEditor.TextAlign(),
            new Ext.ux.form.HtmlEditor.IndentOutdent(),
            new Ext.ux.form.HtmlEditor.RemoveFormat(),
	    new Ext.ux.form.HtmlEditor.SpecialKeys(),
            new Ext.ux.form.HtmlEditor.UndoRedo()
        ];
        if (!this.messageEdit.encrypted) {
            this.plugins.push(new Ext.ux.form.HtmlEditor.SpellChecker());
        }

        Tine.Expressomail.ComposeEditor.superclass.initComponent.call(this);
    },

    /**
     * @private
     */
    initEditor: function() {
        Tine.Expressomail.ComposeEditor.superclass.initEditor.apply(this, arguments);
        try{
            var doc = this.getDoc();
            // add fixKeys to treat keys on FF and IE9/10
            if(Ext.isGecko || Ext.isNewIE){
                Ext.EventManager.on(doc, 'keydown', this.fixKeys, this);
            }
        }catch(e){}
    },
    
    // *Fix* Overridding the onRender method, in order to
    // unset the height and width property, so that the
    // layout manager won't consider this field to be of
    // fixed dimension, thus ignoring the flex option
    onRender: function () {
        Tine.Expressomail.ComposeEditor.superclass.onRender.apply(this, arguments);
        delete this.height;
        delete this.width;
        if (this.messageEdit.encrypted) {
            this.getToolbar().addClass('x-html-editor-encrypted');
        }
        else {
            this.getToolbar().addClass('x-html-editor-decrypted');
        }
    }
});

Ext.namespace('Ext.ux.form.HtmlEditor');

/**
 * @class Ext.ux.form.HtmlEditor.EndBlockquote
 * @extends Ext.util.Observable
 *
 * plugin for htmleditor that ends blockquotes on ENTER
 * tested with chrome, sarari, FF4+
 * fallsback for old (non IE) browser which works for easy structures
 * does not work with IE yet
 *
 * TODO move this to ux dir
 */
Ext.ux.form.HtmlEditor.EndBlockquote = Ext.extend(Ext.util.Observable , {

    // private
    init: function(cmp){
            this.cmp = cmp;
            this.cmp.on('initialize', this.onInit, this);
    },

    // private
    onInit: function() {
        if (Ext.isIE) {
            Ext.EventManager.on(this.cmp.getDoc(), {
                'keydown': this.endBlockquoteIE,
                scope: this
            });
        } else if (Ext.isFunction(this.cmp.win.getSelection().modify)) {
            Ext.EventManager.on(this.cmp.getDoc(), {
                'keyup': this.endBlockquoteHTML5,
                scope: this
            });
        } else {
            Ext.EventManager.on(this.cmp.getDoc(), {
                'keydown': this.endBlockquoteHTML4,
                scope: this
            });
        }

    },

    /**
     * on keyup
     *
     * @param {Event} e
     */
    endBlockquoteIE: function(e) {
        if (e.getKey() == e.ENTER) {

            e.stopEvent();
            e.preventDefault();

            var s = this.cmp.win.document.selection,
                r = s.createRange(),
                doc = this.cmp.getDoc(),
                anchor = r.parentElement(),
                level = this.getBlockquoteLevel(anchor),
                scrollTop = doc.body.scrollTop;

            if (level > 0) {
                r.moveStart('word', -1);
                r.moveEnd('textedit');
                var fragment = r.htmlText;
                r.execCommand('Delete');

                var newText = doc.createElement('p'),
                    newTextMark = '###newTextHere###' + Ext.id(),
                    fragmentMark = '###fragmentHere###' + Ext.id();
                newText.innerHTML = newTextMark + fragmentMark;
                doc.body.appendChild(newText);

                r.expand('textedit');
                r.findText(fragmentMark);
                r.select();
                r.pasteHTML(fragment);

                r.expand('textedit');
                r.findText(newTextMark);
                r.select();
                r.execCommand('Delete');

                // reset scroller
                doc.body.scrollTop = scrollTop;

                this.cmp.syncValue();
                this.cmp.deferFocus();
            }

        }

        return;
    },

    /**
     * on keyup
     *
     * @param {Event} e
     */
    endBlockquoteHTML5: function(e) {
        // Chrome, Safari, FF4+
        if (e.getKey() == e.ENTER) {
            var s = this.cmp.win.getSelection(),
                r = s.getRangeAt(0),
                doc = this.cmp.getDoc(),
                level = this.getBlockquoteLevel(s.anchorNode),
                scrollTop = doc.body.scrollTop;

            if (level > 0) {
                // cut from cursor to end of the document
                if (s.anchorNode.nodeName == '#text') {
                    r.setStartBefore(s.anchorNode.parentNode);
                }
                s.modify("move", "backward", "character");
                r.setEndAfter(doc.body.lastChild);
                var fragmet = r.extractContents();

                // insert paragraph for new text and move cursor in
                // NOTE: we need at least one character in the newText to move cursor in
                var newText = doc.createElement('p');
                newText.innerHTML = '&nbsp;';
                doc.body.appendChild(newText);
                s.modify("move", "forward", "character");

                // paste rest of document
                doc.body.appendChild(fragmet);

                // reset scroller
                doc.body.scrollTop = scrollTop;
            }
        }
    },

    /**
     * on keydown
     *
     * @param {Event} e
     */
    endBlockquoteHTML4: function(e) {
        if (e.getKey() == e.ENTER) {
            var s = this.cmp.win.getSelection(),
                r = s.getRangeAt(0),
                doc = this.cmp.getDoc(),
                level = this.getBlockquoteLevel(s.anchorNode);

            if (level > 0) {
                e.stopEvent();
                e.preventDefault();

                this.cmp.win.focus();
                if (level == 1) {
                    this.cmp.insertAtCursor('<br /><blockquote class="felamimail-body-blockquote"><br />');
                    this.cmp.execCmd('outdent');
                    this.cmp.execCmd('outdent');
                } else if (level > 1) {
                    for (var i=0; i < level; i++) {
                        this.cmp.insertAtCursor('<br /><blockquote class="felamimail-body-blockquote">');
                        this.cmp.execCmd('outdent');
                        this.cmp.execCmd('outdent');
                    }
                    var br = doc.createElement('br');
                    r.insertNode(br);
                }
                this.cmp.deferFocus();
            }
        }
    },

    /**
     * get blockquote level helper
     *
     * @param {DOMNode} node
     * @return {Integer}
     */
    getBlockquoteLevel: function(node) {
        var result = 0;

        while (node.nodeName == '#text' || node.tagName.toLowerCase() != 'body') {
            if (node.tagName && node.tagName.toLowerCase() == 'blockquote') {
                result++;
            }
            node = node.parentNode ? node.parentNode : node.parentElement;
        }

        return result;
    }
});

/**
 * @class Ext.ux.form.HtmlEditor.SpecialKeys
 * @extends Ext.util.Observable
 *
 * plugin for htmleditor that fires events for special keys (like CTRL-ENTER and SHIFT-TAB)
 *
 * TODO move this to ux dir
 */
Ext.ux.form.HtmlEditor.SpecialKeys = Ext.extend(Ext.util.Observable , {
    // private
    init: function(cmp){
        this.cmp = cmp;
        this.cmp.on('initialize', this.onInit, this);
    },
    // private
    onInit: function(){
        Ext.EventManager.on(this.cmp.getDoc(), {
            'keydown': this.onKeydown,
            scope: this
        });
    },

    /**
     * on keydown
     *
     * @param {Event} e
     *
     * TODO try to prevent TAB key event from inserting a TAB in the editor
     */
    onKeydown: function(e) {
        if (e.getKey() == e.TAB && e.shiftKey || e.getKey() == e.ENTER && e.ctrlKey) {
            this.cmp.fireEvent('keydown', e);
        }
    }
});

/**
 * @class Ext.ux.form.HtmlEditor.UndoRedo
 * @extends Ext.util.Observable
 *
 * plugin for htmleditor that fires events for undo/redo keys (CTRL-Z and CTRL-Y)
 *
 * TODO move this to ux dir
 */
Ext.ux.form.HtmlEditor.UndoRedo = Ext.extend(Ext.util.Observable , {
    volume: -1,
    history: new Array(),
    index: 0,
    placeholder: 0,
    count: 0,
    ignore: false,

    // private
    init: function(cmp){
        this.cmp = cmp;
        this.cmp.on('initialize', this.onInit, this);
        this.cmp.on('sync', this.onSync, this);
    },
    // private
    onInit: function(){
        Ext.EventManager.on(this.cmp.getDoc(), {
            'keydown': this.onKeydown,
            scope: this
        });
    },

    /**
     * on keydown
     *
     * @param {Event} e
     */
    onKeydown: function(e) {
        if (e.ctrlKey) {
            if (e.getKey() == e.Z) {
                this.undo();
                e.preventDefault();
            }
            if (e.getKey() == e.Y) {
                this.redo();
                e.preventDefault();
            }
        }
    },


    /**
     * Protected method that will not generally be called directly. Syncs the contents
     * of the editor iframe with the textarea.
     */
    onSync : function(){
        if(this.cmp.initialized){
            if (this.ignore) {
                this.ignore = false;
            }
            else {
                this.updateHistory();
            }
        }
    },

    updateHistory : function(){
        // get the current html content from the element
        var content = this.cmp.el.dom.value;

        // if no historic records exist yet or content has
        // changed since the last record then
        if (this.index == 0 || this.history[this.index] == undefined ||this.history[this.index].content != content) {
            var last_index = this.index;

            // if size of rollbacks has been reached then drop
            // the oldest record from the array
            if (this.count == this.volume) {
                this.history.shift();
                this.placeholder--;
            }

            // else increment the index
            else {
                this.index++;
            }

            // record the changed content and cursor position
            if (!Ext.isIE && !Ext.isIE9) {
                var selection = this.cmp.getDoc().getSelection();
            }
            try {
                var range;
                if (Ext.isIE || Ext.isIE9)
                    range = null;
                else {
                    // Avoid error in getRangeAt(0)
                    if(selection.rangeCount)
                        range = selection.getRangeAt(0);
                    else
                        // Avoid error in operations using range variable
                        range = document.createRange();
                }
                if (Ext.isGecko) {
                    this.history[this.index] = {
                        content: content,
                        bookmark: {startOffset:range.startOffset,endOffset:range.endOffset,startContainer:range.startContainer,endContainer:range.endContainer}
                    };
                }
                else {
                    this.history[this.index] = {
                        content: content,
                        bookmark: range
                    };
                }
                this.count = this.index;
            }
            catch (e) {
                this.index = last_index;
            }
        }
    },

    // perform the undo
    undo: function() {

        // ensure that there is data to undo
        if (this.index > 1) {

            // decrement the index
            this.index--;

            // if in source edit mode then update the element directly
            if (this.cmp.sourceEditMode) {
                this.resetElement();
            }

            // else update the editor body
            else {
                this.reset();

            // ignore next record request as syncValue is called
            // by Ext.form.HtmlEditor.updateToolBar and we don't
            // want our undo reversed again
            this.ignore = true;

            // update the editor toolbar and return focus
            this.cmp.updateToolbar();
            this.cmp.focus();
            }
        }
    },

    // perform the redo
    redo: function() {

        // ensure that there is data to redo
        if (this.index < this.count) {

            // increment the index
            this.index++;

            // if in source edit mode then update the element directly
            if (this.cmp.sourceEditMode) {
                this.resetElement();
            }

            // else update the editor body
            else {
                this.reset();

                // ignore next record request as syncValue is called
                // by Ext.form.HtmlEditor.updateToolBar and we don't
                // want our redo reversed again
                this.ignore = true;

                // update the editor toolbar and return focus
                this.cmp.updateToolbar();
                this.cmp.focus();
            }
        }
    },

    // updates the editor body
    reset : function() {
        var content = this.history[this.index].content;
        this.cmp.getEditorBody().innerHTML = content;
        if (content=='' && Ext.isGecko) {
            // FF does not let the content be empty and adds a BR tag
            var brs = this.cmp.getEditorBody().getElementsByTagName('br');
            try { this.cmp.getEditorBody().removeChild(brs[0]); }
            catch (e) { }
        }
        this.resetBookmark();
    },

    // updates the element (when in source edit mode)
    resetElement : function() {
        this.cmp.el.dom.value = this.history[this.index].content;
        this.resetBookmark();
    },

    // reposition the cursor
    resetBookmark : function() {
        // checks if there is support for createRange method
        if (this.cmp.getDoc().createRange) {
            var node = this.cmp.getEditorBody().lastChild;
            var range = this.cmp.getDoc().createRange();
            if (node) {
                if (node.tagName=='BR' && node.previousSibling) {
                    // FF uses to add a BR tag at the end of the text
                    node = node.previousSibling;
                }
                var len = node.textContent.length;
                range.selectNodeContents(node);
                range.collapse(false);
            }
            var selection = this.cmp.getDoc().getSelection();
            selection.removeAllRanges();
            selection.addRange(range);
        }
    }

});

/**
 * the Tinebase/js/extFixes code for fixKeys is not needed anymore
 */
Ext.apply(Ext.form.HtmlEditor.prototype, {
    fixKeys : function(){ // load time branching for fastest keydown performance
        return function(e){
            if (e.ctrlKey && e.getKey()==35) {
                this.newLineIfAtEnd();
            }
        };
    }(),
    newLineIfAtEnd: function() {
        if (this.getDoc().createRange) {         // all browsers, except IE before version 9
            // get the last child node inside the editor body
            var dChild = this.getDeeperLastChild(this.getEditorBody());

            var selection = this.getDoc().getSelection();
            
            // add a BR tag to the end of the text if there isn't one yet
            if (selection.rangeCount > 0 && dChild.nodeName != 'BR') {
                var selRange = selection.getRangeAt(0);

                // move the selection to the end
                selRange.setEnd(dChild,dChild.textContent.length);
                selRange.setStartAfter(this.getEditorBody().lastChild);
                
                var n = this.getDoc().createElement('BR');
                selRange.insertNode(n);
                this.syncValue();
                this.deferFocus();
            }
        }
    },
    getDeeperLastChild: function(node) {
        // get the deeper last child node from an node
        if (node.lastChild)
            return this.getDeeperLastChild(node.lastChild);
        else
            return node;
    }
});

/**
 * the ExtJS/src/widgets/form/HtmlEditor code for relayCmd needs to return focus to editor
 */
Ext.apply(Ext.form.HtmlEditor.prototype, {
    relayCmd : function(cmd, value){
        (function(){
            this.focus();
            this.execCmd(cmd, value);
            this.updateToolbar();
            this.deferFocus();
        }).defer(10, this);
    }
});
/*
 * Tine 2.0
 * 
 * @package     Addressbook
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @copyright   Copyright (c) 2007-2013 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.ns('Tine.Addressbook');

/**
 * Contact grid panel
 * 
 * @namespace   Tine.Addressbook
 * @class       Tine.Addressbook.ContactGridPanel
 * @extends     Tine.widgets.grid.GridPanel
 * 
 * <p>Contact Grid Panel</p>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Cornelius Weiss <c.weiss@metaways.de>
 * @copyright   Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Addressbook.ContactGridPanel
 */
Tine.Addressbook.ContactGridPanel = Ext.extend(Tine.widgets.grid.GridPanel, {
    /**
     * record class
     * @cfg {Tine.Addressbook.Model.Contact} recordClass
     */
    recordClass: Tine.Addressbook.Model.Contact,
    
    /**
     * grid specific
     * @private
     */ 
    defaultSortInfo: {field: 'n_fileas', direction: 'ASC'},
    gridConfig: {
        autoExpandColumn: 'n_fileas',
        enableDragDrop: true,
        ddGroup: 'containerDDGroup'
    },
    copyEditAction: true,
    felamimail: false,
    multipleEdit: true,
    duplicateResolvable: true,
    
    /**
     * @cfg {Bool} hasDetailsPanel 
     */
    hasDetailsPanel: true,
    
    /**
     * inits this cmp
     * @private
     */
    initComponent: function() {
        this.recordProxy = Tine.Addressbook.contactBackend;
        
        // check if felamimail is installed and user has run right and wants to use felamimail in adb
        if (Tine.Felamimail && Tine.Tinebase.common.hasRight('run', 'Felamimail') && Tine.Felamimail.registry.get('preferences').get('useInAdb')) {
            this.felamimail = (Tine.Felamimail.registry.get('preferences').get('useInAdb') == 1);
        }
        this.gridConfig.cm = this.getColumnModel();

        if (this.hasDetailsPanel) {
            this.detailsPanel = this.getDetailsPanel();
        }

        Tine.Addressbook.ContactGridPanel.superclass.initComponent.call(this);
    },
    
    /**
     * returns column model
     * 
     * @return Ext.grid.ColumnModel
     * @private
     */
    getColumnModel: function() {
        return new Ext.grid.ColumnModel({
            defaults: {
                sortable: true,
                hidden: true,
                resizable: true
            },
            columns: this.getColumns()
        });
    },
    
    /**
     * returns array with columns
     * 
     * @return {Array}
     */
    getColumns: function() {
        return Tine.Addressbook.ContactGridPanel.getBaseColumns(this.app.i18n)
            .concat(this.getModlogColumns().concat(this.getCustomfieldColumns()));
    },
    
    /**
     * @private
     */
    initActions: function() {
        this.actions_exportContact = new Ext.Action({
            requiredGrant: 'exportGrant',
            text: String.format(this.app.i18n.ngettext('Export {0}', 'Export {0}', 50), this.i18nRecordsName),
            singularText: String.format(this.app.i18n.ngettext('Export {0}', 'Export {0}', 1), this.i18nRecordName),
            pluralText:  String.format(this.app.i18n.ngettext('Export {0}', 'Export {0}', 1), this.i18nRecordsName),
            translationObject: this.app.i18n,
            iconCls: 'action_export',
            scope: this,
            disabled: true,
            allowMultiple: true,
            menu: {
                items: [
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as PDF'),
                        iconCls: 'action_exportAsPdf',
                        format: 'pdf',
                        exportFunction: 'Addressbook.exportContacts',
                        gridPanel: this
                    }),
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as CSV'),
                        iconCls: 'tinebase-action-export-csv',
                        format: 'csv',
                        exportFunction: 'Addressbook.exportContacts',
                        gridPanel: this
                    }),
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as ODS'),
                        format: 'ods',
                        iconCls: 'tinebase-action-export-ods',
                        exportFunction: 'Addressbook.exportContacts',
                        gridPanel: this
                    }),
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as XLS'),
                        format: 'xls',
                        iconCls: 'tinebase-action-export-xls',
                        exportFunction: 'Addressbook.exportContacts',
                        gridPanel: this
                    }),
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as DOC'),
                        format: 'doc',
                        iconCls: 'tinebase-action-export-doc',
                        exportFunction: 'Addressbook.exportContacts',
                        gridPanel: this
                    }),
                    new Tine.widgets.grid.ExportButton({
                        text: this.app.i18n._('Export as ...'),
                        iconCls: 'tinebase-action-export-xls',
                        exportFunction: 'Addressbook.exportContacts',
                        showExportDialog: true,
                        gridPanel: this
                    })
                ]
            }
        });

        this.actions_import = new Ext.Action({
            //requiredGrant: 'addGrant',
            text: this.app.i18n._('Import contacts'),
            disabled: false,
            handler: this.onImport,
            iconCls: 'action_import',
            scope: this,
            allowMultiple: true
        });
        
        // register actions in updater
        this.actionUpdater.addActions([
            this.actions_exportContact,
            this.actions_import
        ]);
        
        Tine.Addressbook.ContactGridPanel.superclass.initActions.call(this);
    },

    /**
     * get default / selected addressbook container
     *
     * @returns {Object|Tine.Tinebase.Model.Container}
     */
    getDefaultContainer: function() {
        return this.app.getMainScreen().getWestPanel().getContainerTreePanel().getDefaultContainer('defaultAddressbook');
    },
    
    /**
     * add custom items to action toolbar
     * 
     * @return {Object}
     */
    getActionToolbarItems: function() {
        return [
            {
                xtype: 'buttongroup',
                columns: 1,
                frame: false,
                items: [
                    this.actions_exportContact,
                    this.actions_import
                ]
            }
        ];
    },
    
    /**
     * add custom items to context menu
     * 
     * @return {Array}
     */
    getContextMenuItems: function() {
        var items = [
            '-',
            this.actions_exportContact,
            '-'
        ];
        
        return items;
    },

    /**
     * returns details panel
     * 
     * @private
     * @return {Tine.Addressbook.ContactGridDetailsPanel}
     */
    getDetailsPanel: function() {
        return new Tine.Addressbook.ContactGridDetailsPanel({
            gridpanel: this,
            il8n: this.app.i18n,
            felamimail: this.felamimail
        });
    }
});

// Static Methods

/**
 * tid renderer
 * 
 * @private
 * @return {String} HTML
 */
Tine.Addressbook.ContactGridPanel.contactTypeRenderer = function(data, cell, record) {
    var i18n = Tine.Tinebase.appMgr.get('Addressbook').i18n,
        hasAccount = ((record.get && record.get('account_id')) || record.account_id),
        cssClass = hasAccount ? 'renderer_typeAccountIcon' : 'renderer_typeContactIcon',
        qtipText = Tine.Tinebase.common.doubleEncode(hasAccount ? i18n._('Contact of a user account') : i18n._('Contact'));
    
    return '<div ext:qtip="' + qtipText + '" style="background-position:0px;" class="' + cssClass + '">&#160</div>';
};

Tine.Addressbook.ContactGridPanel.displayNameRenderer = function(data) {
    var i18n = Tine.Tinebase.appMgr.get('Addressbook').i18n;
    return data ? Ext.util.Format.htmlEncode(data) : ('<div class="renderer_displayNameRenderer_noName">' + i18n._('No name') + '</div>');
};

Tine.Addressbook.ContactGridPanel.countryRenderer = function(data) {
    data = Locale.getTranslationData('CountryList', data);
    return Ext.util.Format.htmlEncode(data);
};

/**
 * Statically constructs the columns used to represent a contact. Reused by ListMemberGridPanel
 *
 */
Tine.Addressbook.ContactGridPanel.getBaseColumns = function(i18n) {
    return [
        { id: 'type', header: i18n._('Type'), dataIndex: 'type', width: 30, renderer: Tine.Addressbook.ContactGridPanel.contactTypeRenderer.createDelegate(this), hidden: false },
        { id: 'tags', header: i18n._('Tags'), dataIndex: 'tags', width: 50, renderer: Tine.Tinebase.common.tagsRenderer, sortable: false, hidden: false  },
        { id: 'salutation', header: i18n._('Salutation'), dataIndex: 'salutation', renderer: Tine.Tinebase.widgets.keyfield.Renderer.get('Addressbook', 'contactSalutation') },
        {
            id: 'container_id',
            dataIndex: 'container_id',
            header: Tine.Addressbook.Model.Contact.getContainerName(),
            width: 150,
            renderer: Tine.Tinebase.common.containerRenderer
        },
        { id: 'n_prefix', header: i18n._('Title'), dataIndex: 'n_prefix', width: 80 },
        { id: 'n_middle', header: i18n._('Middle Name'), dataIndex: 'n_middle', width: 80 },
        { id: 'n_family', header: i18n._('Last Name'), dataIndex: 'n_family' },
        { id: 'n_given', header: i18n._('First Name'), dataIndex: 'n_given', width: 80 },
        { id: 'n_fn', header: i18n._('Full Name'), dataIndex: 'n_fn', renderer: Tine.Addressbook.ContactGridPanel.displayNameRenderer },
        { id: 'n_fileas', header: i18n._('Display Name'), dataIndex: 'n_fileas', hidden: false, renderer: Tine.Addressbook.ContactGridPanel.displayNameRenderer},
        { id: 'org_name', header: i18n._('Company'), dataIndex: 'org_name', width: 120, hidden: false },
        { id: 'org_unit', header: i18n._('Unit'), dataIndex: 'org_unit'  },
        { id: 'title', header: i18n._('Job Title'), dataIndex: 'title' },
//            { id: 'role', header: i18n._('Job Role'), dataIndex: 'role' },
//            { id: 'room', header: i18n._('Room'), dataIndex: 'room' },
        { id: 'adr_one_street', header: i18n._('Street'), dataIndex: 'adr_one_street' },
        { id: 'adr_one_locality', header: i18n._('City'), dataIndex: 'adr_one_locality', width: 150, hidden: false },
        { id: 'adr_one_region', header: i18n._('Region'), dataIndex: 'adr_one_region' },
        { id: 'adr_one_postalcode', header: i18n._('Postalcode'), dataIndex: 'adr_one_postalcode' },
        { id: 'adr_one_countryname', header: i18n._('Country'), dataIndex: 'adr_one_countryname', renderer: Tine.Addressbook.ContactGridPanel.countryRenderer },
        { id: 'adr_two_street', header: i18n._('Street (private)'), dataIndex: 'adr_two_street' },
        { id: 'adr_two_locality', header: i18n._('City (private)'), dataIndex: 'adr_two_locality' },
        { id: 'adr_two_region', header: i18n._('Region (private)'), dataIndex: 'adr_two_region' },
        { id: 'adr_two_postalcode', header: i18n._('Postalcode (private)'), dataIndex: 'adr_two_postalcode' },
        { id: 'adr_two_countryname', header: i18n._('Country (private)'), dataIndex: 'adr_two_countryname', renderer: Tine.Addressbook.ContactGridPanel.countryRenderer },
        { id: 'email', header: i18n._('Email'), dataIndex: 'email', width: 150, hidden: false },
        { id: 'tel_work', header: i18n._('Phone'), dataIndex: 'tel_work', hidden: false },
        { id: 'tel_cell', header: i18n._('Mobile'), dataIndex: 'tel_cell', hidden: false },
        { id: 'tel_fax', header: i18n._('Fax'), dataIndex: 'tel_fax' },
        { id: 'tel_car', header: i18n._('Car phone'), dataIndex: 'tel_car' },
        { id: 'tel_pager', header: i18n._('Pager'), dataIndex: 'tel_pager' },
        { id: 'tel_home', header: i18n._('Phone (private)'), dataIndex: 'tel_home' },
        { id: 'tel_fax_home', header: i18n._('Fax (private)'), dataIndex: 'tel_fax_home' },
        { id: 'tel_cell_private', header: i18n._('Mobile (private)'), dataIndex: 'tel_cell_private' },
        { id: 'email_home', header: i18n._('Email (private)'), dataIndex: 'email_home' },
        { id: 'url', header: i18n._('Web'), dataIndex: 'url' },
        { id: 'url_home', header: i18n._('URL (private)'), dataIndex: 'url_home' },
        { id: 'note', header: i18n._('Note'), dataIndex: 'note' },
        { id: 'tz', header: i18n._('Timezone'), dataIndex: 'tz' },
        { id: 'geo', header: i18n._('Geo'), dataIndex: 'geo' },
        { id: 'bday', header: i18n._('Birthday'), dataIndex: 'bday', renderer: Tine.Tinebase.common.dateRenderer }
    ];
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.ns('Tine.Expressomail');

/**
 * Contact grid panel
 * 
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.ContactGridPanel
 * @extends     Tine.Addressbook.ContactGridPanel
 * 
 * <p>Contact Grid Panel</p>
 * <p>
 * </p>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.ContactGridPanel
 */
Tine.Expressomail.ContactGridPanel = Ext.extend(Tine.Addressbook.ContactGridPanel, {

    hasDetailsPanel: false,
    hasFavoritesPanel: false,
    hasQuickSearchFilterToolbarPlugin: false,
    stateId: 'ExpressomailContactGrid',
    
    query: null,
    
    gridConfig: {
        autoExpandColumn: 'n_fn',
        enableDragDrop: false
    },
    
    /**
     * the message record with recipients
     * @type Tine.Expressomail.Model.Message
     */
    messageRecord: null,
    
    /**
     * expressomail app, needed for translations
     * @type Tine.Expressomail.Application
     */
    expressomailApp: null,
    
    /**
     * inits this cmp
     * @private
     */
    initComponent: function() {
        this.addEvents(
            /**
             * @event addcontacts
             * Fired when contacts are added
             */
            'addcontacts'
        );
        
        this.app = Tine.Tinebase.appMgr.get('Addressbook');
        this.expressomailApp = Tine.Tinebase.appMgr.get('Expressomail');
        
        Tine.Expressomail.ContactGridPanel.superclass.initComponent.call(this);
        
        this.grid.on('rowdblclick', this.onRowDblClick, this);
        this.grid.on('cellclick', this.onCellClick, this);
        this.store.on('load', this.onContactStoreLoad, this);
    },
    
    /**
     * initialises the filter panel 
     * 
     * @param {Object} config
     */
    initFilterPanel: function(config) {
        
        
        this.defaultFilters = 
                            [{"field":"query","operator":"contains","value":this.query? this.query :'@'},
                             {"field":"container_id","operator":"in","value":Tine.Addressbook.registry.get('preferences').get('autosearchAddressbook')
                            }]
        
        Tine.Expressomail.ContactGridPanel.superclass.initFilterPanel.call(this, {
            filterFieldWidth: 100,
            filterValueWidth: 100
        });
    },
    
    /**
     * returns array with columns
     * 
     * @return {Array}
     */
    getColumns: function() {
        var columns = Tine.Expressomail.ContactGridPanel.superclass.getColumns.call(this);
        
        // hide all columns except name/company/email/email_home (?)
        Ext.each(columns, function(column) {
            if (['n_fn', 'org_name', 'email'].indexOf(column.dataIndex) === -1) {
                column.hidden = true;
            }
        });
        
        this.radioTpl = new Ext.XTemplate('<input',
            ' name="' + this.id + '_{id}"',
            ' value="{type}"',
            ' type="radio"',
            ' autocomplete="off"',
            ' class="x-form-radio x-form-field"',
            ' {checked}',
        '>');
        
        Ext.each(['To', 'Cc', 'Bcc', 'None'], function(type) { // _('None')
            columns.push({
                header: this.expressomailApp.i18n._(type),
                dataIndex: Ext.util.Format.lowercase(type),
                width: 50,
                hidden: false,
                renderer: this.typeRadioRenderer.createDelegate(this, [type], 0)
            });
            
        }, this);
            
        return columns;
    },
    
    /**
     * render type radio buttons in grid
     * 
     * @param {String} type
     * @param {String} value
     * @param {Object} metaData
     * @param {Object} record
     * @param {Number} rowIndex
     * @param {Number} colIndex
     * @param {Store} store
     * @return {String}
     */
    typeRadioRenderer: function(type, value, metaData, record, rowIndex, colIndex, store) {
        if (! record.hasEmail()) {
            return '';
        }
        
        var lowerType = Ext.util.Format.lowercase(type);
        
        return this.radioTpl.apply({
            id: record.id, 
            type: lowerType,
            checked: lowerType === 'none' ? 'checked' : ''
        });
    },
    
    /**
     * called after a new set of Records has been loaded
     * 
     * @param  {Ext.data.Store} this.store
     * @param  {Array}          loaded records
     * @param  {Array}          load options
     * @return {Void}
     */
    onContactStoreLoad: function(store, records, options) {
        Ext.each(records, function(record) {
            Ext.each(['to', 'cc', 'bcc'], function(type) {
                if (this.messageRecord.data[type].indexOf(Tine.Expressomail.getEmailStringFromContact(record)) !== -1) {
                    this.setTypeRadio(record, type);
                }
            }, this);
        }, this);
    },
    
    /**
     * cell click handler -> update recipients in record
     * 
     * @param {Grid} grid
     * @param {Number} row
     * @param {Number} col
     * @param {Event} e
     */
    onCellClick: function(grid, row, col, e) {
        var contact = this.store.getAt(row),
            typeToSet = this.grid.getColumnModel().getDataIndex(col)
        
        if (['to', 'cc', 'bcc', 'none'].indexOf(typeToSet) === -1) {
            // some other column has been clicked
            return;
        }
        
        if (! contact.hasEmail() && typeToSet !== 'none') {
            this.setTypeRadio(contact, 'none');
        } else {
            this.setTypeRadio(contact, typeToSet);
            this.updateRecipients(contact, typeToSet);
        }
    },
    
    /**
     * update recipient
     * 
     * @param {Tine.Addressbook.Model.Contact} contact
     * @param {String} typeToSet
     */
    updateRecipients: function(contact, typeToSet) {
        var email = Tine.Expressomail.getEmailStringFromContact(contact),
            found = false;
            
        Ext.each(['to', 'cc', 'bcc'], function(type) {
            if (this.messageRecord.data[type].indexOf(email) !== -1) {
                if (type !== typeToSet) {
                    this.messageRecord.data[type].remove(email);
                } else {
                    found = true;
                }
            }
        }, this);
        
        if (! found && typeToSet !== 'none') {
            Tine.log.debug('Tine.Expressomail.ContactGridPanel::updateRecipients() - adding email ' + email + ' to ' + typeToSet);
            this.messageRecord.data[typeToSet].push(email);
        }
    },
    
    /**
     * update type radio buttons dom
     * 
     * @param {Array} records of type Tine.Addressbook.Model.Contact
     * @param {String} type
     */
    setTypeRadio: function(records, type) {
        var rs = [].concat(records);
        
        Ext.each(rs, function(r) {
            if (r.hasEmail() || type === 'none') {
                Ext.select('input[name=' + this.id + '_' + r.id + ']', this.grid.el).each(function(el) {
                    el.dom.checked = type === el.dom.value;
                });
                this.updateRecipients(r, type);
            }
        }, this);
    },
    
    /**
     * Return CSS class to apply to rows depending upon email set or not
     * 
     * @param {Tine.Addressbook.Model.Contact} record
     * @param {Integer} index
     * @return {String}
     */
    getViewRowClass: function(record, index) {
        var className = '';
        
        if (! record.hasEmail()) {
            className = 'expressomail-no-email';
        }
        
        return className;
    },
    
    /**
     * @private
     */
    initActions: function() {
        this.actions_addAsTo = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.expressomailApp.i18n._('Add as "To"'),
            disabled: true,
            iconCls: 'action_add',
            actionUpdater: this.updateRecipientActions,
            handler: this.onAddContact.createDelegate(this, ['to']),
            allowMultiple: true,
            scope: this
        });

        this.actions_addAsCc = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.expressomailApp.i18n._('Add as "Cc"'),
            disabled: true,
            iconCls: 'action_add',
            actionUpdater: this.updateRecipientActions,
            handler: this.onAddContact.createDelegate(this, ['cc']),
            allowMultiple: true,
            scope: this
        });

        this.actions_addAsBcc = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.expressomailApp.i18n._('Add as "Bcc"'),
            disabled: true,
            iconCls: 'action_add',
            actionUpdater: this.updateRecipientActions,
            handler: this.onAddContact.createDelegate(this, ['bcc']),
            allowMultiple: true,
            scope: this
        });
        
        this.actions_setToNone = new Ext.Action({
            requiredGrant: 'readGrant',
            text: this.expressomailApp.i18n._('Remove from recipients'),
            disabled: true,
            iconCls: 'action_delete',
            actionUpdater: this.updateRecipientActions,
            handler: this.onAddContact.createDelegate(this, ['none']),
            allowMultiple: true,
            scope: this
        });
        
        //register actions in updater
        this.actionUpdater.addActions([
            this.actions_addAsTo,
            this.actions_addAsCc,
            this.actions_addAsBcc,
            this.actions_setToNone
        ]);
    },
    
    /**
     * updates context menu
     * 
     * @param {Ext.Action} action
     * @param {Object} grants grants sum of grants
     * @param {Object} records
     */
    updateRecipientActions: function(action, grants, records) {
        if (records.length > 0) {
            var emptyEmails = true;
            for (var i=0; i < records.length; i++) {
                if (records[i].hasEmail()) {
                    emptyEmails = false;
                    break;
                }
            }
            
            action.setDisabled(emptyEmails);
        } else {
            action.setDisabled(true);
        }
    },
    
    /**
     * on add contact -> fires addcontacts event and passes rows + type
     * 
     * @param {String} type
     */
    onAddContact: function(type) {
        var sm = this.grid.getSelectionModel(),
            selectedRows = sm.getSelections();
            
        this.setTypeRadio(selectedRows, type);

        // search contacts if all pages are selected (filter select)
        if (sm.isFilterSelect) {
            this['AddressLoadMask'] = new Ext.LoadMask(Ext.getBody(), {msg: this.expressomailApp.i18n._('Loading Mail Addresses')});
            this['AddressLoadMask'].show();
            
            var contact = null;
            Tine.Addressbook.searchContacts(sm.getSelectionFilter(), null, function(response) {
                Ext.each(response.results, function(contactData) {
                    contact = new Tine.Addressbook.Model.Contact(contactData);
                    this.updateRecipients(contact, type);
                }, this);
                
                this['AddressLoadMask'].hide();
            }.createDelegate(this));
        }
    },
    
    /**
     * row doubleclick handler
     * 
     * @param {} grid
     * @param {} row
     * @param {} e
     */
    onRowDblClick: function(grid, row, e) {
        this.onAddContact('to');
    }, 
    
    /**
     * returns rows context menu
     * 
     * @return {Ext.menu.Menu}
     */
    getContextMenu: function() {
        if (! this.contextMenu) {
            var items = [
                this.actions_addAsTo,
                this.actions_addAsCc,
                this.actions_addAsBcc,
                this.actions_setToNone
            ];
            this.contextMenu = new Ext.menu.Menu({items: items});
        }
        return this.contextMenu;
    }
});

Ext.reg('expressomailcontactgrid', Tine.Expressomail.ContactGridPanel);
/*
 * Tine 2.0
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2011 Metaways Infosystems GmbH (http://www.metaways.de)
 */
Ext.ns('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.RecipientPickerFavoritePanel
 * @extends     Ext.tree.TreePanel
 * 
 * <p>PersistentFilter Picker Panel</p>
 * 
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.RecipientPickerFavoritePanel
 */
Tine.Expressomail.RecipientPickerFavoritePanel = Ext.extend(Tine.widgets.persistentfilter.PickerPanel, {
    
    collapsible: true,
    baseCls: 'ux-arrowcollapse',
    animCollapse: true,
    titleCollapse:true,
    draggable : true,
    autoScroll: false,
    
    /**
     * no context menu
     * @type Function
     */
    onContextMenu: Ext.emptyFn,
    
    /**
     * @private
     */
    initComponent: function() {
        this.title = this.app.i18n._('Recipient filter');
        
        this.store = new Ext.data.ArrayStore({
            fields: Tine.widgets.persistentfilter.model.PersistentFilter.getFieldDefinitions(),
            sortInfo: {field: 'name', direction: 'ASC'}
        });
        
        var label = '';
        Ext.each(['all', 'to', 'cc', 'bcc'], function(field) {
            switch (field) {
                case 'all':
                    label = this.app.i18n._('All recipients');
                    break;
                default:
                    label = String.format(this.app.i18n._('"{0}" recipients'), field);
                    break;
            }
            this.store.add([new Tine.widgets.persistentfilter.model.PersistentFilter({
                filters: field,
                name: label,
                model: 'Addressbook_Model_Contact',
                application_id: this.app.id,
                id: Ext.id()
            })]);
        }, this);

        
        this.filterNode = new Ext.tree.AsyncTreeNode({
            id: '_recipientFilter',
            leaf: false,
            expanded: true
        });
        
        Tine.Expressomail.RecipientPickerFavoritePanel.superclass.initComponent.call(this);
    },
    
    /**
     * load grid from saved filter
     * 
     * -> overwritten to allow to dynamically update email filter
     * 
     *  @param {Tine.widgets.persistentfilter.model.PersistentFilter} persistentFilter
     */
    onFilterSelect: function(persistentFilter) {
        var emailRecipients = [];

        switch (persistentFilter.get('filters')) {
            case 'all':
                Ext.each(['to', 'cc', 'bcc'], function(field) {
                    emailRecipients = emailRecipients.concat(this.grid.messageRecord.data[field]);
                }, this);
                break;
            default:
                emailRecipients = this.grid.messageRecord.data[persistentFilter.get('filters')];
                break;
        }
        
        var filterValue = [], emailRegExp = /<([^>]*)/, filter = [{field: 'container_id', operator: 'in', value: []}];
        Ext.each(emailRecipients, function(email) {
            emailRegExp.exec(email);
            if (RegExp.$1 != '') {
                filterValue.push(RegExp.$1)
            }
        }, this);
        if (filterValue.length > 0) {
            filter = [{field: 'email_query', operator: 'in', value: filterValue}];
        }

        var updatedPersistentFilter = new Tine.widgets.persistentfilter.model.PersistentFilter({
            filters: filter,
            name: persistentFilter.get('name'),
            model: 'Addressbook_Model_Contact',
            application_id: this.app.id,
            id: Ext.id()
        });

        Tine.Expressomail.RecipientPickerFavoritePanel.superclass.onFilterSelect.call(this, updatedPersistentFilter);
    }
});
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */
 
Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.RecipientPickerDialog
 * @extends     Tine.widgets.dialog.EditDialog
 * 
 * <p>Message Compose Dialog</p>
 * <p>This dialog is for searching contacts in the addressbook and adding them to the recipient list in the email compose dialog.</p>
 * <p>
 * </p>
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new RecipientPickerDialog
 */
 Tine.Expressomail.RecipientPickerDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {
    
    /**
     * @private
     */
    windowNamePrefix: 'RecipientPickerWindow_',
    appName: 'Expressomail',
    recordClass: Tine.Expressomail.Model.Message,
    recordProxy: Tine.Expressomail.messageBackend,
    loadRecord: false,
    evalGrants: false,
    mode: 'local',
    
    bodyStyle:'padding:0px',
    
    /**
     * overwrite update toolbars function (we don't have record grants)
     * @private
     */
    updateToolbars: Ext.emptyFn,
    
    
    query: null,
    
    /**
     * @private
     */
    onRecordLoad: function() {
        // interrupt process flow till dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }
        Ext.each(['to', 'cc', 'bcc'], function(type) {
                if (this.record.data[type].indexOf(this.query) !== -1) {
                    this.record.data[type].remove(this.query);
                }
        }, this);
        
        var subject = (this.record.get('subject') != '') ? this.record.get('subject') : this.app.i18n._('(new message)');
        this.window.setTitle(String.format(this.app.i18n._('Select recipients for "{0}"'), subject));
        
        this.loadMask.hide();
    },

    /**
     * returns dialog
     * 
     * NOTE: when this method gets called, all initialisation is done.
     * 
     * @return {Object}
     * @private
     */
    getFormItems: function() {
        var adbApp = Tine.Tinebase.appMgr.get('Addressbook');
        
        this.treePanel = new Tine.widgets.container.TreePanel({
            allowMultiSelection: true,
            region: 'west',
            filterMode: 'filterToolbar',
            recordClass: Tine.Addressbook.Model.Contact,
            app: adbApp,
            width: 200,
            minSize: 100,
            maxSize: 300,
            border: false,
            enableDrop: false
        });
        
        this.contactGrid = new Tine.Expressomail.ContactGridPanel({
            region: 'center',
            messageRecord: this.record,
            app: adbApp,
            query: this.query,
            plugins: [this.treePanel.getFilterPlugin()]
        });
        
        this.westPanel = new Tine.widgets.mainscreen.WestPanel({
            app: adbApp,
            hasFavoritesPanel: true,
            ContactTreePanel: this.treePanel,
            ContactFilterPanel: new Tine.widgets.persistentfilter.PickerPanel({
                filter: [{field: 'model', operator: 'equals', value: 'Addressbook_Model_ContactFilter'}],
                app: adbApp,
                grid: this.contactGrid
            }),
            additionalItems: [ new Tine.Expressomail.RecipientPickerFavoritePanel({
                app: this.app,
                grid: this.contactGrid
            })]
        });
        
        return {
            border: false,
            layout: 'border',
            items: [{
                cls: 'tine-mainscreen-centerpanel-west',
                region: 'west',
                stateful: false,
                layout: 'border',
                split: true,
                width: 200,
                minSize: 100,
                maxSize: 300,
                border: false,
                collapsible: true,
                collapseMode: 'mini',
                header: false,
                items: [{
                    border: false,
                    region: 'center',
                    items: [ this.westPanel ]
                }]
            }, this.contactGrid]
        };
    }
});

/**
 * Expressomail Edit Popup
 * 
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.RecipientPickerDialog.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 1000,
        height: 600,
        name: Tine.Expressomail.RecipientPickerDialog.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.RecipientPickerDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schüle <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010-2012 Metaways Infosystems GmbH (http://www.metaways.de)
 * 
 */
Ext.ns('Tine.Expressomail');

/**
 * @namespace   Tine.widgets.container
 * @class       Tine.Expressomail.FolderFilterModel
 * @extends     Tine.widgets.grid.FilterModel
 * 
 * @author      Philipp Schuele <p.schuele@metaways.de>
 */
Tine.Expressomail.FolderFilterModel = Ext.extend(Tine.widgets.grid.PickerFilter, {

    /**
     * @cfg 
     */
    operators: ['in', 'notin'],
    field: 'path',
    
    /**
     * @private
     */
    initComponent: function() {
        this.label = this.app.i18n._('Folder');
        
        this.multiselectFieldConfig = {
            labelField: 'path',
            selectionWidget: new Tine.Expressomail.FolderSelectTriggerField({
                allAccounts: true
            }),
            recordClass: Tine.Expressomail.Model.Folder,
            valueStore: this.app.getFolderStore(),
            
            /**
             * functions
             */
            labelRenderer: Tine.Expressomail.GridPanel.prototype.accountAndFolderRenderer.createDelegate(this),
            initSelectionWidget: function() {
                this.selectionWidget.onSelectFolder = this.addRecord.createDelegate(this);
            },
            isSelectionVisible: function() {
                return this.selectionWidget.selectPanel && ! this.selectionWidget.selectPanel.isDestroyed        
            },
            getRecordText: function(value) {
                var path = (Ext.isString(value)) ? value : (value.path) ? value.path : '/' + value.id,
                    index = this.valueStore.findExact('path', path),
                    record = this.valueStore.getAt(index),
                    text = null;
                
                if (! record) {
                    // try account
                    var accountId = path.substr(1, 40);
                    record = this.app.getAccountStore().getById(accountId);
                }
                if (record) {
                    this.currentValue.push(path);
                    // always copy/clone record because it can't exist in 2 different stores
                    this.store.add(record.copy());
                    text = this.labelRenderer(record.id, {}, record);
                } else {
                    text = path;
                    this.currentValue.push(path);
                }
                
                return text;
            }
        };

        Tine.Expressomail.FolderFilterModel.superclass.initComponent.call(this);
    }
});

Tine.widgets.grid.FilterToolbar.FILTERS['tine.expressomail.folder.filtermodel'] = Tine.Expressomail.FolderFilterModel;
/*
 * Tine 2.0
 * 
 * @package     Admin
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010 Metaways Infosystems GmbH (http://www.metaways.de)
 *
 */

Ext.namespace('Tine.Expressomail');

/**
 * admin settings panel
 * 
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.AdminPanel
 * @extends     Tine.widgets.dialog.AdminPanel
 * 
 * <p>Admin Admin Panel</p>
 * <p><pre>
 * </pre></p>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Philipp Schuele <p.schuele@metaways.de>
 * @copyright   Copyright (c) 2010 Metaways Infosystems GmbH (http://www.metaways.de)
 * 
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressomail.AdminPanel
 */
Tine.Expressomail.AdminPanel = Ext.extend(Tine.widgets.dialog.AdminPanel, {
    /**
     * @private
     */
    appName: 'Expressomail',
    
    /**
     * get config items
     * 
     * @return {Array}
     */
    getConfigItems: function() {
        return [[
            {
                name: 'imapSearchMaxResults',
                fieldLabel: this.app.i18n._('Max Results in Search Messages'),
                xtype: 'numberfield',
                value: null,
                minValue: 1,
                maxValue: 9999,
                allowBlank: false
            },
            {
                name: 'autoSaveDraftsInterval',
                fieldLabel: this.app.i18n._('Interval (in seconds) for Auto Saving Drafts (0 to disable)'),
                xtype: 'numberfield',
                value: null,
                minValue: 0,
                maxValue: 9999,
                allowBlank: false
            },
            {
                name: 'reportPhishingEmail',
                fieldLabel: this.app.i18n._('Email to which to report phishing'),
                xtype: 'textfield',
                value: null,
                maxLength: 64,
                allowBlank: true
            }
        ]];
    }
});

/**
 * admin panel on update function
 * 
 * TODO         update registry without reloading the mainscreen
 */
Tine.Expressomail.AdminPanel.onUpdate = function() {
    // reload mainscreen to make sure registry gets updated
    window.location = window.location.href.replace(/#+.*/, '');
}

/**
 * Admin admin settings popup
 * 
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressomail.AdminPanel.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 600,
        height: 400,
        name: Tine.Expressomail.AdminPanel.prototype.windowNamePrefix + Ext.id(),
        contentPanelConstructor: 'Tine.Expressomail.AdminPanel',
        contentPanelConstructorConfig: config
    });
    return window;
};
/*
 * Tine 2.0
 * 
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Mário César Kolling <mario.kolling@serpro.gov.br>
 * @copyright   Copyright (c) 2009-2013 Serpro (http://www.serpro.gov.br)
 */

Ext.namespace('Tine.Expressomail');

/**
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.SignatureAppletPanel
 * @extends     Ext.widgets.Panel
 * 
 * @author      Mário César Kolling <mario.kolling@serpro.gov.br>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * 
 * @param       {Object} config
 * @constructor
 * Create a new AppletPanel
 */

 Tine.Expressomail.SignatureAppletPanel = Ext.extend(Ext.Panel, {
    
    messageModel: null,
    
    signedMessageModel: null, // todo: create this model.
    
    initComponent: function()
    {
        Tine.Expressomail.SignatureAppletPanel.superclass.initComponent.call(this);
    }
    
//    // methods
//    toApplet: function()
//    {
//        // call Applet passing => this.messageModel.toString();
//
//    },
//    
//    fromApplet: function(response)
//    {
//        // get response and send the signed message
//    }
    
});/**
 * @author Shea Frederick - http://www.vinylfox.com
 * @class Ext.ux.form.HtmlEditor.Table
 * @extends Ext.util.Observable
 * <p>A plugin that creates a button on the HtmlEditor for making simple tables.</p>
 */
Ext.ux.form.HtmlEditor.Table = Ext.extend(Ext.util.Observable, {
    // Table language text
    langTitle       : 'Insert Table',
    langInsert      : 'Insert',
    langCancel      : 'Cancel',
    langRows        : 'Rows',
    langColumns     : 'Columns',
    langBorder      : 'Border',
    langCellLabel   : 'Label Cells',
    lang100PcWidth    : '100% Width',
    // private
    cmd: 'table',
    /**
     * @cfg {Boolean} showCellLocationText
     * Set true to display row and column informational text inside of newly created table cells.
     */
    showCellLocationText: false,
    /**
     * @cfg {Boolean} set100PcWidth
     * Set true to insert a 100% width table.
     */
    set100PcWidth: false,
    /**
     * @cfg {String} cellLocationText
     * The string to display inside of newly created table cells.
     */
    cellLocationText: '{0}&nbsp;-&nbsp;{1}',
    /**
     * @cfg {Array} tableBorderOptions
     * A nested array of value/display options to present to the user for table border style. Defaults to a simple list of 5 varrying border types.
     */
    tableBorderOptions: [['none', 'None'], ['1px solid #000', 'Solid Thin'], ['2px solid #000', 'Solid Thick'], ['1px dashed #000', 'Dashed'], ['1px dotted #000', 'Dotted']],
    
    // private
    init: function(cmp){
        this.cmp = cmp;
        this.cmp.on('render', this.onRender, this);
        this.tableBorderOptions = [['none', _('None')], ['1px solid #000', _('Solid Thin')], ['2px solid #000', _('Solid Thick')], ['1px dashed #000', _('Dashed')], ['1px dotted #000', _('Dotted')]];
    },
    // private
    onRender: function(){
        var btn = this.cmp.getToolbar().addButton({
            iconCls: 'x-edit-table',
            handler: function(){
                if (!this.tableWindow){
                    this.tableWindow = new Ext.Window({
                        title: _(this.langTitle),
                        closeAction: 'hide',
                        width: 235,
                        items: [{
                            itemId: 'insert-table',
                            xtype: 'form',
                            border: false,
                            plain: true,
                            bodyStyle: 'padding: 10px;',
                            labelWidth: 85,
                            labelAlign: 'right',
                            items: [{
                                xtype: 'numberfield',
                                allowBlank: false,
                                allowDecimals: false,
                                fieldLabel: _(this.langRows),
                                name: 'row',
                                width: 60
                            }, {
                                xtype: 'numberfield',
                                allowBlank: false,
                                allowDecimals: false,
                                fieldLabel: _(this.langColumns),
                                name: 'col',
                                width: 60
                            }, {
                                xtype: 'combo',
                                fieldLabel: _(this.langBorder),
                                name: 'border',
                                forceSelection: true,
                                mode: 'local',
                                store: new Ext.data.ArrayStore({
                                    autoDestroy: true,
                                    fields: ['spec', 'val'],
                                    data: this.tableBorderOptions
                                }),
                                triggerAction: 'all',
                                value: 'none',
                                displayField: 'val',
                                valueField: 'spec',
                                anchor: '-15'
                            }, {
                            	xtype: 'checkbox',
                            	fieldLabel: _(this.langCellLabel),
                            	checked: this.showCellLocationText,
                            	listeners: {
                            		check: function(){
                            			this.showCellLocationText = !this.showCellLocationText;
                            		},
                            		scope: this
                            	}
                            }, {
                            	xtype: 'checkbox',
                            	fieldLabel: _(this.lang100PcWidth),
                            	checked: this.set100PcWidth,
                            	listeners: {
                            		check: function(){
                            			this.set100PcWidth = !this.set100PcWidth;
                            		},
                            		scope: this
                            	}
                            }]
                        }],
                        buttons: [{
                            text: _(this.langInsert),
                            handler: function(){
                                var frm = this.tableWindow.getComponent('insert-table').getForm();
                                if (frm.isValid()) {
                                    var border = frm.findField('border').getValue();
                                    var rowcol = [frm.findField('row').getValue(), frm.findField('col').getValue()];
                                    if (rowcol.length == 2 && rowcol[0] > 0 && rowcol[1] > 0) {
                                        var colwidth = Math.floor(100/rowcol[1]);
                                        var html = "<table style='border-collapse: collapse;" + (this.set100PcWidth ? "width:100%;" : "") + "'>";
                                        var cellText = '&nbsp;';
                                        if (this.showCellLocationText){ cellText = this.cellLocationText; }
                                        for (var row = 0; row < rowcol[0]; row++) {
                                            html += "<tr>";
                                            for (var col = 0; col < rowcol[1]; col++) {
                                                html += "<td " + (border=='none' ? "class='editor-wysiwyg-table' " : "") + "width='" + colwidth + "%' style='border: " + border + ";'>" + String.format(cellText, (row+1), String.fromCharCode(col+65)) + "</td>";
                                            }
                                            html += "</tr>";
                                        }
                                        html += "</table><br/>";
                                        this.cmp.insertAtCursor(html);
                                        if (Ext.isIE || Ext.isNewIE) {
                                            var r = this.cmp.currentRange || this.cmp.getDoc().selection.createRange();
                                            if (r) { 
                                                r.select();
                                            }
                                        }
                                    }
                                    this.tableWindow.hide();
                                }else{
                                    if (!frm.findField('row').isValid()){
                                        frm.findField('row').getEl().frame();
                                    }else if (!frm.findField('col').isValid()){
                                        frm.findField('col').getEl().frame();
                                    }
                                }
                            },
                            scope: this
                        }, {
                            text: _(this.langCancel),
                            handler: function(){
                                this.tableWindow.hide();
                            },
                            scope: this
                        }]
                    });
                
                }else{
                    this.tableWindow.getEl().frame();
                }
                this.tableWindow.show();
            },
            scope: this,
            tooltip: {
                title: _(this.langTitle)
            },
            overflowText: _(this.langTitle)
        });
    }
});
Ext.ux.form.HtmlEditor.UploadImage = Ext.extend(Ext.util.Observable, {
    text_title: 'Insert Image',
    text_error_msgbox_title: 'Error',
    text_error_file_type_not_permitted: 'This file type is not allowed.<br/>Please, select a file of the following extensions: {1}',
    text_note_upload_failed: 'Either a server internal error occurred or the service is unavailable.<br/>You may also check if the file size does not exceed {0} bytes.',
    text_note_upload_error: 'Upload error.<br/>Please check if the file size does not exceed {0} bytes.',
    text_note_uploading: 'Uploading...',
    url : null,
    method : 'Expressomail.uploadImage',
    base64: 'no',
    permitted_extensions: ['jpg', 'jpeg', 'png', 'bmp', 'gif'],
    form : null,
    isLocal: false,
    eid : null,

    init : function(cmp) {
        this.cmp = cmp;
        if(this.cmp.messageEdit) {
            this.eid = this.cmp.messageEdit.eid;
            this.cmp.messageEdit.record.set('embedded_images', this.cmp.messageEdit.record.get('embedded_images')||[]);
        }
        this.isLocal = this.cmp.messageEdit && this.cmp.messageEdit.encrypted;
        this.url = this.isLocal ? 'https://local.expressov3.serpro.gov.br:8998/upload' : 'index.php';
        this.method = this.isLocal ? '' : this.method;
        this.cmp.on('render', this.onRender, this);
        this.on('uploadsuccess', this.uploadsuccess || this.onUploadSuccess, this);
        this.on('uploaderror', this.uploaderror || this.onUploadError, this);
        this.on('uploadfailed', this.uploadfailed || this.onUploadFailed, this);

        // Registering dialog events.
        this.addEvents({
            'uploadsuccess' : true,
            'uploaderror' : true,
            'uploadfailed' : true,
            'uploadstart' : true,
            'uploadcomplete' : true
        });
    },

    setBase64 : function(base64) {
        this.base64 = base64;
    },

    createForm : function()	{
        this.form = Ext.DomHelper.append(this.body, {
            tag: 'form',
            method: 'post',
            action: this.url,
            style: 'position: absolute; left: -100px; top: -100px; width: 100px; height: 100px'
        });
    },

    recreateForm : function() {
        if (this.form) {
            this.form.parentNode.removeChild(this.form);
        }
        this.createForm();
    },

    onRender : function(ct, position) {
        this.cmp.getToolbar().addButton([new Ext.Toolbar.Separator()]);

        this.btn = this.cmp.getToolbar().addButton(new Ext.ux.UploadImage.BrowseButton({
            input_name: 'upload',
            iconCls: 'x-edit-image',
            handler: this.onFileSelected,
            scope: this,
            url: this.url,
            tooltip: {title: _(this.text_title)},
            overflowText: _(this.text_title)
        }));

        this.body = Ext.getBody();
        this.createForm();
    },

    onFileSelected : function() {
        if (this.isPermittedFile()) {
            var input_file = this.btn.detachInputFile();

            input_file.appendTo(this.form);
            input_file.setStyle('width', '100px');
            input_file.dom.disabled = true;

            this.image_file = {
                filename: input_file.dom.value,
                input_element: input_file
            };
            if (!Ext.isIE && !Ext.isNewIE) {
                this.image_file.file = (input_file.dom.files ? input_file.dom.files[0] : null);
            }

            input_file.dom.disabled = false;

            this.uploadFile(this.image_file);
            this.fireUploadStartEvent();
            this.image_file = null;
        }
    },

    uploadFile : function(record) {
        if (this.isLocal && !Ext.isIE && !Ext.isNewIE) {
            var defaultHeaders = {
                "Content-Type"          : "application/octet-stream",
                "X-Tine20-Request-Type" : "HTTPS",
                "X-Requested-With"      : "XMLHttpRequest"
            };

            var xmlData = record.file;

            var fileRecord = this.createFileRecord(record.file);

            var conn = new Ext.data.Connection({
                disableCaching: true,
                method: 'POST',
                url: this.url,
                timeout: 300000, // 5 mins
                defaultHeaders: defaultHeaders
            });

            var transaction = conn.request({
                headers: {
                    "X-File-Name"           : fileRecord.get('name'),
                    "X-File-Type"           : fileRecord.get('type'),
                    "X-File-Size"           : fileRecord.get('size')
                },
                params: {
                    "eid": this.eid
                },
                xmlData: xmlData,
                success: this.onAjaxSuccess.createDelegate(this, null, true),
                failure: this.onAjaxFailure.createDelegate(this, null, true)
            });

            conn = null;
        }
        else {
            this.base_params = { 
                method: this.method,
                base64: this.base64,
                requestType: 'HTTP',
                eid: this.eid
            };
            var options = {
                url : this.url,
                params: {
                    method: this.method,
                    base64: this.base64,
                    requestType: 'HTTPS',
                    eid: this.eid
                },
                method : 'POST',
                form : this.form,
                isUpload : true,
                success : this.onAjaxSuccess.createDelegate(this),
                failure : this.onAjaxFailure.createDelegate(this),
                scope : this,
                record: record
            };

            if (this.isLocal) {
                // form upload with post message response
                var postMessageCallback = function(event){
                    console.log('event.origin: '+event.origin);
                    if (event.origin != "https://local.expressov3.serpro.gov.br:8998"){
                        Tine.log.err("postMessage: Origin not allowed");
                        options.failure({responseText : event.data}, options);
                        return;
                    }

                    if (Ext.isIE) {
                        window.dettachEvent('onmessage', postMessageCallback);
                    } else {
                        window.removeEventListener('message', postMessageCallback);
                    }

                    options.success({responseText : event.data}, options);
                    //this.onUploadSuccess({responseText : event.data}, options, this.fileRecord);

                }.createDelegate(this);
                if (Ext.isIE) {
                    window.attachEvent('onmessage', postMessageCallback);
                } else {
                    window.addEventListener('message', postMessageCallback);
                }
            }

            Ext.Ajax.request(options);
        }
    },

    createFileRecord: function(file) {

        var fileRecord = new Ext.ux.file.Upload.file({
            name: file.name ? file.name : file.fileName,  // safari and chrome use the non std. fileX props
            type: (file.type ? file.type : file.fileType), // missing if safari and chrome
            size: 0,
            status: "uploading",
            progress: 0,
            input: file
        });

        return fileRecord;

    },

    getFileExtension : function(filename) {
        var result = null;
        var parts = filename.split('.');
        if (parts.length > 1) {
            result = parts.pop();
        }
        return result;
    },

    isPermittedFileType : function(filename) {
        var result = true;
        if (this.permitted_extensions.length > 0) {
            result = this.permitted_extensions.indexOf(this.getFileExtension(filename)) != -1;
        }
        return result;
    },

    isPermittedFile : function() {
        var result = false;
        var filename = this.btn.getInputFile().dom.value;

        if (this.isPermittedFileType(filename.toLowerCase())) {
            result = true;
        }
        else {
            this.showMessage(_(this.text_error_msgbox_title),String.format(_(this.text_error_file_type_not_permitted),filename,this.permitted_extensions.join(', ')));
            result = false;
        }

        return result;
    },

    fireUploadStartEvent : function() {
        this.wait = Ext.MessageBox.wait(_(this.text_note_uploading), _('Please wait!'));
        this.fireEvent('uploadstart', this);
    },

    fireUploadSuccessEvent : function(data) {
        this.fireEvent('uploadsuccess', this, data.record.filename, data.response);
    },

    fireUploadErrorEvent : function(data) {
        this.fireEvent('uploaderror', this, data.record.filename, data.response);
    },

    fireUploadFailedEvent : function(data) {
        this.fireEvent('uploadfailed', this, data.record.filename, data.response);
    },

    fireUploadCompleteEvent : function() {
        if (this.wait) {
            this.wait.hide();
        }
        this.fireEvent('uploadcomplete', this);
    },

    onAjaxSuccess : function(response, options) {
        var json_response = {
            'success' : false,
            'error' : _(this.text_note_upload_failed)
        }
        try {
            var rt = response.responseText;
            var filter = rt.match(/^<pre>((?:.|\n)*)<\/pre>$/i);
            if (filter) {
                rt = filter[1];
            }
            json_response = Ext.util.JSON.decode(rt);
        }
        catch (e) {
            if (this.isLocal) {
                return;
            }
        }

        if (this.isLocal) {
            json_response = {
                tempFile: json_response.tempFile,
                id: json_response.tempFile.eid,
                size: json_response.tempFile.size,
                success: (json_response.status=='success')
            };
            if (!options.record) {
                options.record = {
                    filename: options.xmlData.name,
                    file: options.xmlData
                };
            }
        }

        var data = {
            record: options.record,
            response: json_response
        }

        this.recreateForm();
        if ('success' in json_response && json_response.success) {
            this.fireUploadSuccessEvent(data);
        }
        else if ('method' in json_response && json_response.method) {
            this.fireUploadErrorEvent(data);
        }
        else {
            this.fireUploadFailedEvent(data);
        }
        this.fireUploadCompleteEvent();
    },

    onAjaxFailure : function(response, options) {
        if (!options.record) {
            options.record = {
                filename: options.xmlData.name,
                file: options.xmlData
            };
        }
        var data = {
            record : options.record,
            response : {
                'success' : false,
                'error' : _(this.text_note_upload_failed)
            }
        }

        this.recreateForm();
        this.fireUploadFailedEvent(data);
        this.fireUploadCompleteEvent();
    },

    onUploadSuccess : function(dialog, filename, resp_data, record) {
        var fileName = filename.replace(/[a-zA-Z]:[\\\/]fakepath[\\\/]/, '');
        var html;
        if (this.isLocal) {
            html = '<img alt="'+fileName+'" src="https://local.expressov3.serpro.gov.br:8998/'+resp_data.tempFile.eid+'"/>';
        }
        else {
            html = '<img alt="'+fileName+'" src="index.php?method=Expressomail.showTempImage&tempImageId='+resp_data.id+'"/>';
        }
        if (!dialog.cmp.activated) {
            dialog.cmp.getEditorBody().focus();
            dialog.cmp.onFirstFocus();
        }

        if(dialog.cmp.messageEdit)
            var images = dialog.cmp.messageEdit.record.get('embedded_images');
        resp_data.name = filename;
        images.push(resp_data);
        //dialog.cmp.messageEdit.record.set('embedded_images', images);

        dialog.cmp.insertAtCursor(html);
    },

    onUploadError : function(dialog, filename, resp_data, record) {
        var fileName = filename.replace(/[a-zA-Z]:[\\\/]fakepath[\\\/]/, '');
        this.showMessage(_(this.text_error_msgbox_title),String.format(_(this.text_note_upload_error),resp_data.maxsize));
    },

    onUploadFailed : function(dialog, filename, resp_data, record) {
        var fileName = filename.replace(/[a-zA-Z]:[\\\/]fakepath[\\\/]/, '');
        this.showMessage(_(this.text_error_msgbox_title),String.format(_(this.text_note_upload_failed),resp_data.maxsize));
    },

    showMessage : function(title, message) {
        this.wait = null;
        Ext.Msg.alert(title,message);
    }

});

/**
 * Ext.ux.UploadImage namespace.
 */
Ext.namespace('Ext.ux.UploadImage');

/**
 * File upload browse button.
 *
 * @class Ext.ux.UploadImage.BrowseButton
 */
Ext.ux.UploadImage.BrowseButton = Ext.extend(Ext.Button, {
    input_name : 'file',
    input_file : null,
    original_handler : null,
    original_scope : null,
    hideParent : true,

    /**
     * @access private
     */
    initComponent : function() {
        Ext.ux.UploadImage.BrowseButton.superclass.initComponent.call(this);
        this.original_handler = this.handler || null;
        this.original_scope = this.scope || window;
        this.handler = null;
        this.scope = null;
    },

    /**
     * @access private
     */
    onRender : function(ct, position) {
        Ext.ux.UploadImage.BrowseButton.superclass.onRender.call(this, ct, position);
        this.createInputFile();
    },

    /**
     * @access private
     */
    onDestroy : function() {
    Ext.ux.UploadImage.BrowseButton.superclass.onDestroy.call(this);
    if(this.container) {
        this.container.remove();
        }
    },

    /**
     * @access private
     */
    createInputFile : function() {
        var button_container = this.el.child('em' /* JYJ '.x-btn-center'*/);
        button_container.position('relative');
        this.wrap = this.el.child('em');

        this.input_file = this.wrap.createChild({
            tag: 'input',
            type: 'file',
            size: 1,
            name: this.input_name || Ext.id(this.el),
            style: "position: absolute; display: block; border: none; cursor: pointer; margin-top: -4px; margin-left: -4px; z-index: 2;"
        });
        this.input_file.setOpacity(0.0);

        var button_box = button_container.getBox();
        this.input_file.setStyle('font-size', (button_box.width * 0.5) + 'px');

        var input_box = this.input_file.getBox();

        this.input_file.setWidth(button_box.width + 11 + 'px');
        this.input_file.setTop('0px');
        this.input_file.setHeight(button_box.height + 8 + 'px');
        this.input_file.setOpacity(0.0);

        if (this.handleMouseEvents) {
            this.input_file.on('mouseover', this.onMouseOver, this);
            this.input_file.on('mousedown', this.onMouseDown, this);
        }

        if(this.tooltip){
            if(typeof this.tooltip == 'object'){
                Ext.QuickTips.register(Ext.apply({target: this.input_file}, this.tooltip));
            }
            else {
                this.input_file.dom[this.tooltipType] = this.tooltip;
            }
        }

        this.input_file.on('change', this.onInputFileChange, this);
        this.input_file.on('click', function(e) {e.stopPropagation();});
        this.wrap.setStyle('overflow','hidden');
    },

    /**
     * @access public
     */
    detachInputFile : function(no_create) {
        var result = this.input_file;

        no_create = no_create || false;

        if (typeof this.tooltip == 'object') {
            Ext.QuickTips.unregister(this.input_file);
        }
        else {
            this.input_file.dom[this.tooltipType] = null;
        }
        this.input_file.removeAllListeners();
        this.input_file = null;

        if (!no_create) {
            this.createInputFile();
        }
        return result;
    },

    /**
     * @access public
     */
    getInputFile : function() {
        return this.input_file;
    },

    /**
     * @access public
     */
    disable : function() {
        Ext.ux.UploadImage.BrowseButton.superclass.disable.call(this);
        this.input_file.dom.disabled = true;
    },

    /**
     * @access public
     */
    enable : function() {
        Ext.ux.UploadImage.BrowseButton.superclass.enable.call(this);
        this.input_file.dom.disabled = false;
    },

    /**
     * @access public
     */
    destroy : function() {
        var input_file = this.detachInputFile(true);
        input_file.remove();
        input_file = null;
        Ext.ux.UploadImage.BrowseButton.superclass.destroy.call(this);
    },

    /**
    * @access private
    */
    onInputFileChange : function() {
        if (this.original_handler) {
            this.original_handler.call(this.original_scope, this);
        }
    },

    monitorMouseOver : function(e){
        if( e.target != this.el.dom && !e.within(this.el) && e.target != this.input_file.dom ){
            if(this.monitoringMouseOver){
                this.doc.un('mouseover', this.monitorMouseOver, this);
                this.monitoringMouseOver = false;
            }
            this.onMouseOut(e);
        }
    }
});
/**
 * @author Fernando Lages
 * @class Ext.ux.form.HtmlEditor.SpellChecker
 * @extends Ext.util.Observable
 * <p>A plugin that creates a button on the HtmlEditor for GoogieSpell spell checker.</p>
 */
Ext.ux.form.HtmlEditor.SpellChecker = Ext.extend(Ext.util.Observable, {
    // Spell Checker language text
    langTitle: 'Spell Checker',
    langText: 'Verifica a ortografia do texto pelos padrões da linguagem.', //Spell checks texts by choosen language standards.
    spellchecker: null,
    // private
    init: function(cmp){
        this.cmp = cmp;
        this.cmp.on('render', this.onRender, this);
        this.cmp.on('beforedestroy', this.onDestroy, this);
    },
    getLanguages: function() {
        var langs = {"pt_BR": "Portugu&#234;s (Brasil)"};
        langs = Tine.Expressomail.registry.get('aspellDicts') || langs;
        return langs;
    },
    getCurrentLanguage: function() {
        return (this.currLang || "pt_BR");
    },
    setCurrentLanguage: function(lang) {
        this.currLang = lang;
    },
    onDestroy: function() {
        if (this.spellchecker)
            this.spellchecker.hideErrorWindow();
    },
    onRender: function() {
        var cmp = this.cmp;
        var menu = [];
        var langs = this.getLanguages();
        var checked = true;
        for (var dict in langs) {
            checked = (dict=='pt_BR');
            menu.push({
                text: langs[dict],
                iconCls: 'email-icon',
                group: 'language',
                lang: dict,
                checked: checked,
                handler: this.setCurrentLanguage.createDelegate(this,[dict]),
                scope: this
            });
        }
        this.btn = this.cmp.getToolbar().addButton({
            xtype: 'tbsplit',
            id: 'spellchecker_button',
            itemId: 'spellchecker',
            cls: 'x-btn-icon',
            iconCls: 'x-edit-spellchecker',
            enableToggle: true,
            menu: menu,
            handler: function(){
                this.onShow();
                if (this.btn.pressed) {
                    this.cmp.disableItems(true);
                    this.cmp.tb.items.get('sourceedit').setDisabled(true);
                    this.cmp.tb.items.get('spellchecker').setDisabled(false);
                    this.spellchecker.spellCheck(this.getCurrentLanguage());
                }
                else {
                    this.cmp.disableItems(false);
                    this.spellchecker.resumeEditing();
                }
            },
            scope: this,
            tooltip: {
                title: _(this.langTitle),
                text: _(this.langText)
            },
            overflowText: _(this.langTitle)
        });
        this.spellchecker = new Ext.GoogieSpell("Expressomail/images/editorplugins/", "Expressomail.CheckSpelling",this);

        this.spellchecker.lang_chck_spell = "";
        this.setCurrentLanguage("pt_BR");
    },
    onShow: function(){
        this.spellchecker.setButton(this.btn.btnEl);
        this.spellchecker.setIsHTML(this.cmp.getEditorBody());
        this.spellchecker.setEditor(this.cmp);
        this.spellchecker.setTextarea(this.cmp.getEditorBody());
    }
});

/****
 GoogieSpell
     LICENSE
        Creative Commons Attribution-NonCommercial-ShareAlike
        http://creativecommons.org/licenses/by-nc-sa/3.0/
        To use GoogieSpell in commercial products you need to buy a license from here:
        http://orangoo.com/labs/GoogieSpell/Buy_License/
     AUTHOR
         Amir Salihefendic (http://amix.dk) - amix@amix.dk

 Modified: 19/09/13 10:30:00 
 By: Fernando Lages
****/
function GoogieSpell(img_dir, server_url, parentObj) {
//    var cookie_value;
    var lang;
//    cookie_value = getCookie('language');
//
//    if(cookie_value != null)
//        GOOGIE_CUR_LANG = cookie_value;
//    else
//        GOOGIE_CUR_LANG = GOOGIE_DEFAULT_LANG;

    this.img_dir = img_dir;
    this.server_url = server_url;
    this.parentObj = parentObj;
    this.ta_scroll_top = 0;
    this.el_scroll_top = 0;
    this.edit_layer_dbl_click = true;

    this.lang_chck_spell = "Check spelling";
    this.lang_revert = _("Reverter para"); //_("Revert to")
    this.lang_close = _("Close");
    this.lang_rsm_edt = _("Continuar editando"); //_("Resume editing")
    this.lang_no_error_found = _("Nenhum erro ortográfico encontrado"); //_("No spelling errors found")
    this.lang_no_suggestions = _("Sem sugestões"); //_("No suggestions")
    
    //Counters
    this.cnt_errors = 0;
    this.cnt_errors_fixed = 0;

    //Set document on click to hide the language and error menu
    var fn = function(e) {
        var elm = getEventElm(e);
        if (!isDefined(elm)) return;
        if(elm.googie_action_btn != "1" && this.isErrorWindowShown())
            this.hideErrorWindow();
    };
    GoogieSpell.addEventListener(document,"click",fn.createDelegate(this));
}

GoogieSpell.addEventListener = function(obj,ev,argument) {
    if (Ext.isIE) {
        obj.attachEvent("on"+ev,argument);
    }
    else {
        obj.addEventListener(ev,argument,false);
    }
}

// Request functions
GoogieSpell.escapeSepcial = function(val) {
    return val.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
}

// Spell checking functions
GoogieSpell.prototype.parseResult = function(r_text) {
    //Returns an array
    var re_split_text = /\t/g;

    var matched_c = r_text.results;
    var results = new Array();

    if(matched_c == null)
        return results;
    
    for(var i=0; i < matched_c.length; i++) {
        var item = new Array();
        this.errorFound();

        //Get attributes
        item['attrs'] = {
            'o': matched_c[i].o,
            'l': matched_c[i].l,
            's': matched_c[i].s
        };

        //Get suggestions
        item['suggestions'] = new Array();
        var split_t = matched_c[i].suggestions;
        for(var k=0; k < split_t.length; k++) {
        if(split_t[k] != "")
            item['suggestions'].push(split_t[k]);
        }
        results.push(item);
    }
    return results;
}

// Counters
GoogieSpell.prototype.errorFixed = function() { 
    this.cnt_errors_fixed++; 
}
GoogieSpell.prototype.errorFound = function() {this.cnt_errors++;}

// Error menu functions
GoogieSpell.prototype.createErrorWindow = function() {
    this.error_window = document.createElement('DIV');
    this.error_window.className = "googie_window";
    this.error_window.googie_action_btn = "1";
}

GoogieSpell.prototype.isErrorWindowShown = function() {
    return this.error_window != null && this.error_window.style.visibility == "visible";
}

GoogieSpell.prototype.hideErrorWindow = function() {
    try {
        this.error_window.style.visibility = "hidden";
        if(this.error_window_iframe)
            this.error_window_iframe.style.visibility = "hidden";
    }
    catch(e) {}
}

GoogieSpell.prototype.updateOriginalText = function(offset, old_value, new_value, id) {
    var part_1 = this.original_text.substring(0, offset);
    var part_2 = this.original_text.substring(offset+old_value.length);
    this.original_text = part_1 + new_value + part_2;
    this.setValue(this.text_area, this.original_text);
    var add_2_offset = new_value.length - old_value.length;
    for(var j=0; j < this.results.length; j++) {
        //Don't edit the offset of the current item
        if(j != id && j > id){
            this.results[j]['attrs']['o'] += add_2_offset;
        }
    }
}

GoogieSpell.prototype.saveOldValue = function(elm, old_value) {
    elm.is_changed = true;
    elm.old_value = old_value;
}

GoogieSpell.prototype.correctError = function(id, elm, l_elm, /*optional*/ rm_pre_space) {
    var old_value = elm.innerHTML;
    var new_value = l_elm.innerHTML;
    var offset = this.results[id]['attrs']['o'];

    if(rm_pre_space) {
        var pre_length = elm.previousSibling.innerHTML;
        elm.previousSibling.innerHTML = pre_length.slice(0, pre_length.length-1);
        old_value = " " + old_value;
        offset--;
    }

    this.hideErrorWindow();

    this.updateOriginalText(offset, old_value, new_value, id);

    elm.innerHTML = new_value;
    elm.style.color = "green";
    elm.is_corrected = true;

    this.results[id]['attrs']['l'] = new_value.length;

    if(!isDefined(elm.old_value))
        this.saveOldValue(elm, old_value);
    
    this.errorFixed();
}

GoogieSpell.prototype.showErrorWindow = function(elm, id) {
    var me = this;

    this.error_window.innerHTML = "";

    var table = document.createElement('TABLE');
    table.setAttribute('class', 'googie_list');
    table.googie_action_btn = "1";
    var list = document.createElement('TBODY');

    //Build up the result list
    var suggestions = this.results[id]['suggestions'];
    var offset = this.results[id]['attrs']['o'];
    var len = this.results[id]['attrs']['l'];

    if(suggestions.length == 0) {
        var row = document.createElement('TR');
        var item = document.createElement('TD');
        item.style['cursor'] = 'default';
        var dummy = document.createElement('SPAN');
        dummy.innerHTML = this.lang_no_suggestions;
        AJSappendChildNodes(item, document.createTextNode(dummy.innerHTML));
        item.googie_action_btn = "1";
        row.appendChild(item);
        list.appendChild(row);
    }

    for(i=0; i < suggestions.length; i++) {
        var row = document.createElement('TR');
        var item = document.createElement('TD');
        var dummy = document.createElement('SPAN');
        dummy.innerHTML = suggestions[i];
        item.appendChild(document.createTextNode(dummy.innerHTML));

        var fn = function(e) {
            var l_elm = getEventElm(e);
            this.correctError(id, elm, l_elm);
        };

        GoogieSpell.addEventListener(item,"click", fn.createDelegate(this));

        item.onmouseover = GoogieSpell.item_onmouseover;
        item.onmouseout = GoogieSpell.item_onmouseout;
        row.appendChild(item);
        list.appendChild(row);
    }

//    //Append the edit box
//    var edit_row = document.createElement('DIV');
//    edit_row.style['cursor'] = 'default';
//
//    var edit_input = document.createElement('INPUT');
//    edit_input.style['width'] = '120px'; 
//    edit_input.style['margin'] = '0'; 
//    edit_input.style['padding'] = '0';
//    edit_input.style['value'] = elm.innerHTML;
//    edit_input.googie_action_btn = "1";
//
//    var onsub = function () {
//        if(edit_input.value != "") {
//            if(!isDefined(elm.old_value))
//                this.saveOldValue(elm, elm.innerHTML);
//
//            this.updateOriginalText(offset, elm.innerHTML, edit_input.value, id);
//            elm.style.color = "green";
//            elm.is_corrected = true;
//            elm.innerHTML = edit_input.value;
//
//            this.hideErrorWindow();
//        }
//        return false;
//    };
//
//    var ok_pic = document.createElement('IMG');
//    ok_pic.setAttribute('src', this.img_dir + "ok.gif");
//    ok_pic.style['width'] = '32px'; 
//    ok_pic.style['height'] = '16px'; 
//    ok_pic.style['margin-left'] = '2px'; 
//    ok_pic.style['margin-right'] = '2px'; 
//    ok_pic.style['cursor'] = 'pointer';
//    var edit_form = document.createElement('FORM');
//    edit_form.style['margin'] = '0'; 
//    edit_form.style['padding'] = '0';
//    edit_form.style['cursor'] = 'default;'; 
//    edit_form.appendChild(edit_input);
//    edit_form.appendChild(ok_pic);
//
//    edit_form.googie_action_btn = "1";
//
//    GoogieSpell.addEventListener(edit_form,"submit", onsub.createDelegate(this));
//    GoogieSpell.addEventListener(ok_pic,"click", onsub.createDelegate(this));
//
//    edit_row.appendChild(edit_form);

    table.appendChild(list);
    var div_list = document.createElement('DIV');
    div_list.setAttribute('class', 'googie_divlist');
    div_list.appendChild(table);
    this.error_window.appendChild(div_list);
//    this.error_window.appendChild(edit_row);
    
    //The element has changed, append the revert
    if(elm.is_changed && elm.innerHTML != elm.old_value) {
        var old_value = elm.old_value;

        var fn = function(e) { 
            this.updateOriginalText(offset, elm.innerHTML, old_value, id);
            elm.is_corrected = true;
            elm.style.color = "#b91414";
            elm.innerHTML = old_value;
            this.hideErrorWindow();
        };
        AJSappendChildNodes(this.error_window, this.createRevertButton(fn, old_value));

    }

    //Close button
    AJSappendChildNodes(this.error_window, this.createCloseButton(this.hideErrorWindow));
    var max_height = this.edit_layer.style.height || this.edit_layer.clientHeight;
    max_height = (max_height>this.error_window.clientHeight ? this.error_window.clientHeight : max_height);
    div_list.style.maxHeight = "180px";
    if (this.error_window.offsetTop+this.error_window.clientHeight > Ext.getBody().dom.clientHeight) {
        this.error_window.style.top = (this.error_window.offsetTop - this.error_window.clientHeight - 20)+"px";
    }

    //Dummy for IE - dropdown bug fix
    if((Ext.isIE || Ext.isIE9) && !this.error_window_iframe) {
        var iframe = document.createElement('IFRAME');
        iframe.style['position'] = 'absolute'; 
        iframe.style['z-index'] = '0';
        AJSappendChildNodes(Ext.getBody(), iframe);
        this.error_window_iframe = iframe;
    }
    if(Ext.isIE || Ext.isIE9) {
        var iframe = this.error_window_iframe;
        iframe.style['top'] = this.error_window.offsetTop;
        iframe.style['left'] = this.error_window.offsetLeft;

        iframe.style['width'] = this.error_window.offsetWidth;
        iframe.style['height'] = this.error_window.offsetHeight;

        iframe.style.visibility = "visible";
    }

    //Set focus on the last element --
    var link = this.createFocusLink('link');
    var tr1 = document.createElement('TR');
    var td1 = document.createElement('TD');
    td1.style['text-align'] = 'right'; 
    td1.style['font-size'] = '1px'; 
    td1.style['height'] = '1px'; 
    td1.style['margin'] = '0'; 
    td1.style['padding'] = '0';
    td1.appendChild(link);
    tr1.appendChild(td1);
    list.appendChild(tr1);

    var abs_pos = absolutePosition(elm);
    abs_pos.y -= this.edit_layer.scrollTop;
    this.error_window.style.visibility = "visible";

    var top_pos = abs_pos.y+20;
    if (top_pos+150>this.edit_layer.clientOffset) {
        top_pos = this.edit_layer.clientOffset - 150;
    }
    this.error_window.style.top = (top_pos)+'px';
    this.error_window.style.left = (abs_pos.x)+'px';
    link.focus();

    div_list.scrollTop = 0;
}

// Edit layer (the layer where the suggestions are stored)
GoogieSpell.prototype.createEditLayer = function(width, height) {
    this.edit_layer = document.createElement('DIV');
    this.edit_layer.setAttribute('class','googie_edit_layer');

    //Set the style so it looks like edit areas
    this.edit_layer.className = this.editor.getEditorBody().className;
    this.edit_layer.style.fontFamily = this.editor.getEditorBody().style.fontFamily;
    this.edit_layer.style.fontSize = this.editor.getEditorBody().style.fontSize;
    this.edit_layer.style.border = "none";
    this.edit_layer.style.backgroundColor = "#fff";
    this.edit_layer.style.padding = "3px";
    this.edit_layer.style.margin = "0px";
    this.edit_layer.style.padding = this.editor.getEditorBody().style.padding;
    this.edit_layer.style.margin = this.editor.getEditorBody().style.margin;

    this.edit_layer.style.width = (width-8)+'px';

    if(this.text_area.nodeName.toLowerCase() != "input" || this.getValue(this.text_area) == "") {
        this.edit_layer.style.overflow = "auto";
        this.edit_layer.style.height = (height-6)+'px';
    }
    else {
        this.edit_layer.style.overflow = "hidden";
    }

    if(this.edit_layer_dbl_click) {
        var me = this;
        var fn = function(e) {
            if(getEventElm(e).className != "googie_link" && !me.isErrorWindowShown()) {
                me.resumeEditing();
                var fn1 = function() {
                    me.text_area.focus();
                    fn1 = null;
                };
                setTimeout(fn1, 10);
            }
            return false;
        };
        this.edit_layer.ondblclick = fn;
        fn = null;
    }
}

GoogieSpell.prototype.resumeEditing = function() {
    this.switch_lan_pic.style.display = "inline";

    if(this.edit_layer)
        this.el_scroll_top = this.edit_layer.scrollTop;

    this.hideErrorWindow();

    this.spell_span.className = "googie_no_style";

    if(!this.ignore) {
        //Remove the EDIT_LAYER
        try {
            this.edit_layer.parentNode.removeChild(this.edit_layer);
        }
        catch(e) {
        }

        this.editor.getContainer().style.display = '';

        if(this.el_scroll_top != undefined)
            this.text_area.scrollTop = this.el_scroll_top;
    }

    this.checkSpellingState(false);
}

GoogieSpell.prototype.createErrorLink = function(text, id) {
    var elm = document.createElement('SPAN');
    elm.setAttribute('class', 'googie_link');

    var me = this;
    var d = function (e) {
        me.showErrorWindow(elm, id);
        d = null;
        return false;
    };
    GoogieSpell.addEventListener(elm,"click",d);

    elm.googie_action_btn = "1";
    elm.g_id = id;
    elm.is_corrected = false;
    elm.oncontextmenu = d;
    elm.innerHTML = text;
    return elm;
}

GoogieSpell.createPart = function(txt_part) {
    if(txt_part == " ")
        return document.createTextNode();
    var result = document.createElement('SPAN');

    var is_first = true;
    var is_safari = (navigator.userAgent.toLowerCase().indexOf("safari") != -1);

    var part = document.createElement('SPAN');
    txt_part = GoogieSpell.escapeSepcial(txt_part);
    txt_part = txt_part.replace(/\n/g, "<br>");
    txt_part = txt_part.replace(/    /g, " &nbsp;");
    txt_part = txt_part.replace(/^ /g, "&nbsp;");
    txt_part = txt_part.replace(/ $/g, "&nbsp;");
    
    part.innerHTML = txt_part;

    return part;
}

GoogieSpell.prototype.showErrorsInIframe = function() {
    var output = document.createElement('DIV');
    output.style.textAlign = "left";
    var pointer = 0;
    var results = this.results;

    if(results.length > 0) {
        // HTML support
        var outputBuffer = '';
        for(var i=0; i < results.length; i++) {
            var offset = results[i]['attrs']['o'];
            var len = results[i]['attrs']['l'];
            
            var part_1_text = this.original_text.substring(pointer, offset);
            // HTML support
            if (this.getIsHTML()) {
                outputBuffer += part_1_text;
            } else {
                var part_1 = GoogieSpell.createPart(part_1_text);
                output.appendChild(part_1);
            }
            pointer += offset - pointer;
            
            //If the last child was an error, then insert some space
            var err_link = this.createErrorLink(this.original_text.substr(offset, len), i);
            this.error_links.push(err_link);
            // HTML support
            if (this.getIsHTML()) {
                // placeholder for insertErrLinks()
                outputBuffer += '<var class="err_hook">&nbsp;</var> ';
            } else {
                output.appendChild(err_link);
            }
            pointer += len;
        }
        //Insert the rest of the original text
        var part_2_text = this.original_text.substr(pointer, this.original_text.length);

        // HTML support
        if (this.getIsHTML()) {
            outputBuffer += part_2_text;
            output.innerHTML = outputBuffer;
            this.insertErrLinks(output, this.error_links);
        } else {
            var part_2 = GoogieSpell.createPart(part_2_text);
            output.appendChild(part_2);
        }
    }
    else
        output.innerHTML = this.original_text;

    AJSappendChildNodes(this.edit_layer,output);

    //Hide text area
    this.text_area_bottom = this.text_area.offsetTop + this.text_area.offsetHeight;

    this.editor.getContainer().style['display'] = 'none';

    this.editor.getContainer().parentNode.insertBefore(this.edit_layer, this.editor.getContainer());

    this.edit_layer.scrollTop = this.ta_scroll_top;
}

// Choose language menu
GoogieSpell.prototype.createSpellDiv = function() {
    var chk_spell = document.createElement('SPAN');
    chk_spell.setAttribute('class','googie_check_spelling_link');

    chk_spell.innerHTML = this.lang_chck_spell;
    var spell_img = null;
    spell_img = document.createElement('IMG');
    spell_img.setAttribute('src',this.img_dir + "spellc.gif");
    var span = document.createElement('SPAN');
    span.innerHTML = spell_img + chk_spell;
    return span;
}

// State functions
GoogieSpell.prototype.flashNoSpellingErrorState = function() {
    Ext.getCmp('spellchecker_button').toggle();
    this.parentObj.cmp.disableItems(false);
    this.resumeEditing();
    Ext.MessageBox.alert(_('Spell Checker'), _('Nenhum erro ortográfico encontrado')); //_("No spelling errors found")
}

GoogieSpell.prototype.resumeEditingState = function() {
    //Change link text to resume
    this.switch_lan_pic.style['display'] = 'none';
    var dummy = document.createElement('IMG');
    dummy.setAttribute('src', this.img_dir + "blank.gif");
    dummy.style['height'] = '16px'; 
    dummy.style['width'] = '1px';
    var rsm = document.createElement('SPAN');
    rsm.innerHTML = this.lang_rsm_edt;
    var span = document.createElement('SPAN');
    span.appendChild(dummy);
    span.appendChild(rsm);
    replaceChildNodes(this.spell_span, span);

    var fn = function(e) {
        this.resumeEditing();
    }
    this.spell_span.onclick = fn.createDelegate(this);

    this.spell_span.className = "googie_resume_editing";

    try {this.edit_layer.scrollTop = this.ta_scroll_top;}
    catch(e) { }
}

GoogieSpell.prototype.checkSpellingState = function(fire) {
    this.switch_lan_pic = document.createElement('SPAN');

    var span_chck = this.createSpellDiv();
    var fn = function() {
        this.spellCheck();
    };

    span_chck.onclick = fn.createDelegate(this);

    this.spell_span = span_chck;
    replaceChildNodes(this.spell_container, span_chck, document.createElement('SPAN'), this.switch_lan_pic);
}

// Misc. functions
GoogieSpell.item_onmouseover = function(e) {
    var elm = getEventElm(e);
    if(elm.className != "googie_list_revert" && elm.className != "googie_list_close")
        elm.className = "googie_list_onhover";
    else
        elm.parentNode.className = "googie_list_onhover";
}
GoogieSpell.item_onmouseout = function(e) {
    var elm = getEventElm(e);
    if(elm.className != "googie_list_revert" && elm.className != "googie_list_close")
        elm.className = "googie_list_onout";
    else
        elm.parentNode.className = "googie_list_onout";
}

GoogieSpell.prototype.createRevertButton = function(c_fn, old_value) {
    return this.createButton(this.lang_revert + " " + old_value, 'googie_list_revert', c_fn.createDelegate(this));
}

GoogieSpell.prototype.createCloseButton = function(c_fn) {
    return this.createButton(this.lang_close, 'googie_list_close', c_fn.createDelegate(this));
}

GoogieSpell.prototype.createButton = function(name, css_class, c_fn) {
    var btn = document.createElement('DIV');

    btn.onmouseover = GoogieSpell.item_onmouseover;
    btn.onmouseout = GoogieSpell.item_onmouseout;

    var spn_btn;
    if(css_class != "") {
        spn_btn = document.createElement('SPAN');
        spn_btn.setAttribute('class', css_class);
        spn_btn.innerHTML = name;
    }
    else {
        spn_btn = document.createTextNode(name);
    }
    btn.appendChild(spn_btn);
    GoogieSpell.addEventListener(btn,"click", c_fn);

    return btn;
}

GoogieSpell.prototype.createFocusLink = function(name) {
    var a1 = document.createElement('A');
    a1.setAttribute('href', 'javascript:;');
    a1.setAttribute('name', 'name');
    return a1;
}

function setCookie(name, value, expires, path, domain, secure) {
  var curCookie = name + "=" + escape(value) +
      ((expires) ? "; expires=" + expires.toGMTString() : "") +
      ((path) ? "; path=" + path : "") +
      ((domain) ? "; domain=" + domain : "") +
      ((secure) ? "; secure" : "");
  document.cookie = curCookie;
}

function getCookie(name) {
  var dc = document.cookie;
  var prefix = name + "=";
  var begin = dc.indexOf("; " + prefix);
  if (begin == -1) {
    begin = dc.indexOf(prefix);
    if (begin != 0) return null;
  } else
    begin += 2;
  var end = document.cookie.indexOf(";", begin);
  if (end == -1)
    end = dc.length;
  return unescape(dc.substring(begin + prefix.length, end));
}

function getKeys(obj) {
    var keys = [];
    for (var prop in obj) {
        keys.push(prop);
    }
    return keys;
}

function getValues(obj) {
    var values = [];
    for (var prop in obj) {
        values.push(obj[prop]);
    }
    return values;
}

function isDefined(o) {
    return (o != "undefined" && o != null)
}

function getEventElm(e) {
    if(e && !e.type && !e.keyCode)
        return e;
    var targ;
    if (!e) var e = window.event;
    if (!e) return e;
    if (e.target) 
        targ = e.target;
    else if (e.srcElement) 
        targ = e.srcElement;
    if (targ && targ.nodeType == 3) // defeat Safari bug
        targ = targ.parentNode;
    return targ;
}

function map(list, fn,/*optional*/ start_index, end_index) {
    var i = 0, l = list.length;
    if(start_index)
            i = start_index;
    if(end_index)
            l = end_index;
    for(i; i < l; i++) {
        var val = fn(list[i], i);
        if(val != undefined)
            return val;
    }
}

function isString(obj) {
    return (typeof obj == 'string');
}

function isNumber(obj) {
    return (typeof obj == 'number');
}

function AJSappendChildNodes(elm/*, elms...*/) {
    if(arguments.length >= 2) {
        map(arguments, function(n) {
            if(isString(n))
                n = document.createTextNode(n);
            if(isDefined(n))
                elm.appendChild(n);
        }, 1);
    }
    return elm;
}

function replaceChildNodes(elm/*, elms...*/) {
    var child;
    while ((child = elm.firstChild))
        swapDOM(child, null);

    if (arguments.length < 2)
        return elm;
    else
        return AJSappendChildNodes.apply(null, arguments);
    return elm;
}

function swapDOM(dest, src) {
    dest = getElement(dest);
    var parent = dest.parentNode;
    if (src) {
        src = getElement(src);
        parent.replaceChild(src, dest);
    } else {
        parent.removeChild(dest);
    }

    return src;
}

function getElement(id) {
    if(isString(id) || isNumber(id))
        return document.getElementById(id);
    else
        return id;
}

function absolutePosition(elm) {
    if(!elm)
        return {x: 0, y: 0};

    if(elm.scrollLeft)
        return {x: elm.scrollLeft, y: elm.scrollTop};
    else if(elm.clientX)
        return {x: elm.clientX, y: elm.clientY};

    var posObj = {'x': elm.offsetLeft, 'y': elm.offsetTop};

    if(elm.offsetParent) {
        var next = elm.offsetParent;
        while(next) {
            posObj.x += next.offsetLeft;
            posObj.y += next.offsetTop;
            next = next.offsetParent;
        }
    }
    // safari bug
    if (Ext.isSafari && elm.style.position == 'absolute' ) {
        posObj.x -= document.body.offsetLeft;
        posObj.y -= document.body.offsetTop;
    }
    return posObj;
}

// extends GoogieSpell
Ext.GoogieSpell = Ext.extend(GoogieSpell, {
    editor: null,
    setButton: function(btn) {
        this.btnEl = btn;
    },
    setEditor: function(ed){
        this.editor = ed;
    },
    setTextarea: function(id){
        if(typeof(id) == "string") {
            this.text_area = Ext.get(id);
        }
        else {
            this.text_area = id;
        }

        var r_width, r_height;

        if(this.text_area != null) {
            if(!Ext.isDefined(this.spell_container)) {
                var spell_container = new Ext.Element(document.createElement('TD'));
                this.spell_container = spell_container;
            }
            this.checkSpellingState();
        }
        else {
            Ext.MessageBox.alert(_('Errors'), _("Área de edição não encontrada."));
        }
    },
    getEditAreaValue: function(ta){
        return (ta.value ? ta.value : this.editor.getValue());
    },
    getValue: function(ta) {
        return this.getEditAreaValue(ta);
    },
    setValue: function(ta, value) {
        if (this.getIsHTML()) {
            this.editor.setValue(value);
        } 
        else {
            ta.value = value;
        }
    },
    spellCheck: function(lang) {
        var me = this;

        this.cnt_errors_fixed = 0;
        this.cnt_errors = 0;

        this.original_text = this.getValue(this.text_area);
        
        this.btnEl.addClass("loading-indicator");

        this.error_links = [];
        this.ta_scroll_top = this.text_area.scrollTop;

        this.createEditLayer(this.editor.getContainer().clientWidth,this.editor.getContainer().offsetHeight);

        this.createErrorWindow();
        Ext.getBody().appendChild(this.error_window);

        try {netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");} 
        catch (e) { }

        this.spell_span.onclick = null;

        this.document_fragment = document.createDocumentFragment();
        var div = document.createElement('div');
        div.innerHTML = this.original_text;
        this.document_fragment.appendChild(div);

        //Create request
        var params = {
            application: 'Expressomail',
            method: 'Expressomail.checkSpelling',
            lang: lang,
            data: this.original_text
        };
        Ext.Ajax.request({
            params: params,
            scope: this,
            success: function(res_txt) {
                var r_text = Ext.util.JSON.decode(res_txt.responseText);
                this.btnEl.removeClass("loading-indicator");
                if (r_text.error == 0) {
                    me.results = me.parseResult(r_text);

                    if(me.results != null && me.results.length>0) {
                        //Before parsing be sure that errors were found
                        me.showErrorsInIframe();
                        me.resumeEditingState();
                    }
                    else {
                        me.flashNoSpellingErrorState();
                    }
                }
                else {
                    var err_desc = r_text.description ? r_text.description : _("Erro inesperado na verificação ortográfica. Tente novamente mais tarde.");
                    Ext.MessageBox.alert(_('Spell Checker'), err_desc);
                }
                this.btnEl.dom.style.backgroundImage = '';
            },
            failure: function(res_txt, req) {
                this.btnEl.removeClass("loading-indicator");
                Ext.MessageBox.alert(_('Spell Checker'), _("Um erro ocorreu no servidor. Tente novamente mais tarde."));

                me.checkSpellingState();
                this.btnEl.dom.style['background-image'] = btn_image;
            }
        });
    },
    //GoogieSpell HTML support / switch between modes
    isHTML: false,
    setIsHTML: function(el) {
        var origSupport = new RegExp('input|textarea', 'i');
        this.isHTML = origSupport.test(el.nodeName) ? false : true;
    },
    getIsHTML: function() {
        return this.isHTML;
    },
    insertErrLinks: function(output, err_links) {
        var all_vars = output.getElementsByTagName('var');
        var all_vars_len = all_vars.length;
        var err_count = 0;
        var err_len = err_links.length;
        for (var i = 0; i < all_vars_len; i++) {
            if (all_vars[i].className == 'err_hook') {
                //	Replace &nbsp; with span.    is needed for IE implementation of innerHTML
                all_vars[i].replaceChild(err_links[err_count], all_vars[i].firstChild);
                err_count++;
                if (err_count == err_len) break;
            }
        }
    }
});
/**
 * @author Fernando Lages
 * @class Ext.ux.form.HtmlEditor.TextAlign
 * @extends Ext.util.Observable
 * <p>A plugin that creates a menu on the HtmlEditor to align paragraphs.</p>
 */
Ext.ux.form.HtmlEditor.TextAlign = Ext.extend(Ext.util.Observable, {
    // Table language text
    alignTitle       : 'Text Alignment',
    alignText        : 'Align the text',

    // private
    init: function(cmp){
        this.cmp = cmp;
        this.cmp.on('render', this.onRender, this);
        this.cmp.on('initialize', this.onInitialize, this);
    },

    // private
    onInitialize : function(){
        // Destroying the component during/before initEditor can cause issues.
        try{
            var dbody = this.cmp.getEditorBody(),
                doc,
                fn;

            doc = this.cmp.getDoc();

            /*
             * We need to use createDelegate here, because when using buffer, the delayed task is added
             * as a property to the function. When the listener is removed, the task is deleted from the function.
             * Since onEditorEvent is shared on the prototype, if we have multiple html editors, the first time one of the editors
             * is destroyed, it causes the fn to be deleted from the prototype, which causes errors. Essentially, we're just anonymizing the function.
             */
            fn = this.onEditorClick.createDelegate(this);
            Ext.EventManager.on(doc, {
                mousedown: fn,
                dblclick: fn,
                click: fn,
                keyup: fn,
                buffer:100
            });
        }
        catch(e){}
    },

    // private
    onRender: function(){
        var aligns = {"Left": "Left align", "Center": "Center align", "Right": "Right align", "Full": "Fully justify"};
        var menu = [];
        var checked = true;

        for (var align in aligns) {
            menu.push({
                itemId : 'justify'+align.toLowerCase(),
                enableToggle: true,
                handler: this.cmp.relayBtnCmd,
                text: this._(aligns[align]),
                hideLabel: true,
                iconCls: 'x-edit-justify'+align.toLowerCase(),
                group: 'alignment',
                pressed: true,
                toggle: this.toggleAlignButton,
                scope: this.cmp
            });
        }
        this.btn = this.cmp.getToolbar().addButton({
            xtype: 'tbsplit',
            id: 'textalign_button',
            itemId: 'textalign',
            cls: 'x-btn-icon',
            iconCls: 'x-edit-justifyleft',
            enableToggle: true,
            menu: { items: menu },
            arrowHandler: function(){
                this.onEditorEvent();
            },
            hideBorders: false,
            hideLabel: true,
            showSeparator: true,
            handler: function(){
                this.onEditorEvent();
            },
            scope: this,
            tooltip: {
                title: this._(this.alignTitle),
                text: this._(this.alignText)
            }
        });
    },
    // private
    onEditorClick: function(e) {
        var doc = this.cmp.getDoc();
        Ext.each(this.btn.menu.items, function(b, i){
            b = this.btn.menu.items.get(i);
            if (doc.queryCommandState(b.itemId)) {
                this.btn.btnEl.dom.className = 'x-btn-text x-edit-'+b.itemId;
                this.btn.iconCls = 'x-edit-'+b.itemId;
            }
        }, this);
    },
    // private
    onEditorEvent: function(e) {
        if (e) {
            this.btn.btnEl.dom.className = 'x-btn-text x-edit-'+e.itemId;
            this.btn.iconCls = 'x-edit-'+e.itemId;
        }
        var doc = this.cmp.getDoc();
        Ext.each(this.btn.menu.items, function(b, i){
            b = this.btn.menu.items.get(i);
            b.toggle(doc.queryCommandState(b.itemId));
        }, this);
    },
    /**
     * If a state it passed, it becomes the pressed state otherwise the current state is toggled.
     * @param {Boolean} state (optional) Force a particular state
     * @return {Ext.Button} this
     */
    toggleAlignButton : function(state){
        state = state === undefined ? !this.pressed : !!state;
        if(state !== this.pressed || true) {
            if(this.rendered){
                this.el[state ? 'addClass' : 'removeClass']('x-btn-pressed');
                this.el[state ? 'addClass' : 'removeClass']('x-menu-item-selected');
            }
            this.fireEvent('toggle', this, state);
            if(this.toggleHandler){
                this.toggleHandler.call(this.scope || this, this, state);
            }
        }
        return this;
    },
    // private
    /**
     * Shortcut for translation on Expressomail's scope
     * @param {String} msgid - string to be translated
     * @returns {String} translated string
     */
    _ : function(msgid) {
        return this.cmp.messageEdit.app.i18n._(msgid);
    }
});

Ext.ns('Tine.Expressomail');

Tine.Expressomail.ReadConfirmationRecord = Tine.Tinebase.data.Record.create([
   {name: 'id', type: 'int'},
   {name: 'to'},
   {name: 'subject'},
   {name: 'from_name'},
   {name: 'from_email'},
   {name: 'received', type: 'date', dateFormat: Date.patterns.ISO8601Long},
   {name: 'read_time', type: 'date', dateFormat: Date.patterns.ISO8601Long}
   
], {
    appName: 'Expressomail',
    modelName: 'ReadConfirmation',
    idProperty: 'id'
});

Tine.Expressomail.ReadConfirmationRecordProxy  = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressomail',
    modelName: 'ReadConfirmation',
    recordClass: Tine.Expressomail.ReadConfirmationRecord
});



/**
 * read confirmation grid panel
 * 
 * @namespace   Tine.Expressomail
 * @class       Tine.Expressomail.ReadConfirmationDetailsPanel
 * @extends     Tine.widgets.grid.DetailsPanel
 * 
 * <p>Tinebase Expressomail ReadConfirmationDetailsPanel</p>
 * <p><pre>
 * </pre></p>
 * 
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * 
 * Create a new Tine.Expressomail.ReadConfirmationDetailsPanel
 * 
 */

Tine.Expressomail.ReadConfirmationDetailsPanel = Ext.extend(Tine.widgets.grid.DetailsPanel, {

    
    /**
     * @cfg {Object} preparedPart
     * server prepared text/webconference Invite part 
     */
    preparedPart: null,
    
    
    /**
     * @property statusActions
     * @type Array
     */
    statusActions:[],
    
    /**
     * init this component
     */
    initComponent: function() {
        
        this.app = Tine.Tinebase.appMgr.get('Expressomail');
        this.record = Tine.Expressomail.ReadConfirmationRecordProxy.recordReader({
                        responseText: Ext.util.JSON.encode(this.preparedPart.preparedData)
                    });
        
        this.initToolbar();

        this.on('afterrender', this.showEmail, this);

        Tine.Expressomail.ReadConfirmationDetailsPanel.superclass.initComponent.call(this);
    },
    
    /**
     * Invite action toolbar
     */
    initToolbar: function() {
        var singleRecordPanel = this.getSingleRecordPanel();
        
        this.actions = [];
        this.statusActions = [];
        
        
        this.statusActions.push (this.acceptAction);
        this.actions = this.actions.concat(this.statusActions);
        
        // add more actions here (no spam / apply / crush / send event / ...)
        
        this.clause = new Ext.Toolbar.TextItem({
            text: ''
        });
        this.tbar = new Ext.Toolbar({
            items: [{
                xtype: 'tbitem',
                cls: 'ExpressomailIconCls',
                width: 16,
                height: 16,
                style: 'margin: 3px 5px 2px 5px;'
            },
            this.clause,
            '->'
            ]//.concat(this.actions)
        });
    },
    
    /**
     * show/layout Email panel
     */
    showEmail: function() {
        
        
        var singleRecordPanel = this.getSingleRecordPanel();
        
        this.clause.setText(this.app.i18n._('Reading Confirmation:') + ' ' + this.record.get('subject'));
        
        this.getLoadMask().hide();
        singleRecordPanel.setVisible(true);
        singleRecordPanel.setHeight(150);
        
        singleRecordPanel.loadRecord(this.record);
    },
    
  
    createOffset: function() {
        var date = new Date();
        var sign = (date.getTimezoneOffset() > 0) ? "-" : "+";
        var offset = Math.abs(date.getTimezoneOffset());
        var hours = Math.floor(offset / 60);
            hours = hours < 10 ? '0' + hours : hours;
        var minutes = offset % 60;
            minutes = minutes < 10 ? '0' + minutes : minutes;
        return '(GMT'+sign + hours + ":" + minutes + ')';
    },
    
    /**
     * renders datetime in user browser timezone
     * 
     * @param {Date} dt
     * @return {String}
     */
    datetimeRenderer: function(dt) {
        var offset = new Date().getTimezoneOffset();
        dt.setTime(dt.getTime() - (offset * 60 * 1000));
  
        return Tine.Tinebase.common.dateRenderer(dt) + ' ' + dt.format('H:i:s') + ' ' + this.createOffset();
    },
    /**
     * main email details panel
     * 
     * @return {Ext.ux.display.DisplayPanel}
     */
    getSingleRecordPanel: function() {
        if (! this.singleRecordPanel) {
	    var _this = this;
            this.singleRecordPanel = new Ext.ux.display.DisplayPanel ({

                layout: 'fit',
                border: false,
                items: [{
                    layout: 'vbox',
                    border: false,
                    layoutConfig: {
                        align:'stretch'
                    },
                    items: [{
                        layout: 'hbox',
                        flex: 0,
                        height: 16,
                        border: false,
                        style: 'padding-left: 5px; padding-right: 5px',
                        layoutConfig: {
                            align:'stretch'
                        },
                        items: []
                    }, {
                        layout: 'hbox',
                        flex: 1,
                        border: false,
                        layoutConfig: {
                            padding:'5',
                            align:'stretch'
                        },
                        defaults:{
                            margins:'0 5 0 0'
                        },
                        items: [
                            {
                                flex: 2,
                                layout: 'ux.display',
                                labelWidth: 100,
                                layoutConfig: {
                                    background: 'solid'
                                },
                                items: [
                                    {
                                        xtype: 'label',
                                        html: this.app.i18n._('Your message:') + ' ' + this.record.get('subject')
                                    },
                                    {
                                        xtype: 'label',
                                        html:'<br/>' + this.app.i18n._('Received on') + ' ' + this.datetimeRenderer(this.record.get('received'))
                                    },
                                    {
                                        xtype: 'label',
                                        html: '<br/>' + this.app.i18n._('Was read by:') + ' ' + this.record.get('from_name')  +  ' < ' + this.record.get('from_email') +' > ' + this.app.i18n._('on') + ' ' + this.datetimeRenderer(this.record.get('read_time'))
                                    }
                                ]
                            }
                            ]
                    }]
                }]
            });
        }
        
        return this.singleRecordPanel;
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressomail
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Fernando Lages <fernando.lages@serpro.gov.br>
 * @copyright   Copyright (c) 2009-2014 Serpro (http://serpro.gov.br)
 */

/**
 *
 * @class Tine.MultiWindow
 * @extends Ext.Window
 * for now it works pretty much the same as Ext.Window
 * but implements a notification tool on top of window
 * TODO: implement multiple windows feature
 */
Tine.MultiWindow = Ext.extend(Ext.Window, {
    notifiable : false,

    /**
     * initialize tools
     */
    initTools : function(){
        if(this.headerAsText && this.notifiable){
            this.addTool({
                id: 'notify',
                handler: this.notify.createDelegate(this, [])
            });
        }
        Tine.MultiWindow.superclass.initTools.call(this);
    },

    /**
     * fire event notify
     */
    notify : function(){
        this.fireEvent('notify', this);
        return this;
    },

    /**
     * set tools notify
     */
    setNotify : function(notify, what){
        if(this.header && this.headerAsText && this.notifiable) {
            if (what) {
                notify_class = 'x-tool-notify-'+what;
            }
            else {
                if (notify==='') {
                    notify_class = 'x-tool-notify-clear';
                }
                else {
                    notify_class = 'x-tool-notify-default';
                }
            }
            this.tools.notify.removeClass('x-tool-notify-clear');
            this.tools.notify.removeClass('x-tool-notify-default');
            this.tools.notify.removeClass('x-tool-notify-error');
            this.tools.notify.addClass(notify_class);
            this.tools.notify.dom.innerHTML = notify;
            this.fireEvent('notifychange', this, notify);
        }
        return this;
    }

});

Ext.override(Ext.ux.WindowFactory, {
    /**
     * @private
     */
    getExtWindow: function (c) {
        // add titleBar
        c.height = c.height + 20;
        // border width
        c.width = c.width + 16;

        //limit the window size
        c.height = Math.min(Ext.getBody().getBox().height, c.height);
        c.width = Math.min(Ext.getBody().getBox().width, c.width);

        c.layout = c.layout || 'fit';
        c.items = {
            layout: 'card',
            border: false,
            activeItem: 0,
            isWindowMainCardPanel: true,
            items: [this.getCenterPanel(c)]
        }

        // we can only handle one window yet
        c.modal = true;

        // do not allow drag&drop outside container limits
        c.constrain = true;

        var win = new Tine.MultiWindow(c);
        c.items.items[0].window = win;

        // if initShow property is present and it is set to false don't show window, just return reference
        if (c.hasOwnProperty('initShow') && c.initShow === false) {
            return win;
        }

        win.show();
        return win;
    }
});